(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    838091,      22218]
NotebookOptionsPosition[    830907,      21973]
NotebookOutlinePosition[    831316,      21991]
CellTagsIndexPosition[    831273,      21988]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"Copyright", " ", "2013", " ", "Thomas", " ", "Trogdon"}], " ", 
   "*)"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
   "This", " ", "software", " ", "is", " ", "distributed", " ", "under", " ", 
    "the", " ", "terms", " ", "of", " ", "the", " ", "GNU", " ", "General", 
    " ", "Public", " ", "License"}], " ", "*)"}]}]], "Input"],

Cell[CellGroupData[{

Cell["Documentation", "Section",
 InitializationGroup->True,
 CellChangeTimes->{{3.513708165818142*^9, 3.51370816752608*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Needs", "[", "\"\<RiemannHilbert`\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"HillKdV", "::", "usage"}], "=", 
   "\"\<HillKdV[q,n] gives the eigenvalues of the modified Zakharov-Shabat \
scattering (including continuous spectrum) problem with n Fourier modes using \
Hill's method.\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"LocatePolesKdV", "::", "usage"}], "=", 
   "\"\<LocatePoles[q,n] gives the true eigenvalues of the modified \
Zakharov-Shabat scattering problem.\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"KdV", "::", "usage"}], "=", 
   "\"\<KdV[i][x,t] = {out,\[CapitalPhi],rhp1,rhp2,timestring} where out is \
the solution of KdV, \[CapitalPhi] is the solution of the Riemann-Hilbert \
problem, rhp1 and rhp2 are the Riemann-Hilbert problems and timestring is the \
string of computation times.  This is done using the deformation in Region i.\
\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"KdVAuto", "::", "usage"}], "=", 
  "\"\<KdVAuto[x,t] returns the solution of the KdV equation with deformation \
selected automagically.  Many things must be initialized.  Sample code:\n\n\t\
q[x_]:=.1 Exp[-x^2];\[IndentingNewLine]Defocusing[];\n\t\
H=ScatteringMatrixFiniteKdV[q,50,6];\[IndentingNewLine]aa//Clear;bb//Clear;\
\[IndentingNewLine]aa[k_]:=aa[k]=H[k][[1,1]];\[IndentingNewLine]bb[k_]:=bb[k]=\
H[k][[2,1]];\n\t\
SetParams[.6,.1,10.^(-9),15,30];\[IndentingNewLine]h[k_]:=3/(1+Abs[k/2+1/3.2]^\
8);\[IndentingNewLine]Setrsamp[h];\[IndentingNewLine]Settimeflag[False];\
\[IndentingNewLine]\[Nu]=Getnu[];\[IndentingNewLine]\[Rho][k_]:=bb[k]/aa[k];\n\
\tup=I(\[Nu]+.0001);m=40;el=8;\[IndentingNewLine]f={Fun[\[Rho],Line[{-el,0}-\
up],m],Fun[\[Rho],Line[{0,el}-up],m],Fun[\[Rho],Line[{el,0}+up],m],Fun[\[Rho],\
Line[{0,-el}+up],m]};\n\n\t\
up=I(\[Nu]+.0001);m=40;el=8;\[IndentingNewLine]f={Fun[\[Rho],Line[{-el,0}-up],\
m],Fun[\[Rho],Line[{0,el}-up],m],Fun[\[Rho],Line[{el,0}+up],m],Fun[\[Rho],\
Line[{0,-el}+up],m]};\n\
SetScatteringData[aa,bb,m\[Rho],LocatePoles[q,40]]\>\""}], "\
\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Focusing", "::", "usage"}], "=", 
   "\"\<Focusing[], switch to focusing KdV\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Defocusing", "::", "usage"}], "=", 
   "\"\<Defocusing[], switch to focusing KdV\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"SetScatteringData", "::", "usage"}], "=", 
   "\"\<SetScatteringData[a,b,\[Rho],poles] or \
SetScatteringData[a,b,\[Rho],poles,normingConstants] sets the scattering data \
for the KdV equation\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"SetParams", "::", "usage"}], "=", 
   "\"\<SetParams[\[Nu],rad,globalTol,smallN,bigN] sets the parameters for \
the rest of the code:\n\t\[Nu]: half of width of strip of analyticity\n\trad: \
radius of soliton contours\n\tglobalTol: contour truncation tolerance\n\t\
smallN: small number of collocation points\n\tbigN: big number of collocation \
points\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"GetScatteringData", "::", "usage"}], "=", 
   "\"\<GetScatteringData[] = {\[Rho],poles,normingConstants}\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ScatteringMatrixFiniteKdV", "::", "usage"}], "=", 
   "\"\<ScatteringMatrixFiniteKdV[q,n,L] returns a function f[w] that \
produces an approximation of the scattering matrix at w. The parameter n \
indicates the number of collocation points and L sets the problem on [-L,L]\n\
Sample code:\n\tq[x_]:=.1 Exp[-x^2+I x];\[IndentingNewLine]Defocusing[];\n\t\
H=ScatteringMatrixFiniteKdV[q,50,6];\[IndentingNewLine]aa//Clear;bb//Clear;\
\[IndentingNewLine]aa[k_]:=aa[k]=H[k][[1,1]];\[IndentingNewLine]bb[k_]:=bb[k]=\
H[k][[2,1]];\n\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Getnu", "::", "usage"}], "=", "\"\<Getnu[] returns \[Nu]\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Setrsamp", "::", "usage"}], "=", 
   "\"\<Setrsamp[h] sets a function that should be chosen so that that \
|\[Rho]| < h\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"timestring", "::", "usage"}], "=", 
   "\"\<A string updated by KdV[i][x,t]\n(x,t) 1) Construct: --time to \
construct rhp1-- 1) Solve: --time to solve rhp1-- 2) Construct: --time to \
construct rhp2-- 2) Solve: --time to solve rhp2--\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Settimeflag", "::", "usage"}], "=", 
   "\"\<Settimeflag[bool] bool = True forces timestring to be printed each \
time KdV[i][x,t] is called\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"startift", ";"}], "\[IndentingNewLine]", 
 RowBox[{"Setsingflag", ";"}], "\[IndentingNewLine]", 
 RowBox[{"Getsingflag", ";"}], "\[IndentingNewLine]", 
 RowBox[{"domainOutput", ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ReflectionCoefficientKdV", "::", "usage"}], "=", 
    "\"\<just for testing\>\""}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Begin", "[", "\"\<Private`\>\"", "]"}], ";"}]}], "Input",
 InitializationGroup->True,
 CellChangeTimes->{{3.5137081968140993`*^9, 3.513708220106812*^9}, {
   3.513708251598336*^9, 3.513708260607749*^9}, {3.513709188920638*^9, 
   3.513709189830554*^9}, {3.5137102470949373`*^9, 3.513710248122636*^9}, {
   3.513710327213766*^9, 3.513710327375476*^9}, {3.5137107005416193`*^9, 
   3.513710704633794*^9}, {3.513892211328*^9, 3.513892244064*^9}, {
   3.513944038153119*^9, 3.5139440504772377`*^9}, {3.51490865283358*^9, 
   3.514908688869083*^9}, {3.5149088447367496`*^9, 3.514908847957034*^9}, {
   3.514908890493555*^9, 3.5149088915294123`*^9}, {3.514912537924096*^9, 
   3.514912538602947*^9}, 3.514912651568637*^9, {3.514913420363991*^9, 
   3.514913422351489*^9}, {3.514915238203745*^9, 3.514915240325727*^9}, {
   3.5149165696337*^9, 3.5149165710458717`*^9}, {3.514916828081748*^9, 
   3.5149168303085814`*^9}, {3.51492264908776*^9, 3.5149226496893663`*^9}, {
   3.514923459235902*^9, 3.514923473879924*^9}, {3.514923609606861*^9, 
   3.5149236678379297`*^9}, 3.5149237785404778`*^9, {3.5149239632762337`*^9, 
   3.514923985264545*^9}, 3.514924035992598*^9, {3.5150107064700003`*^9, 
   3.51501070749*^9}, {3.515010820084*^9, 3.515010822774*^9}, {
   3.515011739966*^9, 3.5150117447060003`*^9}, {3.515011811398*^9, 
   3.515011814668*^9}, 3.515016621426*^9, {3.515155006646846*^9, 
   3.515155012398736*^9}, {3.515161405388542*^9, 3.5151614071047297`*^9}, {
   3.515161506975882*^9, 3.515161507859984*^9}, {3.515162979110601*^9, 
   3.51516298439431*^9}, {3.515163681303184*^9, 3.5151636836334467`*^9}, {
   3.5151639040428743`*^9, 3.515163905490437*^9}, {3.515164066077634*^9, 
   3.5151640679776783`*^9}, {3.515182527641205*^9, 3.5151825295557623`*^9}, {
   3.5151898348264*^9, 3.5151898365736*^9}, {3.5151898668844*^9, 
   3.5151898686628*^9}, {3.515865595644541*^9, 3.515865599028973*^9}, {
   3.516018665879665*^9, 3.516018669923935*^9}, {3.516019454499804*^9, 
   3.516019457143765*^9}, {3.5160203425168753`*^9, 3.516020344083581*^9}, {
   3.516029273628706*^9, 3.516029277905128*^9}, {3.516029836359334*^9, 
   3.516029839868938*^9}, {3.51603168179848*^9, 3.516031684249277*^9}, {
   3.5160317359840117`*^9, 3.5160317372417727`*^9}, {3.516031839641303*^9, 
   3.516031839825748*^9}, {3.51604593957542*^9, 3.5160459415743923`*^9}, {
   3.5160491405591393`*^9, 3.516049143223133*^9}, {3.516049475447588*^9, 
   3.5160494773697643`*^9}, {3.516049789077999*^9, 3.516049790980497*^9}, {
   3.5161107536323442`*^9, 3.516110755122838*^9}, {3.5163721439439573`*^9, 
   3.5163721529521523`*^9}, {3.516386237320636*^9, 3.51638624850243*^9}, {
   3.5163862792664013`*^9, 3.5163862795321913`*^9}, {3.516386360236644*^9, 
   3.516386376811532*^9}, {3.516397335101952*^9, 3.516397337025401*^9}, {
   3.516397704562858*^9, 3.516397704765292*^9}, {3.516407718106127*^9, 
   3.516407720463643*^9}, {3.516410444818138*^9, 3.516410447464739*^9}, {
   3.516410875117381*^9, 3.516410876217999*^9}, {3.516410944587029*^9, 
   3.5164109483536577`*^9}, {3.516461126501289*^9, 3.5164611299490423`*^9}, {
   3.516461294707374*^9, 3.516461297166636*^9}, {3.516461327659837*^9, 
   3.516461327740819*^9}, {3.516463561809222*^9, 3.516463565124407*^9}, {
   3.516476398141786*^9, 3.516476398874007*^9}, {3.5164766625639257`*^9, 
   3.516476665579368*^9}, {3.5164793645522947`*^9, 3.5164793676290827`*^9}, {
   3.516479604034827*^9, 3.5164796054708357`*^9}, {3.516479833783655*^9, 
   3.5164798356595087`*^9}, 3.516493622096712*^9, 3.517748695954481*^9, {
   3.517774143133275*^9, 3.5177741451763573`*^9}, {3.5178396880058126`*^9, 
   3.517839689202875*^9}, {3.517858591597598*^9, 3.5178585973752937`*^9}, {
   3.5182917712009277`*^9, 3.518291774979272*^9}, {3.518375651840969*^9, 
   3.518375655792688*^9}, {3.518450285472134*^9, 3.518450290187779*^9}, {
   3.518450400407465*^9, 3.518450401236301*^9}, {3.51845107524128*^9, 
   3.518451076733898*^9}, {3.518699044664328*^9, 3.518699047060379*^9}, {
   3.518699684222773*^9, 3.5186996898253717`*^9}, {3.5187010806642723`*^9, 
   3.518701087045055*^9}, {3.5369573365322113`*^9, 3.536957339500263*^9}, {
   3.5374101042618*^9, 3.5374101079877996`*^9}, {3.5463652236104*^9, 
   3.5463652586904*^9}, {3.568127948432659*^9, 3.5681279954027*^9}, {
   3.568131041313644*^9, 3.56813105227133*^9}, {3.5681313274174757`*^9, 
   3.568131327977957*^9}, {3.5681318705130177`*^9, 3.568131872855075*^9}, {
   3.568331583995309*^9, 3.568331588709426*^9}, {3.5683345365889673`*^9, 
   3.56833456945201*^9}, {3.5683795107727127`*^9, 3.568379522027073*^9}, {
   3.568380843326642*^9, 3.568380848847931*^9}, {3.5683809186136827`*^9, 
   3.568380920584309*^9}, {3.56838204438026*^9, 3.568382053079723*^9}, {
   3.568382187521057*^9, 3.568382209994052*^9}, {3.568394824550172*^9, 
   3.5683948263246813`*^9}, {3.568403062179324*^9, 3.568403065339301*^9}, {
   3.568405963369748*^9, 3.5684059652093363`*^9}, {3.568406222105316*^9, 
   3.568406224637711*^9}, {3.5684065828519373`*^9, 3.568406583043234*^9}, {
   3.5684067424388933`*^9, 3.568406745038039*^9}, {3.568409437330915*^9, 
   3.568409438933494*^9}, 3.568409638254784*^9, {3.56841277222714*^9, 
   3.568412772822226*^9}, {3.568414396519912*^9, 3.5684143987223177`*^9}, {
   3.5684147809599857`*^9, 3.568414782570952*^9}, {3.5685821506842937`*^9, 
   3.568582152281106*^9}, {3.568583073760479*^9, 3.568583074525279*^9}, {
   3.568583488438321*^9, 3.5685834891377773`*^9}, {3.568584951484253*^9, 
   3.568584955198462*^9}, {3.568585807038322*^9, 3.568585807907267*^9}, {
   3.568639283876072*^9, 3.568639285575286*^9}, {3.56863938326729*^9, 
   3.5686394131265697`*^9}, {3.568639875435589*^9, 3.568639876151482*^9}, {
   3.5686423542251883`*^9, 3.568642354949008*^9}, {3.5686425702793417`*^9, 
   3.568642573954401*^9}, {3.568642834419856*^9, 3.568642836170411*^9}, {
   3.568651914004204*^9, 3.568651915490437*^9}, {3.5686520061501207`*^9, 
   3.56865201112029*^9}, {3.568652314637508*^9, 3.568652325335346*^9}, {
   3.5686582529324007`*^9, 3.568658255725828*^9}, {3.5687315847022047`*^9, 
   3.568731592965949*^9}, {3.568764308714259*^9, 3.568764335789035*^9}, {
   3.63455077047145*^9, 3.634550770900894*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Setup", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.546363064814*^9, 3.546363067493*^9}, {
  3.565745327189906*^9, 3.5657453277187843`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Focusing", "[", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"\[Lambda]", "=", "1"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Defocusing", "[", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"\[Lambda]", "=", 
      RowBox[{"-", "1"}]}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Defocusing", "[", "]"}], ";"}], 
  RowBox[{"(*", 
   RowBox[{
   "Parameter", " ", "left", " ", "over", " ", "from", " ", "NLS", " ", "and",
     " ", "mKdV"}], "*)"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"SetScatteringData", "[", 
    RowBox[{"aa_", ",", "bb_", ",", "rr_", ",", "ma_", ",", "mc_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"r", "[", "k_", "]"}], ":=", 
       RowBox[{"rr", "[", "k", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"A", "=", "ma"}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"\[Tau]", "[", "k_", "]"}], ":=", 
       RowBox[{"1", "+", 
        RowBox[{"\[Lambda]", " ", 
         RowBox[{"r", "[", "k", "]"}], 
         RowBox[{"cc", "[", 
          RowBox[{"r", "[", 
           RowBox[{"cc", "[", "k", "]"}], "]"}], "]"}]}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"\[Tau]", "[", 
        RowBox[{"_", "?", "InfinityQ"}], "]"}], ":=", "1"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"\[Tau]", "[", "k_List", "]"}], ":=", 
       RowBox[{"\[Tau]", "/@", "k"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"c", "=", "mc"}], ";", "\[IndentingNewLine]", 
      RowBox[{"singflag", " ", "=", " ", 
       RowBox[{"ZeroQ", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"rr", "[", "0.", "]"}], " ", "+", "1."}], "//", "Abs"}], 
        "]"}]}], ";"}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"SetScatteringData", "[", 
    RowBox[{"aa_", ",", "bb_", ",", "rr_", ",", "ma_", ",", "q_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"r", "[", "k_", "]"}], ":=", 
       RowBox[{"rr", "[", "k", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"A", "=", "ma"}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"\[Tau]", "[", "k_", "]"}], ":=", 
       RowBox[{"1", "+", 
        RowBox[{"\[Lambda]", " ", 
         RowBox[{"r", "[", "k", "]"}], 
         RowBox[{"cc", "[", 
          RowBox[{"r", "[", 
           RowBox[{"cc", "[", "k", "]"}], "]"}], "]"}]}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"\[Tau]", "[", 
        RowBox[{"_", "?", "InfinityQ"}], "]"}], ":=", "1"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"\[Tau]", "[", "k_List", "]"}], ":=", 
       RowBox[{"\[Tau]", "/@", "k"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"c", " ", "=", " ", 
       RowBox[{"Residues", "[", 
        RowBox[{"q", ",", "ma", ",", "0"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"singflag", " ", "=", " ", 
       RowBox[{"ZeroQ", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"rr", "[", "0.", "]"}], " ", "+", "1."}], "//", "Abs"}], 
        "]"}]}], ";"}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"SetParams", "[", 
    RowBox[{"nu_", ",", "rad1_", ",", "tol_", ",", "sN_", ",", "bN_"}], "]"}],
    ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"\[Nu]", "=", "nu"}], ";", " ", 
      RowBox[{"(*", 
       RowBox[{
       "parameter", " ", "for", " ", "width", " ", "of", " ", "strip", " ", 
        "of", " ", "analyticity"}], "*)"}], "\[IndentingNewLine]", 
      RowBox[{"rad", "=", "rad1"}], ";", "\[IndentingNewLine]", 
      RowBox[{"globalTol", "=", "tol"}], ";", "\[IndentingNewLine]", 
      RowBox[{"smallN", "=", "sN"}], ";", "\[IndentingNewLine]", 
      RowBox[{"bigN", "=", "bN"}], ";"}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"GetScatteringData", "[", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"r", ",", "A", ",", "c"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Getnu", "[", "]"}], ":=", "\[Nu]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Setrsamp", "[", "h_", "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", "}"}], ",", 
      RowBox[{
       RowBox[{"rsamp", "=", "h"}], ";", 
       RowBox[{"dom", "=", 
        RowBox[{
         RowBox[{
          RowBox[{"Truncate", "[", "globalTol", "]"}], "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"Log", "[", 
               RowBox[{"1", "-", 
                RowBox[{
                 RowBox[{"Abs", "[", 
                  RowBox[{"h", "[", "#", "]"}], "]"}], "^", "2"}]}], "]"}], "/.", 
              RowBox[{
               RowBox[{"Underflow", "[", "]"}], "\[Rule]", "0."}]}], "&"}], 
            ",", 
            RowBox[{"Line", "[", 
             RowBox[{"{", 
              RowBox[{
               RowBox[{"-", "el"}], ",", "el"}], "}"}], "]"}], ",", "100"}], 
           "}"}], "]"}], "[", 
         RowBox[{"[", "1", "]"}], "]"}]}], ";"}]}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Setsingflag", "[", "bool_", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"singflag", " ", "=", " ", "bool"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Setsingflag", "[", "True", "]"}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Getsingflag", "[", "]"}], ":=", "singflag"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Settimeflag", "[", "h_", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"timeflag", "=", "h"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"SetN", "[", "x_", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{
      RowBox[{"bigN", "=", 
       RowBox[{"x", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ";", 
      RowBox[{"smallN", "=", 
       RowBox[{"x", "[", 
        RowBox[{"[", "2", "]"}], "]"}]}], ";"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"expconst", "=", "1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"el", "=", "100"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"bigN", "=", "20"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"smallN", "=", "10"}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5463630695*^9, 3.546363080922*^9}, {3.546363115111*^9, 
   3.546363141222*^9}, {3.5463634376070004`*^9, 3.546363492512*^9}, {
   3.5463635258719997`*^9, 3.546363635209*^9}, {3.5463659693424*^9, 
   3.5463659740664*^9}, {3.5463660119104*^9, 3.5463661504323997`*^9}, {
   3.5463664883764*^9, 3.5463664944384003`*^9}, {3.5463666192464*^9, 
   3.5463666459524*^9}, {3.5463667113484*^9, 3.5463667373324003`*^9}, {
   3.5463676399364*^9, 3.5463676562564*^9}, {3.546390936499014*^9, 
   3.546390947510488*^9}, {3.546401541105667*^9, 3.5464015544196997`*^9}, {
   3.546434422083152*^9, 3.546434437782632*^9}, {3.563910175270574*^9, 
   3.563910220797307*^9}, {3.564799718336*^9, 3.564799720823*^9}, {
   3.5649679651156*^9, 3.5649679782296*^9}, {3.5655714809302*^9, 
   3.5655714994402*^9}, {3.5655715372522*^9, 3.5655715645132*^9}, {
   3.5656148581191998`*^9, 3.5656148586572*^9}, {3.5656173795022*^9, 
   3.5656173983492002`*^9}, {3.5656200626132*^9, 3.5656200714432*^9}, {
   3.5656203941162*^9, 3.5656204067612*^9}, {3.5656338952332*^9, 
   3.5656339009142*^9}, {3.5656343322771997`*^9, 3.5656343385042*^9}, {
   3.5657105818142*^9, 3.5657105820962*^9}, {3.5657106368512*^9, 
   3.5657106371591997`*^9}, {3.5657106923602*^9, 3.5657107507892*^9}, 
   3.5657108510002003`*^9, {3.5657130676252003`*^9, 3.5657130795922003`*^9}, {
   3.5657280484491997`*^9, 3.5657280561782*^9}, {3.5657313869382*^9, 
   3.5657314205002003`*^9}, {3.565744033573463*^9, 3.565744042518571*^9}, {
   3.565745333305562*^9, 3.565745362905344*^9}, {3.5657454278631887`*^9, 
   3.565745444786531*^9}, {3.5657455096709547`*^9, 3.5657455425153027`*^9}, {
   3.565746618412477*^9, 3.565746650258099*^9}, {3.565753896592372*^9, 
   3.56575390058217*^9}, {3.565753960980502*^9, 3.5657539672129097`*^9}, {
   3.565754087442314*^9, 3.565754191303976*^9}, {3.565754253265995*^9, 
   3.565754273708042*^9}, {3.565754484213047*^9, 3.5657544921511374`*^9}, {
   3.565755271816646*^9, 3.565755314332326*^9}, {3.568322803553039*^9, 
   3.568322821334434*^9}, {3.568331042850155*^9, 3.5683311066045713`*^9}, {
   3.568334249311112*^9, 3.568334262380498*^9}, 3.5683343178112087`*^9, {
   3.568379406212199*^9, 3.568379406519261*^9}, {3.568383640859544*^9, 
   3.5683836590364428`*^9}, {3.568392161425585*^9, 3.568392180908291*^9}, {
   3.568394553704639*^9, 3.568394555034494*^9}, 3.568394648757533*^9, {
   3.568400478158368*^9, 3.5684004955352783`*^9}, {3.568402951200943*^9, 
   3.568403058123541*^9}, {3.568406208805195*^9, 3.568406215375626*^9}, {
   3.568406689422266*^9, 3.568406722397731*^9}, 3.56842608135682*^9, {
   3.568426139039611*^9, 3.5684261656985703`*^9}, {3.568426236048396*^9, 
   3.568426296062298*^9}, {3.568585014363508*^9, 3.568585028142354*^9}, {
   3.5687374618113956`*^9, 3.568737462533094*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Reflection Coefficient", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.5407685463534*^9, 3.5407685478054*^9}, {
  3.5657108306542*^9, 3.5657108348501997`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"CME", "[", "x_", "]"}], ":=", 
   RowBox[{"Chop", "[", 
    RowBox[{"x", ",", "$MachineEpsilon"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ReflectionCoefficientKdV", "[", 
    RowBox[{"q_", ",", "n_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "qf", ",", "qfb", ",", "\[Phi]2", ",", "\[Phi]b1", ",", "\[Phi]b2", ",",
        "Dm", ",", "Dm2", ",", "Dmb", ",", "Dmb2", ",", "P", ",", "Pb", ",", 
       "IIm", ",", "IImb"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"qf", "=", 
       RowBox[{"Fun", "[", 
        RowBox[{"q", ",", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "\[Infinity]"}], ",", "0"}], "}"}], "]"}], ",", 
         "n"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"qfb", "=", 
       RowBox[{"Fun", "[", 
        RowBox[{"q", ",", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "\[Infinity]"}], "}"}], "]"}], ",", "n"}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Dm", "=", 
       RowBox[{"DerivativeMatrix", "[", "qf", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Dmb", "=", 
       RowBox[{"DerivativeMatrix", "[", "qfb", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"IIm", "=", 
       RowBox[{"ReduceDimensionIntegrateMatrix", "[", "qf", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"IImb", "=", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"ReduceDimensionIntegrateMatrix", "[", 
             RowBox[{"(", 
              RowBox[{"qfb", "//", "ReverseOrientation"}], ")"}], "]"}], "//",
             "Transpose"}], "//", "Reverse"}], "//", "Transpose"}], "//", 
         "Reverse"}], ")"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"P", "=", 
       RowBox[{"Dm", "+", 
        RowBox[{
         RowBox[{"DiagonalMatrix", "[", 
          RowBox[{"qf", " ", "//", "Values"}], "]"}], ".", "IIm"}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Pb", "=", 
       RowBox[{"Dmb", "+", 
        RowBox[{
         RowBox[{"DiagonalMatrix", "[", 
          RowBox[{"qfb", "//", "Values"}], "]"}], ".", "IImb"}]}]}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"ReflectionCoefficientKdV", "[", 
       RowBox[{"qf", ",", "qfb", ",", "P", ",", "Pb"}], "]"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ReflectionCoefficientKdV", "[", 
     RowBox[{"qf_", ",", "qfb_", ",", "P_", ",", "Pb_"}], "]"}], "[", 
    RowBox[{"_", "?", "NZeroQ"}], "]"}], ":=", 
   RowBox[{"-", "1"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ReflectionCoefficientKdV", "[", 
     RowBox[{"qf_", ",", "qfb_", ",", "P_", ",", "Pb_"}], "]"}], "[", "w_", 
    "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"\[Phi]2", ",", "\[Phi]b1", ",", "\[Phi]b2"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"\[Phi]2", "=", 
       RowBox[{
        RowBox[{"IFun", "[", 
         RowBox[{
          RowBox[{"LinearSolve", "[", 
           RowBox[{
            RowBox[{"P", "-", 
             RowBox[{"2", " ", "I", " ", "w", " ", 
              RowBox[{"IdentityMatrix", "[", 
               RowBox[{"Length", "[", "qf", "]"}], "]"}]}]}], ",", 
            RowBox[{"-", 
             RowBox[{"Chop", "[", 
              RowBox[{
               RowBox[{"Values", "[", "qf", "]"}], ",", "$MachineTolerance"}],
               "]"}]}]}], "]"}], ",", 
          RowBox[{"qf", "//", "Domain"}]}], "]"}], "//", "Integrate"}]}], ";",
       "\[IndentingNewLine]", 
      RowBox[{"\[Phi]b1", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"IFun", "[", 
          RowBox[{
           RowBox[{"LinearSolve", "[", 
            RowBox[{
             RowBox[{"Pb", "+", 
              RowBox[{"2", " ", "I", " ", "w", " ", 
               RowBox[{"IdentityMatrix", "[", 
                RowBox[{"Length", "[", "qfb", "]"}], "]"}]}]}], ",", 
             RowBox[{"-", 
              RowBox[{"Chop", "[", 
               RowBox[{
                RowBox[{"Values", "[", "qfb", "]"}], ",", 
                "$MachineTolerance"}], "]"}]}]}], "]"}], ",", 
           RowBox[{"qfb", "//", "Domain"}]}], "]"}], "//", "Integrate"}], "//",
         "ZeroAtRight"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"\[Phi]b2", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"IFun", "[", 
          RowBox[{
           RowBox[{"LinearSolve", "[", 
            RowBox[{
             RowBox[{"Pb", "-", 
              RowBox[{"2", " ", "I", " ", "w", " ", 
               RowBox[{"IdentityMatrix", "[", 
                RowBox[{"Length", "[", "qfb", "]"}], "]"}]}]}], ",", 
             RowBox[{"-", 
              RowBox[{"Chop", "[", 
               RowBox[{
                RowBox[{"Values", "[", "qfb", "]"}], ",", 
                "$MachineTolerance"}], "]"}]}]}], "]"}], ",", 
           RowBox[{"qfb", "//", "Domain"}]}], "]"}], "//", "Integrate"}], "//",
         "ZeroAtRight"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"LinearSolve", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{"1", "+", 
               RowBox[{"(", 
                RowBox[{"\[Phi]b1", "//", "First"}], ")"}]}], ")"}], " ", ",", 
             RowBox[{"1", "+", 
              RowBox[{"(", 
               RowBox[{"\[Phi]b2", "//", "First"}], ")"}]}]}], "}"}], ",", 
           "\[IndentingNewLine]", "\t", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"\[Phi]b1", "'"}], "+", 
               RowBox[{"I", " ", "w", " ", 
                RowBox[{"(", 
                 RowBox[{"1", "+", "\[Phi]b1"}], ")"}]}]}], "//", "First"}], 
             ",", 
             RowBox[{
              RowBox[{
               RowBox[{"\[Phi]b2", "'"}], "-", 
               RowBox[{"I", " ", "w", " ", 
                RowBox[{"(", 
                 RowBox[{"1", "+", "\[Phi]b2"}], ")"}]}]}], "//", "First"}]}],
             "}"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"1", "+", 
            RowBox[{"(", 
             RowBox[{"\[Phi]2", "//", "Last"}], ")"}]}], ",", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Phi]2", "'"}], "-", 
              RowBox[{"I", " ", "w", " ", 
               RowBox[{"(", 
                RowBox[{"1", "+", "\[Phi]2"}], ")"}]}]}], "//", "Last"}], 
            ")"}]}], "}"}]}], "]"}], "//", 
       RowBox[{
        RowBox[{
         RowBox[{"#", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
         "/", 
         RowBox[{
         "#", "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], 
        "&"}]}]}]}], "\[IndentingNewLine]", "]"}]}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ScatteringMatrixFiniteKdV", "[", 
    RowBox[{"q_", ",", "n_", ",", "el_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "qf", ",", "qfb", ",", "\[Phi]2", ",", "\[Phi]b1", ",", "\[Phi]b2", ",",
        "Dm", ",", "Dm2", ",", "Dmb", ",", "Dmb2", ",", "P", ",", "Pb", ",", 
       "IIm", ",", "IImb"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"qf", "=", 
       RowBox[{"Fun", "[", 
        RowBox[{"q", ",", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "el"}], ",", "0"}], "}"}], "]"}], ",", "n"}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"qfb", "=", 
       RowBox[{"Fun", "[", 
        RowBox[{"q", ",", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "el"}], "}"}], "]"}], ",", "n"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"qf", "//", "Values"}], "//", "First"}], "//", "Abs"}], 
         ",", 
         RowBox[{"qf", "//", "DCTPlot"}]}], "}"}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Dm", "=", 
       RowBox[{"DerivativeMatrix", "[", "qf", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Dmb", "=", 
       RowBox[{"DerivativeMatrix", "[", "qfb", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"IIm", "=", 
       RowBox[{"ReduceDimensionIntegrateMatrix", "[", "qf", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"IImb", "=", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"ReduceDimensionIntegrateMatrix", "[", 
             RowBox[{"(", 
              RowBox[{"qfb", "//", "ReverseOrientation"}], ")"}], "]"}], "//",
             "Transpose"}], "//", "Reverse"}], "//", "Transpose"}], "//", 
         "Reverse"}], ")"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"P", "=", 
       RowBox[{
        RowBox[{"IdentityMatrix", "[", "n", "]"}], "+", 
        RowBox[{"IIm", ".", 
         RowBox[{"DiagonalMatrix", "[", 
          RowBox[{"qf", " ", "//", "Values"}], "]"}], ".", "IIm"}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Pb", "=", 
       RowBox[{
        RowBox[{"IdentityMatrix", "[", "n", "]"}], "+", 
        RowBox[{"IImb", ".", 
         RowBox[{"DiagonalMatrix", "[", 
          RowBox[{"qfb", "//", "Values"}], "]"}], ".", "IImb"}]}]}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"ScatteringMatrixFiniteKdV", "[", 
       RowBox[{
       "qf", ",", "qfb", ",", "P", ",", "Pb", ",", "IIm", ",", "IImb"}], 
       "]"}]}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ScatteringMatrixFiniteKdV", "[", 
     RowBox[{
     "qf_", ",", "qfb_", ",", "P_", ",", "Pb_", ",", "IIm_", ",", "IImb_"}], 
     "]"}], "[", "w_", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"\[Phi]2", ",", "\[Phi]b1", ",", "\[Phi]b2", ",", "\[Phi]1"}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"\[Phi]2", "=", 
       RowBox[{
        RowBox[{"IFun", "[", 
         RowBox[{
          RowBox[{"LinearSolve", "[", 
           RowBox[{
            RowBox[{"P", "-", 
             RowBox[{"2", " ", "I", " ", "w", " ", "IIm"}]}], ",", 
            RowBox[{"-", 
             RowBox[{"Chop", "[", 
              RowBox[{
               RowBox[{"IIm", ".", 
                RowBox[{"Values", "[", "qf", "]"}]}], ",", 
               "$MachineTolerance"}], "]"}]}]}], "]"}], ",", 
          RowBox[{"qf", "//", "Domain"}]}], "]"}], "//", "Integrate"}]}], ";",
       "\[IndentingNewLine]", 
      RowBox[{"\[Phi]1", "=", 
       RowBox[{
        RowBox[{"IFun", "[", 
         RowBox[{
          RowBox[{"LinearSolve", "[", 
           RowBox[{
            RowBox[{"P", "+", 
             RowBox[{"2", " ", "I", " ", "w", " ", "IIm"}]}], ",", 
            RowBox[{"-", 
             RowBox[{"Chop", "[", 
              RowBox[{
               RowBox[{"IIm", ".", 
                RowBox[{"Values", "[", "qf", "]"}]}], ",", 
               "$MachineTolerance"}], "]"}]}]}], "]"}], ",", 
          RowBox[{"qf", "//", "Domain"}]}], "]"}], "//", "Integrate"}]}], ";",
       "\[IndentingNewLine]", 
      RowBox[{"\[Phi]b1", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"IFun", "[", 
          RowBox[{
           RowBox[{"LinearSolve", "[", 
            RowBox[{
             RowBox[{"Pb", "+", 
              RowBox[{"2", " ", "I", " ", "w", " ", "IImb"}]}], ",", 
             RowBox[{"-", 
              RowBox[{"Chop", "[", 
               RowBox[{
                RowBox[{"IImb", ".", 
                 RowBox[{"Values", "[", "qfb", "]"}]}], ",", 
                "$MachineTolerance"}], "]"}]}]}], "]"}], ",", 
           RowBox[{"qfb", "//", "Domain"}]}], "]"}], "//", "Integrate"}], "//",
         "ZeroAtRight"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"\[Phi]b2", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"IFun", "[", 
          RowBox[{
           RowBox[{"LinearSolve", "[", 
            RowBox[{
             RowBox[{"Pb", "-", 
              RowBox[{"2", " ", "I", " ", "w", " ", "IImb"}]}], ",", 
             RowBox[{"-", 
              RowBox[{"Chop", "[", 
               RowBox[{
                RowBox[{"IImb", ".", 
                 RowBox[{"Values", "[", "qfb", "]"}]}], ",", 
                "$MachineTolerance"}], "]"}]}]}], "]"}], ",", 
           RowBox[{"qfb", "//", "Domain"}]}], "]"}], "//", "Integrate"}], "//",
         "ZeroAtRight"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Inverse", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"1", "+", 
              RowBox[{"(", 
               RowBox[{"\[Phi]b1", "//", "First"}], ")"}]}], ")"}], " ", ",", 
            
            RowBox[{"1", "+", 
             RowBox[{"(", 
              RowBox[{"\[Phi]b2", "//", "First"}], ")"}]}]}], "}"}], ",", 
          "\[IndentingNewLine]", "\t", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[Phi]b1", "'"}], "+", 
              RowBox[{"I", " ", "w", " ", 
               RowBox[{"(", 
                RowBox[{"1", "+", "\[Phi]b1"}], ")"}]}]}], "//", "First"}], 
            ",", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Phi]b2", "'"}], "-", 
              RowBox[{"I", " ", "w", " ", 
               RowBox[{"(", 
                RowBox[{"1", "+", "\[Phi]b2"}], ")"}]}]}], "//", "First"}]}], 
           "}"}]}], "}"}], "]"}], ".", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"1", "+", 
            RowBox[{"(", 
             RowBox[{"\[Phi]1", "//", "Last"}], ")"}]}], ",", 
           RowBox[{"1", "+", 
            RowBox[{"(", 
             RowBox[{"\[Phi]2", "//", "Last"}], ")"}]}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Phi]1", "'"}], "+", 
              RowBox[{"I", " ", "w", " ", 
               RowBox[{"(", 
                RowBox[{"1", "+", "\[Phi]1"}], ")"}]}]}], "//", "Last"}], 
            ")"}], ",", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              RowBox[{"\[Phi]2", "'"}], "-", 
              RowBox[{"I", " ", "w", " ", 
               RowBox[{"(", 
                RowBox[{"1", "+", "\[Phi]2"}], ")"}]}]}], "//", "Last"}], 
            ")"}]}], "}"}]}], "}"}]}]}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5407710794914*^9, 3.5407710807813997`*^9}, {
   3.5407711951854*^9, 3.5407712059854*^9}, {3.5407712856654*^9, 
   3.5407713737354*^9}, {3.540827282042986*^9, 3.540827333330905*^9}, {
   3.54083242807616*^9, 3.5408324852837*^9}, {3.540832522921386*^9, 
   3.5408325930834007`*^9}, {3.540832938848312*^9, 3.540832960757762*^9}, {
   3.540832994793727*^9, 3.5408330192890873`*^9}, {3.540833097880999*^9, 
   3.540833104398241*^9}, {3.5408331497947607`*^9, 3.540833157137931*^9}, {
   3.540833189894033*^9, 3.540833239916215*^9}, {3.5408333534680634`*^9, 
   3.540833375999275*^9}, {3.540833439903254*^9, 3.540833457391617*^9}, 
   3.5409039382404003`*^9, {3.5409378931194*^9, 3.5409379107794*^9}, {
   3.5409397365383997`*^9, 3.5409397675624*^9}, 3.5409398277864*^9, {
   3.5409399836464*^9, 3.5409399892994003`*^9}, {3.5409422127864*^9, 
   3.5409422401823997`*^9}, {3.541851859895*^9, 3.541851909367*^9}, {
   3.5418522254849997`*^9, 3.541852231517*^9}, {3.5418533264820004`*^9, 
   3.541853326612*^9}, {3.5463628431070004`*^9, 3.5463628486429996`*^9}, {
   3.5463662630164003`*^9, 3.5463662636764*^9}, {3.5656340548582*^9, 
   3.5656340797651997`*^9}, {3.5656341116712*^9, 3.5656341309472*^9}, {
   3.5656341613592*^9, 3.5656341633571997`*^9}, {3.5657105213012*^9, 
   3.5657105408712*^9}, {3.5657557671034803`*^9, 3.5657557769551697`*^9}, {
   3.5657559916336403`*^9, 3.5657559932735453`*^9}, {3.568127174586278*^9, 
   3.568127201459868*^9}, {3.568379501128624*^9, 3.56837950300025*^9}, {
   3.568379701211008*^9, 3.5683797677471523`*^9}, {3.568379960324254*^9, 
   3.568380122207906*^9}, {3.568380591410409*^9, 3.568380604484015*^9}, {
   3.568380830132427*^9, 3.5683808305159397`*^9}, {3.5683809537842703`*^9, 
   3.5683809540021667`*^9}, {3.56838120172148*^9, 3.568381202093484*^9}, {
   3.56838128806918*^9, 3.568381293271595*^9}, {3.568381541782577*^9, 
   3.568381542034951*^9}, {3.568381588515732*^9, 3.5683815915633287`*^9}, {
   3.568381861259313*^9, 3.568382027589775*^9}, {3.5683821499122887`*^9, 
   3.5683821788058023`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Eigenvalues", "Section",
 InitializationGroup->True,
 CellChangeTimes->{{3.503928097550683*^9, 3.503928099975516*^9}, {
  3.504030654349873*^9, 3.5040306549409733`*^9}, {3.568127373491021*^9, 
  3.568127376717606*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"HillKdV", "[", 
    RowBox[{"F_", ",", "n_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "c2", ",", "c1", ",", "c0", ",", "f2", ",", "f1", ",", "f0", ",", "t", 
       ",", "A", ",", "mu", ",", "L", ",", "P", ",", "fun"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"mu", " ", "=", " ", "0."}], ";", "\[IndentingNewLine]", 
      RowBox[{"L", " ", "=", " ", "Pi"}], ";", "\[IndentingNewLine]", 
      RowBox[{"P", " ", "=", "1"}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"f2", "[", "x_", "]"}], ":=", " ", 
       RowBox[{"4", "*", 
        RowBox[{
         RowBox[{"Cos", "[", 
          RowBox[{"x", "/", "2"}], "]"}], "^", "4"}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"f1", "[", "x_", "]"}], ":=", " ", 
       RowBox[{
        RowBox[{"-", "4"}], "*", 
        RowBox[{
         RowBox[{"Cos", "[", 
          RowBox[{"x", "/", "2"}], "]"}], "^", "3"}], "*", 
        RowBox[{"Sin", "[", 
         RowBox[{"x", "/", "2"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"t", "[", "x_", "]"}], " ", ":=", " ", 
       RowBox[{"Tan", "[", 
        RowBox[{"x", "/", "2"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"f0", "[", "x_", "]"}], ":=", " ", 
       RowBox[{"F", "[", 
        RowBox[{"t", "[", "x", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"fun", "=", " ", 
       RowBox[{"{", 
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{"f0", ",", "f1", ",", "f2"}], "}"}], "}"}], "}"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"A", "=", " ", 
       RowBox[{"Hill", "[", 
        RowBox[{"mu", ",", "L", ",", "n", ",", "P", ",", "fun"}], "]"}]}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"Eigensystem", "[", 
       RowBox[{"Chop", "[", 
        RowBox[{"A", ",", "$MachineEpsilon"}], "]"}], "]"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"LocatePolesKdV", "[", 
    RowBox[{"F_", ",", "n_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"eval", ",", "s", ",", "i", ",", "sys", ",", "vecs"}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"sys", "=", 
       RowBox[{"HillKdV", "[", 
        RowBox[{"F", ",", "n"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"eval", "=", 
       RowBox[{"sys", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"s", " ", "=", " ", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"vecs", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"i", "=", "1"}], ",", 
        RowBox[{"i", "\[LessEqual]", 
         RowBox[{
          RowBox[{"2", "*", "n"}], "+", "1"}]}], ",", 
        RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"Re", "[", 
             RowBox[{"eval", "[", 
              RowBox[{"[", "i", "]"}], "]"}], "]"}], ">", 
            RowBox[{"10", "^", 
             RowBox[{"(", 
              RowBox[{"-", "6"}], ")"}]}]}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"s", "=", 
             RowBox[{"Join", "[", 
              RowBox[{"s", ",", 
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"eval", "[", 
                  RowBox[{"[", "i", "]"}], "]"}], "//", "Re"}], "}"}]}], 
              "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"vecs", "=", " ", 
             RowBox[{"Join", "[", 
              RowBox[{"vecs", ",", 
               RowBox[{"{", 
                RowBox[{"sys", "[", 
                 RowBox[{"[", 
                  RowBox[{"2", ",", "i"}], "]"}], "]"}], "}"}]}], "]"}]}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Sqrt", "[", 
       RowBox[{
        RowBox[{"-", "s"}], "//", "Chop"}], "]"}]}]}], "\[IndentingNewLine]", 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Residues", "[", 
    RowBox[{"f_", ",", "k_", ",", "x_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "qf", ",", "qfb", ",", "\[Phi]2", ",", "\[Phi]b1", ",", "\[Phi]b2", ",",
        "Dm", ",", "Dm2", ",", "Dmb", ",", "Dmb2", ",", "P", ",", "Pb", ",", 
       "\[Phi]2p", ",", "\[Phi]2m", ",", "l", ",", "r", ",", "b", ",", "i"}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"l", " ", "=", " ", 
       RowBox[{"Length", "[", "k", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"r", " ", "=", " ", 
       RowBox[{"ConstantArray", "[", 
        RowBox[{"0", ",", "l"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"qf", "=", 
       RowBox[{"Fun", "[", 
        RowBox[{"f", ",", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "\[Infinity]"}], ",", "x"}], "}"}], "]"}], ",", 
         "10"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"qfb", "=", 
       RowBox[{"Fun", "[", 
        RowBox[{"f", ",", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{"x", ",", "\[Infinity]"}], "}"}], "]"}], ",", "10"}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Dm", "=", 
       RowBox[{"DerivativeMatrix", "[", "qf", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Dm2", "=", 
       RowBox[{"Dm", ".", "Dm"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Dmb", "=", 
       RowBox[{"DerivativeMatrix", "[", "qfb", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Dmb2", "=", 
       RowBox[{"Dmb", ".", "Dmb"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"P", "=", 
       RowBox[{"AddBasisElement", "[", 
        RowBox[{
         RowBox[{"Dm2", "+", 
          RowBox[{"DiagonalMatrix", "[", 
           RowBox[{"Values", "[", "qf", " ", "]"}], "]"}]}], ",", "1"}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Pb", "=", 
       RowBox[{"AddBasisElement", "[", 
        RowBox[{
         RowBox[{"Dmb2", "+", 
          RowBox[{"DiagonalMatrix", "[", 
           RowBox[{"Values", "[", "qfb", " ", "]"}], "]"}]}], ",", 
         RowBox[{"-", "1"}], ",", 
         RowBox[{"-", "1"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"i", "=", "1"}], ",", 
        RowBox[{"i", "\[LessEqual]", "l"}], ",", 
        RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"\[Phi]2", "=", 
          RowBox[{"IFun", "[", 
           RowBox[{
            RowBox[{"LeastSquares", "[", 
             RowBox[{
              RowBox[{"P", "-", 
               RowBox[{"2", " ", "I", " ", 
                RowBox[{"k", "[", 
                 RowBox[{"[", "i", "]"}], "]"}], " ", "Dm"}]}], ",", 
              RowBox[{"-", 
               RowBox[{"Chop", "[", 
                RowBox[{
                 RowBox[{"Values", "[", "qf", "]"}], ",", 
                 "$MachineTolerance"}], "]"}]}]}], "]"}], ",", 
            RowBox[{"qf", "//", "Domain"}]}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"\[Phi]b1", "=", 
          RowBox[{"IFun", "[", 
           RowBox[{
            RowBox[{"LeastSquares", "[", 
             RowBox[{
              RowBox[{"Pb", "+", 
               RowBox[{"2", " ", "I", " ", 
                RowBox[{"k", "[", 
                 RowBox[{"[", "i", "]"}], "]"}], " ", "Dmb"}]}], ",", 
              RowBox[{"-", 
               RowBox[{"Chop", "[", 
                RowBox[{
                 RowBox[{"Values", "[", "qfb", "]"}], ",", 
                 "$MachineTolerance"}], "]"}]}]}], "]"}], ",", 
            RowBox[{"qfb", "//", "Domain"}]}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"b", " ", "=", " ", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"1", " ", "+", " ", 
             RowBox[{"(", 
              RowBox[{"\[Phi]2", " ", "//", "Last"}], ")"}]}], ")"}], 
           RowBox[{
            RowBox[{"Exp", "[", 
             RowBox[{
              RowBox[{"-", "2"}], "I", " ", 
              RowBox[{"k", "[", 
               RowBox[{"[", "i", "]"}], "]"}], "x"}], "]"}], "/", 
            RowBox[{"(", 
             RowBox[{"1", "+", 
              RowBox[{"(", 
               RowBox[{"\[Phi]b1", " ", "//", "First"}], ")"}]}], ")"}]}]}]}],
          ";", "\[IndentingNewLine]", 
         RowBox[{"\[Phi]2m", "=", 
          RowBox[{"IFun", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"1", "+", "\[Phi]2"}], "//", "Values"}], ")"}], 
               RowBox[{"Exp", "[", 
                RowBox[{
                 RowBox[{"-", "I"}], "*", 
                 RowBox[{"k", "[", 
                  RowBox[{"[", "i", "]"}], "]"}], " ", 
                 RowBox[{"(", 
                  RowBox[{"\[Phi]2", "//", "Points"}], ")"}]}], "]"}]}], 
              ")"}], "^", "2"}], ",", 
            RowBox[{"qf", "//", "Domain"}]}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"\[Phi]2p", "=", 
          RowBox[{"IFun", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"(", 
              RowBox[{"b", "*", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"1", "+", "\[Phi]b1"}], "//", "Values"}], ")"}], 
               RowBox[{"Exp", "[", 
                RowBox[{"I", "*", 
                 RowBox[{"k", "[", 
                  RowBox[{"[", "i", "]"}], "]"}], " ", 
                 RowBox[{"(", 
                  RowBox[{"\[Phi]b1", "//", "Points"}], ")"}]}], "]"}]}], 
              ")"}], "^", "2"}], ",", 
            RowBox[{"qfb", "//", "Domain"}]}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"r", "[", 
           RowBox[{"[", "i", "]"}], "]"}], "=", 
          RowBox[{"I", " ", 
           RowBox[{
            RowBox[{"b", "^", "2"}], "/", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"DomainIntegrate", "[", "\[Phi]2p", "]"}], "+", 
              RowBox[{"DomainIntegrate", "[", "\[Phi]2m", "]"}]}], 
             ")"}]}]}]}], ";"}]}], "\[IndentingNewLine]", "]"}], ";", 
      "\[IndentingNewLine]", "r"}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.568383399024926*^9, 3.5683834101787*^9}, {
  3.568383553912117*^9, 3.5683835545291*^9}, {3.5683921466554937`*^9, 
  3.568392147045281*^9}, {3.568763624391408*^9, 3.568763642205597*^9}, {
  3.568763676304141*^9, 3.568763679877295*^9}, {3.568763730997913*^9, 
  3.568763737168064*^9}, {3.5687637702414293`*^9, 3.5687637877613907`*^9}, {
  3.568763842427932*^9, 3.568763850038035*^9}, {3.568763892214501*^9, 
  3.568763919569477*^9}, {3.5687640155782223`*^9, 3.568764015824513*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Jump Functions", "Section",
 CellChangeTimes->{{3.568128904921782*^9, 3.56812890695053*^9}}],

Cell[CellGroupData[{

Cell["Unscaled Functions", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.5408983275643997`*^9, 3.5408983375383997`*^9}, {
  3.568128918926874*^9, 3.568128920316318*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Theta]", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "z_", "]"}], ":=", 
   RowBox[{"2", " ", "I", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"4", " ", "t", " ", 
       SuperscriptBox["z", "3"]}], "+", 
      RowBox[{"x", " ", "z"}]}], ")"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"rb", "[", "k_", "]"}], ":=", 
   RowBox[{"\[Lambda]", " ", 
    RowBox[{"cc", "[", 
     RowBox[{"r", "[", 
      RowBox[{"cc", "[", "k", "]"}], "]"}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"rbsamp", "[", "k_", "]"}], ":=", 
   RowBox[{"\[Lambda]", " ", 
    RowBox[{"cc", "[", 
     RowBox[{"rsamp", "[", 
      RowBox[{"cc", "[", "k", "]"}], "]"}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Tau]samp", "[", "k_", "]"}], ":=", 
    RowBox[{"1", "+", 
     RowBox[{"\[Lambda]", " ", 
      RowBox[{"rsamp", "[", "k", "]"}], 
      RowBox[{"cc", "[", 
       RowBox[{"rsamp", "[", 
        RowBox[{"cc", "[", "k", "]"}], "]"}], "]"}]}]}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"L", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", "0"},
      {
       RowBox[{
        FractionBox[
         RowBox[{"r", "[", "k", "]"}], 
         RowBox[{"\[Tau]", "[", "k", "]"}]], 
        RowBox[{"Exp", "[", 
         RowBox[{
          RowBox[{"\[Theta]", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Lsamp", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {"1", "0"},
       {
        RowBox[{
         FractionBox[
          RowBox[{"rsamp", "[", "k", "]"}], 
          RowBox[{"\[Tau]samp", "[", "k", "]"}]], 
         RowBox[{"Exp", "[", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "1"}
      }], ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Lin", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", "0"},
      {
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{"r", "[", "k", "]"}], 
          RowBox[{"\[Tau]", "[", "k", "]"}]]}], 
        RowBox[{"Exp", "[", 
         RowBox[{
          RowBox[{"\[Theta]", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Linsamp", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {"1", "0"},
       {
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{"rsamp", "[", "k", "]"}], 
           RowBox[{"\[Tau]samp", "[", "k", "]"}]]}], 
         RowBox[{"Exp", "[", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "1"}
      }], ")"}]}], ";"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"U", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", 
       RowBox[{
        FractionBox[
         RowBox[{"rb", "[", "k", "]"}], 
         RowBox[{"\[Tau]", "[", "k", "]"}]], 
        RowBox[{"Exp", "[", 
         RowBox[{"-", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "]"}]}]},
      {"0", "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Uin", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{"rb", "[", "k", "]"}], 
          RowBox[{"\[Tau]", "[", "k", "]"}]]}], 
        RowBox[{"Exp", "[", 
         RowBox[{"-", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "]"}]}]},
      {"0", "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Usamp", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", 
       RowBox[{
        FractionBox[
         RowBox[{"rbsamp", "[", "k", "]"}], 
         RowBox[{"\[Tau]samp", "[", "k", "]"}]], 
        RowBox[{"Exp", "[", 
         RowBox[{"-", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "]"}]}]},
      {"0", "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Uinsamp", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {"1", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{"rbsamp", "[", "k", "]"}], 
           RowBox[{"\[Tau]samp", "[", "k", "]"}]]}], 
         RowBox[{"Exp", "[", 
          RowBox[{"-", 
           RowBox[{
            RowBox[{"\[Theta]", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "]"}]}]},
       {"0", "1"}
      }], ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DD", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"\[Tau]", "[", "k", "]"}], "0"},
      {"0", 
       RowBox[{"1", "/", 
        RowBox[{"\[Tau]", "[", "k", "]"}]}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"DDin", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {
        RowBox[{"1", "/", 
         RowBox[{"\[Tau]", "[", "k", "]"}]}], "0"},
       {"0", 
        RowBox[{"\[Tau]", "[", "k", "]"}]}
      }], ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DDsamp", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"\[Tau]samp", "[", "k", "]"}], "0"},
      {"0", 
       RowBox[{"1", "/", 
        RowBox[{"\[Tau]samp", "[", "k", "]"}]}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"DDinsamp", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {
        RowBox[{"1", "/", 
         RowBox[{"\[Tau]samp", "[", "k", "]"}]}], "0"},
       {"0", 
        RowBox[{"\[Tau]samp", "[", "k", "]"}]}
      }], ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"G", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "z_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"1", "+", 
        RowBox[{
         RowBox[{"r", "[", "z", "]"}], 
         RowBox[{"rb", "[", "z", "]"}]}]}], 
       RowBox[{
        RowBox[{"rb", "[", "z", "]"}], 
        RowBox[{"Exp", "[", 
         RowBox[{
          RowBox[{"-", "2"}], " ", "I", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"4", " ", "t", " ", 
             SuperscriptBox["z", "3"]}], "+", 
            RowBox[{"x", " ", "z"}]}], ")"}]}], "]"}]}]},
      {
       RowBox[{
        RowBox[{"r", "[", "z", "]"}], " ", 
        RowBox[{"Exp", "[", 
         RowBox[{"2", " ", "I", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"4", "t", " ", 
             SuperscriptBox["z", "3"]}], "+", " ", 
            RowBox[{"x", " ", "z"}]}], ")"}]}], "]"}]}], "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"G", "[", 
     RowBox[{"_", ",", "_"}], "]"}], "[", 
    RowBox[{"_", "?", "InfinityQ"}], "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1.", "0"},
      {"0", "1."}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Gsamp", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "z_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"1", "+", 
        RowBox[{
         RowBox[{"rsamp", "[", "z", "]"}], 
         RowBox[{"rbsamp", "[", "z", "]"}]}]}], 
       RowBox[{
        RowBox[{"rbsamp", "[", "z", "]"}], 
        RowBox[{"Exp", "[", 
         RowBox[{
          RowBox[{"-", "2"}], " ", "I", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "t", " ", 
             SuperscriptBox["z", "2"]}], "+", 
            RowBox[{"x", " ", "z"}]}], ")"}]}], "]"}]}]},
      {
       RowBox[{
        RowBox[{"rsamp", "[", "z", "]"}], " ", 
        RowBox[{"Exp", "[", 
         RowBox[{"2", " ", "I", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "t", " ", 
             SuperscriptBox["z", "2"]}], "+", " ", 
            RowBox[{"x", " ", "z"}]}], ")"}]}], "]"}]}], "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Gsamp", "[", 
     RowBox[{"_", ",", "_"}], "]"}], "[", 
    RowBox[{"_", "?", "InfinityQ"}], "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1.", "0"},
      {"0", "1."}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"M", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", 
       RowBox[{
        RowBox[{"rb", "[", "k", "]"}], " ", 
        RowBox[{"Exp", "[", 
         RowBox[{"-", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "]"}]}]},
      {"0", "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Msamp", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", 
       RowBox[{
        RowBox[{"rbsamp", "[", "k", "]"}], " ", 
        RowBox[{"Exp", "[", 
         RowBox[{"-", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "]"}]}]},
      {"0", "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Minv", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{"rb", "[", "k", "]"}]}], " ", 
        RowBox[{"Exp", "[", 
         RowBox[{"-", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "]"}]}]},
      {"0", "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Minvsamp", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{"rbsamp", "[", "k", "]"}]}], " ", 
        RowBox[{"Exp", "[", 
         RowBox[{"-", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "]"}]}]},
      {"0", "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"P", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", "0"},
      {
       RowBox[{
        RowBox[{"r", "[", "k", "]"}], " ", 
        RowBox[{"Exp", "[", 
         RowBox[{
          RowBox[{"\[Theta]", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Pin", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", "0"},
      {
       RowBox[{
        RowBox[{"-", 
         RowBox[{"r", "[", "k", "]"}]}], " ", 
        RowBox[{"Exp", "[", 
         RowBox[{
          RowBox[{"\[Theta]", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Psamp", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", "0"},
      {
       RowBox[{
        RowBox[{"rsamp", "[", "k", "]"}], " ", 
        RowBox[{"Exp", "[", 
         RowBox[{
          RowBox[{"\[Theta]", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Pinsamp", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {"1", "0"},
       {
        RowBox[{
         RowBox[{"-", 
          RowBox[{"rsamp", "[", "k", "]"}]}], " ", 
         RowBox[{"Exp", "[", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "1"}
      }], ")"}]}], ";"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"qq", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"out", ",", "i"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"out", " ", "=", " ", "1"}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"i", "=", "1"}], ",", 
        RowBox[{"i", "\[LessEqual]", 
         RowBox[{"Length", "[", "A", "]"}]}], ",", 
        RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Abs", "[", 
              RowBox[{"c", "[", 
               RowBox[{"[", "i", "]"}], "]"}], "]"}], 
             RowBox[{"Exp", "[", 
              RowBox[{"Re", "[", 
               RowBox[{
                RowBox[{"\[Theta]", "[", 
                 RowBox[{"x", ",", "t"}], "]"}], "[", 
                RowBox[{"A", "[", 
                 RowBox[{"[", "i", "]"}], "]"}], "]"}], "]"}], "]"}]}], ">", 
            "expconst"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"out", " ", "=", " ", 
             RowBox[{"out", "*", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"k", "-", 
                 RowBox[{"A", "[", 
                  RowBox[{"[", "i", "]"}], "]"}]}], ")"}], "/", 
               RowBox[{"(", 
                RowBox[{"k", "-", 
                 RowBox[{"Conjugate", "[", 
                  RowBox[{"A", "[", 
                   RowBox[{"[", "i", "]"}], "]"}], "]"}]}], ")"}]}]}]}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", "out"}]}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Q", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{
        RowBox[{"qq", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "0"},
      {"0", 
       RowBox[{"1", "/", 
        RowBox[{
         RowBox[{"qq", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Qin", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"1", "/", 
        RowBox[{
         RowBox[{"qq", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "0"},
      {"0", 
       RowBox[{
        RowBox[{"qq", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Q", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", 
    RowBox[{"_", "?", "InfinityQ"}], "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1.", "0"},
      {"0", "1."}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"LogIntVariable", "[", 
    RowBox[{"s_", ",", "k0_", ",", "z_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"-", "2"}], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"Log", "[", "k0", "]"}]}], " ", 
        RowBox[{"Log", "[", 
         RowBox[{"1", "-", 
          FractionBox["k0", 
           RowBox[{"z", "+", 
            RowBox[{"s", " ", 
             RowBox[{"10.", "^", 
              RowBox[{"(", 
               RowBox[{"-", "30"}], ")"}]}], " ", "I"}]}]]}], "]"}]}], "+", 
       RowBox[{
        RowBox[{"Log", "[", "k0", "]"}], " ", 
        RowBox[{"Log", "[", 
         FractionBox[
          RowBox[{"k0", "+", "z", "+", 
           RowBox[{"s", " ", 
            RowBox[{"10.", "^", 
             RowBox[{"(", 
              RowBox[{"-", "30"}], ")"}]}], " ", "I"}]}], 
          RowBox[{"z", "+", 
           RowBox[{"s", " ", 
            RowBox[{"10.", "^", 
             RowBox[{"(", 
              RowBox[{"-", "30"}], ")"}]}], " ", "I"}]}]], "]"}]}], "+", 
       RowBox[{"PolyLog", "[", 
        RowBox[{"2", ",", 
         RowBox[{"-", 
          FractionBox["k0", 
           RowBox[{"z", "+", 
            RowBox[{"s", " ", 
             RowBox[{"10.", "^", 
              RowBox[{"(", 
               RowBox[{"-", "30"}], ")"}]}], " ", "I"}]}]]}]}], "]"}], "-", 
       RowBox[{"PolyLog", "[", 
        RowBox[{"2", ",", 
         FractionBox["k0", 
          RowBox[{"z", "+", 
           RowBox[{"s", " ", 
            RowBox[{"10.", "^", 
             RowBox[{"(", 
              RowBox[{"-", "30"}], ")"}]}], " ", "I"}]}]]}], "]"}]}], ")"}], 
     "/", 
     RowBox[{"(", 
      RowBox[{"2", "Pi", " ", "I"}], ")"}]}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"LogIntVariable", "[", 
    RowBox[{"k0_", ",", "z_"}], "]"}], ":=", 
   RowBox[{"LogIntVariable", "[", 
    RowBox[{"0.", ",", "k0", ",", "z"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"startift", "[", "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"const", ",", "eps"}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"LL", "=", 
        RowBox[{"dom", "[", 
         RowBox[{"[", 
          RowBox[{"1", ",", "2"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"eps", " ", "=", " ", "$MachineEpsilon"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{"singflag", ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"Clear", "[", 
           RowBox[{"ift", ",", "ifl", ",", "ifr", ",", "\[Delta]t"}], "]"}], 
          ";", "\[IndentingNewLine]", 
          RowBox[{"const", "=", 
           RowBox[{"1", "/", "2"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"ift", "[", "k0_", "]"}], ":=", 
           RowBox[{
            RowBox[{"ift", "[", "k0", "]"}], "=", 
            RowBox[{"Fun", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"Log", "[", 
                RowBox[{
                 RowBox[{"\[Tau]", "[", "#", "]"}], "/", 
                 RowBox[{
                  RowBox[{"Abs", "[", "#", "]"}], "^", "2"}]}], "]"}], "&"}], 
              ",", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"-", 
                  RowBox[{"Min", "[", 
                   RowBox[{"k0", ",", "LL"}], "]"}]}], ",", 
                 RowBox[{"Min", "[", 
                  RowBox[{"k0", ",", "LL"}], "]"}]}], "}"}], "//", "Line"}], 
              ",", 
              RowBox[{"2", "*", "smallN"}]}], "]"}]}]}], ";", 
          RowBox[{"(*", " ", 
           RowBox[{
           "must", " ", "have", " ", "even", " ", "number", " ", "of", " ", 
            "grid", " ", "points"}], "*)"}], "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"ifl", "[", "k0_", "]"}], ":=", 
           RowBox[{
            RowBox[{"ifl", "[", "k0", "]"}], "=", 
            RowBox[{"Fun", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"Log", "[", 
                RowBox[{"\[Tau]", "[", "#", "]"}], "]"}], "&"}], ",", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"-", 
                  RowBox[{"Min", "[", 
                   RowBox[{"k0", ",", "LL"}], "]"}]}], ",", 
                 RowBox[{"-", "const"}]}], "}"}], "//", "Line"}], ",", 
              "bigN"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"ifr", "[", "k0_", "]"}], ":=", 
           RowBox[{
            RowBox[{"ifr", "[", "k0", "]"}], "=", 
            RowBox[{"Fun", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"Log", "[", 
                RowBox[{"\[Tau]", "[", "#", "]"}], "]"}], "&"}], ",", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{"const", ",", 
                 RowBox[{"Min", "[", 
                  RowBox[{"k0", ",", "LL"}], "]"}]}], "}"}], "//", "Line"}], 
              ",", "bigN"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{"\[Delta]t", "[", 
             RowBox[{"x_", ",", "t_"}], "]"}], "[", 
            RowBox[{"s_", ",", "k_"}], "]"}], ":=", 
           RowBox[{"Module", "[", 
            RowBox[{
             RowBox[{"{", "out", "}"}], ",", 
             RowBox[{
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"Abs", "[", 
                  RowBox[{"K0", "[", 
                   RowBox[{"x", ",", "t"}], "]"}], "]"}], "\[LessEqual]", 
                 "const"}], ",", "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{"out", "=", 
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"LogIntVariable", "[", 
                    RowBox[{
                    RowBox[{"Min", "[", 
                    RowBox[{
                    RowBox[{"Abs", "[", 
                    RowBox[{"K0", "[", 
                    RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", "LL"}], "]"}],
                     ",", 
                    RowBox[{"k", "+", 
                    RowBox[{"s", " ", "I", " ", "eps"}]}]}], "]"}], "+", 
                    RowBox[{"Cauchy", "[", 
                    RowBox[{
                    RowBox[{"ift", "[", 
                    RowBox[{"Abs", "[", 
                    RowBox[{"K0", "[", 
                    RowBox[{"x", ",", "t"}], "]"}], "]"}], "]"}], ",", 
                    RowBox[{"k", " ", "+", 
                    RowBox[{"s", " ", "I", " ", "eps"}]}]}], "]"}]}], "//", 
                    "Exp"}], ")"}]}], ";"}], ",", "\[IndentingNewLine]", 
                RowBox[{
                 RowBox[{"out", "=", 
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"LogIntVariable", "[", 
                    RowBox[{"const", ",", "k"}], "]"}], "+", 
                    RowBox[{"Cauchy", "[", 
                    RowBox[{"s", ",", 
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"ift", "[", "const", "]"}], ",", 
                    RowBox[{"ifl", "[", 
                    RowBox[{"Abs", "[", 
                    RowBox[{"K0", "[", 
                    RowBox[{"x", ",", "t"}], "]"}], "]"}], "]"}], ",", 
                    RowBox[{"ifr", "[", 
                    RowBox[{"Abs", "[", 
                    RowBox[{"K0", "[", 
                    RowBox[{"x", ",", "t"}], "]"}], "]"}], "]"}]}], "}"}], 
                    ",", "k"}], " ", "]"}]}], "//", "Exp"}], ")"}]}], ";"}]}],
                "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
              "out"}]}], "\[IndentingNewLine]", "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{"\[Delta]t", "[", 
             RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
           RowBox[{
            RowBox[{
             RowBox[{"\[Delta]t", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "=", 
            RowBox[{"Module", "[", 
             RowBox[{
              RowBox[{"{", "out", "}"}], ",", 
              RowBox[{
               RowBox[{"If", "[", 
                RowBox[{
                 RowBox[{
                  RowBox[{"Abs", "[", 
                   RowBox[{"K0", "[", 
                    RowBox[{"x", ",", "t"}], "]"}], "]"}], "\[LessEqual]", 
                  "const"}], ",", "\[IndentingNewLine]", 
                 RowBox[{
                  RowBox[{"out", "=", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"LogIntVariable", "[", 
                    RowBox[{
                    RowBox[{"Min", "[", 
                    RowBox[{
                    RowBox[{"Abs", "[", 
                    RowBox[{"K0", "[", 
                    RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", "LL"}], "]"}],
                     ",", "k"}], "]"}], "+", 
                    RowBox[{"Cauchy", "[", 
                    RowBox[{
                    RowBox[{"ift", "[", 
                    RowBox[{"Abs", "[", 
                    RowBox[{"K0", "[", 
                    RowBox[{"x", ",", "t"}], "]"}], "]"}], "]"}], ",", "k"}], 
                    " ", "]"}]}], "//", "Exp"}], ")"}]}], ";"}], ",", 
                 "\[IndentingNewLine]", 
                 RowBox[{
                  RowBox[{"out", "=", 
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"LogIntVariable", "[", 
                    RowBox[{"const", ",", "k"}], "]"}], "+", 
                    RowBox[{"Cauchy", "[", 
                    RowBox[{
                    RowBox[{"{", 
                    RowBox[{
                    RowBox[{"ift", "[", "const", "]"}], ",", 
                    RowBox[{"ifl", "[", 
                    RowBox[{"Abs", "[", 
                    RowBox[{"K0", "[", 
                    RowBox[{"x", ",", "t"}], "]"}], "]"}], "]"}], ",", 
                    RowBox[{"ifr", "[", 
                    RowBox[{"Abs", "[", 
                    RowBox[{"K0", "[", 
                    RowBox[{"x", ",", "t"}], "]"}], "]"}], "]"}]}], "}"}], 
                    ",", "k"}], " ", "]"}]}], "//", "Exp"}], ")"}]}], ";"}]}],
                 "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
               "out"}]}], "\[IndentingNewLine]", "]"}]}]}], ";"}], 
         "\[IndentingNewLine]", ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"Clear", "[", 
           RowBox[{"ift", ",", "\[Delta]t"}], "]"}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"ift", "[", "k0_", "]"}], ":=", 
           RowBox[{
            RowBox[{"ift", "[", "k0", "]"}], "=", 
            RowBox[{"Fun", "[", 
             RowBox[{
              RowBox[{
               RowBox[{"Log", "[", 
                RowBox[{"\[Tau]", "[", "#", "]"}], "]"}], "&"}], ",", 
              RowBox[{
               RowBox[{"{", 
                RowBox[{
                 RowBox[{"-", 
                  RowBox[{"Min", "[", 
                   RowBox[{"k0", ",", "LL"}], "]"}]}], ",", 
                 RowBox[{"Min", "[", 
                  RowBox[{"k0", ",", "LL"}], "]"}]}], "}"}], "//", "Line"}], 
              ",", "100"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{"\[Delta]t", "[", 
             RowBox[{"x_", ",", "t_"}], "]"}], "[", 
            RowBox[{"s_", ",", "k_"}], "]"}], ":=", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Cauchy", "[", 
              RowBox[{"s", ",", 
               RowBox[{"ift", "[", 
                RowBox[{"Abs", "[", 
                 RowBox[{"K0", "[", 
                  RowBox[{"x", ",", "t"}], "]"}], "]"}], "]"}], ",", "k"}], 
              "]"}], "//", "Exp"}], ")"}]}], ";", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{"\[Delta]t", "[", 
             RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
           RowBox[{
            RowBox[{
             RowBox[{"\[Delta]t", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "=", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"Cauchy", "[", 
               RowBox[{
                RowBox[{"ift", "[", 
                 RowBox[{"Abs", "[", 
                  RowBox[{"K0", "[", 
                   RowBox[{"x", ",", "t"}], "]"}], "]"}], "]"}], ",", "k"}], 
               "]"}], "//", "Exp"}], ")"}]}]}], ";"}]}], 
        "\[IndentingNewLine]", "]"}], ";"}]}], "\[IndentingNewLine]", "]"}]}],
    ";"}], "\[IndentingNewLine]"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalPhi]t", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "z__", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{
        RowBox[{"\[Delta]t", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "[", "z", "]"}], "0"},
      {"0", 
       FractionBox["1", 
        RowBox[{
         RowBox[{"\[Delta]t", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "z", "]"}]]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalPhi]tin", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "z__", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"1", "/", 
        RowBox[{
         RowBox[{"\[Delta]t", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "z", "]"}]}], "0"},
      {"0", 
       RowBox[{
        RowBox[{"\[Delta]t", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "[", "z", "]"}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"K0", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Sqrt", "[", 
    RowBox[{
     RowBox[{"-", "x"}], "/", 
     RowBox[{"(", 
      RowBox[{"12", " ", "t"}], ")"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"K0", "[", 
    RowBox[{"_", ",", 
     RowBox[{"t_", "?", "ZeroQ"}]}], "]"}], ":=", "\[Infinity]"}], 
  ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.538103142394384*^9, 3.538103151534226*^9}, {
   3.5381035800010853`*^9, 3.5381035806070843`*^9}, {3.540653533861*^9, 
   3.540653534101*^9}, {3.5406547841210003`*^9, 3.540654804311*^9}, {
   3.5406548650150003`*^9, 3.540654883401*^9}, {3.5406583132382*^9, 
   3.5406583318182*^9}, {3.5406593163606*^9, 3.5406593424706*^9}, {
   3.5406606785154*^9, 3.5406606829554*^9}, {3.540670035786598*^9, 
   3.540670045469647*^9}, {3.5406701523856287`*^9, 3.540670159109951*^9}, {
   3.540671043884227*^9, 3.5406710445151052`*^9}, {3.54067107622948*^9, 
   3.540671117438383*^9}, 3.540671223981215*^9, 3.540673675932254*^9, {
   3.5406742109124002`*^9, 3.5406742112953997`*^9}, {3.5406777028814*^9, 
   3.5406777049323997`*^9}, {3.5406826023554*^9, 3.5406826068993998`*^9}, {
   3.5406828924653997`*^9, 3.5406828948314*^9}, {3.5406831061644*^9, 
   3.5406831144644003`*^9}, {3.5406844599924*^9, 3.5406844614054003`*^9}, {
   3.5406847746944*^9, 3.5406847750914*^9}, {3.5406849669924*^9, 
   3.5406849737824*^9}, {3.5406851745424*^9, 3.5406852291823997`*^9}, {
   3.5406853996924*^9, 3.5406853998624*^9}, 3.5406876104944*^9, {
   3.5406888923224*^9, 3.5406889113124*^9}, {3.5407250148524*^9, 
   3.5407250177324*^9}, {3.5407255652974*^9, 3.5407255687424*^9}, {
   3.5407264058993998`*^9, 3.5407264153334*^9}, {3.5407270816303997`*^9, 
   3.5407270841644*^9}, {3.5407272503754*^9, 3.5407272625214*^9}, {
   3.5407277902693996`*^9, 3.5407277906574*^9}, {3.5407280626984*^9, 
   3.5407280632244*^9}, {3.540732091156747*^9, 3.540732094732853*^9}, {
   3.5407337077895193`*^9, 3.540733708610265*^9}, {3.540734314953412*^9, 
   3.540734315469853*^9}, {3.540734447769846*^9, 3.540734448600122*^9}, {
   3.5407364063724413`*^9, 3.5407364098191147`*^9}, {3.5407369996439257`*^9, 
   3.54073700378605*^9}, {3.540737269754858*^9, 3.540737270296258*^9}, {
   3.5407374577807903`*^9, 3.540737460562327*^9}, 3.5407388991890717`*^9, {
   3.540739981122607*^9, 3.540739982269462*^9}, {3.540740493804627*^9, 
   3.540740495873988*^9}, {3.540740792568321*^9, 3.540740792845584*^9}, {
   3.540741429871428*^9, 3.540741430139724*^9}, {3.540743857872079*^9, 
   3.5407438622771*^9}, {3.540749237001852*^9, 3.5407492383893623`*^9}, {
   3.5407608990193996`*^9, 3.5407609003594*^9}, {3.5407636848574*^9, 
   3.5407637029354*^9}, {3.5407637497794*^9, 3.5407637906194*^9}, {
   3.5407639569974003`*^9, 3.5407639572774*^9}, 3.5407685188574*^9, {
   3.5407690527953997`*^9, 3.5407690690114*^9}, {3.540839541133526*^9, 
   3.540839547433312*^9}, {3.5409029240854*^9, 3.5409029383283997`*^9}, 
   3.5409035117384*^9, 3.5409416746544*^9, 3.5656171862981997`*^9, {
   3.5656174193571997`*^9, 3.5656174476022*^9}, {3.5656175091342*^9, 
   3.5656175348802*^9}, {3.5656175759962*^9, 3.5656176103392*^9}, {
   3.5656176410592003`*^9, 3.5656176662782*^9}, {3.5656177086092*^9, 
   3.5656177351282*^9}, {3.5656204709592*^9, 3.5656205127052*^9}, {
   3.5656212498292*^9, 3.5656214052762003`*^9}, {3.5656411683152*^9, 
   3.5656411980062*^9}, {3.5656586528082*^9, 3.5656587202482*^9}, {
   3.565754330008767*^9, 3.565754333067934*^9}, {3.5681288606726007`*^9, 
   3.568128864037929*^9}, {3.568129172366681*^9, 3.568129210978828*^9}, {
   3.568129281093484*^9, 3.568129290380556*^9}, {3.568129652490979*^9, 
   3.568129663654998*^9}, {3.56813263902489*^9, 3.568132648010124*^9}, {
   3.568132691574448*^9, 3.56813270059894*^9}, {3.5681455250180607`*^9, 
   3.5681455292247887`*^9}, {3.5681471446438637`*^9, 3.568147147953083*^9}, {
   3.568147241915513*^9, 3.568147324679017*^9}, {3.568147364300024*^9, 
   3.5681473662966757`*^9}, {3.568147541341443*^9, 3.568147544281726*^9}, {
   3.568147752436235*^9, 3.568147766837874*^9}, {3.5681487126240683`*^9, 
   3.568148725181775*^9}, {3.568332505130286*^9, 3.568332564706992*^9}, {
   3.568333948645219*^9, 3.568334002967908*^9}, {3.5683788423046618`*^9, 
   3.568378869836611*^9}, {3.5683789776998377`*^9, 3.5683789828934517`*^9}, {
   3.56840309240283*^9, 3.5684031190900106`*^9}, {3.56840490216264*^9, 
   3.568404947105895*^9}, {3.5684049938814707`*^9, 3.568405018480193*^9}, {
   3.568405104267029*^9, 3.5684051044619226`*^9}, {3.568405141189176*^9, 
   3.568405277129157*^9}, {3.5684053731333942`*^9, 3.568405404593604*^9}, {
   3.568405503753558*^9, 3.5684055592137117`*^9}, {3.568405689374742*^9, 
   3.568405689658036*^9}, {3.5684058600139437`*^9, 3.568405860620275*^9}, {
   3.568406187976935*^9, 3.568406188194696*^9}, {3.568406395432805*^9, 
   3.568406404044888*^9}, {3.568406452860858*^9, 3.568406457080192*^9}, {
   3.5684065000722713`*^9, 3.5684065061550493`*^9}, {3.568406827941753*^9, 
   3.56840684192846*^9}, {3.568406976423539*^9, 3.568407007201652*^9}, {
   3.568407053162166*^9, 3.568407054760188*^9}, {3.568407105228881*^9, 
   3.568407110701915*^9}, {3.568407149723048*^9, 3.568407153705937*^9}, {
   3.568413834370887*^9, 3.568413849151699*^9}, {3.568413892056828*^9, 
   3.568414187013473*^9}, {3.568414222992474*^9, 3.568414320323882*^9}, {
   3.568414436620501*^9, 3.568414470100946*^9}, {3.568422053427067*^9, 
   3.5684220605591917`*^9}, {3.568423007586426*^9, 3.568423029136219*^9}, {
   3.568423779433147*^9, 3.568423784068293*^9}, {3.568423861658251*^9, 
   3.568423867328888*^9}, {3.568424367544805*^9, 3.568424369736207*^9}, {
   3.568424641773058*^9, 3.56842464279823*^9}, {3.568424803573354*^9, 
   3.56842480993932*^9}, {3.5684263321982193`*^9, 3.568426333549808*^9}, {
   3.568426373901404*^9, 3.568426386543342*^9}, {3.568426547387495*^9, 
   3.5684265922842693`*^9}, {3.568426919737748*^9, 3.5684269239698687`*^9}, {
   3.568427037863954*^9, 3.56842704368211*^9}, {3.5684271925537043`*^9, 
   3.5684271931032743`*^9}, {3.568427272363194*^9, 3.568427281885763*^9}, {
   3.568474527964669*^9, 3.568474556811685*^9}, {3.56847471869173*^9, 
   3.568474752028996*^9}, {3.568475889763123*^9, 3.568475892206497*^9}, {
   3.56848009680085*^9, 3.56848012747853*^9}, {3.568480218400028*^9, 
   3.5684802274342623`*^9}, {3.5685759065704107`*^9, 3.568575916324731*^9}, {
   3.568664885590089*^9, 3.568664906356408*^9}, {3.56873747529211*^9, 
   3.568737483016428*^9}, {3.568737639073501*^9, 3.568737639419819*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Differentiated Unscaled Functions", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.5408983275643997`*^9, 3.5408983375383997`*^9}, {
  3.568128918926874*^9, 3.568128920316318*^9}, {3.5683789913759212`*^9, 
  3.568378995567071*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Theta]x", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "z_", "]"}], ":=", 
    RowBox[{"2", " ", "I", " ", "z"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Lx", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {"0", "0"},
       {
        RowBox[{
         FractionBox[
          RowBox[{"r", "[", "k", "]"}], 
          RowBox[{"\[Tau]", "[", "k", "]"}]], 
         RowBox[{
          RowBox[{"\[Theta]x", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], 
         RowBox[{"Exp", "[", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "0"}
      }], ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Linx", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {"0", "0"},
       {
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{"r", "[", "k", "]"}], 
           RowBox[{"\[Tau]", "[", "k", "]"}]]}], 
         RowBox[{
          RowBox[{"\[Theta]x", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], 
         RowBox[{"Exp", "[", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "0"}
      }], ")"}]}], ";"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Ux", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"0", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{
          RowBox[{"\[Theta]x", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], 
        FractionBox[
         RowBox[{"rb", "[", "k", "]"}], 
         RowBox[{"\[Tau]", "[", "k", "]"}]], 
        RowBox[{"Exp", "[", 
         RowBox[{"-", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "]"}]}]},
      {"0", "0"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Uinx", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {"0", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Theta]x", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], 
         FractionBox[
          RowBox[{"rb", "[", "k", "]"}], 
          RowBox[{"\[Tau]", "[", "k", "]"}]], 
         RowBox[{"Exp", "[", 
          RowBox[{"-", 
           RowBox[{
            RowBox[{"\[Theta]", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "]"}]}]},
       {"0", "0"}
      }], ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Gx", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "z_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"0", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{
          RowBox[{"\[Theta]x", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "z", "]"}]}], 
        RowBox[{"rb", "[", "z", "]"}], 
        RowBox[{"Exp", "[", 
         RowBox[{
          RowBox[{"-", "2"}], " ", "I", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"4", " ", "t", " ", 
             SuperscriptBox["z", "3"]}], "+", 
            RowBox[{"x", " ", "z"}]}], ")"}]}], "]"}]}]},
      {
       RowBox[{
        RowBox[{
         RowBox[{"\[Theta]x", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "z", "]"}], 
        RowBox[{"r", "[", "z", "]"}], " ", 
        RowBox[{"Exp", "[", 
         RowBox[{"2", " ", "I", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"4", "t", " ", 
             SuperscriptBox["z", "3"]}], "+", " ", 
            RowBox[{"x", " ", "z"}]}], ")"}]}], "]"}]}], "0"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Mx", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"0", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{
          RowBox[{"\[Theta]x", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], 
        RowBox[{"rb", "[", "k", "]"}], " ", 
        RowBox[{"Exp", "[", 
         RowBox[{"-", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "]"}]}]},
      {"0", "0"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Minvx", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"0", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Theta]x", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], 
        RowBox[{"rb", "[", "k", "]"}], " ", 
        RowBox[{"Exp", "[", 
         RowBox[{"-", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "]"}]}]},
      {"0", "0"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Px", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"0", "0"},
      {
       RowBox[{
        RowBox[{
         RowBox[{"\[Theta]x", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], 
        RowBox[{"r", "[", "k", "]"}], " ", 
        RowBox[{"Exp", "[", 
         RowBox[{
          RowBox[{"\[Theta]", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "0"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Pinx", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"0", "0"},
      {
       RowBox[{
        RowBox[{"-", 
         RowBox[{
          RowBox[{"\[Theta]x", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], 
        RowBox[{"r", "[", "k", "]"}], " ", 
        RowBox[{"Exp", "[", 
         RowBox[{
          RowBox[{"\[Theta]", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "0"}
     }], ")"}]}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.538103142394384*^9, 3.538103151534226*^9}, {
   3.5381035800010853`*^9, 3.5381035806070843`*^9}, {3.540653533861*^9, 
   3.540653534101*^9}, {3.5406547841210003`*^9, 3.540654804311*^9}, {
   3.5406548650150003`*^9, 3.540654883401*^9}, {3.5406583132382*^9, 
   3.5406583318182*^9}, {3.5406593163606*^9, 3.5406593424706*^9}, {
   3.5406606785154*^9, 3.5406606829554*^9}, {3.540670035786598*^9, 
   3.540670045469647*^9}, {3.5406701523856287`*^9, 3.540670159109951*^9}, {
   3.540671043884227*^9, 3.5406710445151052`*^9}, {3.54067107622948*^9, 
   3.540671117438383*^9}, 3.540671223981215*^9, 3.540673675932254*^9, {
   3.5406742109124002`*^9, 3.5406742112953997`*^9}, {3.5406777028814*^9, 
   3.5406777049323997`*^9}, {3.5406826023554*^9, 3.5406826068993998`*^9}, {
   3.5406828924653997`*^9, 3.5406828948314*^9}, {3.5406831061644*^9, 
   3.5406831144644003`*^9}, {3.5406844599924*^9, 3.5406844614054003`*^9}, {
   3.5406847746944*^9, 3.5406847750914*^9}, {3.5406849669924*^9, 
   3.5406849737824*^9}, {3.5406851745424*^9, 3.5406852291823997`*^9}, {
   3.5406853996924*^9, 3.5406853998624*^9}, 3.5406876104944*^9, {
   3.5406888923224*^9, 3.5406889113124*^9}, {3.5407250148524*^9, 
   3.5407250177324*^9}, {3.5407255652974*^9, 3.5407255687424*^9}, {
   3.5407264058993998`*^9, 3.5407264153334*^9}, {3.5407270816303997`*^9, 
   3.5407270841644*^9}, {3.5407272503754*^9, 3.5407272625214*^9}, {
   3.5407277902693996`*^9, 3.5407277906574*^9}, {3.5407280626984*^9, 
   3.5407280632244*^9}, {3.540732091156747*^9, 3.540732094732853*^9}, {
   3.5407337077895193`*^9, 3.540733708610265*^9}, {3.540734314953412*^9, 
   3.540734315469853*^9}, {3.540734447769846*^9, 3.540734448600122*^9}, {
   3.5407364063724413`*^9, 3.5407364098191147`*^9}, {3.5407369996439257`*^9, 
   3.54073700378605*^9}, {3.540737269754858*^9, 3.540737270296258*^9}, {
   3.5407374577807903`*^9, 3.540737460562327*^9}, 3.5407388991890717`*^9, {
   3.540739981122607*^9, 3.540739982269462*^9}, {3.540740493804627*^9, 
   3.540740495873988*^9}, {3.540740792568321*^9, 3.540740792845584*^9}, {
   3.540741429871428*^9, 3.540741430139724*^9}, {3.540743857872079*^9, 
   3.5407438622771*^9}, {3.540749237001852*^9, 3.5407492383893623`*^9}, {
   3.5407608990193996`*^9, 3.5407609003594*^9}, {3.5407636848574*^9, 
   3.5407637029354*^9}, {3.5407637497794*^9, 3.5407637906194*^9}, {
   3.5407639569974003`*^9, 3.5407639572774*^9}, 3.5407685188574*^9, {
   3.5407690527953997`*^9, 3.5407690690114*^9}, {3.540839541133526*^9, 
   3.540839547433312*^9}, {3.5409029240854*^9, 3.5409029383283997`*^9}, 
   3.5409035117384*^9, 3.5409416746544*^9, 3.5656171862981997`*^9, {
   3.5656174193571997`*^9, 3.5656174476022*^9}, {3.5656175091342*^9, 
   3.5656175348802*^9}, {3.5656175759962*^9, 3.5656176103392*^9}, {
   3.5656176410592003`*^9, 3.5656176662782*^9}, {3.5656177086092*^9, 
   3.5656177351282*^9}, {3.5656204709592*^9, 3.5656205127052*^9}, {
   3.5656212498292*^9, 3.5656214052762003`*^9}, {3.5656411683152*^9, 
   3.5656411980062*^9}, {3.5656586528082*^9, 3.5656587202482*^9}, {
   3.565754330008767*^9, 3.565754333067934*^9}, {3.5681288606726007`*^9, 
   3.568128864037929*^9}, {3.568129172366681*^9, 3.568129210978828*^9}, {
   3.568129281093484*^9, 3.568129290380556*^9}, {3.568129652490979*^9, 
   3.568129663654998*^9}, {3.56813263902489*^9, 3.568132648010124*^9}, {
   3.568132691574448*^9, 3.56813270059894*^9}, {3.5681455250180607`*^9, 
   3.5681455292247887`*^9}, {3.5681471446438637`*^9, 3.568147147953083*^9}, {
   3.568147241915513*^9, 3.568147324679017*^9}, {3.568147364300024*^9, 
   3.5681473662966757`*^9}, {3.568147541341443*^9, 3.568147544281726*^9}, {
   3.568147752436235*^9, 3.568147766837874*^9}, {3.5681487126240683`*^9, 
   3.568148725181775*^9}, {3.568332505130286*^9, 3.568332564706992*^9}, {
   3.568333948645219*^9, 3.568334002967908*^9}, {3.5683788423046618`*^9, 
   3.568378869836611*^9}, {3.5683789776998377`*^9, 3.56837931511119*^9}, {
   3.568406946707398*^9, 3.568406966480369*^9}, {3.568413205481204*^9, 
   3.568413205973881*^9}, {3.5685723217601557`*^9, 3.5685723577803917`*^9}, {
   3.568648108538783*^9, 3.568648115463829*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Scaled Functions", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.5408983275643997`*^9, 3.5408983375383997`*^9}, {
  3.568128918926874*^9, 3.568128920316318*^9}, {3.568571824553351*^9, 
  3.568571825480629*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Ls", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"L", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Lsamps", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Lsamp", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Lins", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Lin", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Linsamps", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Linsamp", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Us", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"U", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Uins", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Uin", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Usamps", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Usamp", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Uinsamps", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Uinsamp", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DDs", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"DD", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DDins", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"DDin", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DDsampss", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"DDsamp", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DDinsamps", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"DDinsamp", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{
    RowBox[{"Gs", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"G", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Gsamps", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Gsamp", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Ms", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"M", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Msamps", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Msamp", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Minvs", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Minv", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Minvsamps", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Minvsamp", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Ps", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"P", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Psamps", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Psamp", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Pins", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Pin", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Pinsamps", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Pinsamp", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Qs", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Q", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalPhi]ts", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"\[CapitalPhi]t", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalPhi]tins", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"\[CapitalPhi]tin", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.538103142394384*^9, 3.538103151534226*^9}, {
   3.5381035800010853`*^9, 3.5381035806070843`*^9}, {3.540653533861*^9, 
   3.540653534101*^9}, {3.5406547841210003`*^9, 3.540654804311*^9}, {
   3.5406548650150003`*^9, 3.540654883401*^9}, {3.5406583132382*^9, 
   3.5406583318182*^9}, {3.5406593163606*^9, 3.5406593424706*^9}, {
   3.5406606785154*^9, 3.5406606829554*^9}, {3.540670035786598*^9, 
   3.540670045469647*^9}, {3.5406701523856287`*^9, 3.540670159109951*^9}, {
   3.540671043884227*^9, 3.5406710445151052`*^9}, {3.54067107622948*^9, 
   3.540671117438383*^9}, 3.540671223981215*^9, 3.540673675932254*^9, {
   3.5406742109124002`*^9, 3.5406742112953997`*^9}, {3.5406777028814*^9, 
   3.5406777049323997`*^9}, {3.5406826023554*^9, 3.5406826068993998`*^9}, {
   3.5406828924653997`*^9, 3.5406828948314*^9}, {3.5406831061644*^9, 
   3.5406831144644003`*^9}, {3.5406844599924*^9, 3.5406844614054003`*^9}, {
   3.5406847746944*^9, 3.5406847750914*^9}, {3.5406849669924*^9, 
   3.5406849737824*^9}, {3.5406851745424*^9, 3.5406852291823997`*^9}, {
   3.5406853996924*^9, 3.5406853998624*^9}, 3.5406876104944*^9, {
   3.5406888923224*^9, 3.5406889113124*^9}, {3.5407250148524*^9, 
   3.5407250177324*^9}, {3.5407255652974*^9, 3.5407255687424*^9}, {
   3.5407264058993998`*^9, 3.5407264153334*^9}, {3.5407270816303997`*^9, 
   3.5407270841644*^9}, {3.5407272503754*^9, 3.5407272625214*^9}, {
   3.5407277902693996`*^9, 3.5407277906574*^9}, {3.5407280626984*^9, 
   3.5407280632244*^9}, {3.540732091156747*^9, 3.540732094732853*^9}, {
   3.5407337077895193`*^9, 3.540733708610265*^9}, {3.540734314953412*^9, 
   3.540734315469853*^9}, {3.540734447769846*^9, 3.540734448600122*^9}, {
   3.5407364063724413`*^9, 3.5407364098191147`*^9}, {3.5407369996439257`*^9, 
   3.54073700378605*^9}, {3.540737269754858*^9, 3.540737270296258*^9}, {
   3.5407374577807903`*^9, 3.540737460562327*^9}, 3.5407388991890717`*^9, {
   3.540739981122607*^9, 3.540739982269462*^9}, {3.540740493804627*^9, 
   3.540740495873988*^9}, {3.540740792568321*^9, 3.540740792845584*^9}, {
   3.540741429871428*^9, 3.540741430139724*^9}, {3.540743857872079*^9, 
   3.5407438622771*^9}, {3.540749237001852*^9, 3.5407492383893623`*^9}, {
   3.5407608990193996`*^9, 3.5407609003594*^9}, {3.5407636848574*^9, 
   3.5407637029354*^9}, {3.5407637497794*^9, 3.5407637906194*^9}, {
   3.5407639569974003`*^9, 3.5407639572774*^9}, 3.5407685188574*^9, {
   3.5407690527953997`*^9, 3.5407690690114*^9}, {3.540839541133526*^9, 
   3.540839547433312*^9}, {3.5409029240854*^9, 3.5409029383283997`*^9}, 
   3.5409035117384*^9, 3.5409416746544*^9, 3.5656171862981997`*^9, {
   3.5656174193571997`*^9, 3.5656174476022*^9}, {3.5656175091342*^9, 
   3.5656175348802*^9}, {3.5656175759962*^9, 3.5656176103392*^9}, {
   3.5656176410592003`*^9, 3.5656176662782*^9}, {3.5656177086092*^9, 
   3.5656177351282*^9}, {3.5656204709592*^9, 3.5656205127052*^9}, {
   3.5656212498292*^9, 3.5656214052762003`*^9}, {3.5656411683152*^9, 
   3.5656411980062*^9}, {3.5656586528082*^9, 3.5656587202482*^9}, {
   3.565754330008767*^9, 3.565754333067934*^9}, {3.5681288606726007`*^9, 
   3.568128864037929*^9}, {3.568129172366681*^9, 3.568129210978828*^9}, {
   3.568129281093484*^9, 3.568129290380556*^9}, {3.568129652490979*^9, 
   3.568129663654998*^9}, {3.56813263902489*^9, 3.568132648010124*^9}, {
   3.568132691574448*^9, 3.56813270059894*^9}, {3.5681455250180607`*^9, 
   3.5681455292247887`*^9}, {3.5681471446438637`*^9, 3.568147147953083*^9}, {
   3.568147241915513*^9, 3.568147324679017*^9}, {3.568147364300024*^9, 
   3.5681473662966757`*^9}, {3.568147541341443*^9, 3.568147544281726*^9}, {
   3.568147752436235*^9, 3.568147766837874*^9}, {3.5681487126240683`*^9, 
   3.568148725181775*^9}, {3.568332505130286*^9, 3.568332564706992*^9}, {
   3.568333948645219*^9, 3.568334002967908*^9}, {3.5683788423046618`*^9, 
   3.568378869836611*^9}, {3.5683789776998377`*^9, 3.5683789828934517`*^9}, {
   3.56840309240283*^9, 3.5684031190900106`*^9}, {3.56840490216264*^9, 
   3.568404947105895*^9}, {3.5684049938814707`*^9, 3.568405018480193*^9}, {
   3.568405104267029*^9, 3.5684051044619226`*^9}, {3.568405141189176*^9, 
   3.568405277129157*^9}, {3.5684053731333942`*^9, 3.568405404593604*^9}, {
   3.568405503753558*^9, 3.5684055592137117`*^9}, {3.568405689374742*^9, 
   3.568405689658036*^9}, {3.5684058600139437`*^9, 3.568405860620275*^9}, {
   3.568406187976935*^9, 3.568406188194696*^9}, {3.568406395432805*^9, 
   3.568406404044888*^9}, {3.568406452860858*^9, 3.568406457080192*^9}, {
   3.5684065000722713`*^9, 3.5684065061550493`*^9}, {3.568406827941753*^9, 
   3.56840684192846*^9}, {3.568406976423539*^9, 3.568407007201652*^9}, {
   3.568407053162166*^9, 3.568407054760188*^9}, {3.568407105228881*^9, 
   3.568407110701915*^9}, {3.568407149723048*^9, 3.568407153705937*^9}, {
   3.568413834370887*^9, 3.568413849151699*^9}, {3.568413892056828*^9, 
   3.568414187013473*^9}, {3.568414222992474*^9, 3.568414320323882*^9}, {
   3.568414436620501*^9, 3.568414470100946*^9}, {3.568422053427067*^9, 
   3.5684220605591917`*^9}, {3.568423007586426*^9, 3.568423029136219*^9}, {
   3.568423779433147*^9, 3.568423784068293*^9}, {3.568423861658251*^9, 
   3.568423867328888*^9}, {3.568424367544805*^9, 3.568424369736207*^9}, {
   3.568424641773058*^9, 3.56842464279823*^9}, {3.568424803573354*^9, 
   3.56842480993932*^9}, {3.5684263321982193`*^9, 3.568426333549808*^9}, {
   3.568426373901404*^9, 3.568426386543342*^9}, {3.568426547387495*^9, 
   3.5684265922842693`*^9}, {3.568426919737748*^9, 3.5684269239698687`*^9}, {
   3.568427037863954*^9, 3.56842704368211*^9}, {3.5684271925537043`*^9, 
   3.5684271931032743`*^9}, {3.568427272363194*^9, 3.568427281885763*^9}, {
   3.568474527964669*^9, 3.568474556811685*^9}, {3.56847471869173*^9, 
   3.568474752028996*^9}, {3.568475889763123*^9, 3.568475892206497*^9}, {
   3.56848009680085*^9, 3.56848012747853*^9}, {3.568480218400028*^9, 
   3.5684802274342623`*^9}, {3.56857182985888*^9, 3.568572235308062*^9}, {
   3.5686442137711143`*^9, 3.56864421520441*^9}, {3.5686465849060802`*^9, 
   3.5686465945952168`*^9}, {3.5686649316966953`*^9, 3.568664942738905*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Differentiated Scaled Functions", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.5408983275643997`*^9, 3.5408983375383997`*^9}, {
  3.568128918926874*^9, 3.568128920316318*^9}, {3.568571824553351*^9, 
  3.568571825480629*^9}, {3.5685722534837217`*^9, 3.5685722561136007`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Lsx", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Lx", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Linsx", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Linx", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Usx", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Ux", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Uinsx", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Uinx", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Gsx", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Gx", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Msx", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Mx", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Minvsx", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Minvx", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Psx", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Px", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Pinsx", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Pinx", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "k"}], "]"}]}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.538103142394384*^9, 3.538103151534226*^9}, {
   3.5381035800010853`*^9, 3.5381035806070843`*^9}, {3.540653533861*^9, 
   3.540653534101*^9}, {3.5406547841210003`*^9, 3.540654804311*^9}, {
   3.5406548650150003`*^9, 3.540654883401*^9}, {3.5406583132382*^9, 
   3.5406583318182*^9}, {3.5406593163606*^9, 3.5406593424706*^9}, {
   3.5406606785154*^9, 3.5406606829554*^9}, {3.540670035786598*^9, 
   3.540670045469647*^9}, {3.5406701523856287`*^9, 3.540670159109951*^9}, {
   3.540671043884227*^9, 3.5406710445151052`*^9}, {3.54067107622948*^9, 
   3.540671117438383*^9}, 3.540671223981215*^9, 3.540673675932254*^9, {
   3.5406742109124002`*^9, 3.5406742112953997`*^9}, {3.5406777028814*^9, 
   3.5406777049323997`*^9}, {3.5406826023554*^9, 3.5406826068993998`*^9}, {
   3.5406828924653997`*^9, 3.5406828948314*^9}, {3.5406831061644*^9, 
   3.5406831144644003`*^9}, {3.5406844599924*^9, 3.5406844614054003`*^9}, {
   3.5406847746944*^9, 3.5406847750914*^9}, {3.5406849669924*^9, 
   3.5406849737824*^9}, {3.5406851745424*^9, 3.5406852291823997`*^9}, {
   3.5406853996924*^9, 3.5406853998624*^9}, 3.5406876104944*^9, {
   3.5406888923224*^9, 3.5406889113124*^9}, {3.5407250148524*^9, 
   3.5407250177324*^9}, {3.5407255652974*^9, 3.5407255687424*^9}, {
   3.5407264058993998`*^9, 3.5407264153334*^9}, {3.5407270816303997`*^9, 
   3.5407270841644*^9}, {3.5407272503754*^9, 3.5407272625214*^9}, {
   3.5407277902693996`*^9, 3.5407277906574*^9}, {3.5407280626984*^9, 
   3.5407280632244*^9}, {3.540732091156747*^9, 3.540732094732853*^9}, {
   3.5407337077895193`*^9, 3.540733708610265*^9}, {3.540734314953412*^9, 
   3.540734315469853*^9}, {3.540734447769846*^9, 3.540734448600122*^9}, {
   3.5407364063724413`*^9, 3.5407364098191147`*^9}, {3.5407369996439257`*^9, 
   3.54073700378605*^9}, {3.540737269754858*^9, 3.540737270296258*^9}, {
   3.5407374577807903`*^9, 3.540737460562327*^9}, 3.5407388991890717`*^9, {
   3.540739981122607*^9, 3.540739982269462*^9}, {3.540740493804627*^9, 
   3.540740495873988*^9}, {3.540740792568321*^9, 3.540740792845584*^9}, {
   3.540741429871428*^9, 3.540741430139724*^9}, {3.540743857872079*^9, 
   3.5407438622771*^9}, {3.540749237001852*^9, 3.5407492383893623`*^9}, {
   3.5407608990193996`*^9, 3.5407609003594*^9}, {3.5407636848574*^9, 
   3.5407637029354*^9}, {3.5407637497794*^9, 3.5407637906194*^9}, {
   3.5407639569974003`*^9, 3.5407639572774*^9}, 3.5407685188574*^9, {
   3.5407690527953997`*^9, 3.5407690690114*^9}, {3.540839541133526*^9, 
   3.540839547433312*^9}, {3.5409029240854*^9, 3.5409029383283997`*^9}, 
   3.5409035117384*^9, 3.5409416746544*^9, 3.5656171862981997`*^9, {
   3.5656174193571997`*^9, 3.5656174476022*^9}, {3.5656175091342*^9, 
   3.5656175348802*^9}, {3.5656175759962*^9, 3.5656176103392*^9}, {
   3.5656176410592003`*^9, 3.5656176662782*^9}, {3.5656177086092*^9, 
   3.5656177351282*^9}, {3.5656204709592*^9, 3.5656205127052*^9}, {
   3.5656212498292*^9, 3.5656214052762003`*^9}, {3.5656411683152*^9, 
   3.5656411980062*^9}, {3.5656586528082*^9, 3.5656587202482*^9}, {
   3.565754330008767*^9, 3.565754333067934*^9}, {3.5681288606726007`*^9, 
   3.568128864037929*^9}, {3.568129172366681*^9, 3.568129210978828*^9}, {
   3.568129281093484*^9, 3.568129290380556*^9}, {3.568129652490979*^9, 
   3.568129663654998*^9}, {3.56813263902489*^9, 3.568132648010124*^9}, {
   3.568132691574448*^9, 3.56813270059894*^9}, {3.5681455250180607`*^9, 
   3.5681455292247887`*^9}, {3.5681471446438637`*^9, 3.568147147953083*^9}, {
   3.568147241915513*^9, 3.568147324679017*^9}, {3.568147364300024*^9, 
   3.5681473662966757`*^9}, {3.568147541341443*^9, 3.568147544281726*^9}, {
   3.568147752436235*^9, 3.568147766837874*^9}, {3.5681487126240683`*^9, 
   3.568148725181775*^9}, {3.568332505130286*^9, 3.568332564706992*^9}, {
   3.568333948645219*^9, 3.568334002967908*^9}, {3.5683788423046618`*^9, 
   3.568378869836611*^9}, {3.5683789776998377`*^9, 3.5683789828934517`*^9}, {
   3.56840309240283*^9, 3.5684031190900106`*^9}, {3.56840490216264*^9, 
   3.568404947105895*^9}, {3.5684049938814707`*^9, 3.568405018480193*^9}, {
   3.568405104267029*^9, 3.5684051044619226`*^9}, {3.568405141189176*^9, 
   3.568405277129157*^9}, {3.5684053731333942`*^9, 3.568405404593604*^9}, {
   3.568405503753558*^9, 3.5684055592137117`*^9}, {3.568405689374742*^9, 
   3.568405689658036*^9}, {3.5684058600139437`*^9, 3.568405860620275*^9}, {
   3.568406187976935*^9, 3.568406188194696*^9}, {3.568406395432805*^9, 
   3.568406404044888*^9}, {3.568406452860858*^9, 3.568406457080192*^9}, {
   3.5684065000722713`*^9, 3.5684065061550493`*^9}, {3.568406827941753*^9, 
   3.56840684192846*^9}, {3.568406976423539*^9, 3.568407007201652*^9}, {
   3.568407053162166*^9, 3.568407054760188*^9}, {3.568407105228881*^9, 
   3.568407110701915*^9}, {3.568407149723048*^9, 3.568407153705937*^9}, {
   3.568413834370887*^9, 3.568413849151699*^9}, {3.568413892056828*^9, 
   3.568414187013473*^9}, {3.568414222992474*^9, 3.568414320323882*^9}, {
   3.568414436620501*^9, 3.568414470100946*^9}, {3.568422053427067*^9, 
   3.5684220605591917`*^9}, {3.568423007586426*^9, 3.568423029136219*^9}, {
   3.568423779433147*^9, 3.568423784068293*^9}, {3.568423861658251*^9, 
   3.568423867328888*^9}, {3.568424367544805*^9, 3.568424369736207*^9}, {
   3.568424641773058*^9, 3.56842464279823*^9}, {3.568424803573354*^9, 
   3.56842480993932*^9}, {3.5684263321982193`*^9, 3.568426333549808*^9}, {
   3.568426373901404*^9, 3.568426386543342*^9}, {3.568426547387495*^9, 
   3.5684265922842693`*^9}, {3.568426919737748*^9, 3.5684269239698687`*^9}, {
   3.568427037863954*^9, 3.56842704368211*^9}, {3.5684271925537043`*^9, 
   3.5684271931032743`*^9}, {3.568427272363194*^9, 3.568427281885763*^9}, {
   3.568474527964669*^9, 3.568474556811685*^9}, {3.56847471869173*^9, 
   3.568474752028996*^9}, {3.568475889763123*^9, 3.568475892206497*^9}, {
   3.56848009680085*^9, 3.56848012747853*^9}, {3.568480218400028*^9, 
   3.5684802274342623`*^9}, {3.56857182985888*^9, 3.568572303838901*^9}, {
   3.568572400852068*^9, 3.5685724598235826`*^9}, {3.5686480657942123`*^9, 
   3.568648070723565*^9}}]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Region 0: Undeformed", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.538443201592167*^9, 3.538443212095222*^9}, {
   3.5656156095022*^9, 3.5656156113522*^9}, 3.5656208942292*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"J", "[", "0", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"G", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"G", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}]}], "}"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "el"}], ",", "0"}], "}"}], "]"}], ",", 
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{"0", ",", "el"}], "}"}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"2", "*", "bigN"}], ",", 
       RowBox[{"2", "*", "bigN"}]}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Jx", "[", "0", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"Gx", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Gx", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}]}], "}"}], 
     ",", 
     RowBox[{
      RowBox[{
       RowBox[{"J", "[", "0", "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], "[", 
      RowBox[{"[", "2", "]"}], "]"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"J", "[", "0", "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], "[", 
      RowBox[{"[", "3", "]"}], "]"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Jsamp", "[", "0", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"Gsamp", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Gsamp", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}]}], "}"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "el"}], ",", "0"}], "}"}], "]"}], ",", 
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{"0", ",", "el"}], "}"}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"2", "*", "bigN"}], ",", 
       RowBox[{"2", "*", "bigN"}]}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Jadapt", "[", "0", "]"}], "[", 
   RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"Adapt", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Jsamp", "[", "0", "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", "globalTol"}], "]"}], "[", 
   RowBox[{
    RowBox[{"J", "[", "0", "]"}], "[", 
    RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Jadaptx", "[", "0", "]"}], "[", 
   RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"Adapt", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Jsamp", "[", "0", "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", "globalTol"}], "]"}], "[", 
   RowBox[{
    RowBox[{"Jx", "[", "0", "]"}], "[", 
    RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5406576785462*^9, 3.5406577161661997`*^9}, {
   3.5406582201982*^9, 3.5406582273082*^9}, 3.5406586749382*^9, 
   3.5407609243294*^9, {3.5408422052614202`*^9, 3.540842205632593*^9}, {
   3.540842429716239*^9, 3.540842429846467*^9}, {3.5409332732904*^9, 
   3.5409333148454*^9}, {3.541861619785755*^9, 3.541861623452838*^9}, {
   3.541861656327001*^9, 3.541861658902041*^9}, {3.5423735388424*^9, 
   3.5423735413424*^9}, {3.5424025499314003`*^9, 3.5424025521113997`*^9}, {
   3.5656156244642*^9, 3.5656156262852*^9}, {3.5656183001152*^9, 
   3.5656183043241997`*^9}, {3.5656205355512*^9, 3.5656205799202003`*^9}, {
   3.5656206584592*^9, 3.5656206776342*^9}, {3.5656208891191998`*^9, 
   3.5656208914672003`*^9}, {3.5656305467581997`*^9, 3.5656305490722*^9}, {
   3.5657106489052*^9, 3.5657106655822*^9}, {3.568387083875744*^9, 
   3.568387112304347*^9}, {3.568471300492754*^9, 3.568471329438607*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Region 1: Soliton", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.5681287930307693`*^9, 3.5681287971735277`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"c1", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"\[Nu]", "-", 
        RowBox[{"Abs", "[", 
         RowBox[{"K0", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ">", "0"}], " ", "&&", " ", 
      RowBox[{"x", " ", "\[GreaterEqual]", " ", "0"}]}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "el"}], "+", 
           RowBox[{"I", " ", "\[Nu]"}]}], ",", 
          RowBox[{
           RowBox[{
            RowBox[{"Sqrt", "[", "2", "]"}], 
            RowBox[{"Exp", "[", 
             RowBox[{"3", "I", " ", 
              RowBox[{"Pi", "/", "4"}]}], "]"}], 
            RowBox[{"(", 
             RowBox[{"\[Nu]", "-", 
              RowBox[{"Abs", "[", 
               RowBox[{"K0", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ")"}]}], " ", "+", 
           " ", 
           RowBox[{"I", " ", 
            RowBox[{"Abs", "[", 
             RowBox[{"K0", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}]}], "}"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Sqrt", "[", "2", "]"}], 
            RowBox[{"Exp", "[", 
             RowBox[{"3", "I", " ", 
              RowBox[{"Pi", "/", "4"}]}], "]"}], 
            RowBox[{"Abs", "[", 
             RowBox[{"\[Nu]", "-", 
              RowBox[{"Abs", "[", 
               RowBox[{"K0", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}]}], " ", "+", 
           " ", 
           RowBox[{"I", " ", 
            RowBox[{"Abs", "[", 
             RowBox[{"K0", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], ",", 
          RowBox[{"I", " ", 
           RowBox[{"Abs", "[", 
            RowBox[{"K0", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], "}"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"I", " ", 
           RowBox[{"Abs", "[", 
            RowBox[{"K0", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ",", 
          RowBox[{
           RowBox[{
            RowBox[{"Sqrt", "[", "2", "]"}], 
            RowBox[{"Exp", "[", 
             RowBox[{"I", " ", 
              RowBox[{"Pi", "/", "4"}]}], "]"}], 
            RowBox[{"Abs", "[", 
             RowBox[{"\[Nu]", "-", 
              RowBox[{"Abs", "[", 
               RowBox[{"K0", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}]}], " ", "+", 
           " ", 
           RowBox[{"I", " ", 
            RowBox[{"Abs", "[", 
             RowBox[{"K0", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}]}], "}"}], "]"}], ",", 
       
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Sqrt", "[", "2", "]"}], 
            RowBox[{"Exp", "[", 
             RowBox[{"I", " ", 
              RowBox[{"Pi", "/", "4"}]}], "]"}], 
            RowBox[{"Abs", "[", 
             RowBox[{"\[Nu]", "-", 
              RowBox[{"Abs", "[", 
               RowBox[{"K0", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}]}], " ", "+", 
           " ", 
           RowBox[{"I", " ", 
            RowBox[{"Abs", "[", 
             RowBox[{"K0", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], ",", 
          RowBox[{"el", "+", 
           RowBox[{"I", " ", "\[Nu]"}]}]}], "}"}], "]"}]}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "el"}], "+", 
           RowBox[{"I", " ", "\[Nu]"}]}], ",", 
          RowBox[{"I", " ", "\[Nu]"}]}], "}"}], "]"}], ",", 
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"I", " ", "\[Nu]"}], ",", 
          RowBox[{
           RowBox[{"I", " ", "\[Nu]"}], " ", "+", " ", "el"}]}], "}"}], 
        "]"}]}], "}"}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"J", "[", "1", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Abs", "[", 
       RowBox[{"K0", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], "<", "\[Infinity]"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Join", "[", 
        RowBox[{
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"P", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{"Length", "[", 
              RowBox[{"c1", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "}"}]}], "]"}], ",", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"M", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{"Length", "[", 
              RowBox[{"c1", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
        "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Join", "[", 
        RowBox[{
         RowBox[{"c1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ",", 
         RowBox[{
          RowBox[{"c1", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "//", "LineConjugate"}]}], "]"}], 
       ",", "\[IndentingNewLine]", 
       RowBox[{"Table", "[", 
        RowBox[{"bigN", ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "1", ",", 
           RowBox[{"2", "*", 
            RowBox[{"Length", "[", 
             RowBox[{"c1", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], "}"}]}], "]"}]}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"P", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"P", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"M", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"M", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}]}], "}"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "el"}], " ", ",", "0"}], "}"}], "+", 
           RowBox[{"I", " ", "\[Nu]"}]}], "]"}], ",", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"0", ",", "el"}], "}"}], "+", 
           RowBox[{"I", " ", "\[Nu]"}]}], "]"}], ",", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "el"}], " ", ",", "0"}], "}"}], "-", 
           RowBox[{"I", " ", "\[Nu]"}]}], "]"}], ",", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"0", " ", ",", "el"}], "}"}], "-", 
           RowBox[{"I", " ", "\[Nu]"}]}], "]"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"bigN", ",", "bigN", ",", "bigN", ",", "bigN"}], "}"}]}], 
      "}"}]}], "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Jx", "[", "1", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}], ",", 
     RowBox[{
      RowBox[{"Jumps", "=", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"K0", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}], "<", "\[Infinity]"}], ",", 
         
         RowBox[{"Join", "[", 
          RowBox[{
           RowBox[{"Table", "[", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Px", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
             RowBox[{"{", 
              RowBox[{"i", ",", "1", ",", 
               RowBox[{"Length", "[", 
                RowBox[{"c1", "[", 
                 RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "}"}]}], "]"}], ",", 
           RowBox[{"Table", "[", 
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"Mx", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
             RowBox[{"{", 
              RowBox[{"i", ",", "1", ",", 
               RowBox[{"Length", "[", 
                RowBox[{"c1", "[", 
                 RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
          "]"}], ",", "\[IndentingNewLine]", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Px", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{
            RowBox[{
             RowBox[{"Px", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{
            RowBox[{
             RowBox[{"Mx", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{
            RowBox[{
             RowBox[{"Mx", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}]}], 
          "}"}]}], "\[IndentingNewLine]", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Domains", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"J", "[", "1", "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], "[", 
        RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"NumPts", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"J", "[", "1", "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], "[", 
        RowBox[{"[", "3", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Jsamp", "[", "1", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", "\[IndentingNewLine]", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Abs", "[", 
       RowBox[{"K0", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], "<", "\[Infinity]"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Join", "[", 
        RowBox[{
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Psamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{"Length", "[", 
              RowBox[{"c1", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "}"}]}], "]"}], ",", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Msamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{"Length", "[", 
              RowBox[{"c1", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
        "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Join", "[", 
        RowBox[{
         RowBox[{"c1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ",", 
         RowBox[{
          RowBox[{"c1", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "//", "LineConjugate"}]}], "]"}], 
       ",", "\[IndentingNewLine]", 
       RowBox[{"Table", "[", 
        RowBox[{"bigN", ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "1", ",", 
           RowBox[{"2", "*", 
            RowBox[{"Length", "[", 
             RowBox[{"c1", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], "}"}]}], "]"}]}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"Psamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"Psamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"Msamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"Msamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}]}], "}"}], 
       ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "el"}], " ", ",", "0"}], "}"}], "+", 
           RowBox[{"I", " ", "\[Nu]"}]}], "]"}], ",", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"0", ",", "el"}], "}"}], "+", 
           RowBox[{"I", " ", "\[Nu]"}]}], "]"}], ",", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "el"}], " ", ",", "0"}], "}"}], "-", 
           RowBox[{"I", " ", "\[Nu]"}]}], "]"}], ",", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"0", " ", ",", "el"}], "}"}], "-", 
           RowBox[{"I", " ", "\[Nu]"}]}], "]"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"bigN", ",", "bigN", ",", "bigN", ",", "bigN"}], "}"}]}], 
      "}"}]}], "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Jadapt", "[", "1", "]"}], "[", 
   RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"Adapt", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Jsamp", "[", "1", "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", "globalTol"}], "]"}], "[", 
   RowBox[{
    RowBox[{"J", "[", "1", "]"}], "[", 
    RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Jadaptx", "[", "1", "]"}], "[", 
   RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"Adapt", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Jsamp", "[", "1", "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", "globalTol"}], "]"}], "[", 
   RowBox[{
    RowBox[{"Jx", "[", "1", "]"}], "[", 
    RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGAQAWIQvUOvfO2qqa8cFSPjNoNoCTuhHSDacJr5LhBdc2zr
YTA979gREM3V9+sCiF52r+8/iE77kcKyGiR/6bQQiM74/FwcRLtwb1EF0ZN3
TtYC0T/iZa1A9IZ6oRgQnXP5XAqIviL+KBtE3/rjWAiir/XpfgLRbJ9fg+ma
ok1sa4B0edRxMD31MbciiGb6Jwqmv73SkOCa9srRtKceTD+OP/2ZF0hv2dcs
xAek9ZfIyIBo3pAfYHqOwaYKfiC9MValGkSLzpYRMQDSz7R/KoPoKTfVtUC0
XByjNojWsatbFggyT85jA4he/irTKRRIR8mUgmkAlPWlzQ==
  "]]
}, Closed]],

Cell[CellGroupData[{

Cell["Region 2: Painleve (scaled)(unfinished)", "Section",
 CellChangeTimes->{{3.5681287930307693`*^9, 3.5681287971735277`*^9}, {
  3.568140784197513*^9, 3.568140792715275*^9}, {3.5681424559224987`*^9, 
  3.568142457636324*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"c2", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "el"}], "+", 
         RowBox[{"I", " ", "\[Nu]"}]}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"3", 
            RowBox[{"Pi", "/", "4"}], "I"}], "]"}], "\[Nu]"}], "-", 
         RowBox[{"Abs", "[", 
          RowBox[{"K0", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], "}"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"3", 
            RowBox[{"Pi", "/", "4"}], "I"}], "]"}], "\[Nu]"}], "-", 
         RowBox[{"Abs", "[", 
          RowBox[{"K0", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ",", 
        RowBox[{"-", 
         RowBox[{"Abs", "[", 
          RowBox[{"K0", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], "}"}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{"Abs", "[", 
          RowBox[{"K0", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ",", 
        RowBox[{"Abs", "[", 
         RowBox[{"K0", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "}"}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"K0", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"1", 
            RowBox[{"Pi", "/", "4"}], "I"}], "]"}], "\[Nu]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"K0", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], "}"}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"1", 
            RowBox[{"Pi", "/", "4"}], "I"}], "]"}], "\[Nu]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"K0", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"I", " ", "\[Nu]"}], " ", "+", " ", "el"}]}], "}"}], "]"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "el"}], "-", 
         RowBox[{"I", " ", "\[Nu]"}]}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"5", 
            RowBox[{"Pi", "/", "4"}], "I"}], "]"}], "\[Nu]"}], "-", 
         RowBox[{"Abs", "[", 
          RowBox[{"K0", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], "}"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"5", 
            RowBox[{"Pi", "/", "4"}], "I"}], "]"}], "\[Nu]"}], "-", 
         RowBox[{"Abs", "[", 
          RowBox[{"K0", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ",", 
        RowBox[{"-", 
         RowBox[{"Abs", "[", 
          RowBox[{"K0", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], "}"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"K0", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "1"}], 
            RowBox[{"Pi", "/", "4"}], "I"}], "]"}], "\[Nu]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"K0", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], "}"}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "1"}], 
            RowBox[{"Pi", "/", "4"}], "I"}], "]"}], "\[Nu]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"K0", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "I"}], " ", "\[Nu]"}], " ", "+", " ", "el"}]}], "}"}], 
      "]"}]}], "\[IndentingNewLine]", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"J", "[", "2", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"P", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"P", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"G", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"P", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"P", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"M", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"M", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"M", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"M", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}]}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"c2", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Table", "[", 
      RowBox[{"smallN", ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", 
         RowBox[{"Length", "[", 
          RowBox[{"c2", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "}"}]}], "]"}]}], "}"}]}],
   ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Jsamp", "[", "2", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"Psamp", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Psamp", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Gsamp", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Psamp", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Psamp", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Msamp", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Msamp", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Msamp", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Msamp", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}]}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"c2", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Table", "[", 
      RowBox[{"smallN", ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", 
         RowBox[{"Length", "[", 
          RowBox[{"c2", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "}"}]}], "]"}]}], "}"}]}],
   ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Jadapt", "[", "2", "]"}], "[", 
   RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"Adapt", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Jsamp", "[", "2", "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", "globalTol"}], "]"}], "[", 
   RowBox[{
    RowBox[{"J", "[", "2", "]"}], "[", 
    RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], "Input"]
}, Closed]],

Cell[CellGroupData[{

Cell["Region 2.5: Painleve (unscaled)", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.5681287930307693`*^9, 3.5681287971735277`*^9}, {
  3.568140784197513*^9, 3.568140801577532*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"c2", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "el"}], "+", 
         RowBox[{"I", " ", "\[Nu]"}]}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"3", 
            RowBox[{"Pi", "/", "4"}], "I"}], "]"}], "\[Nu]"}], "-", 
         RowBox[{"Abs", "[", 
          RowBox[{"K0", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], "}"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"3", 
            RowBox[{"Pi", "/", "4"}], "I"}], "]"}], "\[Nu]"}], "-", 
         RowBox[{"Abs", "[", 
          RowBox[{"K0", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ",", 
        RowBox[{"-", 
         RowBox[{"Abs", "[", 
          RowBox[{"K0", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], "}"}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{"Abs", "[", 
          RowBox[{"K0", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ",", 
        RowBox[{"Abs", "[", 
         RowBox[{"K0", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "}"}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"K0", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"1", 
            RowBox[{"Pi", "/", "4"}], "I"}], "]"}], "\[Nu]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"K0", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], "}"}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"1", 
            RowBox[{"Pi", "/", "4"}], "I"}], "]"}], "\[Nu]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"K0", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"I", " ", "\[Nu]"}], " ", "+", " ", "el"}]}], "}"}], "]"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "el"}], "-", 
         RowBox[{"I", " ", "\[Nu]"}]}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"5", 
            RowBox[{"Pi", "/", "4"}], "I"}], "]"}], "\[Nu]"}], "-", 
         RowBox[{"Abs", "[", 
          RowBox[{"K0", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], "}"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"5", 
            RowBox[{"Pi", "/", "4"}], "I"}], "]"}], "\[Nu]"}], "-", 
         RowBox[{"Abs", "[", 
          RowBox[{"K0", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ",", 
        RowBox[{"-", 
         RowBox[{"Abs", "[", 
          RowBox[{"K0", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], "}"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"K0", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "1"}], 
            RowBox[{"Pi", "/", "4"}], "I"}], "]"}], "\[Nu]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"K0", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], "}"}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "1"}], 
            RowBox[{"Pi", "/", "4"}], "I"}], "]"}], "\[Nu]"}], "+", 
         RowBox[{"Abs", "[", 
          RowBox[{"K0", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "I"}], " ", "\[Nu]"}], " ", "+", " ", "el"}]}], "}"}], 
      "]"}]}], "\[IndentingNewLine]", "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"J", "[", "2.5", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"P", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"P", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"G", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"P", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"P", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"M", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"M", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"M", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"M", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}]}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"c2", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Table", "[", 
      RowBox[{"smallN", ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", 
         RowBox[{"Length", "[", 
          RowBox[{"c2", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "}"}]}], "]"}]}], "}"}]}],
   ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Jx", "[", "2.5", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"Px", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Px", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Gx", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Px", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Px", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Mx", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Mx", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Mx", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Mx", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}]}], "}"}], 
     ",", 
     RowBox[{
      RowBox[{
       RowBox[{"J", "[", "2.5", "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], "[", 
      RowBox[{"[", "2", "]"}], "]"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"J", "[", "2.5", "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], "[", 
      RowBox[{"[", "3", "]"}], "]"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Jsamp", "[", "2.5", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"Psamp", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Psamp", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Gsamp", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Psamp", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Psamp", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Msamp", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Msamp", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Msamp", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Msamp", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}]}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"c2", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Table", "[", 
      RowBox[{"smallN", ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "1", ",", 
         RowBox[{"Length", "[", 
          RowBox[{"c2", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "}"}]}], "]"}]}], "}"}]}],
   ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Jadapt", "[", "2.5", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"Adapt", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Jsamp", "[", "2.5", "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", "globalTol"}], "]"}], "[", 
    RowBox[{
     RowBox[{"J", "[", "2.5", "]"}], "[", 
     RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Jadaptx", "[", "2.5", "]"}], "[", 
   RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"Adapt", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Jsamp", "[", "2.5", "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", "globalTol"}], "]"}], "[", 
   RowBox[{
    RowBox[{"Jx", "[", "2.5", "]"}], "[", 
    RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->CompressedData["
1:eJxTTMoPSmViYGAQBWIQvUOvfO2qqa8cFSPjNoNoCTuhHSDacJr5LhBdc2zr
YTA979gREM3V9+sCiF52r+8/iE77kcKyGiR/6bQQiM74/FwcRLtwb1EF0ZN3
TtYC0T/iZa1A9IZ6oRgQnXP5XAqIviL+KBtE3/rjWAiir/XpfgLRbJ9fg+ma
ok1sa4B0edRxMD31MbciiGb6Jwqm1zW8d9gA4v//lgKi1aO+ZIBoqbu1FSD6
CWtKO4hmfP22F0Rvl6w9B6IbhCZcA9G/3n67BaLtvgW9A9FmRsICG4G0wRoF
QRB95J2aEc+0V47HlhiC6V0vI4wMgDT7nwOmINqzWWFTIJCe//nsThANAKAv
upg=
  "]]
}, Closed]],

Cell[CellGroupData[{

Cell["Region 3: Dispersive", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.56814327368044*^9, 3.568143277482564*^9}, {
  3.568335025816708*^9, 3.568335033072962*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"Valid", " ", "for"}], " ", "|", "K0", "|", " ", 
    RowBox[{">", " ", "\[Nu]"}]}], "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"if", "[", "t_", "]"}], ":=", 
     RowBox[{".25", "/", 
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{"Sqrt", "[", "t", "]"}]}], ")"}]}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"rays", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "3", " ", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}], ",", 
           RowBox[{
            RowBox[{"if", "[", "t", "]"}], " ", 
            RowBox[{"Exp", "[", 
             RowBox[{"I", " ", "3", " ", 
              RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}]}], "}"}], "]"}], ",", 
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "5", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}], ",", 
           RowBox[{
            RowBox[{"if", "[", "t", "]"}], " ", 
            RowBox[{"Exp", "[", 
             RowBox[{"I", " ", "5", " ", 
              RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}]}], "}"}], "]"}], ",", 
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{"if", "[", "t", "]"}], " ", 
            RowBox[{"Exp", "[", 
             RowBox[{
              RowBox[{"-", "I"}], " ", 
              RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}], ",", " ", 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}], "}"}], "]"}], ",", 
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{"if", "[", "t", "]"}], " ", 
            RowBox[{"Exp", "[", 
             RowBox[{"I", " ", 
              RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}], ",", 
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}], "}"}], "]"}]}], "}"}], 
      "*", "\[Nu]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"square", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "3", " ", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}], ",", 
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "5", " ", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}], "}"}], "]"}], ",", 
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "5", " ", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}], ",", 
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "7", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}], "}"}], "]"}], ",", 
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "7", " ", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}], ",", 
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}], "}"}], "]"}], ",", 
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}], ",", 
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "3", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}], "}"}], "]"}]}], "}"}], 
      "*", 
      RowBox[{"if", "[", "t", "]"}], " ", "\[Nu]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"stencil", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", " ", 
     RowBox[{
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"rays", "[", 
         RowBox[{"x", ",", "t"}], "]"}], ",", 
        RowBox[{"square", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "]"}], "*", 
      RowBox[{"Sqrt", "[", "2", "]"}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"extendL", " ", "=", " ", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "el"}], " ", "+", " ", 
           RowBox[{"I", " ", "\[Nu]"}]}], ",", 
          RowBox[{
           RowBox[{"Sqrt", "[", "2", "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{"3", " ", 
             RowBox[{"Pi", "/", "4"}], "I"}], "]"}], " ", "\[Nu]"}]}], "}"}], 
        "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "el"}], " ", "-", " ", 
           RowBox[{"I", " ", "\[Nu]"}]}], ",", 
          RowBox[{
           RowBox[{"Sqrt", "[", "2", "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{"5", 
             RowBox[{"Pi", "/", "4"}], "I"}], "]"}], " ", "\[Nu]"}]}], "}"}], 
        "]"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"extendR", "=", " ", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"Sqrt", "[", "2", "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"Pi", "/", "4"}], "I"}], "]"}], " ", "\[Nu]"}], ",", " ", 
          RowBox[{"el", " ", "+", " ", 
           RowBox[{"I", " ", "\[Nu]"}]}]}], "}"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"Sqrt", "[", "2", "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"-", "Pi"}], "/", "4"}], "I"}], "]"}], " ", "\[Nu]"}], 
          ",", 
          RowBox[{"el", "-", " ", 
           RowBox[{"I", " ", "\[Nu]"}]}]}], "}"}], "]"}]}], "}"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"connect", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", " ", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"-", 
            RowBox[{"Abs", "[", 
             RowBox[{"K0", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "]"}]}], " ", "+", " ", 
           RowBox[{
            RowBox[{"Sqrt", "[", "2", "]"}], "\[Nu]", " ", 
            RowBox[{"Exp", "[", 
             RowBox[{"I", " ", 
              RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
          RowBox[{
           RowBox[{"Abs", "[", 
            RowBox[{"K0", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}], " ", "+", " ", 
           RowBox[{
            RowBox[{"Sqrt", "[", "2", "]"}], "\[Nu]", " ", 
            RowBox[{"Exp", "[", 
             RowBox[{"3", "I", " ", 
              RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"-", 
            RowBox[{"Abs", "[", 
             RowBox[{"K0", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "]"}]}], " ", "+", " ", 
           RowBox[{
            RowBox[{"Sqrt", "[", "2", "]"}], "\[Nu]", " ", 
            RowBox[{"Exp", "[", 
             RowBox[{
              RowBox[{"-", "I"}], " ", 
              RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
          RowBox[{
           RowBox[{"Abs", "[", 
            RowBox[{"K0", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}], " ", "+", " ", 
           RowBox[{
            RowBox[{"Sqrt", "[", "2", "]"}], "\[Nu]", " ", 
            RowBox[{"Exp", "[", 
             RowBox[{
              RowBox[{"-", "3"}], "I", " ", 
              RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}]}], 
      "}"}]}], ";"}], "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"ctr", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"Join", "[", 
      RowBox[{
       RowBox[{"extendL", "-", 
        RowBox[{"K0", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], ",", 
       RowBox[{
        RowBox[{"stencil", "[", 
         RowBox[{"x", ",", "t"}], "]"}], " ", "-", " ", 
        RowBox[{"K0", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], ",", 
       RowBox[{"connect", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"stencil", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "+", 
        RowBox[{"K0", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], ",", 
       RowBox[{"extendR", " ", "+", " ", 
        RowBox[{"K0", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"DDI", "[", 
       RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Im", "[", "k", "]"}], " ", "\[GreaterEqual]", "0"}], ",", 
       RowBox[{
        RowBox[{"DD", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], ",", 
       RowBox[{"IdentityMatrix", "[", "2", "]"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"DDIin", "[", 
       RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Im", "[", "k", "]"}], " ", "\[GreaterEqual]", "0"}], ",", 
       RowBox[{
        RowBox[{"DDin", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], ",", 
       RowBox[{"IdentityMatrix", "[", "2", "]"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"DDIx", "[", 
       RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"DDIinx", "[", 
       RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "0"}], "}"}]}], "}"}]}], ";"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"DDIsamp", "[", 
       RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Im", "[", "k", "]"}], " ", "\[GreaterEqual]", "0"}], ",", 
       RowBox[{
        RowBox[{"DDsamp", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], ",", 
       RowBox[{"IdentityMatrix", "[", "2", "]"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"DDIinsamp", "[", 
       RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Im", "[", "k", "]"}], " ", "\[GreaterEqual]", "0"}], ",", 
       RowBox[{
        RowBox[{"DDinsamp", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], ",", 
       RowBox[{"IdentityMatrix", "[", "2", "]"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"J", "[", "3.3", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Jumps", "=", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"U", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"U", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}]}], 
          "}"}]}], ";", 
        RowBox[{"Domains", "=", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Line", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{"-", "el"}], " ", ",", "0"}], "}"}], "+", 
             RowBox[{"I", " ", "\[Nu]"}]}], "]"}], ",", 
           RowBox[{"Line", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"0", ",", "el"}], "}"}], "+", 
             RowBox[{"I", " ", "\[Nu]"}]}], "]"}], ",", 
           RowBox[{"Line", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{"-", "el"}], " ", ",", "0"}], "}"}], "-", 
             RowBox[{"I", " ", "\[Nu]"}]}], "]"}], ",", 
           RowBox[{"Line", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"0", " ", ",", "el"}], "}"}], "-", 
             RowBox[{"I", " ", "\[Nu]"}]}], "]"}]}], "}"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"NumPts", "=", 
         RowBox[{"{", 
          RowBox[{"bigN", ",", "bigN", ",", "bigN", ",", "bigN"}], "}"}]}], 
        ";", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}]}]}], 
      "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Jsamp", "[", "3.3", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Jumps", "=", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Usamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Usamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}]}], 
          "\[IndentingNewLine]", "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Domains", "=", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Line", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{"-", "el"}], " ", ",", "0"}], "}"}], "+", 
             RowBox[{"I", " ", "\[Nu]"}]}], "]"}], ",", 
           RowBox[{"Line", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"0", ",", "el"}], "}"}], "+", 
             RowBox[{"I", " ", "\[Nu]"}]}], "]"}], ",", 
           RowBox[{"Line", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{"-", "el"}], " ", ",", "0"}], "}"}], "-", 
             RowBox[{"I", " ", "\[Nu]"}]}], "]"}], ",", 
           RowBox[{"Line", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"0", " ", ",", "el"}], "}"}], "-", 
             RowBox[{"I", " ", "\[Nu]"}]}], "]"}]}], "}"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"NumPts", "=", 
         RowBox[{"{", 
          RowBox[{"bigN", ",", "bigN", ",", "bigN", ",", "bigN"}], "}"}]}], 
        ";", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}]}]}], 
      "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Jx", "[", "3.3", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Jumps", "=", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Ux", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Ux", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Lx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Lx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}]}], 
          "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"Jumps", ",", 
          RowBox[{
           RowBox[{
            RowBox[{"J", "[", "3.3", "]"}], "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", 
           RowBox[{"[", "2", "]"}], "]"}], ",", 
          RowBox[{
           RowBox[{
            RowBox[{"J", "[", "3.3", "]"}], "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", 
           RowBox[{"[", "3", "]"}], "]"}]}], "}"}]}]}], "\[IndentingNewLine]",
       "]"}]}], ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"J", "[", "3.2", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Jumps", "=", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"P", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"M", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "extendL"}], "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"P", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"M", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"U", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "rays", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Minv", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"DDIin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Uin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"DDin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "square", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"U", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           " ", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "connect"}], "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"U", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"M", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"P", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "rays", " ", "R"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"DDIin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Minv", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Uin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"DDin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "square", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"P", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"M", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}]}], 
          RowBox[{"(*", 
           RowBox[{"end", " ", "extendR"}], "*)"}], "\[IndentingNewLine]", 
          "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Domains", " ", "=", " ", 
         RowBox[{"ctr", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"NumPts", " ", "=", " ", 
         RowBox[{"Table", "[", 
          RowBox[{"smallN", ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{"Length", "[", "Domains", "]"}]}], "}"}]}], "]"}]}], ";",
         "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}]}]}], 
      "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Jsamp", "[", "3.2", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Jumps", "=", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Psamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Msamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "extendL"}], "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Psamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Msamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Usamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "rays", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "square", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Usamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           " ", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "connect"}], "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Usamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Msamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Psamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "rays", " ", "R"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "square", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Psamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Msamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}]}], 
          RowBox[{"(*", 
           RowBox[{"end", " ", "extendR"}], "*)"}], "\[IndentingNewLine]", 
          "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Domains", " ", "=", " ", 
         RowBox[{"ctr", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"NumPts", " ", "=", " ", 
         RowBox[{"Table", "[", 
          RowBox[{"smallN", ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{"Length", "[", "Domains", "]"}]}], "}"}]}], "]"}]}], ";",
         "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}]}]}], 
      "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Jx", "[", "3.2", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Jumps", "=", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Px", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Mx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "extendL"}], "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Px", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Mx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Lx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Ux", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "rays", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"\[CapitalPhi]t", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
              RowBox[{
               RowBox[{"Minvx", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
              RowBox[{
               RowBox[{"L", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "+", 
             RowBox[{
              RowBox[{
               RowBox[{"\[CapitalPhi]t", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
              RowBox[{
               RowBox[{"Minv", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
              RowBox[{
               RowBox[{"Lx", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}]}], "&"}], 
           ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Lx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"DDIinx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Uinx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"DDin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "square", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Ux", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Lx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           " ", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "connect"}], "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Ux", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Lx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Mx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Px", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "rays", " ", "R"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"DDIinx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Lx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"\[CapitalPhi]t", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
              RowBox[{
               RowBox[{"Minvx", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
              RowBox[{
               RowBox[{"L", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "+", 
             RowBox[{
              RowBox[{
               RowBox[{"\[CapitalPhi]t", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
              RowBox[{
               RowBox[{"Minv", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
              RowBox[{
               RowBox[{"Lx", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}]}], "&"}], 
           ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Uinx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"DDin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "square", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Px", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Mx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}]}], 
          RowBox[{"(*", 
           RowBox[{"end", " ", "extendR"}], "*)"}], "\[IndentingNewLine]", 
          "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"Jumps", ",", 
          RowBox[{
           RowBox[{
            RowBox[{"J", "[", "3.2", "]"}], "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", 
           RowBox[{"[", "2", "]"}], "]"}], ",", 
          RowBox[{
           RowBox[{
            RowBox[{"J", "[", "3.2", "]"}], "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", 
           RowBox[{"[", "3", "]"}], "]"}]}], "}"}]}]}], "\[IndentingNewLine]",
       "]"}]}], ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Jadapt", "[", "3.2", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{
      RowBox[{"Adapt", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Jsamp", "[", "3.2", "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], ",", "globalTol"}], "]"}], "[", 
      RowBox[{
       RowBox[{"J", "[", "3.2", "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ";"}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.568144739351859*^9, 3.568144768075987*^9}, {
   3.568145313447761*^9, 3.5681453327796707`*^9}, {3.568145376573141*^9, 
   3.568145381688393*^9}, {3.5681454371313972`*^9, 3.5681455150174026`*^9}, {
   3.568145569123678*^9, 3.568145640898922*^9}, 3.568145735969737*^9, 
   3.5681457842697573`*^9, {3.568145966381893*^9, 3.5681461110591393`*^9}, {
   3.5681461476277943`*^9, 3.5681462249097977`*^9}, {3.568146329932455*^9, 
   3.568146494179833*^9}, {3.568146527537668*^9, 3.5681467331954527`*^9}, 
   3.568146786806254*^9, {3.568146981968944*^9, 3.568147003303782*^9}, {
   3.5681477923150587`*^9, 3.5681477944311438`*^9}, {3.568147921896377*^9, 
   3.568147927835744*^9}, {3.568149378238656*^9, 3.56814938792607*^9}, {
   3.5681494262633057`*^9, 3.568149477859494*^9}, {3.56814951449235*^9, 
   3.568149589294854*^9}, {3.568149620607332*^9, 3.568149648942997*^9}, {
   3.5681497045425453`*^9, 3.568149728649761*^9}, {3.568149850187934*^9, 
   3.568149851216811*^9}, {3.568150013812023*^9, 3.568150074988999*^9}, {
   3.5681501221337748`*^9, 3.5681501268447247`*^9}, {3.568150344328685*^9, 
   3.5681503447013474`*^9}, {3.568150439229638*^9, 3.56815045013251*^9}, {
   3.5681508262619753`*^9, 3.56815083741147*^9}, {3.568150970457507*^9, 
   3.5681510134580917`*^9}, {3.568151094017838*^9, 3.5681510957808857`*^9}, {
   3.568151146755842*^9, 3.568151149432043*^9}, {3.568151183497633*^9, 
   3.568151206989304*^9}, {3.56815123848202*^9, 3.568151258072465*^9}, {
   3.5681520173106527`*^9, 3.568152080041749*^9}, {3.568152159235017*^9, 
   3.568152162839055*^9}, {3.568152233382656*^9, 3.568152250193955*^9}, {
   3.568152306478895*^9, 3.56815231568089*^9}, {3.5681523480442343`*^9, 
   3.5681523496745243`*^9}, {3.568333271971957*^9, 3.5683334679126863`*^9}, {
   3.568407233456438*^9, 3.568407489365816*^9}, {3.568471521060493*^9, 
   3.5684715647611523`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"Valid", " ", "if"}], " ", "|", "K0", "|", 
    RowBox[{"<", " ", "\[Nu]"}]}], "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"skewrays", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "3", " ", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}], ",", 
           RowBox[{
            RowBox[{"if", "[", "t", "]"}], " ", 
            RowBox[{"Exp", "[", 
             RowBox[{"I", " ", "3", " ", 
              RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}]}], "}"}], "]"}], ",", 
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "5", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}], ",", 
           RowBox[{
            RowBox[{"if", "[", "t", "]"}], " ", 
            RowBox[{"Exp", "[", 
             RowBox[{"I", " ", "5", " ", 
              RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}]}], "}"}], "]"}], ",", 
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{"if", "[", "t", "]"}], " ", 
            RowBox[{"Exp", "[", 
             RowBox[{
              RowBox[{"-", "I"}], " ", 
              RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}], ",", 
           RowBox[{
            RowBox[{
             RowBox[{"Abs", "[", 
              RowBox[{"K0", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "]"}], "/", "\[Nu]"}], " ", 
            RowBox[{"Exp", "[", 
             RowBox[{
              RowBox[{"-", "I"}], " ", 
              RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}]}], "}"}], "]"}], ",", 
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{"if", "[", "t", "]"}], " ", 
            RowBox[{"Exp", "[", 
             RowBox[{"I", " ", 
              RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}], ",", 
           RowBox[{
            RowBox[{
             RowBox[{"Abs", "[", 
              RowBox[{"K0", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "]"}], "/", "\[Nu]"}], " ", 
            RowBox[{"Exp", "[", 
             RowBox[{"I", " ", 
              RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}]}], "}"}], "]"}]}], 
       "}"}], "*", "\[Nu]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"stencilskewL", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", " ", 
     RowBox[{
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"skewrays", "[", 
         RowBox[{"x", ",", "t"}], "]"}], ",", 
        RowBox[{"square", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "]"}], "*", 
      RowBox[{"Sqrt", "[", "2", "]"}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"stencilskewR", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", " ", 
     RowBox[{
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"skewrays", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}], "//", "LineReverse"}], ",", 
        RowBox[{"square", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "]"}], "*", 
      RowBox[{"Sqrt", "[", "2", "]"}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"ctrskew", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"Join", "[", 
      RowBox[{
       RowBox[{"extendL", "-", 
        RowBox[{"K0", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], ",", 
       RowBox[{
        RowBox[{"stencilskewL", "[", 
         RowBox[{"x", ",", "t"}], "]"}], " ", "-", " ", 
        RowBox[{"K0", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], ",", 
       RowBox[{
        RowBox[{"stencilskewR", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "+", 
        RowBox[{"K0", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], ",", 
       RowBox[{"extendR", " ", "+", " ", 
        RowBox[{"K0", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"J", "[", "3.1", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Jumps", "=", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"P", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"M", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "extendL"}], "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"P", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"M", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"U", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "rays", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Minv", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"DDIin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Uin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"DDin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "square", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"M", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"P", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"U", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{
            "end", " ", "rays", " ", "R", " ", "with", " ", "rotation"}], 
            "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"DDIin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Minv", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Uin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"DDin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "square", " ", "R"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"P", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"M", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}]}], 
          RowBox[{"(*", 
           RowBox[{"end", " ", "extendR"}], "*)"}], "\[IndentingNewLine]", 
          "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Domains", " ", "=", " ", 
         RowBox[{"ctrskew", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"NumPts", " ", "=", " ", 
         RowBox[{"Table", "[", 
          RowBox[{"smallN", ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{"Length", "[", "Domains", "]"}]}], "}"}]}], "]"}]}], ";",
         "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}]}]}], 
      "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Jsamp", "[", "3.1", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Jumps", "=", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Psamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Msamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "extendL"}], "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Psamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Msamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Usamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "rays", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "square", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Msamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Psamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Usamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "rays", " ", "R"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "square", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Psamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Msamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}]}], 
          RowBox[{"(*", 
           RowBox[{"end", " ", "extendR"}], "*)"}], "\[IndentingNewLine]", 
          "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Domains", " ", "=", " ", 
         RowBox[{"ctrskew", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"NumPts", " ", "=", " ", 
         RowBox[{"Table", "[", 
          RowBox[{"smallN", ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{"Length", "[", "Domains", "]"}]}], "}"}]}], "]"}]}], ";",
         "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}]}]}], 
      "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Jx", "[", "3.1", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Jumps", "=", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Px", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Mx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "extendL"}], "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Px", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Mx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Lx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Ux", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "rays", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"\[CapitalPhi]t", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
              RowBox[{
               RowBox[{"Minvx", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
              RowBox[{
               RowBox[{"L", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "+", 
             RowBox[{
              RowBox[{
               RowBox[{"\[CapitalPhi]t", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
              RowBox[{
               RowBox[{"Minv", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
              RowBox[{
               RowBox[{"Lx", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}]}], "&"}], 
           ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Lx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"DDIinx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Uinx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"DDin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "square", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Mx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Px", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Ux", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Lx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{
            "end", " ", "rays", " ", "R", " ", "with", " ", "rotation"}], 
            "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"DDIinx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Lx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{
               RowBox[{"\[CapitalPhi]t", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
              RowBox[{
               RowBox[{"Minvx", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
              RowBox[{
               RowBox[{"L", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "+", 
             RowBox[{
              RowBox[{
               RowBox[{"\[CapitalPhi]t", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
              RowBox[{
               RowBox[{"Minv", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
              RowBox[{
               RowBox[{"Lx", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}]}], "&"}], 
           ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Uinx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"DDin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "square", " ", "R"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Px", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Mx", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}]}], 
          RowBox[{"(*", 
           RowBox[{"end", " ", "extendR"}], "*)"}], "\[IndentingNewLine]", 
          "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"Jumps", ",", 
          RowBox[{
           RowBox[{
            RowBox[{"J", "[", "3.1", "]"}], "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", 
           RowBox[{"[", "2", "]"}], "]"}], ",", 
          RowBox[{
           RowBox[{
            RowBox[{"J", "[", "3.1", "]"}], "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", 
           RowBox[{"[", "3", "]"}], "]"}]}], "}"}]}]}], "\[IndentingNewLine]",
       "]"}]}], ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Jadapt", "[", "3.1", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{
      RowBox[{"Adapt", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Jsamp", "[", "3.1", "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], ",", "globalTol"}], "]"}], "[", 
      RowBox[{
       RowBox[{"J", "[", "3.1", "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"J", "[", "3", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"\[Piecewise]", GridBox[{
        {
         RowBox[{
          RowBox[{"J", "[", "3.2", "]"}], "[", 
          RowBox[{"x", ",", "t"}], "]"}], 
         RowBox[{"el", ">", 
          RowBox[{"Abs", "[", 
           RowBox[{"K0", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}], ">", "\[Nu]"}]},
        {
         RowBox[{
          RowBox[{"J", "[", "3.1", "]"}], "[", 
          RowBox[{"x", ",", "t"}], "]"}], 
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"K0", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}], "\[LessEqual]", 
          "\[Nu]"}]},
        {
         RowBox[{
          RowBox[{"J", "[", "3.3", "]"}], "[", 
          RowBox[{"x", ",", "t"}], "]"}], "True"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxItemSize->{
        "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
         "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.84]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.4]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}]}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Jx", "[", "3", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"\[Piecewise]", GridBox[{
        {
         RowBox[{
          RowBox[{"Jx", "[", "3.2", "]"}], "[", 
          RowBox[{"x", ",", "t"}], "]"}], 
         RowBox[{"el", ">", 
          RowBox[{"Abs", "[", 
           RowBox[{"K0", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}], ">", "\[Nu]"}]},
        {
         RowBox[{
          RowBox[{"Jx", "[", "3.1", "]"}], "[", 
          RowBox[{"x", ",", "t"}], "]"}], 
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"K0", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}], "\[LessEqual]", 
          "\[Nu]"}]},
        {
         RowBox[{
          RowBox[{"Jx", "[", "3.3", "]"}], "[", 
          RowBox[{"x", ",", "t"}], "]"}], "True"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxItemSize->{
        "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
         "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.84]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.4]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}]}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Jsamp", "[", "3", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"\[Piecewise]", GridBox[{
        {
         RowBox[{
          RowBox[{"Jsamp", "[", "3.2", "]"}], "[", 
          RowBox[{"x", ",", "t"}], "]"}], 
         RowBox[{"el", ">", 
          RowBox[{"Abs", "[", 
           RowBox[{"K0", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}], ">", "\[Nu]"}]},
        {
         RowBox[{
          RowBox[{"Jsamp", "[", "3.1", "]"}], "[", 
          RowBox[{"x", ",", "t"}], "]"}], 
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"K0", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}], "\[LessEqual]", 
          "\[Nu]"}]},
        {
         RowBox[{
          RowBox[{"Jsamp", "[", "3.3", "]"}], "[", 
          RowBox[{"x", ",", "t"}], "]"}], "True"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxItemSize->{
        "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
         "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.84]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.4]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}]}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Jadapt", "[", "3", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{
      RowBox[{"Adapt", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Jsamp", "[", "3", "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], ",", "globalTol"}], "]"}], "[", 
      RowBox[{
       RowBox[{"J", "[", "3", "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
   
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Jadaptx", "[", "3", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{
      RowBox[{"Adapt", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Jsamp", "[", "3", "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], ",", "globalTol"}], "]"}], "[", 
      RowBox[{
       RowBox[{"Jx", "[", "3", "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ";"}], 
   "\[IndentingNewLine]"}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.568150148091033*^9, 3.568150159471627*^9}, {
   3.56815054877111*^9, 3.5681505495548697`*^9}, {3.568150867831676*^9, 
   3.568150872179638*^9}, {3.5681511298883057`*^9, 3.568151135829864*^9}, {
   3.568151212815825*^9, 3.568151353493661*^9}, {3.568151435714768*^9, 
   3.568151440788952*^9}, {3.5681514777263937`*^9, 3.568151480162973*^9}, {
   3.568151813635438*^9, 3.568151813822794*^9}, {3.568152388630576*^9, 
   3.568152412512949*^9}, {3.56833351018124*^9, 3.568333584301177*^9}, {
   3.568407509527516*^9, 3.5684075585357723`*^9}, {3.568407607044567*^9, 
   3.568407611865266*^9}, 3.568410906800005*^9, {3.568471574941979*^9, 
   3.568471609083024*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Region 3: Dispersive (scale-shift)(unfinished)", "Section",
 CellChangeTimes->{{3.56814327368044*^9, 3.568143277482564*^9}, {
  3.568335025816708*^9, 3.568335091661737*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"Valid", " ", "for"}], " ", "|", "K0", "|", " ", 
    RowBox[{">", " ", "\[Nu]"}]}], "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"if", "[", "t_", "]"}], ":=", 
     RowBox[{".25", "/", 
      RowBox[{"(", 
       RowBox[{"1", "+", 
        RowBox[{"Sqrt", "[", "t", "]"}]}], ")"}]}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"rays", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "3", " ", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}], ",", 
           RowBox[{
            RowBox[{"if", "[", "t", "]"}], " ", 
            RowBox[{"Exp", "[", 
             RowBox[{"I", " ", "3", " ", 
              RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}]}], "}"}], "]"}], ",", 
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "5", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}], ",", 
           RowBox[{
            RowBox[{"if", "[", "t", "]"}], " ", 
            RowBox[{"Exp", "[", 
             RowBox[{"I", " ", "5", " ", 
              RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}]}], "}"}], "]"}], ",", 
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{"if", "[", "t", "]"}], " ", 
            RowBox[{"Exp", "[", 
             RowBox[{
              RowBox[{"-", "I"}], " ", 
              RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}], ",", " ", 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}], "}"}], "]"}], ",", 
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{"if", "[", "t", "]"}], " ", 
            RowBox[{"Exp", "[", 
             RowBox[{"I", " ", 
              RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}], ",", 
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}], "}"}], "]"}]}], "}"}], 
      "*", "\[Nu]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"square", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "3", " ", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}], ",", 
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "5", " ", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}], "}"}], "]"}], ",", 
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "5", " ", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}], ",", 
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "7", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}], "}"}], "]"}], ",", 
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "7", " ", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}], ",", 
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}], "}"}], "]"}], ",", 
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}], ",", 
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "3", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}], "}"}], "]"}]}], "}"}], 
      "*", 
      RowBox[{"if", "[", "t", "]"}], " ", "\[Nu]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"stencil", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", " ", 
     RowBox[{
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"rays", "[", 
         RowBox[{"x", ",", "t"}], "]"}], ",", 
        RowBox[{"square", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "]"}], "*", 
      RowBox[{"Sqrt", "[", "2", "]"}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"extendL", " ", "=", " ", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "el"}], " ", "+", " ", 
           RowBox[{"I", " ", "\[Nu]"}]}], ",", 
          RowBox[{
           RowBox[{"Sqrt", "[", "2", "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{"3", " ", 
             RowBox[{"Pi", "/", "4"}], "I"}], "]"}], " ", "\[Nu]"}]}], "}"}], 
        "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "el"}], " ", "-", " ", 
           RowBox[{"I", " ", "\[Nu]"}]}], ",", 
          RowBox[{
           RowBox[{"Sqrt", "[", "2", "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{"5", 
             RowBox[{"Pi", "/", "4"}], "I"}], "]"}], " ", "\[Nu]"}]}], "}"}], 
        "]"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"extendR", "=", " ", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"Sqrt", "[", "2", "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"Pi", "/", "4"}], "I"}], "]"}], " ", "\[Nu]"}], ",", " ", 
          RowBox[{"el", " ", "+", " ", 
           RowBox[{"I", " ", "\[Nu]"}]}]}], "}"}], "]"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"Sqrt", "[", "2", "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"-", "Pi"}], "/", "4"}], "I"}], "]"}], " ", "\[Nu]"}], 
          ",", 
          RowBox[{"el", "-", " ", 
           RowBox[{"I", " ", "\[Nu]"}]}]}], "}"}], "]"}]}], "}"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"connect", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", " ", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"-", 
            RowBox[{"Abs", "[", 
             RowBox[{"K0", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "]"}]}], " ", "+", " ", 
           RowBox[{
            RowBox[{"Sqrt", "[", "2", "]"}], "\[Nu]", " ", 
            RowBox[{"Exp", "[", 
             RowBox[{"I", " ", 
              RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
          RowBox[{
           RowBox[{"Abs", "[", 
            RowBox[{"K0", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}], " ", "+", " ", 
           RowBox[{
            RowBox[{"Sqrt", "[", "2", "]"}], "\[Nu]", " ", 
            RowBox[{"Exp", "[", 
             RowBox[{"3", "I", " ", 
              RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"-", 
            RowBox[{"Abs", "[", 
             RowBox[{"K0", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "]"}]}], " ", "+", " ", 
           RowBox[{
            RowBox[{"Sqrt", "[", "2", "]"}], "\[Nu]", " ", 
            RowBox[{"Exp", "[", 
             RowBox[{
              RowBox[{"-", "I"}], " ", 
              RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
          RowBox[{
           RowBox[{"Abs", "[", 
            RowBox[{"K0", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}], " ", "+", " ", 
           RowBox[{
            RowBox[{"Sqrt", "[", "2", "]"}], "\[Nu]", " ", 
            RowBox[{"Exp", "[", 
             RowBox[{
              RowBox[{"-", "3"}], "I", " ", 
              RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}]}], 
      "}"}]}], ";"}], "\n", 
   RowBox[{
    RowBox[{
     RowBox[{"ctr", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"Join", "[", 
      RowBox[{
       RowBox[{"extendL", "-", 
        RowBox[{"K0", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], ",", 
       RowBox[{
        RowBox[{"stencil", "[", 
         RowBox[{"x", ",", "t"}], "]"}], " ", "-", " ", 
        RowBox[{"K0", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], ",", 
       RowBox[{"connect", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"stencil", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "+", 
        RowBox[{"K0", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], ",", 
       RowBox[{"extendR", " ", "+", " ", 
        RowBox[{"K0", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"DDI", "[", 
       RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Im", "[", "k", "]"}], " ", "\[GreaterEqual]", "0"}], ",", 
       RowBox[{
        RowBox[{"DD", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], ",", 
       RowBox[{"IdentityMatrix", "[", "2", "]"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"DDIin", "[", 
       RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Im", "[", "k", "]"}], " ", "\[GreaterEqual]", "0"}], ",", 
       RowBox[{
        RowBox[{"DDin", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], ",", 
       RowBox[{"IdentityMatrix", "[", "2", "]"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"DDIsamp", "[", 
       RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Im", "[", "k", "]"}], " ", "\[GreaterEqual]", "0"}], ",", 
       RowBox[{
        RowBox[{"DDsamp", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], ",", 
       RowBox[{"IdentityMatrix", "[", "2", "]"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"DDIinsamp", "[", 
       RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Im", "[", "k", "]"}], " ", "\[GreaterEqual]", "0"}], ",", 
       RowBox[{
        RowBox[{"DDinsamp", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], ",", 
       RowBox[{"IdentityMatrix", "[", "2", "]"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"J", "[", "3.3", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Jumps", "=", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"U", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"U", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}]}], 
          "}"}]}], ";", 
        RowBox[{"Domains", "=", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Line", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{"-", "el"}], " ", ",", "0"}], "}"}], "+", 
             RowBox[{"I", " ", "\[Nu]"}]}], "]"}], ",", 
           RowBox[{"Line", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"0", ",", "el"}], "}"}], "+", 
             RowBox[{"I", " ", "\[Nu]"}]}], "]"}], ",", 
           RowBox[{"Line", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{"-", "el"}], " ", ",", "0"}], "}"}], "-", 
             RowBox[{"I", " ", "\[Nu]"}]}], "]"}], ",", 
           RowBox[{"Line", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"0", " ", ",", "el"}], "}"}], "-", 
             RowBox[{"I", " ", "\[Nu]"}]}], "]"}]}], "}"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"NumPts", "=", 
         RowBox[{"{", 
          RowBox[{"bigN", ",", "bigN", ",", "bigN", ",", "bigN"}], "}"}]}], 
        ";", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}]}]}], 
      "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Jsamp", "[", "3.3", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Jumps", "=", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Usamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Usamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}]}], 
          "\[IndentingNewLine]", "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Domains", "=", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Line", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{"-", "el"}], " ", ",", "0"}], "}"}], "+", 
             RowBox[{"I", " ", "\[Nu]"}]}], "]"}], ",", 
           RowBox[{"Line", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"0", ",", "el"}], "}"}], "+", 
             RowBox[{"I", " ", "\[Nu]"}]}], "]"}], ",", 
           RowBox[{"Line", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{
               RowBox[{"-", "el"}], " ", ",", "0"}], "}"}], "-", 
             RowBox[{"I", " ", "\[Nu]"}]}], "]"}], ",", 
           RowBox[{"Line", "[", 
            RowBox[{
             RowBox[{"{", 
              RowBox[{"0", " ", ",", "el"}], "}"}], "-", 
             RowBox[{"I", " ", "\[Nu]"}]}], "]"}]}], "}"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"NumPts", "=", 
         RowBox[{"{", 
          RowBox[{"bigN", ",", "bigN", ",", "bigN", ",", "bigN"}], "}"}]}], 
        ";", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}]}]}], 
      "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"J", "[", "3.2", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Jumps", "=", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"P", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"M", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "extendL"}], "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"P", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"M", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"U", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "rays", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Minv", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"DDIin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Uin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"DDin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "square", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"U", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           " ", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "connect"}], "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"U", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"M", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"P", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "rays", " ", "R"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"DDIin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Minv", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Uin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"DDin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "square", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"P", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"M", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}]}], 
          RowBox[{"(*", 
           RowBox[{"end", " ", "extendR"}], "*)"}], "\[IndentingNewLine]", 
          "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Domains", " ", "=", " ", 
         RowBox[{"ctr", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"NumPts", " ", "=", " ", 
         RowBox[{"Table", "[", 
          RowBox[{"smallN", ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{"Length", "[", "Domains", "]"}]}], "}"}]}], "]"}]}], ";",
         "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}]}]}], 
      "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Jsamp", "[", "3.2", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Jumps", "=", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Psamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Msamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "extendL"}], "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Psamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Msamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Usamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "rays", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "square", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Usamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           " ", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "connect"}], "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Usamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Msamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Psamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "rays", " ", "R"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "square", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Psamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Msamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}]}], 
          RowBox[{"(*", 
           RowBox[{"end", " ", "extendR"}], "*)"}], "\[IndentingNewLine]", 
          "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Domains", " ", "=", " ", 
         RowBox[{"ctr", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"NumPts", " ", "=", " ", 
         RowBox[{"Table", "[", 
          RowBox[{"smallN", ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{"Length", "[", "Domains", "]"}]}], "}"}]}], "]"}]}], ";",
         "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}]}]}], 
      "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Jadapt", "[", "3.2", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{
      RowBox[{"Adapt", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Jsamp", "[", "3.2", "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], ",", "globalTol"}], "]"}], "[", 
      RowBox[{
       RowBox[{"J", "[", "3.2", "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ";"}]}]}]], "Input",
 CellChangeTimes->{{3.568144739351859*^9, 3.568144768075987*^9}, {
   3.568145313447761*^9, 3.5681453327796707`*^9}, {3.568145376573141*^9, 
   3.568145381688393*^9}, {3.5681454371313972`*^9, 3.5681455150174026`*^9}, {
   3.568145569123678*^9, 3.568145640898922*^9}, 3.568145735969737*^9, 
   3.5681457842697573`*^9, {3.568145966381893*^9, 3.5681461110591393`*^9}, {
   3.5681461476277943`*^9, 3.5681462249097977`*^9}, {3.568146329932455*^9, 
   3.568146494179833*^9}, {3.568146527537668*^9, 3.5681467331954527`*^9}, 
   3.568146786806254*^9, {3.568146981968944*^9, 3.568147003303782*^9}, {
   3.5681477923150587`*^9, 3.5681477944311438`*^9}, {3.568147921896377*^9, 
   3.568147927835744*^9}, {3.568149378238656*^9, 3.56814938792607*^9}, {
   3.5681494262633057`*^9, 3.568149477859494*^9}, {3.56814951449235*^9, 
   3.568149589294854*^9}, {3.568149620607332*^9, 3.568149648942997*^9}, {
   3.5681497045425453`*^9, 3.568149728649761*^9}, {3.568149850187934*^9, 
   3.568149851216811*^9}, {3.568150013812023*^9, 3.568150074988999*^9}, {
   3.5681501221337748`*^9, 3.5681501268447247`*^9}, {3.568150344328685*^9, 
   3.5681503447013474`*^9}, {3.568150439229638*^9, 3.56815045013251*^9}, {
   3.5681508262619753`*^9, 3.56815083741147*^9}, {3.568150970457507*^9, 
   3.5681510134580917`*^9}, {3.568151094017838*^9, 3.5681510957808857`*^9}, {
   3.568151146755842*^9, 3.568151149432043*^9}, {3.568151183497633*^9, 
   3.568151206989304*^9}, {3.56815123848202*^9, 3.568151258072465*^9}, {
   3.5681520173106527`*^9, 3.568152080041749*^9}, {3.568152159235017*^9, 
   3.568152162839055*^9}, {3.568152233382656*^9, 3.568152250193955*^9}, {
   3.568152306478895*^9, 3.56815231568089*^9}, {3.5681523480442343`*^9, 
   3.5681523496745243`*^9}, {3.568333271971957*^9, 3.5683334679126863`*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"Valid", " ", "if"}], " ", "|", "K0", "|", 
    RowBox[{"<", " ", "\[Nu]"}]}], "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"skewrays", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", " ", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "3", " ", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}], ",", 
           RowBox[{
            RowBox[{"if", "[", "t", "]"}], " ", 
            RowBox[{"Exp", "[", 
             RowBox[{"I", " ", "3", " ", 
              RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}]}], "}"}], "]"}], ",", 
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "5", 
             RowBox[{"Pi", " ", "/", "4"}]}], "]"}], ",", 
           RowBox[{
            RowBox[{"if", "[", "t", "]"}], " ", 
            RowBox[{"Exp", "[", 
             RowBox[{"I", " ", "5", " ", 
              RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}]}], "}"}], "]"}], ",", 
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{"if", "[", "t", "]"}], " ", 
            RowBox[{"Exp", "[", 
             RowBox[{
              RowBox[{"-", "I"}], " ", 
              RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}], ",", 
           RowBox[{
            RowBox[{
             RowBox[{"Abs", "[", 
              RowBox[{"K0", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "]"}], "/", "\[Nu]"}], " ", 
            RowBox[{"Exp", "[", 
             RowBox[{
              RowBox[{"-", "I"}], " ", 
              RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}]}], "}"}], "]"}], ",", 
        RowBox[{"Line", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{"if", "[", "t", "]"}], " ", 
            RowBox[{"Exp", "[", 
             RowBox[{"I", " ", 
              RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}], ",", 
           RowBox[{
            RowBox[{
             RowBox[{"Abs", "[", 
              RowBox[{"K0", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "]"}], "/", "\[Nu]"}], " ", 
            RowBox[{"Exp", "[", 
             RowBox[{"I", " ", 
              RowBox[{"Pi", " ", "/", "4"}]}], "]"}]}]}], "}"}], "]"}]}], 
       "}"}], "*", "\[Nu]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"stencilskewL", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", " ", 
     RowBox[{
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"skewrays", "[", 
         RowBox[{"x", ",", "t"}], "]"}], ",", 
        RowBox[{"square", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "]"}], "*", 
      RowBox[{"Sqrt", "[", "2", "]"}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"stencilskewR", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", " ", 
     RowBox[{
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"skewrays", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}], "//", "LineReverse"}], ",", 
        RowBox[{"square", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "]"}], "*", 
      RowBox[{"Sqrt", "[", "2", "]"}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"ctrskew", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"Join", "[", 
      RowBox[{
       RowBox[{"extendL", "-", 
        RowBox[{"K0", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], ",", 
       RowBox[{
        RowBox[{"stencilskewL", "[", 
         RowBox[{"x", ",", "t"}], "]"}], " ", "-", " ", 
        RowBox[{"K0", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], ",", 
       RowBox[{
        RowBox[{"stencilskewR", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "+", 
        RowBox[{"K0", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], ",", 
       RowBox[{"extendR", " ", "+", " ", 
        RowBox[{"K0", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"J", "[", "3.1", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Jumps", "=", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"P", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"M", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "extendL"}], "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"P", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"M", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"U", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "rays", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Minv", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"DDIin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Uin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"DDin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "square", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"M", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"P", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"U", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{
            "end", " ", "rays", " ", "R", " ", "with", " ", "rotation"}], 
            "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"DDIin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Minv", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"L", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"Uin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"DDin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           
           RowBox[{"(*", 
            RowBox[{"end", " ", "square", " ", "R"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"P", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"\[CapitalPhi]t", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"M", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
             RowBox[{
              RowBox[{"\[CapitalPhi]tin", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}]}], 
          RowBox[{"(*", 
           RowBox[{"end", " ", "extendR"}], "*)"}], "\[IndentingNewLine]", 
          "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Domains", " ", "=", " ", 
         RowBox[{"ctrskew", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"NumPts", " ", "=", " ", 
         RowBox[{"Table", "[", 
          RowBox[{"smallN", ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{"Length", "[", "Domains", "]"}]}], "}"}]}], "]"}]}], ";",
         "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}]}]}], 
      "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Jsamp", "[", "3.1", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Jumps", "=", 
         RowBox[{"{", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Psamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Msamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "extendL"}], "*)"}], "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Psamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Msamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Usamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "rays", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "square", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Msamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Psamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Usamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "rays", " ", "R"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Lsamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           RowBox[{"(*", 
            RowBox[{"end", " ", "square", " ", "L"}], "*)"}], 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Psamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{
             RowBox[{"Msamp", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}]}], 
          RowBox[{"(*", 
           RowBox[{"end", " ", "extendR"}], "*)"}], "\[IndentingNewLine]", 
          "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Domains", " ", "=", " ", 
         RowBox[{"ctrskew", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"NumPts", " ", "=", " ", 
         RowBox[{"Table", "[", 
          RowBox[{"smallN", ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "1", ",", 
             RowBox[{"Length", "[", "Domains", "]"}]}], "}"}]}], "]"}]}], ";",
         "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}]}]}], 
      "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Jadapt", "[", "3.1", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{
      RowBox[{"Adapt", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Jsamp", "[", "3.1", "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], ",", "globalTol"}], "]"}], "[", 
      RowBox[{
       RowBox[{"J", "[", "3.1", "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"J", "[", "3", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"\[Piecewise]", GridBox[{
        {
         RowBox[{
          RowBox[{"J", "[", "3.2", "]"}], "[", 
          RowBox[{"x", ",", "t"}], "]"}], 
         RowBox[{"el", ">", 
          RowBox[{"Abs", "[", 
           RowBox[{"K0", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}], ">", "\[Nu]"}]},
        {
         RowBox[{
          RowBox[{"J", "[", "3.1", "]"}], "[", 
          RowBox[{"x", ",", "t"}], "]"}], 
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"K0", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}], "\[LessEqual]", 
          "\[Nu]"}]},
        {
         RowBox[{
          RowBox[{"J", "[", "3.3", "]"}], "[", 
          RowBox[{"x", ",", "t"}], "]"}], "True"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxItemSize->{
        "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
         "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.84]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.4]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}]}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Jsamp", "[", "3", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"\[Piecewise]", GridBox[{
        {
         RowBox[{
          RowBox[{"Jsamp", "[", "3.2", "]"}], "[", 
          RowBox[{"x", ",", "t"}], "]"}], 
         RowBox[{"el", ">", 
          RowBox[{"Abs", "[", 
           RowBox[{"K0", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}], ">", "\[Nu]"}]},
        {
         RowBox[{
          RowBox[{"Jsamp", "[", "3.1", "]"}], "[", 
          RowBox[{"x", ",", "t"}], "]"}], 
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{"K0", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}], "\[LessEqual]", 
          "\[Nu]"}]},
        {
         RowBox[{
          RowBox[{"Jsamp", "[", "3.3", "]"}], "[", 
          RowBox[{"x", ",", "t"}], "]"}], "True"}
       },
       GridBoxAlignment->{
        "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
          "RowsIndexed" -> {}},
       GridBoxItemSize->{
        "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
         "RowsIndexed" -> {}},
       GridBoxSpacings->{"Columns" -> {
           Offset[0.27999999999999997`], {
            Offset[0.84]}, 
           Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
           Offset[0.2], {
            Offset[0.4]}, 
           Offset[0.2]}, "RowsIndexed" -> {}}]}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Jadapt", "[", "3", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{
      RowBox[{"Adapt", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Jsamp", "[", "3", "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], ",", "globalTol"}], "]"}], "[", 
      RowBox[{
       RowBox[{"J", "[", "3", "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ";"}], 
   "\[IndentingNewLine]"}]}]], "Input",
 CellChangeTimes->{{3.568150148091033*^9, 3.568150159471627*^9}, {
  3.56815054877111*^9, 3.5681505495548697`*^9}, {3.568150867831676*^9, 
  3.568150872179638*^9}, {3.5681511298883057`*^9, 3.568151135829864*^9}, {
  3.568151212815825*^9, 3.568151353493661*^9}, {3.568151435714768*^9, 
  3.568151440788952*^9}, {3.5681514777263937`*^9, 3.568151480162973*^9}, {
  3.568151813635438*^9, 3.568151813822794*^9}, {3.568152388630576*^9, 
  3.568152412512949*^9}, {3.56833351018124*^9, 3.568333584301177*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Region 4: Collisionless Shock", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.568556583304967*^9, 3.5685565921082973`*^9}}],

Cell[CellGroupData[{

Cell["Find a and b", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.568552120247838*^9, 3.568552123479507*^9}, {
  3.568571262982126*^9, 3.568571265457403*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"a", "//", "Clear"}], ";", 
  RowBox[{"anew", "//", "Clear"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"K0", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Sqrt", "[", 
    RowBox[{
     RowBox[{"-", "x"}], "/", 
     RowBox[{"(", 
      RowBox[{"12", "t"}], ")"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"mNewtonMethod", "[", 
    RowBox[{"f_", ",", "fp_", ",", "x0_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "ret", "}"}], ",", "\[IndentingNewLine]", "\t", 
     RowBox[{
      RowBox[{
       RowBox[{"ret", "[", "0", "]"}], "=", "x0"}], ";", 
      "\[IndentingNewLine]", "\t", 
      RowBox[{
       RowBox[{"ret", "[", "n_", "]"}], ":=", 
       RowBox[{
        RowBox[{"ret", "[", "n", "]"}], "=", 
        RowBox[{
         RowBox[{"ret", "[", 
          RowBox[{"n", "-", "1"}], "]"}], "-", 
         FractionBox[
          RowBox[{"f", "[", 
           RowBox[{"ret", "[", 
            RowBox[{"n", "-", "1"}], "]"}], "]"}], 
          RowBox[{"fp", "[", 
           RowBox[{"ret", "[", 
            RowBox[{"n", "-", "1"}], "]"}], "]"}]]}]}]}], ";", 
      "\[IndentingNewLine]", "ret"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"mNewtonMethod", "[", 
    RowBox[{"f_", ",", "x0_"}], "]"}], ":=", 
   RowBox[{"mNewtonMethod", "[", 
    RowBox[{"f", ",", 
     RowBox[{"f", "'"}], ",", "x0"}], "]"}]}], ";", 
  RowBox[{
   RowBox[{"mFFT", "[", "F_", "]"}], ":=", 
   RowBox[{"Fourier", "[", 
    RowBox[{"F", ",", 
     RowBox[{"FourierParameters", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "1"}], ",", 
        RowBox[{"-", "1"}]}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"InvmFFT", "[", "F_", "]"}], ":=", 
   RowBox[{"InverseFourier", "[", 
    RowBox[{"F", ",", 
     RowBox[{"FourierParameters", "\[Rule]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "1"}], ",", "1"}], "}"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"FFTShift", "[", "V_", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "n", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"n", " ", "=", " ", 
       RowBox[{
        RowBox[{"Length", "[", "V", "]"}], "/", "2"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Join", "[", 
       RowBox[{
        RowBox[{"V", "[", 
         RowBox[{"[", 
          RowBox[{
           RowBox[{"n", "+", "1"}], ";;"}], "]"}], "]"}], ",", 
        RowBox[{"V", "[", 
         RowBox[{"[", 
          RowBox[{"1", ";;", "n"}], "]"}], "]"}]}], "]"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ToCosineSeries", "[", "L_", "]"}], ":=", " ", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"c", ",", "l", ",", "m"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"l", " ", "=", " ", 
       RowBox[{"Length", "[", "L", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Mod", "[", 
          RowBox[{"l", ",", "2"}], "]"}], "\[Equal]", "1"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"m", " ", "=", " ", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{"l", "-", "1"}], ")"}], "/", "2"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"c", " ", "=", " ", 
          RowBox[{"ConstantArray", "[", 
           RowBox[{"0.", ",", 
            RowBox[{"m", "+", "1"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"c", " ", "=", " ", 
          RowBox[{"L", "[", 
           RowBox[{"[", 
            RowBox[{"1", ";;", 
             RowBox[{"m", "+", "1"}]}], "]"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"c", "[", 
           RowBox[{"[", 
            RowBox[{"2", ";;"}], "]"}], "]"}], "=", 
          RowBox[{"2", "*", 
           RowBox[{"c", "[", 
            RowBox[{"[", 
             RowBox[{"2", ";;"}], "]"}], "]"}]}]}], ";"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"m", " ", "=", " ", 
          RowBox[{"l", "/", "2"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"c", " ", "=", " ", 
          RowBox[{"ConstantArray", "[", 
           RowBox[{"0.", ",", 
            RowBox[{"m", "+", "1"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"c", " ", "=", " ", 
          RowBox[{"L", "[", 
           RowBox[{"[", 
            RowBox[{"1", ";;", 
             RowBox[{"m", "+", "2"}]}], "]"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"c", " ", "[", 
           RowBox[{"[", 
            RowBox[{"2", ";;", 
             RowBox[{"-", "2"}]}], "]"}], "]"}], "=", 
          RowBox[{"2", "*", 
           RowBox[{"c", "[", 
            RowBox[{"[", 
             RowBox[{"2", ";;", 
              RowBox[{"-", "2"}]}], "]"}], "]"}]}]}], ";"}]}], "]"}], ";", 
      "\[IndentingNewLine]", "c"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"mDCT", "[", "L_", "]"}], ":=", 
   RowBox[{
    RowBox[{"mFFT", "[", 
     RowBox[{
      RowBox[{"Join", "[", 
       RowBox[{"L", ",", 
        RowBox[{"Reverse", "[", 
         RowBox[{"L", "[", 
          RowBox[{"[", 
           RowBox[{"2", ";;", 
            RowBox[{"-", "2"}]}], "]"}], "]"}], "]"}]}], "]"}], "//", 
      "FFTShift"}], "]"}], "//", "ToCosineSeries"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ChebPts", "[", "n_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Cos", "[", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"1.0", "*", "Pi", "*", 
        RowBox[{"i", "/", 
         RowBox[{"(", 
          RowBox[{"n", "-", "1"}], ")"}]}]}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "0", ",", 
         RowBox[{"n", "-", "1"}]}], "}"}]}], "]"}], "]"}], "//", 
    "Reverse"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Tn2Un", "[", "n_", "]"}], ":=", " ", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "out", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"n", "\[Equal]", "0"}], ",", 
        RowBox[{
         RowBox[{"out", " ", "=", " ", 
          RowBox[{"{", "1", "}"}]}], ";"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"out", "=", 
          RowBox[{"Table", "[", 
           RowBox[{
            RowBox[{"2", "*", 
             RowBox[{"Mod", "[", 
              RowBox[{
               RowBox[{"i", "+", "n", "+", "1"}], ",", "2"}], "]"}]}], ",", 
            RowBox[{"{", 
             RowBox[{"i", ",", "0", ",", "n"}], "}"}]}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"out", "[", 
           RowBox[{"[", "1", "]"}], "]"}], "=", 
          RowBox[{
           RowBox[{"out", "[", 
            RowBox[{"[", "1", "]"}], "]"}], "/", "2"}]}], ";"}]}], "]"}], ";",
       "\[IndentingNewLine]", "out"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Tn2UnMat", "[", "n_", "]"}], ":=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"Join", "[", 
      RowBox[{
       RowBox[{"Tn2Un", "[", "i", "]"}], ",", 
       RowBox[{"ConstantArray", "[", 
        RowBox[{"0.", ",", 
         RowBox[{"n", "-", "1", "-", "i"}]}], "]"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "0", ",", 
       RowBox[{"n", "-", "1"}]}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Un2TnCoef", "[", "n_", "]"}], ":=", 
   RowBox[{"Transpose", "[", 
    RowBox[{"Tn2UnMat", "[", "n", "]"}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"sp", "[", 
     RowBox[{"a_", ",", "b_"}], "]"}], "[", "x_", "]"}], ":=", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"b", "-", "a"}], ")"}], "/", "2"}], " ", "x"}], " ", "+", " ", 
    
    RowBox[{
     RowBox[{"(", 
      RowBox[{"b", "+", "a"}], ")"}], "/", "2"}]}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ef", "[", 
     RowBox[{"a_", ",", "b_"}], "]"}], "[", "x_", "]"}], ":=", 
   RowBox[{"Sqrt", "[", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"sp", "[", 
         RowBox[{"a", ",", "b"}], "]"}], "[", "x", "]"}], "+", "a"}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"sp", "[", 
         RowBox[{"a", ",", "b"}], "]"}], "[", "x", "]"}], "+", "b"}], ")"}], 
     RowBox[{"(", 
      RowBox[{"1", "/", "4"}], ")"}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", "-", "b"}], ")"}], "^", "2"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"anew", "//", "Clear"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"EllipInt", "[", 
    RowBox[{"a_", ",", "b_", ",", "n_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{
     RowBox[{"LinearSolve", "[", 
      RowBox[{
       RowBox[{"Un2TnCoef", "[", 
        RowBox[{"n", "+", "1"}], "]"}], ",", 
       RowBox[{"mDCT", "[", 
        RowBox[{
         RowBox[{"ef", "[", 
          RowBox[{"a", ",", "b"}], "]"}], "[", 
         RowBox[{"ChebPts", "[", "n", "]"}], "]"}], "]"}]}], "]"}], "[", 
     RowBox[{"[", "1", "]"}], "]"}], "*", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"b", "-", "a"}], ")"}], "/", "2"}], " ", "*", " ", 
    RowBox[{"Pi", "/", "2"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ZeroFunction", "[", 
    RowBox[{"s_", ",", "c_", ",", "n_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "F", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"F", "[", "A_", "]"}], ":=", 
       RowBox[{"s", " ", "-", 
        RowBox[{"24", "*", 
         RowBox[{"EllipInt", "[", 
          RowBox[{"A", ",", 
           RowBox[{"Sqrt", "[", 
            RowBox[{"c", "-", 
             RowBox[{"A", "^", "2"}]}], "]"}], ",", "n"}], "]"}]}]}]}], ";", 
      "\[IndentingNewLine]", "F"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ZeroFun", "[", 
    RowBox[{"s_", ",", "c_", ",", "n_"}], "]"}], ":=", 
   RowBox[{"Fun", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"ZeroFunction", "[", 
        RowBox[{"s", ",", "c", ",", "n"}], "]"}], "[", "#", "]"}], "&"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{"0", ",", 
        RowBox[{"Sqrt", "[", 
         RowBox[{"c", "/", "2"}], "]"}]}], "}"}], "]"}], ",", "n"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Finda", "[", 
    RowBox[{"s_", ",", "c_", ",", "n_"}], "]"}], ":=", " ", 
   RowBox[{"mNewtonMethod", "[", 
    RowBox[{
     RowBox[{"ZeroFun", "[", 
      RowBox[{"s", ",", "c", ",", "n"}], "]"}], ",", 
     RowBox[{
      RowBox[{"Sqrt", "[", 
       RowBox[{"c", "/", "2"}], "]"}], "/", "2"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"aNewton", "[", "s_", "]"}], ":=", 
   RowBox[{
    RowBox[{"aNewton", "[", "s", "]"}], "=", 
    RowBox[{
     RowBox[{
      RowBox[{"Finda", "[", 
       RowBox[{"s", ",", "2", ",", "20"}], "]"}], "[", "50", "]"}], "//", 
     "Re"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ESTZ", "[", "x_", "]"}], ":=", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"x", "\[Equal]", 
      RowBox[{"{", "}"}]}], ",", 
     RowBox[{"{", "0", "}"}], ",", "x"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"a", "[", "s_", "]"}], ":=", 
   RowBox[{
    RowBox[{"a", "[", "s", "]"}], "=", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"ZeroFun", "[", 
        RowBox[{"s", ",", "2", ",", "20"}], "]"}], "//", "Roots"}], "//", 
      "ESTZ"}], "//", 
     RowBox[{
      RowBox[{"#", "[", 
       RowBox[{"[", "1", "]"}], "]"}], "&"}]}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"b", "[", "s_", "]"}], ":=", 
   RowBox[{"Sqrt", "[", 
    RowBox[{"2", "-", 
     RowBox[{
      RowBox[{"a", "[", "s", "]"}], "^", "2"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ComposeFunction", "[", 
    RowBox[{"X_", ",", "T_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"X", "[", 
     RowBox[{"T", "[", "#", "]"}], "]"}], "&"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"tau", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"t", " ", 
    RowBox[{
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "^", "3"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"s", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"Log", "[", 
      RowBox[{
       RowBox[{"K0", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "^", "2"}], "]"}]}], "/", 
    RowBox[{"tau", "[", 
     RowBox[{"x", ",", "t"}], "]"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"b", "[", "s_", "]"}], ":=", " ", 
   RowBox[{"Sqrt", "[", 
    RowBox[{"2", "-", 
     RowBox[{
      RowBox[{"a", "[", "s", "]"}], "^", "2"}]}], "]"}]}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.568553397555667*^9, {3.568553449403825*^9, 3.56855345203559*^9}, {
   3.568553772837674*^9, 3.568553818750169*^9}, {3.568554052643714*^9, 
   3.568554063818709*^9}, {3.568554160134904*^9, 3.568554161391015*^9}, {
   3.568554205088578*^9, 3.568554211839402*^9}, {3.568554324036275*^9, 
   3.5685544847216873`*^9}, {3.568554601326907*^9, 3.568554752249818*^9}, {
   3.5685547965331593`*^9, 3.568554799270925*^9}, {3.568554846408194*^9, 
   3.5685548784821177`*^9}, 3.5685553326636887`*^9, {3.568555426372445*^9, 
   3.5685554997613087`*^9}, {3.568556654363599*^9, 3.568556699083126*^9}, {
   3.568556765603969*^9, 3.5685567777584047`*^9}, {3.568556811337154*^9, 
   3.568556864286745*^9}, {3.568556978774241*^9, 3.568557019646803*^9}, 
   3.568557128104793*^9, {3.5685604088845882`*^9, 3.568560423468215*^9}, {
   3.568570447942103*^9, 3.568570448377982*^9}, {3.568571181392728*^9, 
   3.568571181747373*^9}, {3.568571293253327*^9, 3.568571303543618*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["g-function, parametrices and extra jump matrices", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.56857132967124*^9, 3.56857133960244*^9}, {
  3.568571398229066*^9, 3.5685714114505*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"\[Theta]p", "[", 
    RowBox[{"k_", ",", "x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"2", "I", " ", "x", " ", 
     RowBox[{"K0", "[", 
      RowBox[{"x", ",", "t"}], "]"}]}], " ", "+", " ", 
    RowBox[{"24", " ", "I", " ", 
     RowBox[{"k", "^", "2"}], "*", 
     RowBox[{
      RowBox[{"K0", "[", 
       RowBox[{"x", ",", "t"}], "]"}], "^", "3"}], "*", " ", "t"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ci", "//", "Clear"}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"ci", "[", 
    RowBox[{"xx_", ",", "tt_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"ci", "[", 
     RowBox[{"xx", ",", "tt"}], "]"}], "=", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Fun", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"\[Theta]p", "[", 
           RowBox[{"#", ",", "xx", ",", "tt"}], "]"}]}], "&"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Line", "[", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", 
              RowBox[{"b", "[", 
               RowBox[{"s", "[", 
                RowBox[{"xx", ",", "tt"}], "]"}], "]"}]}], ",", 
             RowBox[{"-", 
              RowBox[{"a", "[", 
               RowBox[{"s", "[", 
                RowBox[{"xx", ",", "tt"}], "]"}], "]"}]}]}], "}"}], "]"}], 
          ",", 
          RowBox[{"Line", "[", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"a", "[", 
              RowBox[{"s", "[", 
               RowBox[{"xx", ",", "tt"}], "]"}], "]"}], ",", 
             RowBox[{"b", "[", 
              RowBox[{"s", "[", 
               RowBox[{"xx", ",", "tt"}], "]"}], "]"}]}], "}"}], "]"}]}], 
         "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"30", ",", "30"}], "}"}]}], "]"}], "//", 
      "CauchyInverseCurves"}], ")"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"CSGFUN", "[", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", "\n", 
     RowBox[{
      RowBox[{"gfun", "//", "Clear"}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"gfun", "[", 
        RowBox[{"p_", ",", "z_", ",", "xx_", ",", "tt_"}], "]"}], ":=", 
       RowBox[{
        RowBox[{"gfun", "[", 
         RowBox[{"p", ",", "z", ",", "xx", ",", "tt"}], "]"}], "=", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"CauchyInverseIntegral", "[", 
            RowBox[{"p", ",", "#", ",", "z"}], "]"}], "&"}], "/@", 
          RowBox[{"ci", "[", 
           RowBox[{"xx", ",", "tt"}], "]"}]}], "//", "Total"}]}]}], ";"}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"gfunsamp", "[", 
     RowBox[{"p_", ",", "z_", ",", "xx_", ",", "tt_"}], "]"}], ":=", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Abs", "[", "z", "]"}], "<", "4"}], ",", 
      RowBox[{".1", 
       RowBox[{"Abs", "[", 
        RowBox[{"tau", "[", 
         RowBox[{"xx", ",", "tt"}], "]"}], "]"}], 
       RowBox[{"Sign", "[", 
        RowBox[{"Im", "[", "z", "]"}], "]"}]}], ",", "0"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]", "[", 
      RowBox[{"-", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"Exp", "[", 
        RowBox[{"gfun", "[", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "k", ",", "x", ",", "t"}], "]"}], "]"}], 
       "0"},
      {"0", 
       RowBox[{"Exp", "[", 
        RowBox[{"-", 
         RowBox[{"gfun", "[", 
          RowBox[{
           RowBox[{"-", "1"}], ",", "k", ",", "x", ",", "t"}], "]"}]}], 
        "]"}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"Exp", "[", 
        RowBox[{"gfun", "[", 
         RowBox[{"1", ",", "k", ",", "x", ",", "t"}], "]"}], "]"}], "0"},
      {"0", 
       RowBox[{"Exp", "[", 
        RowBox[{"-", 
         RowBox[{"gfun", "[", 
          RowBox[{"1", ",", "k", ",", "x", ",", "t"}], "]"}]}], "]"}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]samp", "[", 
      RowBox[{"-", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"Exp", "[", 
        RowBox[{"gfunsamp", "[", 
         RowBox[{
          RowBox[{"-", "1"}], ",", "k", ",", "x", ",", "t"}], "]"}], "]"}], 
       "0"},
      {"0", 
       RowBox[{"Exp", "[", 
        RowBox[{"-", 
         RowBox[{"gfunsamp", "[", 
          RowBox[{
           RowBox[{"-", "1"}], ",", "k", ",", "x", ",", "t"}], "]"}]}], 
        "]"}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]samp", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {
        RowBox[{"Exp", "[", 
         RowBox[{"gfunsamp", "[", 
          RowBox[{"1", ",", "k", ",", "x", ",", "t"}], "]"}], "]"}], "0"},
       {"0", 
        RowBox[{"Exp", "[", 
         RowBox[{"-", 
          RowBox[{"gfunsamp", "[", 
           RowBox[{"1", ",", "k", ",", "x", ",", "t"}], "]"}]}], "]"}]}
      }], ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Generate\[Delta]cs", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"k0", ",", "const", ",", "eps"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"\[Delta]cs", "//", "Clear"}], ";", "\[IndentingNewLine]", 
      RowBox[{"if1cs", "//", "Clear"}], ";", "\[IndentingNewLine]", 
      RowBox[{"eps", "=", 
       RowBox[{"10", "^", 
        RowBox[{"(", 
         RowBox[{"-", "14"}], ")"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"k0", " ", "=", " ", 
       RowBox[{"a", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      
      RowBox[{"if1cs", "=", 
       RowBox[{"IFun", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"Log", "[", 
            RowBox[{"\[Tau]", "[", 
             RowBox[{"#", "*", 
              RowBox[{"K0", "[", 
               RowBox[{"x", ",", "t"}], "]"}]}], "]"}], "]"}], "-", 
           RowBox[{"Log", "[", 
            RowBox[{
             RowBox[{"Abs", "[", "#", "]"}], "^", "2"}], "]"}]}], "&"}], ",", 
         
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "k0"}], ",", "k0"}], "}"}], "]"}], ",", "80"}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"\[Delta]cs", "[", 
        RowBox[{
         RowBox[{"k_", "?", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"NZeroQ", "[", 
              RowBox[{"Im", "[", "#", "]"}], "]"}], "&&", 
             RowBox[{
              RowBox[{"-", "k0"}], "\[LessEqual]", 
              RowBox[{"Re", "[", "#", "]"}], "\[LessEqual]", "k0"}]}], "&"}], 
           ")"}]}], ",", "s_"}], "]"}], ":=", 
       RowBox[{
        RowBox[{"\[Delta]cs", "[", 
         RowBox[{"k", ",", "s"}], "]"}], "=", "\[IndentingNewLine]", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"LogIntVariable", "[", 
            RowBox[{"k0", ",", 
             RowBox[{"k", "+", 
              RowBox[{"s", " ", "I", " ", "eps"}]}]}], "]"}], "+", 
           RowBox[{"Cauchy", "[", 
            RowBox[{"s", ",", "if1cs", ",", 
             RowBox[{"k", " ", "+", 
              RowBox[{"s", " ", "I", " ", "eps"}]}]}], "]"}]}], "//", "Exp"}],
          ")"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"\[Delta]cs", "[", 
        RowBox[{"k_", ",", "s_"}], "]"}], ":=", 
       RowBox[{
        RowBox[{"\[Delta]cs", "[", 
         RowBox[{"k", ",", "s"}], "]"}], "=", "\[IndentingNewLine]", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"LogIntVariable", "[", 
            RowBox[{"k0", ",", 
             RowBox[{"k", "+", 
              RowBox[{"s", " ", "I", " ", "eps"}]}]}], "]"}], "+", 
           RowBox[{"Cauchy", "[", 
            RowBox[{"if1cs", ",", "k"}], " ", "]"}]}], "//", "Exp"}], 
         ")"}]}]}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"\[Delta]csk", "//", "Clear"}], ";", "\[IndentingNewLine]", 
      RowBox[{"if1csk", "//", "Clear"}], ";", "\[IndentingNewLine]", 
      RowBox[{"if1csk", "=", 
       RowBox[{"IFun", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Log", "[", 
           RowBox[{
            RowBox[{"\[Tau]", "[", 
             RowBox[{"#", "*", 
              RowBox[{"K0", "[", 
               RowBox[{"x", ",", "t"}], "]"}]}], "]"}], "/", 
            RowBox[{
             RowBox[{"Abs", "[", 
              RowBox[{"#", "*", 
               RowBox[{"K0", "[", 
                RowBox[{"x", ",", "t"}], "]"}]}], "]"}], "^", "2"}]}], "]"}], 
          "&"}], ",", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "k0"}], ",", "k0"}], "}"}], "]"}], ",", "80"}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"\[Delta]csk", "[", 
        RowBox[{
         RowBox[{"k_", "?", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"NZeroQ", "[", 
              RowBox[{"Im", "[", "#", "]"}], "]"}], "&&", 
             RowBox[{
              RowBox[{"-", "k0"}], "\[LessEqual]", 
              RowBox[{"Re", "[", "#", "]"}], "\[LessEqual]", "k0"}]}], "&"}], 
           ")"}]}], ",", "s_"}], "]"}], ":=", 
       RowBox[{
        RowBox[{"\[Delta]csk", "[", 
         RowBox[{"k", ",", "s"}], "]"}], "=", "\[IndentingNewLine]", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"LogIntVariable", "[", 
            RowBox[{"k0", ",", 
             RowBox[{"k", "+", 
              RowBox[{"s", " ", "I", " ", "eps"}]}]}], "]"}], "+", 
           RowBox[{"Cauchy", "[", 
            RowBox[{"s", ",", "if1csk", ",", 
             RowBox[{"k", " ", "+", 
              RowBox[{"s", " ", "I", " ", "eps"}]}]}], "]"}]}], "//", "Exp"}],
          ")"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"\[Delta]csk", "[", 
        RowBox[{"k_", ",", "s_"}], "]"}], ":=", 
       RowBox[{
        RowBox[{"\[Delta]csk", "[", 
         RowBox[{"k", ",", "s"}], "]"}], "=", "\[IndentingNewLine]", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"LogIntVariable", "[", 
            RowBox[{"k0", ",", 
             RowBox[{"k", "+", 
              RowBox[{"s", " ", "I", " ", "eps"}]}]}], "]"}], "+", 
           RowBox[{"Cauchy", "[", 
            RowBox[{"if1csk", ",", "k"}], " ", "]"}]}], "//", "Exp"}], 
         ")"}]}]}], ";"}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Generate\[Zeta]", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", "eps", "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"eps", " ", "=", " ", 
        RowBox[{"$MachineEpsilon", "*", "10"}]}], ";", "\[IndentingNewLine]", 
       
       RowBox[{"\[Zeta]", "//", "Clear"}], ";", "\[IndentingNewLine]", 
       RowBox[{"glim1", " ", "=", " ", 
        RowBox[{"Fun", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"gfun", "[", 
               RowBox[{"1", ",", 
                RowBox[{"#", "+", 
                 RowBox[{"I", " ", "eps"}]}], ",", "x", ",", "t"}], "]"}], 
              "-", 
              RowBox[{"gfun", "[", 
               RowBox[{
                RowBox[{"-", "1"}], ",", 
                RowBox[{"#", "-", 
                 RowBox[{"I", " ", "eps"}]}], ",", "x", ",", "t"}], "]"}]}], 
             ")"}], "+", 
            RowBox[{"Log", "[", 
             RowBox[{"\[Tau]", "[", 
              RowBox[{"#", "*", 
               RowBox[{"K0", "[", 
                RowBox[{"x", ",", "t"}], "]"}]}], "]"}], "]"}]}], "&"}], ",", 
          
          RowBox[{"Line", "[", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", 
              RowBox[{"b", "[", 
               RowBox[{"s", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ",", 
             RowBox[{"-", 
              RowBox[{"a", "[", 
               RowBox[{"s", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], "}"}], "]"}], ",", 
          "150"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"glim2", " ", "=", " ", 
        RowBox[{"Fun", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"gfun", "[", 
               RowBox[{"1", ",", 
                RowBox[{"#", "+", 
                 RowBox[{"I", " ", "eps"}]}], ",", "x", ",", "t"}], "]"}], 
              "-", 
              RowBox[{"gfun", "[", 
               RowBox[{
                RowBox[{"-", "1"}], ",", 
                RowBox[{"#", "-", 
                 RowBox[{"I", " ", "eps"}]}], ",", "x", ",", "t"}], "]"}]}], 
             ")"}], "+", 
            RowBox[{"Log", "[", 
             RowBox[{"\[Tau]", "[", 
              RowBox[{"#", "*", 
               RowBox[{"K0", "[", 
                RowBox[{"x", ",", "t"}], "]"}]}], "]"}], "]"}]}], "&"}], ",", 
          
          RowBox[{"Line", "[", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"a", "[", 
              RowBox[{"s", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
             RowBox[{"b", "[", 
              RowBox[{"s", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "}"}], "]"}], ",", 
          "150"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[Zeta]", "[", 
           RowBox[{"+", "1"}], "]"}], "[", 
          RowBox[{"-", "1"}], "]"}], "[", "k_", "]"}], ":=", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[Zeta]", "[", 
            RowBox[{"+", "1"}], "]"}], "[", 
           RowBox[{"-", "1"}], "]"}], "[", "k", "]"}], "=", 
         RowBox[{
          RowBox[{"Cauchy", "[", 
           RowBox[{"glim1", ",", 
            RowBox[{"k", "+", 
             RowBox[{"I", " ", "eps"}]}]}], "]"}], "//", "Exp"}]}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[Zeta]", "[", 
           RowBox[{"-", "1"}], "]"}], "[", 
          RowBox[{"-", "1"}], "]"}], "[", "k_", "]"}], ":=", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[Zeta]", "[", 
            RowBox[{"-", "1"}], "]"}], "[", 
           RowBox[{"-", "1"}], "]"}], "[", "k", "]"}], "=", 
         RowBox[{
          RowBox[{"Cauchy", "[", 
           RowBox[{"glim1", ",", 
            RowBox[{"k", "-", 
             RowBox[{"I", " ", "eps"}]}]}], "]"}], "//", "Exp"}]}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[Zeta]", "[", 
           RowBox[{"+", "1"}], "]"}], "[", 
          RowBox[{"+", "1"}], "]"}], "[", "k_", "]"}], ":=", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[Zeta]", "[", 
            RowBox[{"+", "1"}], "]"}], "[", 
           RowBox[{"+", "1"}], "]"}], "[", "k", "]"}], "=", 
         RowBox[{
          RowBox[{"Cauchy", "[", 
           RowBox[{"glim2", ",", 
            RowBox[{"k", "+", 
             RowBox[{"I", " ", "eps"}]}]}], "]"}], "//", "Exp"}]}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[Zeta]", "[", 
           RowBox[{"-", "1"}], "]"}], "[", 
          RowBox[{"+", "1"}], "]"}], "[", "k_", "]"}], ":=", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"\[Zeta]", "[", 
            RowBox[{"-", "1"}], "]"}], "[", 
           RowBox[{"+", "1"}], "]"}], "[", "k", "]"}], "=", 
         RowBox[{
          RowBox[{"Cauchy", "[", 
           RowBox[{"glim2", ",", 
            RowBox[{"k", "-", 
             RowBox[{"I", " ", "eps"}]}]}], "]"}], "//", "Exp"}]}]}], ";", 
       "\[IndentingNewLine]", "\[IndentingNewLine]", 
       RowBox[{"w", "//", "Clear"}], ";", "\[IndentingNewLine]", 
       RowBox[{"glim1w", " ", "=", " ", 
        RowBox[{"Fun", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"gfun", "[", 
              RowBox[{"1", ",", 
               RowBox[{"#", "+", 
                RowBox[{"I", " ", "eps"}]}], ",", "x", ",", "t"}], "]"}], "-", 
             RowBox[{"gfun", "[", 
              RowBox[{
               RowBox[{"-", "1"}], ",", 
               RowBox[{"#", "-", 
                RowBox[{"I", " ", "eps"}]}], ",", "x", ",", "t"}], "]"}]}], 
            ")"}], "&"}], ",", 
          RowBox[{"Line", "[", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", 
              RowBox[{"b", "[", 
               RowBox[{"s", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ",", 
             RowBox[{"-", 
              RowBox[{"a", "[", 
               RowBox[{"s", "[", 
                RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], "}"}], "]"}], ",", 
          "50"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"glim2w", " ", "=", " ", 
        RowBox[{"Fun", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"gfun", "[", 
              RowBox[{"1", ",", 
               RowBox[{"#", "+", 
                RowBox[{"I", " ", "eps"}]}], ",", "x", ",", "t"}], "]"}], "-", 
             RowBox[{"gfun", "[", 
              RowBox[{
               RowBox[{"-", "1"}], ",", 
               RowBox[{"#", "-", 
                RowBox[{"I", " ", "eps"}]}], ",", "x", ",", "t"}], "]"}]}], 
            ")"}], "&"}], ",", 
          RowBox[{"Line", "[", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"a", "[", 
              RowBox[{"s", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
             RowBox[{"b", "[", 
              RowBox[{"s", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "}"}], "]"}], ",", 
          "50"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"w", "[", 
           RowBox[{"+", "1"}], "]"}], "[", 
          RowBox[{"-", "1"}], "]"}], "[", "k_", "]"}], ":=", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"w", "[", 
            RowBox[{"+", "1"}], "]"}], "[", 
           RowBox[{"-", "1"}], "]"}], "[", "k", "]"}], "=", 
         RowBox[{
          RowBox[{"Cauchy", "[", 
           RowBox[{"glim1w", ",", 
            RowBox[{"k", "+", 
             RowBox[{"I", " ", "eps"}]}]}], "]"}], "//", "Exp"}]}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"w", "[", 
           RowBox[{"-", "1"}], "]"}], "[", 
          RowBox[{"-", "1"}], "]"}], "[", "k_", "]"}], ":=", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"w", "[", 
            RowBox[{"-", "1"}], "]"}], "[", 
           RowBox[{"-", "1"}], "]"}], "[", "k", "]"}], "=", 
         RowBox[{
          RowBox[{"Cauchy", "[", 
           RowBox[{"glim1w", ",", 
            RowBox[{"k", "-", 
             RowBox[{"I", " ", "eps"}]}]}], "]"}], "//", "Exp"}]}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"w", "[", 
           RowBox[{"+", "1"}], "]"}], "[", 
          RowBox[{"+", "1"}], "]"}], "[", "k_", "]"}], ":=", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"w", "[", 
            RowBox[{"+", "1"}], "]"}], "[", 
           RowBox[{"+", "1"}], "]"}], "[", "k", "]"}], "=", 
         RowBox[{
          RowBox[{"Cauchy", "[", 
           RowBox[{"glim2w", ",", 
            RowBox[{"k", "+", 
             RowBox[{"I", " ", "eps"}]}]}], "]"}], "//", "Exp"}]}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"w", "[", 
           RowBox[{"-", "1"}], "]"}], "[", 
          RowBox[{"+", "1"}], "]"}], "[", "k_", "]"}], ":=", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"w", "[", 
            RowBox[{"-", "1"}], "]"}], "[", 
           RowBox[{"+", "1"}], "]"}], "[", "k", "]"}], "=", 
         RowBox[{
          RowBox[{"Cauchy", "[", 
           RowBox[{"glim2w", ",", 
            RowBox[{"k", "-", 
             RowBox[{"I", " ", "eps"}]}]}], "]"}], "//", "Exp"}]}]}], ";"}]}],
      "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"The", " ", "first"}], " ", "+", 
     RowBox[{
      RowBox[{"/", 
       RowBox[{"-", " ", "indicates"}]}], " ", "limit"}]}], ",", " ", 
    RowBox[{
     RowBox[{"the", " ", "second", " ", "indicates"}], " ", "+", 
     RowBox[{"/", 
      RowBox[{"-", " ", "a"}]}]}], ",", "b"}], " ", 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Z", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"-", "1"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"1", "/", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Zeta]", "[", 
           RowBox[{"+", "1"}], "]"}], "[", 
          RowBox[{"-", "1"}], "]"}], "[", "k", "]"}]}], "0"},
      {"0", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Zeta]", "[", 
          RowBox[{"+", "1"}], "]"}], "[", 
         RowBox[{"-", "1"}], "]"}], "[", "k", "]"}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Z", "[", 
      RowBox[{"-", "1"}], "]"}], "[", 
     RowBox[{"-", "1"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"1", "/", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Zeta]", "[", 
           RowBox[{"-", "1"}], "]"}], "[", 
          RowBox[{"-", "1"}], "]"}], "[", "k", "]"}]}], "0"},
      {"0", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Zeta]", "[", 
          RowBox[{"-", "1"}], "]"}], "[", 
         RowBox[{"-", "1"}], "]"}], "[", "k", "]"}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Z", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"+", "1"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"1", "/", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Zeta]", "[", 
           RowBox[{"+", "1"}], "]"}], "[", 
          RowBox[{"+", "1"}], "]"}], "[", "k", "]"}]}], "0"},
      {"0", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Zeta]", "[", 
          RowBox[{"+", "1"}], "]"}], "[", 
         RowBox[{"+", "1"}], "]"}], "[", "k", "]"}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Z", "[", 
      RowBox[{"-", "1"}], "]"}], "[", 
     RowBox[{"+", "1"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"1", "/", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Zeta]", "[", 
           RowBox[{"-", "1"}], "]"}], "[", 
          RowBox[{"+", "1"}], "]"}], "[", "k", "]"}]}], "0"},
      {"0", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Zeta]", "[", 
          RowBox[{"-", "1"}], "]"}], "[", 
         RowBox[{"+", "1"}], "]"}], "[", "k", "]"}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"W", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"-", "1"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"1", "/", 
        RowBox[{
         RowBox[{
          RowBox[{"w", "[", 
           RowBox[{"+", "1"}], "]"}], "[", 
          RowBox[{"-", "1"}], "]"}], "[", "k", "]"}]}], "0"},
      {"0", 
       RowBox[{
        RowBox[{
         RowBox[{"w", "[", 
          RowBox[{"+", "1"}], "]"}], "[", 
         RowBox[{"-", "1"}], "]"}], "[", "k", "]"}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"W", "[", 
      RowBox[{"-", "1"}], "]"}], "[", 
     RowBox[{"-", "1"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"1", "/", 
        RowBox[{
         RowBox[{
          RowBox[{"w", "[", 
           RowBox[{"-", "1"}], "]"}], "[", 
          RowBox[{"-", "1"}], "]"}], "[", "k", "]"}]}], "0"},
      {"0", 
       RowBox[{
        RowBox[{
         RowBox[{"w", "[", 
          RowBox[{"-", "1"}], "]"}], "[", 
         RowBox[{"-", "1"}], "]"}], "[", "k", "]"}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"W", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"+", "1"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"1", "/", 
        RowBox[{
         RowBox[{
          RowBox[{"w", "[", 
           RowBox[{"+", "1"}], "]"}], "[", 
          RowBox[{"+", "1"}], "]"}], "[", "k", "]"}]}], "0"},
      {"0", 
       RowBox[{
        RowBox[{
         RowBox[{"w", "[", 
          RowBox[{"+", "1"}], "]"}], "[", 
         RowBox[{"+", "1"}], "]"}], "[", "k", "]"}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"W", "[", 
      RowBox[{"-", "1"}], "]"}], "[", 
     RowBox[{"+", "1"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"1", "/", 
        RowBox[{
         RowBox[{
          RowBox[{"w", "[", 
           RowBox[{"-", "1"}], "]"}], "[", 
          RowBox[{"+", "1"}], "]"}], "[", "k", "]"}]}], "0"},
      {"0", 
       RowBox[{
        RowBox[{
         RowBox[{"w", "[", 
          RowBox[{"-", "1"}], "]"}], "[", 
         RowBox[{"+", "1"}], "]"}], "[", "k", "]"}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Zx", "[", "_", "]"}], "[", "_", "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"0", "0"},
      {"0", "0"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Zsamp", "[", "_", "]"}], "[", "_", "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", "1"},
      {"1", "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Wsamp", "[", "_", "]"}], "[", "_", "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", "1"},
      {"1", "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Wx", "[", "_", "]"}], "[", "_", "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"0", "0"},
      {"0", "0"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"cs", "[", 
     RowBox[{"-", "1"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"\[Delta]cs", "[", 
        RowBox[{"k", ",", 
         RowBox[{"-", "1"}]}], "]"}], "0."},
      {"0", 
       RowBox[{"1", "/", 
        RowBox[{"\[Delta]cs", "[", 
         RowBox[{"k", ",", 
          RowBox[{"-", "1"}]}], "]"}]}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]cs", "[", 
     RowBox[{"+", "1"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"\[Delta]cs", "[", 
        RowBox[{"k", ",", 
         RowBox[{"+", "1"}]}], "]"}], "0"},
      {"0", 
       RowBox[{"1", "/", 
        RowBox[{"\[Delta]cs", "[", 
         RowBox[{"k", ",", 
          RowBox[{"+", "1"}]}], "]"}]}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]csx", "[", "_", "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"0", "0"},
      {"0", "0"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]csk", "[", 
     RowBox[{"-", "1"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"\[Delta]csk", "[", 
        RowBox[{"k", ",", 
         RowBox[{"-", "1"}]}], "]"}], "0."},
      {"0", 
       RowBox[{"1", "/", 
        RowBox[{"\[Delta]csk", "[", 
         RowBox[{"k", ",", 
          RowBox[{"-", "1"}]}], "]"}]}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]csk", "[", 
     RowBox[{"+", "1"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"\[Delta]csk", "[", 
        RowBox[{"k", ",", 
         RowBox[{"+", "1"}]}], "]"}], "0"},
      {"0", 
       RowBox[{"1", "/", 
        RowBox[{"\[Delta]csk", "[", 
         RowBox[{"k", ",", 
          RowBox[{"+", "1"}]}], "]"}]}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]cskx", "[", "_", "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"0", "0"},
      {"0", "0"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]k", "[", 
     RowBox[{"-", "1"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"1", "/", 
        RowBox[{"\[Delta]csk", "[", 
         RowBox[{"k", ",", 
          RowBox[{"-", "1"}]}], "]"}]}], "0."},
      {"0", 
       RowBox[{"\[Delta]csk", "[", 
        RowBox[{"k", ",", 
         RowBox[{"-", "1"}]}], "]"}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]k", "[", 
     RowBox[{"+", "1"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"1", "/", 
        RowBox[{"\[Delta]csk", "[", 
         RowBox[{"k", ",", 
          RowBox[{"+", "1"}]}], "]"}]}], "0"},
      {"0", 
       RowBox[{"\[Delta]csk", "[", 
        RowBox[{"k", ",", 
         RowBox[{"+", "1"}]}], "]"}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]kx", "[", "_", "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"0", "0"},
      {"0", "0"}
     }], ")"}]}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.568553397555667*^9, {3.568553449403825*^9, 3.56855345203559*^9}, {
   3.568553772837674*^9, 3.568553818750169*^9}, {3.568554052643714*^9, 
   3.568554063818709*^9}, {3.568554160134904*^9, 3.568554161391015*^9}, {
   3.568554205088578*^9, 3.568554211839402*^9}, {3.568554324036275*^9, 
   3.5685544847216873`*^9}, {3.568554601326907*^9, 3.568554752249818*^9}, {
   3.5685547965331593`*^9, 3.568554799270925*^9}, {3.568554846408194*^9, 
   3.5685548784821177`*^9}, 3.5685553326636887`*^9, {3.568555426372445*^9, 
   3.5685554997613087`*^9}, {3.568556654363599*^9, 3.568556699083126*^9}, {
   3.568556765603969*^9, 3.5685567777584047`*^9}, {3.568556811337154*^9, 
   3.568556864286745*^9}, {3.568556978774241*^9, 3.568557019646803*^9}, 
   3.568557128104793*^9, {3.5685604088845882`*^9, 3.568560423468215*^9}, {
   3.568570447942103*^9, 3.568570448377982*^9}, {3.568571181392728*^9, 
   3.568571181747373*^9}, {3.568571293253327*^9, 3.568571303543618*^9}, {
   3.56857147781658*^9, 3.568571477969314*^9}, {3.568571514211359*^9, 
   3.568571561384326*^9}, {3.568571613423409*^9, 3.568571655406646*^9}, {
   3.568572572274542*^9, 3.5685725725656767`*^9}, {3.568572658513542*^9, 
   3.568572682109009*^9}, {3.568582581804689*^9, 3.568582615112805*^9}, {
   3.568582674766816*^9, 3.568582695001357*^9}, {3.568582753099119*^9, 
   3.568582759883728*^9}, {3.568582799435132*^9, 3.568582803333802*^9}, {
   3.568582849075985*^9, 3.5685828529332533`*^9}, {3.568583931615652*^9, 
   3.568583974673399*^9}, {3.568586084483808*^9, 3.568586101398761*^9}, {
   3.5685861635110493`*^9, 3.568586163844821*^9}, {3.5685862667541018`*^9, 
   3.568586298191765*^9}, {3.5685865941324987`*^9, 3.5685865944450197`*^9}, {
   3.568586677572322*^9, 3.56858667999261*^9}, {3.568586737734747*^9, 
   3.5685867733607407`*^9}, 3.568586808863079*^9, {3.56858684908012*^9, 
   3.5685868624731903`*^9}, {3.568640824211752*^9, 3.568640854163864*^9}, {
   3.568640979332553*^9, 3.568640985539529*^9}, {3.568641017400319*^9, 
   3.5686410259544287`*^9}, {3.568641222057602*^9, 3.5686412336521397`*^9}, 
   3.568641413414571*^9, 3.5686417711905413`*^9, {3.568641840680253*^9, 
   3.568641875344013*^9}, {3.568641990273305*^9, 3.5686419904565363`*^9}, {
   3.568643136264125*^9, 3.5686431519017763`*^9}, {3.568643881811178*^9, 
   3.5686438870211983`*^9}, {3.568643924035589*^9, 3.5686439482528133`*^9}, {
   3.5686440026816397`*^9, 3.568644102693231*^9}, {3.568644143548386*^9, 
   3.5686441523629923`*^9}, {3.568645147672111*^9, 3.568645174432767*^9}, {
   3.568645259281336*^9, 3.568645261461721*^9}, {3.568649015985878*^9, 
   3.56864902760425*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Multiplied jumps", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.568573155255061*^9, 3.568573158829356*^9}}],

Cell[CellGroupData[{

Cell["Normal jumps", "Subsubsection",
 CellChangeTimes->{{3.568643674322945*^9, 3.5686436761966553`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalDelta]ki\[Phi]QinsPsQs\[Phi]\[CapitalDelta]k", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"+", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Ps", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"+", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalDelta]ki\[Phi]QinsPinsQs\[Phi]\[CapitalDelta]k", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"+", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Ps", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"+", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalDelta]ki\[Phi]QinsUinsQs\[Phi]\[CapitalDelta]k", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"+", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Us", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"+", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]k", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"-", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Ms", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"-", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalDelta]ki\[Phi]QinsGsQs\[Phi]\[CapitalDelta]k", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"-", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Gs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"+", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]k", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"-", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Ls", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"-", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsUsQs\[Phi]\[CapitalDelta]k", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{
     RowBox[{"MultFun", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"InvFun", "[", 
         RowBox[{"\[CapitalDelta]k", "[", 
          RowBox[{"+", "1"}], "]"}], "]"}], ",", 
        RowBox[{"InvFun", "[", 
         RowBox[{
          RowBox[{"\[Phi]", "[", 
           RowBox[{"+", "1"}], "]"}], "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
        RowBox[{"InvFun", "[", 
         RowBox[{"Qs", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
        RowBox[{"Us", "[", 
         RowBox[{"x", ",", "t"}], "]"}], ",", 
        RowBox[{"Qs", "[", 
         RowBox[{"x", ",", "t"}], "]"}], ",", 
        RowBox[{
         RowBox[{"\[Phi]", "[", 
          RowBox[{"+", "1"}], "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], ",", 
        RowBox[{"\[CapitalDelta]k", "[", 
         RowBox[{"+", "1"}], "]"}]}], "}"}], "]"}], "[", "k", "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]ki\[Phi]QinsPinsQs\[Phi]\[CapitalDelta]k", "[", 
     RowBox[{"+", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"+", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Ps", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"+", "1"}], "]"}], ",", 
      RowBox[{
       RowBox[{"W", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"+", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]ki\[Phi]QinsUinsQs\[Phi]\[CapitalDelta]k", "[", 
     RowBox[{"+", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"+", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Us", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"+", "1"}], "]"}], ",", 
      RowBox[{
       RowBox[{"Z", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"+", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]k", "[", 
     RowBox[{"+", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"-", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Ms", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"-", "1"}], "]"}], ",", 
      RowBox[{
       RowBox[{"W", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"+", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]k", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"MultFun", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"InvFun", "[", 
        RowBox[{"\[CapitalDelta]k", "[", 
         RowBox[{"-", "1"}], "]"}], "]"}], ",", 
       RowBox[{"InvFun", "[", 
        RowBox[{
         RowBox[{"\[Phi]", "[", 
          RowBox[{"-", "1"}], "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"InvFun", "[", 
        RowBox[{"Qs", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"Ls", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"-", "1"}], "]"}], ",", 
       RowBox[{
        RowBox[{"Z", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"+", "1"}], "]"}]}], "}"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]ki\[Phi]QinsPinsQs\[Phi]\[CapitalDelta]k", "[", 
     RowBox[{"-", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"+", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Ps", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"+", "1"}], "]"}], ",", 
      RowBox[{
       RowBox[{"W", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"-", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]ki\[Phi]QinsUinsQs\[Phi]\[CapitalDelta]k", "[", 
     RowBox[{"-", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"+", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Us", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"+", "1"}], "]"}], ",", 
      RowBox[{
       RowBox[{"Z", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"-", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]k", "[", 
     RowBox[{"-", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"-", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Ms", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"-", "1"}], "]"}], ",", 
      RowBox[{
       RowBox[{"W", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"-", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]k", "[", 
      RowBox[{"-", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"MultFun", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"InvFun", "[", 
        RowBox[{"\[CapitalDelta]k", "[", 
         RowBox[{"-", "1"}], "]"}], "]"}], ",", 
       RowBox[{"InvFun", "[", 
        RowBox[{
         RowBox[{"\[Phi]", "[", 
          RowBox[{"-", "1"}], "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"InvFun", "[", 
        RowBox[{"Qs", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"Ls", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"-", "1"}], "]"}], ",", 
       RowBox[{
        RowBox[{"Z", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"-", "1"}], "]"}]}], "}"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]ki\[Phi]QinsStQs\[Phi]\[CapitalDelta]k", "[", 
     RowBox[{"+", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"+", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"St", "[", 
       RowBox[{"+", "1"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"+", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]ki\[Phi]QinsStQs\[Phi]\[CapitalDelta]k", "[", 
     RowBox[{"-", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"-", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"St", "[", 
       RowBox[{"-", "1"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"-", "1"}], "]"}]}], "}"}], "]"}]}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.513711472307995*^9, 3.51371150688424*^9}, {
   3.513711559410595*^9, 3.513711606964553*^9}, {3.513711648397517*^9, 
   3.513711936409362*^9}, {3.5139442600508213`*^9, 3.513944356354691*^9}, {
   3.513944486092828*^9, 3.5139446411607943`*^9}, {3.513951549753932*^9, 
   3.513951699728924*^9}, {3.5139517397524*^9, 3.5139518129172707`*^9}, {
   3.513952232831436*^9, 3.513952249103897*^9}, {3.513953371131137*^9, 
   3.513953378053*^9}, {3.513953475209831*^9, 3.5139534934360037`*^9}, {
   3.513953588050322*^9, 3.513953588555484*^9}, {3.513953862581882*^9, 
   3.513953869531641*^9}, {3.513953913176754*^9, 3.51395391839988*^9}, {
   3.513954124364752*^9, 3.513954124771585*^9}, {3.513963109152478*^9, 
   3.513963188824473*^9}, {3.513963351166205*^9, 3.513963487001848*^9}, {
   3.513963527828042*^9, 3.51396353762601*^9}, {3.5139635929170723`*^9, 
   3.5139636917580643`*^9}, {3.5139637974682493`*^9, 3.513963802783566*^9}, {
   3.513963881140173*^9, 3.513963913320346*^9}, {3.513964002763595*^9, 
   3.5139640804451647`*^9}, {3.5139641459974127`*^9, 
   3.5139642689184113`*^9}, {3.513964342352106*^9, 3.513964382632668*^9}, {
   3.5139644334878397`*^9, 3.513964445439907*^9}, {3.513965136438246*^9, 
   3.513965142087782*^9}, {3.513968925722913*^9, 3.51396897056528*^9}, {
   3.51482264314016*^9, 3.514822674472425*^9}, {3.514823299014048*^9, 
   3.514823433520165*^9}, {3.5148259836929407`*^9, 3.514826021246005*^9}, {
   3.5148260568909473`*^9, 3.514826093709059*^9}, {3.514829266130872*^9, 
   3.514829267269014*^9}, 3.514907738691799*^9, 3.514909645865958*^9, {
   3.514909725826643*^9, 3.514909749867633*^9}, {3.514909835600062*^9, 
   3.514909849756949*^9}, {3.514909927551237*^9, 3.514909958608143*^9}, {
   3.51490999142306*^9, 3.514910014969368*^9}, {3.514910105977624*^9, 
   3.514910107150621*^9}, {3.514910138545768*^9, 3.514910187921351*^9}, {
   3.514910242162261*^9, 3.514910545155949*^9}, {3.514910580487647*^9, 
   3.514910592845335*^9}, {3.514910650014436*^9, 3.514910654005959*^9}, 
   3.51491115722192*^9, {3.51491234211771*^9, 3.514912347366127*^9}, {
   3.514913668250718*^9, 3.514913684476367*^9}, {3.51491537864764*^9, 
   3.5149154086481857`*^9}, {3.514916004664308*^9, 3.514916111631596*^9}, {
   3.514916343684166*^9, 3.514916350939144*^9}, {3.514919628933803*^9, 
   3.514919629374105*^9}, {3.514919814051456*^9, 3.51491981431933*^9}, {
   3.5149210615292788`*^9, 3.5149210618093157`*^9}, {3.514921376696*^9, 
   3.514921409459235*^9}, {3.514921550975163*^9, 3.514921553118926*^9}, {
   3.5149216002292137`*^9, 3.514921601651573*^9}, {3.514921714355473*^9, 
   3.5149217145617943`*^9}, {3.514921849323975*^9, 3.51492184966441*^9}, {
   3.515014018484*^9, 3.5150140807939997`*^9}, {3.515014414274*^9, 
   3.515014419104*^9}, {3.515014499744*^9, 3.515014556984*^9}, {
   3.516031623738676*^9, 3.51603164993892*^9}, {3.5163613960938*^9, 
   3.5163614987928*^9}, {3.5163621515938*^9, 3.5163622450538*^9}, {
   3.5163623032008*^9, 3.5163623226138*^9}, {3.5163624113227997`*^9, 
   3.5163624414667997`*^9}, {3.5163625423188*^9, 3.5163625712748003`*^9}, {
   3.5163626344598*^9, 3.5163626494598*^9}, {3.5163626897988*^9, 
   3.5163627142928*^9}, {3.5163629553128*^9, 3.5163629783718*^9}, {
   3.5163630349268*^9, 3.5163630547258*^9}, {3.5163630874538*^9, 
   3.5163631165657997`*^9}, {3.5163631505308*^9, 3.5163631901438*^9}, {
   3.5163633288018*^9, 3.5163633291317997`*^9}, {3.5163633932418003`*^9, 
   3.5163634088087997`*^9}, {3.516369647737257*^9, 3.516369666853775*^9}, {
   3.516371046201543*^9, 3.516371081347692*^9}, {3.516371202900103*^9, 
   3.516371207766354*^9}, {3.51637143189148*^9, 3.516371468582984*^9}, {
   3.516371641584278*^9, 3.5163716420861*^9}, 3.516371673320703*^9, {
   3.516371728651638*^9, 3.516371729847763*^9}, {3.516382638594038*^9, 
   3.51638286034973*^9}, {3.516383039819622*^9, 3.516383111119877*^9}, {
   3.516383169368149*^9, 3.516383194658242*^9}, {3.5163838852752867`*^9, 
   3.516383951945406*^9}, {3.5163850545636597`*^9, 3.516385067352995*^9}, {
   3.51638526865587*^9, 3.51638530022193*^9}, {3.5163853567862177`*^9, 
   3.51638542827841*^9}, {3.5163859079277563`*^9, 3.516385912132997*^9}, {
   3.516386199386607*^9, 3.516386213286777*^9}, {3.516386388653233*^9, 
   3.5163863996118517`*^9}, {3.5163867067406063`*^9, 3.516386715759712*^9}, 
   3.516396963584313*^9, {3.516397541951666*^9, 3.5163975535653048`*^9}, 
   3.516457120472746*^9, {3.516457151655912*^9, 3.516457235975576*^9}, {
   3.516457284491362*^9, 3.5164573214514093`*^9}, {3.516457387922867*^9, 
   3.516457454287363*^9}, {3.516457625199349*^9, 3.516457672991251*^9}, {
   3.51645831600738*^9, 3.5164583295493793`*^9}, 3.516458360081263*^9, {
   3.516458779434746*^9, 3.516458802316041*^9}, {3.516459394550309*^9, 
   3.5164595175171347`*^9}, {3.516459604960169*^9, 3.516459625340931*^9}, {
   3.5164596662234*^9, 3.516459666637341*^9}, {3.516459998653577*^9, 
   3.516460006124365*^9}, {3.516460354942031*^9, 3.516460396324935*^9}, {
   3.51646053069876*^9, 3.516460614685628*^9}, {3.5164607527937717`*^9, 
   3.516460897803651*^9}, {3.516462501534315*^9, 3.516462526604961*^9}, {
   3.516462611112651*^9, 3.516462611383815*^9}, {3.516463021154703*^9, 
   3.516463061529895*^9}, {3.5164633717215548`*^9, 3.5164633976508627`*^9}, {
   3.516463429116269*^9, 3.516463481201078*^9}, {3.5164638398603697`*^9, 
   3.5164638593406963`*^9}, {3.516464312638464*^9, 3.516464312912916*^9}, {
   3.516464914674979*^9, 3.516465085200768*^9}, {3.516465337676579*^9, 
   3.516465372496331*^9}, {3.516465405274722*^9, 3.516465406270146*^9}, {
   3.516465552779982*^9, 3.516465588691825*^9}, {3.5164672510061007`*^9, 
   3.516467293942857*^9}, {3.516467474603541*^9, 3.5164675231703978`*^9}, {
   3.51646760024851*^9, 3.516467739960421*^9}, {3.5164677717917137`*^9, 
   3.516467988712447*^9}, {3.516469159296556*^9, 3.516469161955163*^9}, {
   3.5164698074427032`*^9, 3.516469904905406*^9}, {3.5164707554820213`*^9, 
   3.516470756292307*^9}, {3.516471149654396*^9, 3.516471249685629*^9}, {
   3.516471405122237*^9, 3.516471412333644*^9}, {3.516472156919051*^9, 
   3.51647219331638*^9}, {3.516472854954361*^9, 3.5164729609522753`*^9}, {
   3.568572773889769*^9, 3.5685728183678083`*^9}, {3.568573181918784*^9, 
   3.5685732296762877`*^9}, {3.568573342255867*^9, 3.568573749688448*^9}, {
   3.568573787742181*^9, 3.568573813907329*^9}, {3.568573850661203*^9, 
   3.568573919559573*^9}, {3.5685739707975807`*^9, 3.5685739999250727`*^9}, {
   3.568574594174342*^9, 3.568574676348609*^9}, {3.568574730907544*^9, 
   3.568574813674015*^9}, {3.568575547486279*^9, 3.568575589757934*^9}, {
   3.5685834303597527`*^9, 3.568583436207259*^9}, {3.568584066859976*^9, 
   3.5685841512933702`*^9}, {3.568584762036027*^9, 3.568584762654573*^9}, {
   3.5685848204813633`*^9, 3.568584839282544*^9}, 3.568643683007687*^9}]
}, Closed]],

Cell[CellGroupData[{

Cell["Sample jumps", "Subsubsection",
 CellChangeTimes->{{3.568643674322945*^9, 3.5686436761966553`*^9}, {
  3.5686437676644373`*^9, 3.568643768960115*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalDelta]ki\[Phi]QinsPsQs\[Phi]\[CapitalDelta]ksamp", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Psamps", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]samp", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalDelta]ki\[Phi]QinsPinsQs\[Phi]\[CapitalDelta]ksamp", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Psamps", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]samp", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalDelta]ki\[Phi]QinsUinsQs\[Phi]\[CapitalDelta]ksamp", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Usamps", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]samp", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]ksamp", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Msamps", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]samp", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalDelta]ki\[Phi]QinsGsQs\[Phi]\[CapitalDelta]ksamp", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Gsamps", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]samp", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]ksamp", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Lsamps", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]samp", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsUsQs\[Phi]\[CapitalDelta]ksamp", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{
     RowBox[{"MultFun", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"InvFun", "[", 
         RowBox[{
          RowBox[{"\[Phi]samp", "[", 
           RowBox[{"+", "1"}], "]"}], "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
        RowBox[{"Usamps", "[", 
         RowBox[{"x", ",", "t"}], "]"}], ",", 
        RowBox[{
         RowBox[{"\[Phi]samp", "[", 
          RowBox[{"+", "1"}], "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}], "[", "k", "]"}]}], 
   ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
    "\[CapitalDelta]ki\[Phi]QinsPinsQs\[Phi]\[CapitalDelta]ksamp", "[", 
     RowBox[{"+", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Psamps", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]samp", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
    "\[CapitalDelta]ki\[Phi]QinsUinsQs\[Phi]\[CapitalDelta]ksamp", "[", 
     RowBox[{"+", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Usamps", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]samp", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
    "\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]ksamp", "[", 
     RowBox[{"+", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Msamps", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]samp", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]ksamp", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"MultFun", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"InvFun", "[", 
        RowBox[{
         RowBox[{"\[Phi]samp", "[", 
          RowBox[{"-", "1"}], "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"Lsamps", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
    "\[CapitalDelta]ki\[Phi]QinsPinsQs\[Phi]\[CapitalDelta]ksamp", "[", 
     RowBox[{"-", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Psamps", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]samp", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
    "\[CapitalDelta]ki\[Phi]QinsUinsQs\[Phi]\[CapitalDelta]ksamp", "[", 
     RowBox[{"-", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Usamps", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]samp", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
    "\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]ksamp", "[", 
     RowBox[{"-", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Msamps", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]samp", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
    "\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]ksamp", "[", 
     RowBox[{"-", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Lsamps", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]samp", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.513711472307995*^9, 3.51371150688424*^9}, {
   3.513711559410595*^9, 3.513711606964553*^9}, {3.513711648397517*^9, 
   3.513711936409362*^9}, {3.5139442600508213`*^9, 3.513944356354691*^9}, {
   3.513944486092828*^9, 3.5139446411607943`*^9}, {3.513951549753932*^9, 
   3.513951699728924*^9}, {3.5139517397524*^9, 3.5139518129172707`*^9}, {
   3.513952232831436*^9, 3.513952249103897*^9}, {3.513953371131137*^9, 
   3.513953378053*^9}, {3.513953475209831*^9, 3.5139534934360037`*^9}, {
   3.513953588050322*^9, 3.513953588555484*^9}, {3.513953862581882*^9, 
   3.513953869531641*^9}, {3.513953913176754*^9, 3.51395391839988*^9}, {
   3.513954124364752*^9, 3.513954124771585*^9}, {3.513963109152478*^9, 
   3.513963188824473*^9}, {3.513963351166205*^9, 3.513963487001848*^9}, {
   3.513963527828042*^9, 3.51396353762601*^9}, {3.5139635929170723`*^9, 
   3.5139636917580643`*^9}, {3.5139637974682493`*^9, 3.513963802783566*^9}, {
   3.513963881140173*^9, 3.513963913320346*^9}, {3.513964002763595*^9, 
   3.5139640804451647`*^9}, {3.5139641459974127`*^9, 
   3.5139642689184113`*^9}, {3.513964342352106*^9, 3.513964382632668*^9}, {
   3.5139644334878397`*^9, 3.513964445439907*^9}, {3.513965136438246*^9, 
   3.513965142087782*^9}, {3.513968925722913*^9, 3.51396897056528*^9}, {
   3.51482264314016*^9, 3.514822674472425*^9}, {3.514823299014048*^9, 
   3.514823433520165*^9}, {3.5148259836929407`*^9, 3.514826021246005*^9}, {
   3.5148260568909473`*^9, 3.514826093709059*^9}, {3.514829266130872*^9, 
   3.514829267269014*^9}, 3.514907738691799*^9, 3.514909645865958*^9, {
   3.514909725826643*^9, 3.514909749867633*^9}, {3.514909835600062*^9, 
   3.514909849756949*^9}, {3.514909927551237*^9, 3.514909958608143*^9}, {
   3.51490999142306*^9, 3.514910014969368*^9}, {3.514910105977624*^9, 
   3.514910107150621*^9}, {3.514910138545768*^9, 3.514910187921351*^9}, {
   3.514910242162261*^9, 3.514910545155949*^9}, {3.514910580487647*^9, 
   3.514910592845335*^9}, {3.514910650014436*^9, 3.514910654005959*^9}, 
   3.51491115722192*^9, {3.51491234211771*^9, 3.514912347366127*^9}, {
   3.514913668250718*^9, 3.514913684476367*^9}, {3.51491537864764*^9, 
   3.5149154086481857`*^9}, {3.514916004664308*^9, 3.514916111631596*^9}, {
   3.514916343684166*^9, 3.514916350939144*^9}, {3.514919628933803*^9, 
   3.514919629374105*^9}, {3.514919814051456*^9, 3.51491981431933*^9}, {
   3.5149210615292788`*^9, 3.5149210618093157`*^9}, {3.514921376696*^9, 
   3.514921409459235*^9}, {3.514921550975163*^9, 3.514921553118926*^9}, {
   3.5149216002292137`*^9, 3.514921601651573*^9}, {3.514921714355473*^9, 
   3.5149217145617943`*^9}, {3.514921849323975*^9, 3.51492184966441*^9}, {
   3.515014018484*^9, 3.5150140807939997`*^9}, {3.515014414274*^9, 
   3.515014419104*^9}, {3.515014499744*^9, 3.515014556984*^9}, {
   3.516031623738676*^9, 3.51603164993892*^9}, {3.5163613960938*^9, 
   3.5163614987928*^9}, {3.5163621515938*^9, 3.5163622450538*^9}, {
   3.5163623032008*^9, 3.5163623226138*^9}, {3.5163624113227997`*^9, 
   3.5163624414667997`*^9}, {3.5163625423188*^9, 3.5163625712748003`*^9}, {
   3.5163626344598*^9, 3.5163626494598*^9}, {3.5163626897988*^9, 
   3.5163627142928*^9}, {3.5163629553128*^9, 3.5163629783718*^9}, {
   3.5163630349268*^9, 3.5163630547258*^9}, {3.5163630874538*^9, 
   3.5163631165657997`*^9}, {3.5163631505308*^9, 3.5163631901438*^9}, {
   3.5163633288018*^9, 3.5163633291317997`*^9}, {3.5163633932418003`*^9, 
   3.5163634088087997`*^9}, {3.516369647737257*^9, 3.516369666853775*^9}, {
   3.516371046201543*^9, 3.516371081347692*^9}, {3.516371202900103*^9, 
   3.516371207766354*^9}, {3.51637143189148*^9, 3.516371468582984*^9}, {
   3.516371641584278*^9, 3.5163716420861*^9}, 3.516371673320703*^9, {
   3.516371728651638*^9, 3.516371729847763*^9}, {3.516382638594038*^9, 
   3.51638286034973*^9}, {3.516383039819622*^9, 3.516383111119877*^9}, {
   3.516383169368149*^9, 3.516383194658242*^9}, {3.5163838852752867`*^9, 
   3.516383951945406*^9}, {3.5163850545636597`*^9, 3.516385067352995*^9}, {
   3.51638526865587*^9, 3.51638530022193*^9}, {3.5163853567862177`*^9, 
   3.51638542827841*^9}, {3.5163859079277563`*^9, 3.516385912132997*^9}, {
   3.516386199386607*^9, 3.516386213286777*^9}, {3.516386388653233*^9, 
   3.5163863996118517`*^9}, {3.5163867067406063`*^9, 3.516386715759712*^9}, 
   3.516396963584313*^9, {3.516397541951666*^9, 3.5163975535653048`*^9}, 
   3.516457120472746*^9, {3.516457151655912*^9, 3.516457235975576*^9}, {
   3.516457284491362*^9, 3.5164573214514093`*^9}, {3.516457387922867*^9, 
   3.516457454287363*^9}, {3.516457625199349*^9, 3.516457672991251*^9}, {
   3.51645831600738*^9, 3.5164583295493793`*^9}, 3.516458360081263*^9, {
   3.516458779434746*^9, 3.516458802316041*^9}, {3.516459394550309*^9, 
   3.5164595175171347`*^9}, {3.516459604960169*^9, 3.516459625340931*^9}, {
   3.5164596662234*^9, 3.516459666637341*^9}, {3.516459998653577*^9, 
   3.516460006124365*^9}, {3.516460354942031*^9, 3.516460396324935*^9}, {
   3.51646053069876*^9, 3.516460614685628*^9}, {3.5164607527937717`*^9, 
   3.516460897803651*^9}, {3.516462501534315*^9, 3.516462526604961*^9}, {
   3.516462611112651*^9, 3.516462611383815*^9}, {3.516463021154703*^9, 
   3.516463061529895*^9}, {3.5164633717215548`*^9, 3.5164633976508627`*^9}, {
   3.516463429116269*^9, 3.516463481201078*^9}, {3.5164638398603697`*^9, 
   3.5164638593406963`*^9}, {3.516464312638464*^9, 3.516464312912916*^9}, {
   3.516464914674979*^9, 3.516465085200768*^9}, {3.516465337676579*^9, 
   3.516465372496331*^9}, {3.516465405274722*^9, 3.516465406270146*^9}, {
   3.516465552779982*^9, 3.516465588691825*^9}, {3.5164672510061007`*^9, 
   3.516467293942857*^9}, {3.516467474603541*^9, 3.5164675231703978`*^9}, {
   3.51646760024851*^9, 3.516467739960421*^9}, {3.5164677717917137`*^9, 
   3.516467988712447*^9}, {3.516469159296556*^9, 3.516469161955163*^9}, {
   3.5164698074427032`*^9, 3.516469904905406*^9}, {3.5164707554820213`*^9, 
   3.516470756292307*^9}, {3.516471149654396*^9, 3.516471249685629*^9}, {
   3.516471405122237*^9, 3.516471412333644*^9}, {3.516472156919051*^9, 
   3.51647219331638*^9}, {3.516472854954361*^9, 3.5164729609522753`*^9}, {
   3.568572773889769*^9, 3.5685728183678083`*^9}, {3.568573181918784*^9, 
   3.5685732296762877`*^9}, {3.568573342255867*^9, 3.568573749688448*^9}, {
   3.568573787742181*^9, 3.568573813907329*^9}, {3.568573850661203*^9, 
   3.568573919559573*^9}, {3.5685739707975807`*^9, 3.5685739999250727`*^9}, {
   3.568574594174342*^9, 3.568574676348609*^9}, {3.568574730907544*^9, 
   3.568574813674015*^9}, {3.568575547486279*^9, 3.568575589757934*^9}, {
   3.5685834303597527`*^9, 3.568583436207259*^9}, {3.568584066859976*^9, 
   3.5685841512933702`*^9}, {3.568584762036027*^9, 3.568584762654573*^9}, {
   3.5685848204813633`*^9, 3.568584839282544*^9}, 3.568643683007687*^9, {
   3.5686437713675117`*^9, 3.568643810610887*^9}, {3.56864416746951*^9, 
   3.568644179740102*^9}, {3.5686444164253883`*^9, 3.568644714234807*^9}, {
   3.568644799855075*^9, 3.5686449535855503`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Derivatives of jumps", "Subsubsection",
 CellChangeTimes->{{3.568643687267159*^9, 3.568643692073745*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalDelta]ki\[Phi]QinsPsQs\[Phi]\[CapitalDelta]kx", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"+", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Psx", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"+", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalDelta]ki\[Phi]QinsPinsQs\[Phi]\[CapitalDelta]kx", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"+", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Pinsx", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"+", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalDelta]ki\[Phi]QinsUinsQs\[Phi]\[CapitalDelta]kx", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"+", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Uinsx", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"+", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]kx", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"-", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Msx", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"-", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalDelta]ki\[Phi]QinsGsQs\[Phi]\[CapitalDelta]kx", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"-", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Gsx", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"+", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]kx", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"-", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Lsx", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"-", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]ki\[Phi]QinsUsQs\[Phi]\[CapitalDelta]kx", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"MultFun", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"InvFun", "[", 
        RowBox[{"\[CapitalDelta]k", "[", 
         RowBox[{"+", "1"}], "]"}], "]"}], ",", 
       RowBox[{"InvFun", "[", 
        RowBox[{
         RowBox[{"\[Phi]", "[", 
          RowBox[{"+", "1"}], "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"InvFun", "[", 
        RowBox[{"Qs", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"Usx", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"+", "1"}], "]"}]}], "}"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]ki\[Phi]QinsPinsQs\[Phi]\[CapitalDelta]kx", "[", 
     RowBox[{"+", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"+", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Pinsx", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"+", "1"}], "]"}], ",", 
      RowBox[{
       RowBox[{"W", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"+", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]ki\[Phi]QinsUinsQs\[Phi]\[CapitalDelta]kx", "[", 
     RowBox[{"+", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"+", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Uinsx", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"+", "1"}], "]"}], ",", 
      RowBox[{
       RowBox[{"Z", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"+", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]kx", "[", 
     RowBox[{"+", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"-", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Msx", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"-", "1"}], "]"}], ",", 
      RowBox[{
       RowBox[{"W", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"+", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]kx", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"MultFun", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"InvFun", "[", 
        RowBox[{"\[CapitalDelta]k", "[", 
         RowBox[{"-", "1"}], "]"}], "]"}], ",", 
       RowBox[{"InvFun", "[", 
        RowBox[{
         RowBox[{"\[Phi]", "[", 
          RowBox[{"-", "1"}], "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"InvFun", "[", 
        RowBox[{"Qs", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"Lsx", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"-", "1"}], "]"}], ",", 
       RowBox[{
        RowBox[{"Z", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"+", "1"}], "]"}]}], "}"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]ki\[Phi]QinsPinsQs\[Phi]\[CapitalDelta]kx", "[", 
     RowBox[{"-", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"+", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Pinsx", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"+", "1"}], "]"}], ",", 
      RowBox[{
       RowBox[{"W", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"-", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]ki\[Phi]QinsUinsQs\[Phi]\[CapitalDelta]kx", "[", 
     RowBox[{"-", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"+", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Uinsx", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"+", "1"}], "]"}], ",", 
      RowBox[{
       RowBox[{"Z", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"-", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]kx", "[", 
     RowBox[{"-", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"-", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Msx", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"-", "1"}], "]"}], ",", 
      RowBox[{
       RowBox[{"W", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"-", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]kx", "[", 
     RowBox[{"-", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{"\[CapitalDelta]k", "[", 
        RowBox[{"-", "1"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Lsx", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"\[CapitalDelta]k", "[", 
       RowBox[{"-", "1"}], "]"}], ",", 
      RowBox[{
       RowBox[{"Z", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"-", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]"}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.513711472307995*^9, 3.51371150688424*^9}, {
   3.513711559410595*^9, 3.513711606964553*^9}, {3.513711648397517*^9, 
   3.513711936409362*^9}, {3.5139442600508213`*^9, 3.513944356354691*^9}, {
   3.513944486092828*^9, 3.5139446411607943`*^9}, {3.513951549753932*^9, 
   3.513951699728924*^9}, {3.5139517397524*^9, 3.5139518129172707`*^9}, {
   3.513952232831436*^9, 3.513952249103897*^9}, {3.513953371131137*^9, 
   3.513953378053*^9}, {3.513953475209831*^9, 3.5139534934360037`*^9}, {
   3.513953588050322*^9, 3.513953588555484*^9}, {3.513953862581882*^9, 
   3.513953869531641*^9}, {3.513953913176754*^9, 3.51395391839988*^9}, {
   3.513954124364752*^9, 3.513954124771585*^9}, {3.513963109152478*^9, 
   3.513963188824473*^9}, {3.513963351166205*^9, 3.513963487001848*^9}, {
   3.513963527828042*^9, 3.51396353762601*^9}, {3.5139635929170723`*^9, 
   3.5139636917580643`*^9}, {3.5139637974682493`*^9, 3.513963802783566*^9}, {
   3.513963881140173*^9, 3.513963913320346*^9}, {3.513964002763595*^9, 
   3.5139640804451647`*^9}, {3.5139641459974127`*^9, 
   3.5139642689184113`*^9}, {3.513964342352106*^9, 3.513964382632668*^9}, {
   3.5139644334878397`*^9, 3.513964445439907*^9}, {3.513965136438246*^9, 
   3.513965142087782*^9}, {3.513968925722913*^9, 3.51396897056528*^9}, {
   3.51482264314016*^9, 3.514822674472425*^9}, {3.514823299014048*^9, 
   3.514823433520165*^9}, {3.5148259836929407`*^9, 3.514826021246005*^9}, {
   3.5148260568909473`*^9, 3.514826093709059*^9}, {3.514829266130872*^9, 
   3.514829267269014*^9}, 3.514907738691799*^9, 3.514909645865958*^9, {
   3.514909725826643*^9, 3.514909749867633*^9}, {3.514909835600062*^9, 
   3.514909849756949*^9}, {3.514909927551237*^9, 3.514909958608143*^9}, {
   3.51490999142306*^9, 3.514910014969368*^9}, {3.514910105977624*^9, 
   3.514910107150621*^9}, {3.514910138545768*^9, 3.514910187921351*^9}, {
   3.514910242162261*^9, 3.514910545155949*^9}, {3.514910580487647*^9, 
   3.514910592845335*^9}, {3.514910650014436*^9, 3.514910654005959*^9}, 
   3.51491115722192*^9, {3.51491234211771*^9, 3.514912347366127*^9}, {
   3.514913668250718*^9, 3.514913684476367*^9}, {3.51491537864764*^9, 
   3.5149154086481857`*^9}, {3.514916004664308*^9, 3.514916111631596*^9}, {
   3.514916343684166*^9, 3.514916350939144*^9}, {3.514919628933803*^9, 
   3.514919629374105*^9}, {3.514919814051456*^9, 3.51491981431933*^9}, {
   3.5149210615292788`*^9, 3.5149210618093157`*^9}, {3.514921376696*^9, 
   3.514921409459235*^9}, {3.514921550975163*^9, 3.514921553118926*^9}, {
   3.5149216002292137`*^9, 3.514921601651573*^9}, {3.514921714355473*^9, 
   3.5149217145617943`*^9}, {3.514921849323975*^9, 3.51492184966441*^9}, {
   3.515014018484*^9, 3.5150140807939997`*^9}, {3.515014414274*^9, 
   3.515014419104*^9}, {3.515014499744*^9, 3.515014556984*^9}, {
   3.516031623738676*^9, 3.51603164993892*^9}, {3.5163613960938*^9, 
   3.5163614987928*^9}, {3.5163621515938*^9, 3.5163622450538*^9}, {
   3.5163623032008*^9, 3.5163623226138*^9}, {3.5163624113227997`*^9, 
   3.5163624414667997`*^9}, {3.5163625423188*^9, 3.5163625712748003`*^9}, {
   3.5163626344598*^9, 3.5163626494598*^9}, {3.5163626897988*^9, 
   3.5163627142928*^9}, {3.5163629553128*^9, 3.5163629783718*^9}, {
   3.5163630349268*^9, 3.5163630547258*^9}, {3.5163630874538*^9, 
   3.5163631165657997`*^9}, {3.5163631505308*^9, 3.5163631901438*^9}, {
   3.5163633288018*^9, 3.5163633291317997`*^9}, {3.5163633932418003`*^9, 
   3.5163634088087997`*^9}, {3.516369647737257*^9, 3.516369666853775*^9}, {
   3.516371046201543*^9, 3.516371081347692*^9}, {3.516371202900103*^9, 
   3.516371207766354*^9}, {3.51637143189148*^9, 3.516371468582984*^9}, {
   3.516371641584278*^9, 3.5163716420861*^9}, 3.516371673320703*^9, {
   3.516371728651638*^9, 3.516371729847763*^9}, {3.516382638594038*^9, 
   3.51638286034973*^9}, {3.516383039819622*^9, 3.516383111119877*^9}, {
   3.516383169368149*^9, 3.516383194658242*^9}, {3.5163838852752867`*^9, 
   3.516383951945406*^9}, {3.5163850545636597`*^9, 3.516385067352995*^9}, {
   3.51638526865587*^9, 3.51638530022193*^9}, {3.5163853567862177`*^9, 
   3.51638542827841*^9}, {3.5163859079277563`*^9, 3.516385912132997*^9}, {
   3.516386199386607*^9, 3.516386213286777*^9}, {3.516386388653233*^9, 
   3.5163863996118517`*^9}, {3.5163867067406063`*^9, 3.516386715759712*^9}, 
   3.516396963584313*^9, {3.516397541951666*^9, 3.5163975535653048`*^9}, 
   3.516457120472746*^9, {3.516457151655912*^9, 3.516457235975576*^9}, {
   3.516457284491362*^9, 3.5164573214514093`*^9}, {3.516457387922867*^9, 
   3.516457454287363*^9}, {3.516457625199349*^9, 3.516457672991251*^9}, {
   3.51645831600738*^9, 3.5164583295493793`*^9}, 3.516458360081263*^9, {
   3.516458779434746*^9, 3.516458802316041*^9}, {3.516459394550309*^9, 
   3.5164595175171347`*^9}, {3.516459604960169*^9, 3.516459625340931*^9}, {
   3.5164596662234*^9, 3.516459666637341*^9}, {3.516459998653577*^9, 
   3.516460006124365*^9}, {3.516460354942031*^9, 3.516460396324935*^9}, {
   3.51646053069876*^9, 3.516460614685628*^9}, {3.5164607527937717`*^9, 
   3.516460897803651*^9}, {3.516462501534315*^9, 3.516462526604961*^9}, {
   3.516462611112651*^9, 3.516462611383815*^9}, {3.516463021154703*^9, 
   3.516463061529895*^9}, {3.5164633717215548`*^9, 3.5164633976508627`*^9}, {
   3.516463429116269*^9, 3.516463481201078*^9}, {3.5164638398603697`*^9, 
   3.5164638593406963`*^9}, {3.516464312638464*^9, 3.516464312912916*^9}, {
   3.516464914674979*^9, 3.516465085200768*^9}, {3.516465337676579*^9, 
   3.516465372496331*^9}, {3.516465405274722*^9, 3.516465406270146*^9}, {
   3.516465552779982*^9, 3.516465588691825*^9}, {3.5164672510061007`*^9, 
   3.516467293942857*^9}, {3.516467474603541*^9, 3.5164675231703978`*^9}, {
   3.51646760024851*^9, 3.516467739960421*^9}, {3.5164677717917137`*^9, 
   3.516467988712447*^9}, {3.516469159296556*^9, 3.516469161955163*^9}, {
   3.5164698074427032`*^9, 3.516469904905406*^9}, {3.5164707554820213`*^9, 
   3.516470756292307*^9}, {3.516471149654396*^9, 3.516471249685629*^9}, {
   3.516471405122237*^9, 3.516471412333644*^9}, {3.516472156919051*^9, 
   3.51647219331638*^9}, {3.516472854954361*^9, 3.5164729609522753`*^9}, {
   3.568572773889769*^9, 3.5685728183678083`*^9}, {3.568573181918784*^9, 
   3.5685732296762877`*^9}, {3.568573342255867*^9, 3.568573749688448*^9}, {
   3.568573787742181*^9, 3.568573813907329*^9}, {3.568573850661203*^9, 
   3.568573919559573*^9}, {3.5685739707975807`*^9, 3.5685739999250727`*^9}, {
   3.568574594174342*^9, 3.568574676348609*^9}, {3.568574730907544*^9, 
   3.568574813674015*^9}, {3.568575547486279*^9, 3.568575589757934*^9}, {
   3.5685834303597527`*^9, 3.568583436207259*^9}, {3.568584066859976*^9, 
   3.5685841512933702`*^9}, {3.568584762036027*^9, 3.568584762654573*^9}, {
   3.5685848204813633`*^9, 3.568584839282544*^9}, 3.568643683007687*^9, {
   3.5686478992868156`*^9, 3.568647899524404*^9}, {3.568647933613801*^9, 
   3.5686479617678833`*^9}}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["RHP construction", "Subsection",
 CellChangeTimes->{{3.56864670932092*^9, 3.568646712689498*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"InnerModa", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Min", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Sqrt", "[", "2", "]"}], 
      RowBox[{"a", "[", 
       RowBox[{"s", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ",", 
     RowBox[{
      RowBox[{"Sqrt", "[", "2", "]"}], "*", "\[Nu]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"shrinkt", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{
     RowBox[{"Abs", "[", 
      RowBox[{
       RowBox[{"x", "/", "6"}], "*", 
       RowBox[{"K0", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], "^", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "2"}], "/", "3"}], ")"}]}], "/", "2."}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"radat", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Min", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"a", "[", 
       RowBox[{"s", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], "/", "1.2"}], ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"b", "[", 
         RowBox[{"s", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}], "-", 
        RowBox[{"a", "[", 
         RowBox[{"s", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ")"}], "/", "2.2"}], ",", 
     RowBox[{
      RowBox[{"shrinkt", "[", 
       RowBox[{"x", ",", "t"}], "]"}], "/", "2"}], ",", 
     RowBox[{
      RowBox[{"InnerModa", "[", 
       RowBox[{"x", ",", "t"}], "]"}], "/", "2."}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"radbt", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Min", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"a", "[", 
       RowBox[{"s", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], "/", "1.2"}], ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"b", "[", 
         RowBox[{"s", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}], "-", 
        RowBox[{"a", "[", 
         RowBox[{"s", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ")"}], "/", "2.2"}], ",", 
     RowBox[{
      RowBox[{"shrinkt", "[", 
       RowBox[{"x", ",", "t"}], "]"}], "/", "2"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"modulusa", "[", 
   RowBox[{"x_", ",", "t_"}], "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"Sqrt", "[", "2", "]"}], "\[Nu]"}]}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.514825267948341*^9, 3.514825292676878*^9}, {
   3.5148253366078253`*^9, 3.514825427904633*^9}, {3.5148255311926517`*^9, 
   3.5148255943071117`*^9}, {3.514825669646573*^9, 3.514825745629958*^9}, {
   3.5148258988524733`*^9, 3.514825926785774*^9}, {3.5148261039825363`*^9, 
   3.514826258039207*^9}, {3.514826288823626*^9, 3.514826543012664*^9}, {
   3.514826573528026*^9, 3.514826574581785*^9}, {3.514826728774248*^9, 
   3.514826758756071*^9}, {3.5149111916558123`*^9, 3.5149111953371058`*^9}, {
   3.515012840818*^9, 3.515012842308*^9}, {3.515014882502*^9, 
   3.515014960666*^9}, {3.515015599758*^9, 3.515015616958*^9}, {
   3.51501566601*^9, 3.5150156951099997`*^9}, {3.515015753572*^9, 
   3.515015755632*^9}, {3.515016449604*^9, 3.5150164754440002`*^9}, {
   3.515352517005292*^9, 3.515352518198556*^9}, {3.515352576361087*^9, 
   3.5153525768173018`*^9}, {3.516046991290357*^9, 3.516046997450739*^9}, {
   3.516047528477551*^9, 3.516047551533617*^9}, {3.516047585540667*^9, 
   3.516047586374466*^9}, {3.5160478007621*^9, 3.5160478303922567`*^9}, {
   3.5160479997286997`*^9, 3.5160479999812803`*^9}, {3.5160493707196503`*^9, 
   3.516049373787775*^9}, {3.5161161620922937`*^9, 3.516116243178967*^9}, {
   3.516116667401052*^9, 3.516116669227117*^9}, {3.516117193669569*^9, 
   3.516117194488471*^9}, {3.516117229344508*^9, 3.516117231512937*^9}, {
   3.516117362273058*^9, 3.516117362411113*^9}, 3.516118210858127*^9, {
   3.516122500907539*^9, 3.5161225010548067`*^9}, {3.5161225987351503`*^9, 
   3.5161226021411448`*^9}, {3.516123201090465*^9, 3.516123247434527*^9}, {
   3.516123318293346*^9, 3.5161233296183443`*^9}, {3.516123664886915*^9, 
   3.5161236737276907`*^9}, {3.516123811448966*^9, 3.516123812219467*^9}, {
   3.516370160808737*^9, 3.51637016129567*^9}, 3.516370227054289*^9, {
   3.516370368268955*^9, 3.516370371671545*^9}, {3.516370403577023*^9, 
   3.516370430220418*^9}, {3.516370461480939*^9, 3.5163705481935883`*^9}, {
   3.516385809055862*^9, 3.516385833623293*^9}, {3.516385926405691*^9, 
   3.5163859268148317`*^9}, {3.516386453380137*^9, 3.516386454111146*^9}, {
   3.5163865190706177`*^9, 3.516386521379715*^9}, {3.516388684852357*^9, 
   3.516388685899753*^9}, {3.516396368502108*^9, 3.516396370877252*^9}, {
   3.516406857601206*^9, 3.516406859819281*^9}, {3.516407099154993*^9, 
   3.516407099912368*^9}, {3.516407188961028*^9, 3.516407190335492*^9}, {
   3.516408853739092*^9, 3.516408854338091*^9}, {3.516410064590905*^9, 
   3.516410066451586*^9}, {3.5164106800008783`*^9, 3.516410681038292*^9}, {
   3.516410768193162*^9, 3.516410780595913*^9}, {3.516457046239871*^9, 
   3.516457052843502*^9}, {3.5164571421728563`*^9, 3.516457145593808*^9}, {
   3.516465935514337*^9, 3.516465937021831*^9}, 3.516465970101141*^9, {
   3.5164753950579033`*^9, 3.516475404940131*^9}, {3.5685813808683367`*^9, 
   3.5685814053185263`*^9}, {3.568581511814828*^9, 3.568581553993299*^9}, {
   3.56858593809752*^9, 3.568585954961512*^9}}],

Cell[CellGroupData[{

Cell["Full jumps", "Subsubsection",
 CellChangeTimes->{{3.568646722205043*^9, 3.5686467240656652`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "1", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsUsQs\[Phi]\[CapitalDelta]k", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"radat", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"InnerModa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", "*", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "2", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsUsQs\[Phi]\[CapitalDelta]k", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"InnerModa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", "*", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"radat", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "3", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]k", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"radat", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"InnerModa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], "*", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "4", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]k", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"InnerModa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], "*", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"radat", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", "3", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "5", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsGsQs\[Phi]\[CapitalDelta]k", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{"radbt", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "-", 
         RowBox[{"radat", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}]}], "}"}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "6", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsGsQs\[Phi]\[CapitalDelta]k", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{"radat", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"b", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "-", 
         RowBox[{"radbt", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}]}], "}"}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "7", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsPsQs\[Phi]\[CapitalDelta]k", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"b", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"radbt", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"b", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"modulusa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "8", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsPsQs\[Phi]\[CapitalDelta]k", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"modulusa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"radbt", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "9", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]k", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"b", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"radbt", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"b", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"modulusa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJ", "[", "4", "]"}], "[", "10", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]k", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{"b", "[", 
            RowBox[{"s", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
          RowBox[{
           RowBox[{"modulusa", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "*", 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "3"}], "I", " ", 
             RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"b", "[", 
            RowBox[{"s", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
          RowBox[{
           RowBox[{"radbt", "[", 
            RowBox[{"x", ",", "t"}], "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "3"}], "I", " ", 
             RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
      "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"JJ", "[", "4", "]"}], "[", "11", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
        "\[CapitalDelta]ki\[Phi]QinsUinsQs\[Phi]\[CapitalDelta]k", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{"Arc", "[", 
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
         RowBox[{"radat", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", 
           RowBox[{"3", 
            RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}], ",", "smallN"}], 
      "}"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"JJ", "[", "4", "]"}], "[", "12", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"Z", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"+", "1"}], "]"}], ",", 
       RowBox[{"Arc", "[", 
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
         RowBox[{"radat", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"3", 
            RowBox[{"Pi", "/", "4"}]}], ",", "Pi"}], "}"}]}], "]"}], ",", 
       "smallN"}], "}"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"JJ", "[", "4", "]"}], "[", "13", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"Z", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"+", "1"}], "]"}], ",", 
       RowBox[{"Arc", "[", 
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
         RowBox[{"radat", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"Pi", ",", 
           RowBox[{"5", 
            RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}], ",", "smallN"}], 
      "}"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"JJ", "[", "4", "]"}], "[", "14", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
        "\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]k", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{"Arc", "[", 
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
         RowBox[{"radat", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"5", 
            RowBox[{"Pi", "/", "4"}]}], ",", 
           RowBox[{"2", "Pi"}]}], "}"}]}], "]"}], ",", "smallN"}], "}"}]}], 
    ";"}], "*)"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "11", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
      "\[CapitalDelta]ki\[Phi]QinsUinsQs\[Phi]\[CapitalDelta]k", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{"radat", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"radat", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "12", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Z", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"+", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "Pi"}], "]"}]}], "}"}], 
        RowBox[{"radat", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"a", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "13", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Z", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"+", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "Pi"}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "5", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}], "}"}], 
        RowBox[{"radat", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"a", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJ", "[", "4", "]"}], "[", "14", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]k", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "5", " ", 
             RowBox[{"Pi", "/", "4"}]}], "]"}], ",", "1"}], "}"}], 
         RowBox[{"radat", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "+", 
        RowBox[{"a", "[", 
         RowBox[{"s", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
     "}"}]}], ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"JJ", "[", "4", "]"}], "[", "15", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"Z", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"-", "1"}], "]"}], ",", 
       RowBox[{"Arc", "[", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ",", 
         RowBox[{"radat", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", 
           RowBox[{"Pi", "/", "4"}]}], "}"}]}], "]"}], ",", "smallN"}], 
      "}"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"JJ", "[", "4", "]"}], "[", "16", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
        "\[CapitalDelta]ki\[Phi]QinsUinsQs\[Phi]\[CapitalDelta]k", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{"Arc", "[", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ",", 
         RowBox[{"radat", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Pi", "/", "4"}], ",", "Pi"}], "}"}]}], "]"}], ",", 
       "smallN"}], "}"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"JJ", "[", "4", "]"}], "[", "17", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
        "\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]k", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{"Arc", "[", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ",", 
         RowBox[{"radat", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"Pi", ",", 
           RowBox[{"7", 
            RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}], ",", "smallN"}], 
      "}"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"JJ", "[", "4", "]"}], "[", "18", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"Z", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"-", "1"}], "]"}], ",", 
       RowBox[{"Arc", "[", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ",", 
         RowBox[{"radat", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"7", 
            RowBox[{"Pi", "/", "4"}]}], ",", 
           RowBox[{"2", "Pi"}]}], "}"}]}], "]"}], ",", "smallN"}], "}"}]}], 
    ";"}], "*)"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "15", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Z", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"-", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}], "}"}], 
        RowBox[{"radat", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"a", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "16", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
      "\[CapitalDelta]ki\[Phi]QinsUinsQs\[Phi]\[CapitalDelta]k", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "Pi"}], "]"}]}], "}"}], 
        RowBox[{"radat", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"a", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "17", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]k", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "Pi"}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "7", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}], "}"}], 
        RowBox[{"radat", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"a", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJ", "[", "4", "]"}], "[", "18", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"Z", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"-", "1"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "7", 
             RowBox[{"Pi", "/", "4"}]}], "]"}], ",", 
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "2", "Pi"}], "]"}]}], "}"}], 
         RowBox[{"radat", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "-", 
        RowBox[{"a", "[", 
         RowBox[{"s", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
     "}"}]}], ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
  "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"JJ", "[", "4", "]"}], "[", "19", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
        "\[CapitalDelta]ki\[Phi]QinsPinsQs\[Phi]\[CapitalDelta]k", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{"Arc", "[", 
        RowBox[{
         RowBox[{"b", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
         RowBox[{"radbt", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Pi", "/", "4"}], ",", "Pi"}], "}"}]}], "]"}], ",", 
       "smallN"}], "}"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"JJ", "[", "4", "]"}], "[", "20", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
        "\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]k", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{"Arc", "[", 
        RowBox[{
         RowBox[{"b", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
         RowBox[{"radbt", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"Pi", ",", 
           RowBox[{"7", 
            RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}], ",", "smallN"}], 
      "}"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"JJ", "[", "4", "]"}], "[", "21", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"W", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"+", "1"}], "]"}], ",", 
       RowBox[{"Arc", "[", 
        RowBox[{
         RowBox[{"b", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
         RowBox[{"radbt", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{"-", "Pi"}], "/", "4"}], ",", "0"}], "}"}]}], "]"}], ",", 
       "smallN"}], "}"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"JJ", "[", "4", "]"}], "[", "22", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"W", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"+", "1"}], "]"}], ",", 
       RowBox[{"Arc", "[", 
        RowBox[{
         RowBox[{"b", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
         RowBox[{"radbt", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", 
           RowBox[{"Pi", "/", "4"}]}], "}"}]}], "]"}], ",", "smallN"}], 
      "}"}]}], ";"}], "*)"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "19", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
      "\[CapitalDelta]ki\[Phi]QinsPinsQs\[Phi]\[CapitalDelta]k", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Pi", "/", "4"}], ",", "Pi"}], "}"}]}], "]"}], 
        RowBox[{"radbt", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"b", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "20", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]k", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{"Pi", ",", 
            RowBox[{"7", 
             RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}], 
        RowBox[{"radbt", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"b", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "21", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"W", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"+", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "Pi"}], "/", "4"}], ",", "0"}], "}"}]}], "]"}], 
        RowBox[{"radbt", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"b", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJ", "[", "4", "]"}], "[", "22", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"W", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"+", "1"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Exp", "[", 
          RowBox[{"I", 
           RowBox[{"{", 
            RowBox[{"0", ",", 
             RowBox[{"Pi", "/", "4"}]}], "}"}]}], "]"}], 
         RowBox[{"radbt", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "+", 
        RowBox[{"b", "[", 
         RowBox[{"s", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
     "}"}]}], ";"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"JJ", "[", "4", "]"}], "[", "23", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
        "\[CapitalDelta]ki\[Phi]QinsPinsQs\[Phi]\[CapitalDelta]k", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{"Arc", "[", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ",", 
         RowBox[{"radbt", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"0", ",", 
           RowBox[{"3", 
            RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}], ",", "smallN"}], 
      "}"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"JJ", "[", "4", "]"}], "[", "24", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"W", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"-", "1"}], "]"}], ",", 
       RowBox[{"Arc", "[", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ",", 
         RowBox[{"radbt", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"3", 
            RowBox[{"Pi", "/", "4"}]}], ",", "Pi"}], "}"}]}], "]"}], ",", 
       "smallN"}], "}"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"JJ", "[", "4", "]"}], "[", "25", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"W", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"-", "1"}], "]"}], ",", 
       RowBox[{"Arc", "[", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ",", 
         RowBox[{"radbt", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"Pi", ",", 
           RowBox[{"5", 
            RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}], ",", "smallN"}], 
      "}"}]}], ";", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"JJ", "[", "4", "]"}], "[", "26", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
        "\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]k", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{"Arc", "[", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ",", 
         RowBox[{"radbt", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"5", 
            RowBox[{"Pi", "/", "4"}]}], ",", 
           RowBox[{"2", "Pi"}]}], "}"}]}], "]"}], ",", "smallN"}], "}"}]}], 
    ";"}], "*)"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "23", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
      "\[CapitalDelta]ki\[Phi]QinsPinsQs\[Phi]\[CapitalDelta]k", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{"0", ",", 
            RowBox[{"3", 
             RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}], 
        RowBox[{"radbt", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"b", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "24", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"W", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"-", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"3", 
             RowBox[{"Pi", "/", "4"}]}], ",", "Pi"}], "}"}]}], "]"}], 
        RowBox[{"radbt", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"b", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "25", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"W", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"-", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{"Pi", ",", 
            RowBox[{"5", 
             RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}], 
        RowBox[{"radbt", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"b", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJ", "[", "4", "]"}], "[", "26", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]k", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Exp", "[", 
          RowBox[{"I", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"5", 
              RowBox[{"Pi", "/", "4"}]}], ",", 
             RowBox[{"2", "Pi"}]}], "}"}]}], "]"}], 
         RowBox[{"radbt", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "-", 
        RowBox[{"b", "[", 
         RowBox[{"s", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
     "}"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "27", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsPsQs\[Phi]\[CapitalDelta]k", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "el"}], "+", 
         RowBox[{"I", " ", "\[Nu]"}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}], " ", 
          RowBox[{"modulusa", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJ", "[", "4", "]"}], "[", "28", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]k", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "el"}], "-", 
          RowBox[{"I", " ", "\[Nu]"}]}], ",", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"b", "[", 
            RowBox[{"s", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", "3", " ", 
             RowBox[{"Pi", "/", "4"}]}], "]"}], " ", 
           RowBox[{"modulusa", "[", 
            RowBox[{"x", ",", "t"}], "]"}]}]}]}], "}"}], "]"}], ",", 
      "smallN"}], "}"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "29", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsPsQs\[Phi]\[CapitalDelta]k", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"b", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}], " ", 
          RowBox[{"modulusa", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"I", " ", "\[Nu]"}], " ", "+", " ", "el"}]}], "}"}], "]"}], 
     ",", "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJ", "[", "4", "]"}], "[", "30", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]k", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"b", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", 
             RowBox[{"Pi", "/", "4"}]}], "]"}], " ", 
           RowBox[{"modulusa", "[", 
            RowBox[{"x", ",", "t"}], "]"}]}]}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "I"}], " ", "\[Nu]"}], " ", "+", " ", "el"}]}], "}"}],
        "]"}], ",", "smallN"}], "}"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "31", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsUsQs\[Phi]\[CapitalDelta]k", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"InnerModa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", "*", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"InnerModa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", "*", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "4", "]"}], "[", "32", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]k", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"InnerModa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], "*", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"InnerModa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], "*", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.514825267948341*^9, 3.514825292676878*^9}, {
   3.5148253366078253`*^9, 3.514825427904633*^9}, {3.5148255311926517`*^9, 
   3.5148255943071117`*^9}, {3.514825669646573*^9, 3.514825745629958*^9}, {
   3.5148258988524733`*^9, 3.514825926785774*^9}, {3.5148261039825363`*^9, 
   3.514826258039207*^9}, {3.514826288823626*^9, 3.514826543012664*^9}, {
   3.514826573528026*^9, 3.514826574581785*^9}, {3.514826728774248*^9, 
   3.514826758756071*^9}, {3.514828935462126*^9, 3.5148290533272142`*^9}, {
   3.5149096829953327`*^9, 3.514909723142671*^9}, {3.5149098576453648`*^9, 
   3.514909889448564*^9}, {3.514912127190716*^9, 3.51491213623287*^9}, {
   3.514912195772973*^9, 3.514912204226186*^9}, {3.514912303442165*^9, 
   3.5149123042055283`*^9}, 3.514921423035899*^9, {3.5150131270220003`*^9, 
   3.515013127442*^9}, {3.515013592144*^9, 3.515013629584*^9}, {
   3.5150136607939997`*^9, 3.515013665744*^9}, {3.515013883704*^9, 
   3.515013948974*^9}, {3.515014575494*^9, 3.515014585174*^9}, {
   3.515510466019574*^9, 3.5155104702861137`*^9}, {3.516032532081387*^9, 
   3.516032632290653*^9}, 3.516032718421742*^9, {3.516116258863737*^9, 
   3.516116280731205*^9}, {3.5163598914918003`*^9, 3.5163599052258*^9}, {
   3.5163605163677998`*^9, 3.5163605337567997`*^9}, {3.5163605964997997`*^9, 
   3.5163606026288*^9}, {3.5163608507637997`*^9, 3.5163608508698*^9}, {
   3.516382888805026*^9, 3.5163830068406553`*^9}, {3.516383277038055*^9, 
   3.5163833555156527`*^9}, {3.5163833998893337`*^9, 
   3.5163834453055067`*^9}, {3.516383476857959*^9, 3.51638351945615*^9}, {
   3.5163854577936897`*^9, 3.5163856049352503`*^9}, {3.516385952783477*^9, 
   3.516385986208107*^9}, {3.5163867387806797`*^9, 3.516386778242505*^9}, {
   3.516388278925934*^9, 3.516388314455687*^9}, {3.51638840971177*^9, 
   3.51638841247635*^9}, {3.516397565665082*^9, 3.5163976921604767`*^9}, {
   3.516397899028942*^9, 3.516397899322949*^9}, {3.5164076030158243`*^9, 
   3.516407638210791*^9}, {3.516408006778857*^9, 3.516408016021781*^9}, {
   3.51640975918292*^9, 3.516409767769142*^9}, {3.516457465894291*^9, 
   3.516457481056218*^9}, {3.516457559479982*^9, 3.516457610709281*^9}, {
   3.516457646901187*^9, 3.51645764800912*^9}, {3.516457700012754*^9, 
   3.516457786011052*^9}, {3.516457820881053*^9, 3.516458122097102*^9}, 
   3.51645820755171*^9, {3.516458351867426*^9, 3.5164584977762203`*^9}, {
   3.516458531496792*^9, 3.516458550968389*^9}, {3.516458649058075*^9, 
   3.516458748494729*^9}, {3.516458824332374*^9, 3.516458838505898*^9}, {
   3.5164588709359703`*^9, 3.516459125857703*^9}, {3.516459181357025*^9, 
   3.516459184898267*^9}, {3.516459232991765*^9, 3.516459242845731*^9}, {
   3.516459533041588*^9, 3.516459555124057*^9}, {3.516459631924409*^9, 
   3.516459679085615*^9}, {3.516460197000605*^9, 3.516460206808563*^9}, {
   3.5164602992598553`*^9, 3.516460311067243*^9}, {3.5164609129823503`*^9, 
   3.516460968519927*^9}, {3.516462676865677*^9, 3.516462684467883*^9}, {
   3.5164631772156067`*^9, 3.516463276695409*^9}, {3.516463488358333*^9, 
   3.516463493706519*^9}, {3.516463910894813*^9, 3.5164639193960238`*^9}, {
   3.516464097606217*^9, 3.51646419164781*^9}, 3.5164643626396723`*^9, {
   3.516464411015307*^9, 3.516464431083765*^9}, {3.5164646185531282`*^9, 
   3.516464634286481*^9}, {3.516465101313439*^9, 3.5164651163310337`*^9}, {
   3.516465390538862*^9, 3.516465401256546*^9}, {3.516465809366246*^9, 
   3.5164658223352957`*^9}, {3.516468032786314*^9, 3.51646805284263*^9}, {
   3.516468116424253*^9, 3.516468136815833*^9}, {3.516468247182317*^9, 
   3.516468349452812*^9}, {3.5164683848684*^9, 3.516468387922887*^9}, {
   3.5164685154818697`*^9, 3.5164685841468763`*^9}, {3.516469921949301*^9, 
   3.5164699711692543`*^9}, {3.5164700483876743`*^9, 
   3.5164701636501837`*^9}, {3.516471263308015*^9, 3.5164712746994658`*^9}, {
   3.516471352359535*^9, 3.5164713546883163`*^9}, {3.5164714262637653`*^9, 
   3.516471432301978*^9}, {3.516473517538863*^9, 3.516473523403447*^9}, {
   3.516474982691085*^9, 3.516474983982852*^9}, {3.516475459381082*^9, 
   3.5164755286888514`*^9}, {3.5178368227708817`*^9, 3.517836867940639*^9}, 
   3.517836922921733*^9, {3.5685748879394712`*^9, 3.568574942003587*^9}, {
   3.568574975568761*^9, 3.568575073669878*^9}, {3.568575199214121*^9, 
   3.5685753730555277`*^9}, {3.568575430728529*^9, 3.56857544416363*^9}, {
   3.568575639798094*^9, 3.5685757167565727`*^9}, {3.568575765502276*^9, 
   3.568575875843132*^9}, {3.568575964817031*^9, 3.568575996598225*^9}, {
   3.5685815788707113`*^9, 3.5685816231848097`*^9}, {3.568581802957261*^9, 
   3.5685818768577337`*^9}, {3.568583185807536*^9, 3.568583193174665*^9}, {
   3.568583564587915*^9, 3.568583650082548*^9}, {3.568585532768119*^9, 
   3.56858556451052*^9}, {3.56858638617171*^9, 3.568586411272897*^9}, {
   3.568586562870647*^9, 3.5685865750782146`*^9}, 3.568586700783231*^9, {
   3.5685869892028027`*^9, 3.568587050334168*^9}, {3.568587113451311*^9, 
   3.568587177428643*^9}, {3.568587208818534*^9, 3.568587278029192*^9}, {
   3.568587315618803*^9, 3.5685874272201843`*^9}, {3.5685875764556723`*^9, 
   3.568587688191224*^9}, {3.5685877325224733`*^9, 3.568588053803132*^9}, {
   3.568588118269617*^9, 3.5685881777593327`*^9}, {3.568645003388166*^9, 
   3.5686450240213947`*^9}, {3.568646726459577*^9, 3.568646726775237*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Sample jumps", "Subsubsection",
 CellChangeTimes->{{3.56864674561305*^9, 3.5686467471847677`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "1", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsUsQs\[Phi]\[CapitalDelta]ksamp", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"radat", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"InnerModa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", "*", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "2", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsUsQs\[Phi]\[CapitalDelta]ksamp", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"InnerModa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", "*", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"radat", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "3", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]ksamp", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"radat", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"InnerModa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], "*", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "4", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]ksamp", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"InnerModa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], "*", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"radat", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", "3", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "5", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsGsQs\[Phi]\[CapitalDelta]ksamp", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{"radbt", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "-", 
         RowBox[{"radat", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}]}], "}"}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "6", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsGsQs\[Phi]\[CapitalDelta]ksamp", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{"radat", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"b", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "-", 
         RowBox[{"radbt", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}]}], "}"}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "7", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsPsQs\[Phi]\[CapitalDelta]ksamp", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"b", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"radbt", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"b", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"modulusa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "8", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsPsQs\[Phi]\[CapitalDelta]ksamp", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"modulusa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"radbt", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "9", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]ksamp", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"b", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"radbt", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"b", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"modulusa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJsamp", "[", "4", "]"}], "[", "10", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
      "\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]ksamp", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{"b", "[", 
            RowBox[{"s", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
          RowBox[{
           RowBox[{"modulusa", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "*", 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "3"}], "I", " ", 
             RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"b", "[", 
            RowBox[{"s", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
          RowBox[{
           RowBox[{"radbt", "[", 
            RowBox[{"x", ",", "t"}], "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "3"}], "I", " ", 
             RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
      "smallN"}], "}"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "11", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
      "\[CapitalDelta]ki\[Phi]QinsUinsQs\[Phi]\[CapitalDelta]ksamp", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{"radat", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"radat", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "12", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Zsamp", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"+", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "Pi"}], "]"}]}], "}"}], 
        RowBox[{"radat", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"a", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "13", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Zsamp", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"+", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "Pi"}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "5", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}], "}"}], 
        RowBox[{"radat", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"a", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJsamp", "[", "4", "]"}], "[", "14", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{
       "\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]ksamp", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "5", " ", 
             RowBox[{"Pi", "/", "4"}]}], "]"}], ",", "1"}], "}"}], 
         RowBox[{"radat", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "+", 
        RowBox[{"a", "[", 
         RowBox[{"s", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
     "}"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "15", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Zsamp", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"-", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}], "}"}], 
        RowBox[{"radat", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"a", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "16", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
      "\[CapitalDelta]ki\[Phi]QinsUinsQs\[Phi]\[CapitalDelta]ksamp", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "Pi"}], "]"}]}], "}"}], 
        RowBox[{"radat", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"a", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "17", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
      "\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]ksamp", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "Pi"}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "7", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}], "}"}], 
        RowBox[{"radat", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"a", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJsamp", "[", "4", "]"}], "[", "18", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"Zsamp", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"-", "1"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "7", 
             RowBox[{"Pi", "/", "4"}]}], "]"}], ",", 
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "2", "Pi"}], "]"}]}], "}"}], 
         RowBox[{"radat", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "-", 
        RowBox[{"a", "[", 
         RowBox[{"s", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
     "}"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "19", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
      "\[CapitalDelta]ki\[Phi]QinsPinsQs\[Phi]\[CapitalDelta]ksamp", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Pi", "/", "4"}], ",", "Pi"}], "}"}]}], "]"}], 
        RowBox[{"radbt", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"b", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "20", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
      "\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]ksamp", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{"Pi", ",", 
            RowBox[{"7", 
             RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}], 
        RowBox[{"radbt", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"b", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "21", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Wsamp", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"+", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "Pi"}], "/", "4"}], ",", "0"}], "}"}]}], "]"}], 
        RowBox[{"radbt", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"b", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJsamp", "[", "4", "]"}], "[", "22", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"Wsamp", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"+", "1"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Exp", "[", 
          RowBox[{"I", 
           RowBox[{"{", 
            RowBox[{"0", ",", 
             RowBox[{"Pi", "/", "4"}]}], "}"}]}], "]"}], 
         RowBox[{"radbt", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "+", 
        RowBox[{"b", "[", 
         RowBox[{"s", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
     "}"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "23", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{
      "\[CapitalDelta]ki\[Phi]QinsPinsQs\[Phi]\[CapitalDelta]ksamp", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{"0", ",", 
            RowBox[{"3", 
             RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}], 
        RowBox[{"radbt", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"b", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "24", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Wsamp", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"-", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"3", 
             RowBox[{"Pi", "/", "4"}]}], ",", "Pi"}], "}"}]}], "]"}], 
        RowBox[{"radbt", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"b", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "25", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Wsamp", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"-", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{"Pi", ",", 
            RowBox[{"5", 
             RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}], 
        RowBox[{"radbt", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"b", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJsamp", "[", "4", "]"}], "[", "26", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{
       "\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]ksamp", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Exp", "[", 
          RowBox[{"I", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"5", 
              RowBox[{"Pi", "/", "4"}]}], ",", 
             RowBox[{"2", "Pi"}]}], "}"}]}], "]"}], 
         RowBox[{"radbt", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "-", 
        RowBox[{"b", "[", 
         RowBox[{"s", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
     "}"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "27", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsPsQs\[Phi]\[CapitalDelta]ksamp", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "el"}], "+", 
         RowBox[{"I", " ", "\[Nu]"}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}], " ", 
          RowBox[{"modulusa", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJsamp", "[", "4", "]"}], "[", "28", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
      "\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]ksamp", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "el"}], "-", 
          RowBox[{"I", " ", "\[Nu]"}]}], ",", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"b", "[", 
            RowBox[{"s", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", "3", " ", 
             RowBox[{"Pi", "/", "4"}]}], "]"}], " ", 
           RowBox[{"modulusa", "[", 
            RowBox[{"x", ",", "t"}], "]"}]}]}]}], "}"}], "]"}], ",", 
      "smallN"}], "}"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "29", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsPsQs\[Phi]\[CapitalDelta]ksamp", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"b", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}], " ", 
          RowBox[{"modulusa", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"I", " ", "\[Nu]"}], " ", "+", " ", "el"}]}], "}"}], "]"}], 
     ",", "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJsamp", "[", "4", "]"}], "[", "30", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
      "\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]ksamp", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"b", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", 
             RowBox[{"Pi", "/", "4"}]}], "]"}], " ", 
           RowBox[{"modulusa", "[", 
            RowBox[{"x", ",", "t"}], "]"}]}]}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "I"}], " ", "\[Nu]"}], " ", "+", " ", "el"}]}], "}"}],
        "]"}], ",", "smallN"}], "}"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "31", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsUsQs\[Phi]\[CapitalDelta]ksamp", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"InnerModa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", "*", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"InnerModa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", "*", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "4", "]"}], "[", "32", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]ksamp", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"InnerModa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], "*", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"InnerModa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], "*", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.514825267948341*^9, 3.514825292676878*^9}, {
   3.5148253366078253`*^9, 3.514825427904633*^9}, {3.5148255311926517`*^9, 
   3.5148255943071117`*^9}, {3.514825669646573*^9, 3.514825745629958*^9}, {
   3.5148258988524733`*^9, 3.514825926785774*^9}, {3.5148261039825363`*^9, 
   3.514826258039207*^9}, {3.514826288823626*^9, 3.514826543012664*^9}, {
   3.514826573528026*^9, 3.514826574581785*^9}, {3.514826728774248*^9, 
   3.514826758756071*^9}, {3.514828935462126*^9, 3.5148290533272142`*^9}, {
   3.5149096829953327`*^9, 3.514909723142671*^9}, {3.5149098576453648`*^9, 
   3.514909889448564*^9}, {3.514912127190716*^9, 3.51491213623287*^9}, {
   3.514912195772973*^9, 3.514912204226186*^9}, {3.514912303442165*^9, 
   3.5149123042055283`*^9}, 3.514921423035899*^9, {3.5150131270220003`*^9, 
   3.515013127442*^9}, {3.515013592144*^9, 3.515013629584*^9}, {
   3.5150136607939997`*^9, 3.515013665744*^9}, {3.515013883704*^9, 
   3.515013948974*^9}, {3.515014575494*^9, 3.515014585174*^9}, {
   3.515510466019574*^9, 3.5155104702861137`*^9}, {3.516032532081387*^9, 
   3.516032632290653*^9}, 3.516032718421742*^9, {3.516116258863737*^9, 
   3.516116280731205*^9}, {3.5163598914918003`*^9, 3.5163599052258*^9}, {
   3.5163605163677998`*^9, 3.5163605337567997`*^9}, {3.5163605964997997`*^9, 
   3.5163606026288*^9}, {3.5163608507637997`*^9, 3.5163608508698*^9}, {
   3.516382888805026*^9, 3.5163830068406553`*^9}, {3.516383277038055*^9, 
   3.5163833555156527`*^9}, {3.5163833998893337`*^9, 
   3.5163834453055067`*^9}, {3.516383476857959*^9, 3.51638351945615*^9}, {
   3.5163854577936897`*^9, 3.5163856049352503`*^9}, {3.516385952783477*^9, 
   3.516385986208107*^9}, {3.5163867387806797`*^9, 3.516386778242505*^9}, {
   3.516388278925934*^9, 3.516388314455687*^9}, {3.51638840971177*^9, 
   3.51638841247635*^9}, {3.516397565665082*^9, 3.5163976921604767`*^9}, {
   3.516397899028942*^9, 3.516397899322949*^9}, {3.5164076030158243`*^9, 
   3.516407638210791*^9}, {3.516408006778857*^9, 3.516408016021781*^9}, {
   3.51640975918292*^9, 3.516409767769142*^9}, {3.516457465894291*^9, 
   3.516457481056218*^9}, {3.516457559479982*^9, 3.516457610709281*^9}, {
   3.516457646901187*^9, 3.51645764800912*^9}, {3.516457700012754*^9, 
   3.516457786011052*^9}, {3.516457820881053*^9, 3.516458122097102*^9}, 
   3.51645820755171*^9, {3.516458351867426*^9, 3.5164584977762203`*^9}, {
   3.516458531496792*^9, 3.516458550968389*^9}, {3.516458649058075*^9, 
   3.516458748494729*^9}, {3.516458824332374*^9, 3.516458838505898*^9}, {
   3.5164588709359703`*^9, 3.516459125857703*^9}, {3.516459181357025*^9, 
   3.516459184898267*^9}, {3.516459232991765*^9, 3.516459242845731*^9}, {
   3.516459533041588*^9, 3.516459555124057*^9}, {3.516459631924409*^9, 
   3.516459679085615*^9}, {3.516460197000605*^9, 3.516460206808563*^9}, {
   3.5164602992598553`*^9, 3.516460311067243*^9}, {3.5164609129823503`*^9, 
   3.516460968519927*^9}, {3.516462676865677*^9, 3.516462684467883*^9}, {
   3.5164631772156067`*^9, 3.516463276695409*^9}, {3.516463488358333*^9, 
   3.516463493706519*^9}, {3.516463910894813*^9, 3.5164639193960238`*^9}, {
   3.516464097606217*^9, 3.51646419164781*^9}, 3.5164643626396723`*^9, {
   3.516464411015307*^9, 3.516464431083765*^9}, {3.5164646185531282`*^9, 
   3.516464634286481*^9}, {3.516465101313439*^9, 3.5164651163310337`*^9}, {
   3.516465390538862*^9, 3.516465401256546*^9}, {3.516465809366246*^9, 
   3.5164658223352957`*^9}, {3.516468032786314*^9, 3.51646805284263*^9}, {
   3.516468116424253*^9, 3.516468136815833*^9}, {3.516468247182317*^9, 
   3.516468349452812*^9}, {3.5164683848684*^9, 3.516468387922887*^9}, {
   3.5164685154818697`*^9, 3.5164685841468763`*^9}, {3.516469921949301*^9, 
   3.5164699711692543`*^9}, {3.5164700483876743`*^9, 
   3.5164701636501837`*^9}, {3.516471263308015*^9, 3.5164712746994658`*^9}, {
   3.516471352359535*^9, 3.5164713546883163`*^9}, {3.5164714262637653`*^9, 
   3.516471432301978*^9}, {3.516473517538863*^9, 3.516473523403447*^9}, {
   3.516474982691085*^9, 3.516474983982852*^9}, {3.516475459381082*^9, 
   3.5164755286888514`*^9}, {3.5178368227708817`*^9, 3.517836867940639*^9}, 
   3.517836922921733*^9, {3.5685748879394712`*^9, 3.568574942003587*^9}, {
   3.568574975568761*^9, 3.568575073669878*^9}, {3.568575199214121*^9, 
   3.5685753730555277`*^9}, {3.568575430728529*^9, 3.56857544416363*^9}, {
   3.568575639798094*^9, 3.5685757167565727`*^9}, {3.568575765502276*^9, 
   3.568575875843132*^9}, {3.568575964817031*^9, 3.568575996598225*^9}, {
   3.5685815788707113`*^9, 3.5685816231848097`*^9}, {3.568581802957261*^9, 
   3.5685818768577337`*^9}, {3.568583185807536*^9, 3.568583193174665*^9}, {
   3.568583564587915*^9, 3.568583650082548*^9}, {3.568585532768119*^9, 
   3.56858556451052*^9}, {3.56858638617171*^9, 3.568586411272897*^9}, {
   3.568586562870647*^9, 3.5685865750782146`*^9}, 3.568586700783231*^9, {
   3.5685869892028027`*^9, 3.568587050334168*^9}, {3.568587113451311*^9, 
   3.568587177428643*^9}, {3.568587208818534*^9, 3.568587278029192*^9}, {
   3.568587315618803*^9, 3.5685874272201843`*^9}, {3.5685875764556723`*^9, 
   3.568587688191224*^9}, {3.5685877325224733`*^9, 3.568588053803132*^9}, {
   3.568588118269617*^9, 3.5685881777593327`*^9}, 3.568645003388166*^9, {
   3.568645056952374*^9, 3.56864511318191*^9}, {3.568645202222413*^9, 
   3.5686452463478613`*^9}, {3.568645284955329*^9, 3.568645330647273*^9}, {
   3.568646749699089*^9, 3.568646767383565*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Differentiated Jumps", "Subsubsection",
 CellChangeTimes->{{3.568646777515478*^9, 3.568646782201989*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "1", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsUsQs\[Phi]\[CapitalDelta]kx", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"radat", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"InnerModa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", "*", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "2", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsUsQs\[Phi]\[CapitalDelta]kx", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"InnerModa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", "*", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"radat", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "3", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]kx", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"radat", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"InnerModa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], "*", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "4", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]kx", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"InnerModa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], "*", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"radat", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", "3", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "5", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsGsQs\[Phi]\[CapitalDelta]kx", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{"radbt", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "-", 
         RowBox[{"radat", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}]}], "}"}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "6", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsGsQs\[Phi]\[CapitalDelta]kx", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{"radat", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"b", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "-", 
         RowBox[{"radbt", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}]}], "}"}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "7", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsPsQs\[Phi]\[CapitalDelta]kx", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"b", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"radbt", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"b", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"modulusa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "8", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsPsQs\[Phi]\[CapitalDelta]kx", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"modulusa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"radbt", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "9", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]kx", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"b", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"radbt", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"b", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"modulusa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJx", "[", "4", "]"}], "[", "10", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]kx", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{"b", "[", 
            RowBox[{"s", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
          RowBox[{
           RowBox[{"modulusa", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "*", 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "3"}], "I", " ", 
             RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"b", "[", 
            RowBox[{"s", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
          RowBox[{
           RowBox[{"radbt", "[", 
            RowBox[{"x", ",", "t"}], "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "3"}], "I", " ", 
             RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
      "smallN"}], "}"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "11", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[CapitalDelta]ki\[Phi]QinsUinsQs\[Phi]\[CapitalDelta]kx", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{"radat", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"radat", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "12", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Zx", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"+", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "Pi"}], "]"}]}], "}"}], 
        RowBox[{"radat", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"a", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "13", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Zx", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"+", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "Pi"}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "5", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}], "}"}], 
        RowBox[{"radat", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"a", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJx", "[", "4", "]"}], "[", "14", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{
       "\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]kx", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "5", " ", 
             RowBox[{"Pi", "/", "4"}]}], "]"}], ",", "1"}], "}"}], 
         RowBox[{"radat", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "+", 
        RowBox[{"a", "[", 
         RowBox[{"s", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
     "}"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "15", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Zx", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"-", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{"1", ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}], "}"}], 
        RowBox[{"radat", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"a", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "16", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[CapitalDelta]ki\[Phi]QinsUinsQs\[Phi]\[CapitalDelta]kx", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "Pi"}], "]"}]}], "}"}], 
        RowBox[{"radat", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"a", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "17", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]kx", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "Pi"}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "7", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}], "}"}], 
        RowBox[{"radat", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"a", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJx", "[", "4", "]"}], "[", "18", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"Zx", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"-", "1"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "7", 
             RowBox[{"Pi", "/", "4"}]}], "]"}], ",", 
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", "2", "Pi"}], "]"}]}], "}"}], 
         RowBox[{"radat", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "-", 
        RowBox[{"a", "[", 
         RowBox[{"s", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
     "}"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "19", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[CapitalDelta]ki\[Phi]QinsPinsQs\[Phi]\[CapitalDelta]kx", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Pi", "/", "4"}], ",", "Pi"}], "}"}]}], "]"}], 
        RowBox[{"radbt", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"b", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "20", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]kx", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{"Pi", ",", 
            RowBox[{"7", 
             RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}], 
        RowBox[{"radbt", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"b", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "21", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Wx", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"+", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "Pi"}], "/", "4"}], ",", "0"}], "}"}]}], "]"}], 
        RowBox[{"radbt", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"b", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJx", "[", "4", "]"}], "[", "22", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"Wx", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"+", "1"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Exp", "[", 
          RowBox[{"I", 
           RowBox[{"{", 
            RowBox[{"0", ",", 
             RowBox[{"Pi", "/", "4"}]}], "}"}]}], "]"}], 
         RowBox[{"radbt", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "+", 
        RowBox[{"b", "[", 
         RowBox[{"s", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
     "}"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "23", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[CapitalDelta]ki\[Phi]QinsPinsQs\[Phi]\[CapitalDelta]kx", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{"0", ",", 
            RowBox[{"3", 
             RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}], 
        RowBox[{"radbt", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"b", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "24", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Wx", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"-", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"3", 
             RowBox[{"Pi", "/", "4"}]}], ",", "Pi"}], "}"}]}], "]"}], 
        RowBox[{"radbt", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"b", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "25", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Wx", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"-", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{"Pi", ",", 
            RowBox[{"5", 
             RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}], 
        RowBox[{"radbt", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"b", "[", 
        RowBox[{"s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJx", "[", "4", "]"}], "[", "26", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{
       "\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]kx", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Exp", "[", 
          RowBox[{"I", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"5", 
              RowBox[{"Pi", "/", "4"}]}], ",", 
             RowBox[{"2", "Pi"}]}], "}"}]}], "]"}], 
         RowBox[{"radbt", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "-", 
        RowBox[{"b", "[", 
         RowBox[{"s", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], ",", "smallN"}], 
     "}"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "27", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsPsQs\[Phi]\[CapitalDelta]kx", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "el"}], "+", 
         RowBox[{"I", " ", "\[Nu]"}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}], " ", 
          RowBox[{"modulusa", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJx", "[", "4", "]"}], "[", "28", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]kx", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "el"}], "-", 
          RowBox[{"I", " ", "\[Nu]"}]}], ",", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"b", "[", 
            RowBox[{"s", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", "3", " ", 
             RowBox[{"Pi", "/", "4"}]}], "]"}], " ", 
           RowBox[{"modulusa", "[", 
            RowBox[{"x", ",", "t"}], "]"}]}]}]}], "}"}], "]"}], ",", 
      "smallN"}], "}"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "29", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsPsQs\[Phi]\[CapitalDelta]kx", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"b", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}], " ", 
          RowBox[{"modulusa", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"I", " ", "\[Nu]"}], " ", "+", " ", "el"}]}], "}"}], "]"}], 
     ",", "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJx", "[", "4", "]"}], "[", "30", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[CapitalDelta]ki\[Phi]QinsMsQs\[Phi]\[CapitalDelta]kx", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"b", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", 
             RowBox[{"Pi", "/", "4"}]}], "]"}], " ", 
           RowBox[{"modulusa", "[", 
            RowBox[{"x", ",", "t"}], "]"}]}]}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"-", "I"}], " ", "\[Nu]"}], " ", "+", " ", "el"}]}], "}"}],
        "]"}], ",", "smallN"}], "}"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "31", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsUsQs\[Phi]\[CapitalDelta]kx", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"InnerModa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", "*", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"InnerModa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", "*", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "4", "]"}], "[", "32", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[CapitalDelta]ki\[Phi]QinsLsQs\[Phi]\[CapitalDelta]kx", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a", "[", 
           RowBox[{"s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"InnerModa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], "*", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"a", "[", 
          RowBox[{"s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "+", 
         RowBox[{
          RowBox[{"InnerModa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], "*", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.514825267948341*^9, 3.514825292676878*^9}, {
   3.5148253366078253`*^9, 3.514825427904633*^9}, {3.5148255311926517`*^9, 
   3.5148255943071117`*^9}, {3.514825669646573*^9, 3.514825745629958*^9}, {
   3.5148258988524733`*^9, 3.514825926785774*^9}, {3.5148261039825363`*^9, 
   3.514826258039207*^9}, {3.514826288823626*^9, 3.514826543012664*^9}, {
   3.514826573528026*^9, 3.514826574581785*^9}, {3.514826728774248*^9, 
   3.514826758756071*^9}, {3.514828935462126*^9, 3.5148290533272142`*^9}, {
   3.5149096829953327`*^9, 3.514909723142671*^9}, {3.5149098576453648`*^9, 
   3.514909889448564*^9}, {3.514912127190716*^9, 3.51491213623287*^9}, {
   3.514912195772973*^9, 3.514912204226186*^9}, {3.514912303442165*^9, 
   3.5149123042055283`*^9}, 3.514921423035899*^9, {3.5150131270220003`*^9, 
   3.515013127442*^9}, {3.515013592144*^9, 3.515013629584*^9}, {
   3.5150136607939997`*^9, 3.515013665744*^9}, {3.515013883704*^9, 
   3.515013948974*^9}, {3.515014575494*^9, 3.515014585174*^9}, {
   3.515510466019574*^9, 3.5155104702861137`*^9}, {3.516032532081387*^9, 
   3.516032632290653*^9}, 3.516032718421742*^9, {3.516116258863737*^9, 
   3.516116280731205*^9}, {3.5163598914918003`*^9, 3.5163599052258*^9}, {
   3.5163605163677998`*^9, 3.5163605337567997`*^9}, {3.5163605964997997`*^9, 
   3.5163606026288*^9}, {3.5163608507637997`*^9, 3.5163608508698*^9}, {
   3.516382888805026*^9, 3.5163830068406553`*^9}, {3.516383277038055*^9, 
   3.5163833555156527`*^9}, {3.5163833998893337`*^9, 
   3.5163834453055067`*^9}, {3.516383476857959*^9, 3.51638351945615*^9}, {
   3.5163854577936897`*^9, 3.5163856049352503`*^9}, {3.516385952783477*^9, 
   3.516385986208107*^9}, {3.5163867387806797`*^9, 3.516386778242505*^9}, {
   3.516388278925934*^9, 3.516388314455687*^9}, {3.51638840971177*^9, 
   3.51638841247635*^9}, {3.516397565665082*^9, 3.5163976921604767`*^9}, {
   3.516397899028942*^9, 3.516397899322949*^9}, {3.5164076030158243`*^9, 
   3.516407638210791*^9}, {3.516408006778857*^9, 3.516408016021781*^9}, {
   3.51640975918292*^9, 3.516409767769142*^9}, {3.516457465894291*^9, 
   3.516457481056218*^9}, {3.516457559479982*^9, 3.516457610709281*^9}, {
   3.516457646901187*^9, 3.51645764800912*^9}, {3.516457700012754*^9, 
   3.516457786011052*^9}, {3.516457820881053*^9, 3.516458122097102*^9}, 
   3.51645820755171*^9, {3.516458351867426*^9, 3.5164584977762203`*^9}, {
   3.516458531496792*^9, 3.516458550968389*^9}, {3.516458649058075*^9, 
   3.516458748494729*^9}, {3.516458824332374*^9, 3.516458838505898*^9}, {
   3.5164588709359703`*^9, 3.516459125857703*^9}, {3.516459181357025*^9, 
   3.516459184898267*^9}, {3.516459232991765*^9, 3.516459242845731*^9}, {
   3.516459533041588*^9, 3.516459555124057*^9}, {3.516459631924409*^9, 
   3.516459679085615*^9}, {3.516460197000605*^9, 3.516460206808563*^9}, {
   3.5164602992598553`*^9, 3.516460311067243*^9}, {3.5164609129823503`*^9, 
   3.516460968519927*^9}, {3.516462676865677*^9, 3.516462684467883*^9}, {
   3.5164631772156067`*^9, 3.516463276695409*^9}, {3.516463488358333*^9, 
   3.516463493706519*^9}, {3.516463910894813*^9, 3.5164639193960238`*^9}, {
   3.516464097606217*^9, 3.51646419164781*^9}, 3.5164643626396723`*^9, {
   3.516464411015307*^9, 3.516464431083765*^9}, {3.5164646185531282`*^9, 
   3.516464634286481*^9}, {3.516465101313439*^9, 3.5164651163310337`*^9}, {
   3.516465390538862*^9, 3.516465401256546*^9}, {3.516465809366246*^9, 
   3.5164658223352957`*^9}, {3.516468032786314*^9, 3.51646805284263*^9}, {
   3.516468116424253*^9, 3.516468136815833*^9}, {3.516468247182317*^9, 
   3.516468349452812*^9}, {3.5164683848684*^9, 3.516468387922887*^9}, {
   3.5164685154818697`*^9, 3.5164685841468763`*^9}, {3.516469921949301*^9, 
   3.5164699711692543`*^9}, {3.5164700483876743`*^9, 
   3.5164701636501837`*^9}, {3.516471263308015*^9, 3.5164712746994658`*^9}, {
   3.516471352359535*^9, 3.5164713546883163`*^9}, {3.5164714262637653`*^9, 
   3.516471432301978*^9}, {3.516473517538863*^9, 3.516473523403447*^9}, {
   3.516474982691085*^9, 3.516474983982852*^9}, {3.516475459381082*^9, 
   3.5164755286888514`*^9}, {3.5178368227708817`*^9, 3.517836867940639*^9}, 
   3.517836922921733*^9, {3.5685748879394712`*^9, 3.568574942003587*^9}, {
   3.568574975568761*^9, 3.568575073669878*^9}, {3.568575199214121*^9, 
   3.5685753730555277`*^9}, {3.568575430728529*^9, 3.56857544416363*^9}, {
   3.568575639798094*^9, 3.5685757167565727`*^9}, {3.568575765502276*^9, 
   3.568575875843132*^9}, {3.568575964817031*^9, 3.568575996598225*^9}, {
   3.5685815788707113`*^9, 3.5685816231848097`*^9}, {3.568581802957261*^9, 
   3.5685818768577337`*^9}, {3.568583185807536*^9, 3.568583193174665*^9}, {
   3.568583564587915*^9, 3.568583650082548*^9}, {3.568585532768119*^9, 
   3.56858556451052*^9}, {3.56858638617171*^9, 3.568586411272897*^9}, {
   3.568586562870647*^9, 3.5685865750782146`*^9}, 3.568586700783231*^9, {
   3.5685869892028027`*^9, 3.568587050334168*^9}, {3.568587113451311*^9, 
   3.568587177428643*^9}, {3.568587208818534*^9, 3.568587278029192*^9}, {
   3.568587315618803*^9, 3.5685874272201843`*^9}, {3.5685875764556723`*^9, 
   3.568587688191224*^9}, {3.5685877325224733`*^9, 3.568588053803132*^9}, {
   3.568588118269617*^9, 3.5685881777593327`*^9}, 3.568645003388166*^9, {
   3.568645056952374*^9, 3.56864511318191*^9}, {3.568645202222413*^9, 
   3.5686452463478613`*^9}, {3.568645284955329*^9, 3.568645330647273*^9}, {
   3.568646749699089*^9, 3.568646767383565*^9}, {3.5686468184877357`*^9, 
   3.568646856166196*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Assembly", "Subsubsection",
 CellChangeTimes->{{3.56865121529117*^9, 3.568651216471094*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"J", "[", "4", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"r", ",", "end"}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"end", "=", "32"}], ";", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"InnerModa", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "*", 
             RowBox[{"Sin", "[", 
              RowBox[{"Pi", "/", "4"}], "]"}]}], "-", 
            RowBox[{"a", "[", 
             RowBox[{"s", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], "<", 
          RowBox[{"10.", "^", 
           RowBox[{"(", 
            RowBox[{"-", "13"}], ")"}]}]}], ",", 
         RowBox[{"end", "=", 
          RowBox[{"end", "-", "2"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"JJ", "[", "4", "]"}], "[", "i", "]"}], "[", 
           RowBox[{"x", ",", "t"}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "1", ",", "end"}], "}"}]}], "]"}], "//", 
        "Transpose"}]}]}], "\[IndentingNewLine]", "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Jsamp", "[", "4", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"r", ",", "end"}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"end", "=", "32"}], ";", "\[IndentingNewLine]", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"Abs", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"InnerModa", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "*", 
             RowBox[{"Sin", "[", 
              RowBox[{"Pi", "/", "4"}], "]"}]}], "-", 
            RowBox[{"a", "[", 
             RowBox[{"s", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], "<", 
          RowBox[{"10.", "^", 
           RowBox[{"(", 
            RowBox[{"-", "13"}], ")"}]}]}], ",", 
         RowBox[{"end", "=", 
          RowBox[{"end", "-", "2"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"JJsamp", "[", "4", "]"}], "[", "i", "]"}], "[", 
           RowBox[{"x", ",", "t"}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "1", ",", "end"}], "}"}]}], "]"}], "//", 
        "Transpose"}]}]}], "\[IndentingNewLine]", "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Jx", "[", "4", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"r", ",", "end"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"end", "=", "32"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"InnerModa", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "*", 
            RowBox[{"Sin", "[", 
             RowBox[{"Pi", "/", "4"}], "]"}]}], "-", 
           RowBox[{"a", "[", 
            RowBox[{"s", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], "<", 
         RowBox[{"10.", "^", 
          RowBox[{"(", 
           RowBox[{"-", "13"}], ")"}]}]}], ",", 
        RowBox[{"end", "=", 
         RowBox[{"end", "-", "2"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"JJx", "[", "4", "]"}], "[", "i", "]"}], "[", 
          RowBox[{"x", ",", "t"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "1", ",", "end"}], "}"}]}], "]"}], "//", 
       "Transpose"}]}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Jadapt", "[", "4", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"Adapt", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Jsamp", "[", "4", "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", "globalTol"}], "]"}], "[", 
    RowBox[{
     RowBox[{"J", "[", "4", "]"}], "[", 
     RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Jadaptx", "[", "4", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{
     RowBox[{"Adapt", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Jsamp", "[", "4", "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", "globalTol"}], "]"}], "[", 
     RowBox[{
      RowBox[{"Jx", "[", "4", "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"KdVRHPx", "[", "4", "]"}], "[", 
    RowBox[{"n_", ",", "S_", ",", "x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "A", ",", "pole", ",", "res", ",", "rc", ",", "i", ",", "r", ",", 
       "end"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"rc", " ", "=", " ", 
       RowBox[{"S", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"pole", "=", 
       RowBox[{"S", "[", 
        RowBox[{"[", "2", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"res", " ", "=", " ", 
       RowBox[{"S", "[", 
        RowBox[{"[", "3", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"end", "=", "28"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Abs", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"InnerModa", "[", 
             RowBox[{"x", ",", "t", ",", "pole"}], "]"}], "*", 
            RowBox[{"Sin", "[", 
             RowBox[{"Pi", "/", "4"}], "]"}]}], "-", 
           RowBox[{"a", "[", 
            RowBox[{"s", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}]}], "]"}], "<", 
         RowBox[{"10.", "^", 
          RowBox[{"(", 
           RowBox[{"-", "13"}], ")"}]}]}], ",", 
        RowBox[{"end", "=", 
         RowBox[{"end", "-", "2"}]}]}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"r", "=", 
       RowBox[{"Min", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{
           RowBox[{"ChooseRadius", "[", "pole", "]"}], "/", "2"}], ",", 
          ".1"}], "}"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"A", " ", "=", 
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"Jx", "[", "4", "]"}], "[", "i", "]"}], "[", 
          RowBox[{
          "n", ",", "x", ",", "t", ",", "rc", ",", "pole", ",", "res", ",", 
           "0"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "1", ",", "end"}], "}"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"i", " ", "=", "1"}], ",", " ", 
        RowBox[{"i", "\[LessEqual]", 
         RowBox[{"Length", "[", "res", "]"}]}], ",", 
        RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"A", " ", "=", " ", 
          RowBox[{"Join", "[", 
           RowBox[{"A", ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"PoleArray", "[", 
              RowBox[{
               RowBox[{"+", "1"}], ",", "r", ",", "n", ",", 
               RowBox[{
               "\[CapitalDelta]ki\[Phi]QtiStQt\[Phi]\[CapitalDelta]kx", "[", 
                RowBox[{"+", "1"}], "]"}]}], "]"}], "[", 
             RowBox[{
             "x", ",", "t", ",", "rc", ",", "pole", ",", "res", ",", "i"}], 
             "]"}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"PoleArray", "[", 
              RowBox[{
               RowBox[{"-", "1"}], ",", "r", ",", "n", ",", 
               RowBox[{
               "\[CapitalDelta]ki\[Phi]QtiStQt\[Phi]\[CapitalDelta]kx", "[", 
                RowBox[{"-", "1"}], "]"}]}], "]"}], "[", 
             RowBox[{
             "x", ",", "t", ",", "rc", ",", "pole", ",", "res", ",", "i"}], 
             "]"}]}], "]"}]}], ";"}]}], "\[IndentingNewLine]", "]"}], ";", 
      "\[IndentingNewLine]", "A"}]}], "]"}]}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.514825267948341*^9, 3.514825292676878*^9}, {
   3.5148253366078253`*^9, 3.514825427904633*^9}, {3.5148255311926517`*^9, 
   3.5148255943071117`*^9}, {3.514825669646573*^9, 3.514825745629958*^9}, {
   3.5148258988524733`*^9, 3.514825926785774*^9}, {3.5148261039825363`*^9, 
   3.514826258039207*^9}, {3.514826288823626*^9, 3.514826543012664*^9}, {
   3.514826573528026*^9, 3.514826574581785*^9}, {3.514826728774248*^9, 
   3.514826758756071*^9}, {3.514828935462126*^9, 3.5148290533272142`*^9}, {
   3.5149096829953327`*^9, 3.514909723142671*^9}, {3.5149098576453648`*^9, 
   3.514909889448564*^9}, {3.514912127190716*^9, 3.51491213623287*^9}, {
   3.514912195772973*^9, 3.514912204226186*^9}, {3.514912303442165*^9, 
   3.5149123042055283`*^9}, 3.514921423035899*^9, {3.5150131270220003`*^9, 
   3.515013127442*^9}, {3.515013592144*^9, 3.515013629584*^9}, {
   3.5150136607939997`*^9, 3.515013665744*^9}, {3.515013883704*^9, 
   3.515013948974*^9}, {3.515014575494*^9, 3.515014585174*^9}, {
   3.515510466019574*^9, 3.5155104702861137`*^9}, {3.516032532081387*^9, 
   3.516032632290653*^9}, 3.516032718421742*^9, {3.516116258863737*^9, 
   3.516116280731205*^9}, {3.5163598914918003`*^9, 3.5163599052258*^9}, {
   3.5163605163677998`*^9, 3.5163605337567997`*^9}, {3.5163605964997997`*^9, 
   3.5163606026288*^9}, {3.5163608507637997`*^9, 3.5163608508698*^9}, {
   3.516382888805026*^9, 3.5163830068406553`*^9}, {3.516383277038055*^9, 
   3.5163833555156527`*^9}, {3.5163833998893337`*^9, 
   3.5163834453055067`*^9}, {3.516383476857959*^9, 3.51638351945615*^9}, {
   3.5163854577936897`*^9, 3.5163856049352503`*^9}, {3.516385952783477*^9, 
   3.516385986208107*^9}, {3.5163867387806797`*^9, 3.516386778242505*^9}, {
   3.516388278925934*^9, 3.516388314455687*^9}, {3.51638840971177*^9, 
   3.51638841247635*^9}, {3.516397565665082*^9, 3.5163976921604767`*^9}, {
   3.516397899028942*^9, 3.516397899322949*^9}, {3.5164076030158243`*^9, 
   3.516407638210791*^9}, {3.516408006778857*^9, 3.516408016021781*^9}, {
   3.51640975918292*^9, 3.516409767769142*^9}, {3.516457465894291*^9, 
   3.516457481056218*^9}, {3.516457559479982*^9, 3.516457610709281*^9}, {
   3.516457646901187*^9, 3.51645764800912*^9}, {3.516457700012754*^9, 
   3.516457786011052*^9}, {3.516457820881053*^9, 3.516458122097102*^9}, 
   3.51645820755171*^9, {3.516458351867426*^9, 3.5164584977762203`*^9}, {
   3.516458531496792*^9, 3.516458550968389*^9}, {3.516458649058075*^9, 
   3.516458748494729*^9}, {3.516458824332374*^9, 3.516458838505898*^9}, {
   3.5164588709359703`*^9, 3.516459125857703*^9}, {3.516459181357025*^9, 
   3.516459184898267*^9}, {3.516459232991765*^9, 3.516459242845731*^9}, {
   3.516459533041588*^9, 3.516459555124057*^9}, {3.516459631924409*^9, 
   3.516459679085615*^9}, {3.516460197000605*^9, 3.516460206808563*^9}, {
   3.5164602992598553`*^9, 3.516460311067243*^9}, {3.5164609129823503`*^9, 
   3.516460968519927*^9}, {3.516462676865677*^9, 3.516462684467883*^9}, {
   3.5164631772156067`*^9, 3.516463276695409*^9}, {3.516463488358333*^9, 
   3.516463493706519*^9}, {3.516463910894813*^9, 3.5164639193960238`*^9}, {
   3.516464097606217*^9, 3.51646419164781*^9}, 3.5164643626396723`*^9, {
   3.516464411015307*^9, 3.516464431083765*^9}, {3.5164646185531282`*^9, 
   3.516464634286481*^9}, {3.516465101313439*^9, 3.5164651163310337`*^9}, {
   3.516465390538862*^9, 3.516465401256546*^9}, {3.516465809366246*^9, 
   3.5164658223352957`*^9}, {3.516468032786314*^9, 3.51646805284263*^9}, {
   3.516468116424253*^9, 3.516468136815833*^9}, {3.516468247182317*^9, 
   3.516468349452812*^9}, {3.5164683848684*^9, 3.516468387922887*^9}, {
   3.5164685154818697`*^9, 3.5164685841468763`*^9}, {3.516469921949301*^9, 
   3.5164699711692543`*^9}, {3.5164700483876743`*^9, 
   3.5164701636501837`*^9}, {3.516471263308015*^9, 3.5164712746994658`*^9}, {
   3.516471352359535*^9, 3.5164713546883163`*^9}, {3.5164714262637653`*^9, 
   3.516471432301978*^9}, {3.516473517538863*^9, 3.516473523403447*^9}, {
   3.516474982691085*^9, 3.516474983982852*^9}, {3.516475459381082*^9, 
   3.5164755286888514`*^9}, {3.5178368227708817`*^9, 3.517836867940639*^9}, 
   3.517836922921733*^9, {3.5685748879394712`*^9, 3.568574942003587*^9}, {
   3.568574975568761*^9, 3.568575073669878*^9}, {3.568575199214121*^9, 
   3.5685753730555277`*^9}, {3.568575430728529*^9, 3.56857544416363*^9}, {
   3.568575639798094*^9, 3.5685757167565727`*^9}, {3.568575765502276*^9, 
   3.568575875843132*^9}, {3.568575964817031*^9, 3.568575996598225*^9}, {
   3.5685815788707113`*^9, 3.5685816231848097`*^9}, {3.568581802957261*^9, 
   3.5685818768577337`*^9}, {3.568583185807536*^9, 3.568583193174665*^9}, {
   3.568583564587915*^9, 3.568583650082548*^9}, {3.568585532768119*^9, 
   3.56858556451052*^9}, {3.56858638617171*^9, 3.568586411272897*^9}, {
   3.568586562870647*^9, 3.5685865750782146`*^9}, 3.568586700783231*^9, {
   3.5685869892028027`*^9, 3.568587050334168*^9}, {3.568587113451311*^9, 
   3.568587177428643*^9}, {3.568587208818534*^9, 3.568587278029192*^9}, {
   3.568587315618803*^9, 3.5685874272201843`*^9}, {3.5685875764556723`*^9, 
   3.568587688191224*^9}, {3.5685877325224733`*^9, 3.568588053803132*^9}, {
   3.568588118269617*^9, 3.5685881777593327`*^9}, 3.568645003388166*^9, {
   3.56864533834613*^9, 3.568645346114463*^9}, {3.5686454301385927`*^9, 
   3.568645448411878*^9}, {3.56864686230995*^9, 3.5686468698342247`*^9}, {
   3.568651210061797*^9, 3.568651279018854*^9}}]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Region 5: Transition", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.568556583304967*^9, 3.5685565921082973`*^9}, {
  3.568648646130331*^9, 3.568648659262241*^9}}],

Cell[CellGroupData[{

Cell["Find a1 and b1", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.568552120247838*^9, 3.568552123479507*^9}, {
  3.568571262982126*^9, 3.568571265457403*^9}, {3.5686487050098553`*^9, 
  3.568648706535089*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"EllipInt1", "[", 
    RowBox[{"a_", ",", "b_", ",", "n_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"Fun", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Sqrt", "[", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"a", "^", "2"}], "-", 
           RowBox[{"#", "^", "2"}]}], ")"}], 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"b", "^", "2"}], "-", 
           RowBox[{"#", "^", "2"}]}], ")"}]}], "]"}], "&"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"{", 
        RowBox[{"0", ",", "a"}], "}"}], "]"}], ",", "n"}], "]"}], "//", 
    "DomainIntegrate"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"EllipInta1", "[", 
    RowBox[{"a_", ",", "n_"}], "]"}], ":=", 
   RowBox[{"24", "*", 
    RowBox[{"EllipInt1", "[", 
     RowBox[{"a", ",", 
      RowBox[{"Sqrt", "[", 
       RowBox[{"2", "-", 
        RowBox[{"a", "^", "2"}]}], "]"}], ",", "n"}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"RootInt1", "[", 
    RowBox[{"s_", ",", "n_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "F", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"F", "[", "A_", "]"}], ":=", 
       RowBox[{"s", " ", "-", 
        RowBox[{"EllipInta1", "[", 
         RowBox[{"A", ",", "n"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
      "F"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"EllipFun1", "[", 
    RowBox[{"s_", ",", "n_"}], "]"}], ":=", 
   RowBox[{"Fun", "[", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"RootInt1", "[", 
        RowBox[{"s", ",", "n"}], "]"}], "[", "#", "]"}], "&"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{"0", ",", "1"}], "}"}], "]"}], ",", "n"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[Alpha]\[Alpha]Newton1", "[", 
    RowBox[{"s_", ",", "n_"}], "]"}], ":=", " ", 
   RowBox[{"mNewtonMethod", "[", 
    RowBox[{
     RowBox[{"EllipFun1", "[", 
      RowBox[{"s", ",", "n"}], "]"}], ",", ".5"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"a1", "//", "Clear"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"a1", "[", "s_", "]"}], ":=", 
   RowBox[{
    RowBox[{"a1", "[", "s", "]"}], "=", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Alpha]\[Alpha]Newton1", "[", 
       RowBox[{"s", ",", "100"}], "]"}], "[", "100", "]"}], "//", "Re"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"a1", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"a1", "[", 
    RowBox[{".1", "/", 
     RowBox[{"(", 
      RowBox[{"tau", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ")"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"b1", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Sqrt", "[", 
    RowBox[{"2", "-", 
     RowBox[{
      RowBox[{"a1", "[", 
       RowBox[{"x", ",", "t"}], "]"}], "^", "2"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"b1", "[", "s_", "]"}], ":=", 
   RowBox[{"Sqrt", "[", 
    RowBox[{"2", "-", 
     RowBox[{
      RowBox[{"a1", "[", "s", "]"}], "^", "2"}]}], "]"}]}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.568553397555667*^9, {3.568553449403825*^9, 3.56855345203559*^9}, {
   3.568553772837674*^9, 3.568553818750169*^9}, {3.568554052643714*^9, 
   3.568554063818709*^9}, {3.568554160134904*^9, 3.568554161391015*^9}, {
   3.568554205088578*^9, 3.568554211839402*^9}, {3.568554324036275*^9, 
   3.5685544847216873`*^9}, {3.568554601326907*^9, 3.568554752249818*^9}, {
   3.5685547965331593`*^9, 3.568554799270925*^9}, {3.568554846408194*^9, 
   3.5685548784821177`*^9}, 3.5685553326636887`*^9, {3.568555426372445*^9, 
   3.5685554997613087`*^9}, {3.568556654363599*^9, 3.568556699083126*^9}, {
   3.568556765603969*^9, 3.5685567777584047`*^9}, {3.568556811337154*^9, 
   3.568556864286745*^9}, {3.568556978774241*^9, 3.568557019646803*^9}, 
   3.568557128104793*^9, {3.5685604088845882`*^9, 3.568560423468215*^9}, {
   3.568570447942103*^9, 3.568570448377982*^9}, {3.568571181392728*^9, 
   3.568571181747373*^9}, {3.568571293253327*^9, 3.568571303543618*^9}, {
   3.568648680469138*^9, 3.568648716457201*^9}, {3.568648814137488*^9, 
   3.568648870367041*^9}, 3.5686489407346277`*^9, {3.568651957652066*^9, 
   3.5686519675427637`*^9}, {3.568652202192791*^9, 3.56865220251999*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Parametrices, etc.", "Subsection",
 CellChangeTimes->{{3.5686489524171953`*^9, 3.5686489578650503`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ci1", "//", "Clear"}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"ci1", "[", 
    RowBox[{"xx_", ",", "tt_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"ci1", "[", 
     RowBox[{"xx", ",", "tt"}], "]"}], "=", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Fun", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"\[Theta]p", "[", 
           RowBox[{"#", ",", "xx", ",", "tt"}], "]"}]}], "&"}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Line", "[", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", 
              RowBox[{"b1", "[", 
               RowBox[{"xx", ",", "tt"}], "]"}]}], ",", 
             RowBox[{"-", 
              RowBox[{"a1", "[", 
               RowBox[{"xx", ",", "tt"}], "]"}]}]}], "}"}], "]"}], ",", 
          RowBox[{"Line", "[", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"a1", "[", 
              RowBox[{"xx", ",", "tt"}], "]"}], ",", 
             RowBox[{"b1", "[", 
              RowBox[{"xx", ",", "tt"}], "]"}]}], "}"}], "]"}]}], "}"}], ",", 
        
        RowBox[{"{", 
         RowBox[{"30", ",", "30"}], "}"}]}], "]"}], "//", 
      "CauchyInverseCurves"}], ")"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"TransitionGFUN", "[", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", "\n", 
     RowBox[{
      RowBox[{"gfun", "//", "Clear"}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"gfun", "[", 
        RowBox[{"p_", ",", "z_", ",", "xx_", ",", "tt_"}], "]"}], ":=", 
       RowBox[{
        RowBox[{"gfun", "[", 
         RowBox[{"p", ",", "z", ",", "xx", ",", "tt"}], "]"}], "=", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"CauchyInverseIntegral", "[", 
            RowBox[{"p", ",", "#", ",", "z"}], "]"}], "&"}], "/@", 
          RowBox[{"ci1", "[", 
           RowBox[{"xx", ",", "tt"}], "]"}]}], "//", "Total"}]}]}], ";"}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Generate\[Zeta]t", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "eps", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"eps", " ", "=", " ", 
       RowBox[{"$MachineEpsilon", "*", "10"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"\[Zeta]", "//", "Clear"}], ";", "\[IndentingNewLine]", 
      RowBox[{"glim11", " ", "=", " ", 
       RowBox[{"Fun", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"gfun", "[", 
            RowBox[{"1", ",", "#", ",", "x", ",", "t"}], "]"}], "-", 
           RowBox[{"gfun", "[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", "#", ",", "x", ",", "t"}], "]"}]}], 
          "&"}], ",", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"b1", "[", 
              RowBox[{"x", ",", "t"}], "]"}]}], ",", 
            RowBox[{"-", 
             RowBox[{"a1", "[", 
              RowBox[{"x", ",", "t"}], "]"}]}]}], "}"}], "]"}], ",", "50"}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"glim21", " ", "=", " ", 
       RowBox[{"Fun", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"gfun", "[", 
            RowBox[{"1", ",", "#", ",", "x", ",", "t"}], "]"}], "-", 
           RowBox[{"gfun", "[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", "#", ",", "x", ",", "t"}], "]"}]}], 
          "&"}], ",", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"a1", "[", 
             RowBox[{"x", ",", "t"}], "]"}], ",", 
            RowBox[{"b1", "[", 
             RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}], ",", "50"}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"\[Zeta]", "[", 
          RowBox[{"+", "1"}], "]"}], "[", 
         RowBox[{"+", "1"}], "]"}], "[", "k_", "]"}], ":=", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[Zeta]", "[", 
           RowBox[{"+", "1"}], "]"}], "[", 
          RowBox[{"+", "1"}], "]"}], "[", "k", "]"}], "=", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"Cauchy", "[", 
           RowBox[{"glim21", ",", 
            RowBox[{"k", "+", 
             RowBox[{"I", " ", "eps"}]}]}], "]"}], ")"}], "//", "Exp"}]}]}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"\[Zeta]", "[", 
          RowBox[{"-", "1"}], "]"}], "[", 
         RowBox[{"+", "1"}], "]"}], "[", "k_", "]"}], ":=", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[Zeta]", "[", 
           RowBox[{"-", "1"}], "]"}], "[", 
          RowBox[{"+", "1"}], "]"}], "[", "k", "]"}], "=", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"Cauchy", "[", 
           RowBox[{"glim21", ",", 
            RowBox[{"k", "-", 
             RowBox[{"I", " ", "eps"}]}]}], "]"}], ")"}], "//", "Exp"}]}]}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"\[Zeta]", "[", 
          RowBox[{"+", "1"}], "]"}], "[", 
         RowBox[{"-", "1"}], "]"}], "[", "k_", "]"}], ":=", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[Zeta]", "[", 
           RowBox[{"+", "1"}], "]"}], "[", 
          RowBox[{"-", "1"}], "]"}], "[", "k", "]"}], "=", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"Cauchy", "[", 
           RowBox[{"glim11", ",", 
            RowBox[{"k", "+", 
             RowBox[{"I", " ", "eps"}]}]}], "]"}], ")"}], "//", "Exp"}]}]}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"\[Zeta]", "[", 
          RowBox[{"-", "1"}], "]"}], "[", 
         RowBox[{"-", "1"}], "]"}], "[", "k_", "]"}], ":=", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"\[Zeta]", "[", 
           RowBox[{"-", "1"}], "]"}], "[", 
          RowBox[{"-", "1"}], "]"}], "[", "k", "]"}], "=", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"Cauchy", "[", 
           RowBox[{"glim11", ",", 
            RowBox[{"k", "-", 
             RowBox[{"I", " ", "eps"}]}]}], "]"}], ")"}], "//", "Exp"}]}]}], 
      ";"}]}], "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Psi]1", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{
        RowBox[{"-", 
         RowBox[{"ConstInt", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"gfun", "[", 
             RowBox[{"1", ",", "0.", ",", "x", ",", "t"}], "]"}], "-", 
            RowBox[{"gfun", "[", 
             RowBox[{
              RowBox[{"-", "1"}], ",", "0.", ",", "x", ",", "t"}], "]"}]}], 
           ",", 
           RowBox[{"a1", "[", 
            RowBox[{"x", ",", "t"}], "]"}], ",", 
           RowBox[{"k", "+", 
            RowBox[{"I", "*", 
             RowBox[{"10", "^", 
              RowBox[{"(", 
               RowBox[{"-", "16"}], ")"}]}]}]}]}], "]"}]}], "//", "Exp"}], 
       "0"},
      {"0", 
       RowBox[{
        RowBox[{"ConstInt", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"gfun", "[", 
            RowBox[{"1", ",", "0.", ",", "x", ",", "t"}], "]"}], "-", 
           RowBox[{"gfun", "[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", "0.", ",", "x", ",", "t"}], "]"}]}], 
          ",", 
          RowBox[{"a1", "[", 
           RowBox[{"x", ",", "t"}], "]"}], ",", 
          RowBox[{"k", "+", 
           RowBox[{"I", "*", 
            RowBox[{"10", "^", 
             RowBox[{"(", 
              RowBox[{"-", "16"}], ")"}]}]}]}]}], "]"}], "//", "Exp"}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Psi]1", "[", 
      RowBox[{"-", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{
        RowBox[{"-", 
         RowBox[{"ConstInt", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"gfun", "[", 
             RowBox[{"1", ",", "0.", ",", "x", ",", "t"}], "]"}], "-", 
            RowBox[{"gfun", "[", 
             RowBox[{
              RowBox[{"-", "1"}], ",", "0.", ",", "x", ",", "t"}], "]"}]}], 
           ",", 
           RowBox[{"a1", "[", 
            RowBox[{"x", ",", "t"}], "]"}], ",", 
           RowBox[{"k", "-", 
            RowBox[{"I", "*", 
             RowBox[{"10", "^", 
              RowBox[{"(", 
               RowBox[{"-", "16"}], ")"}]}]}]}]}], "]"}]}], "//", "Exp"}], 
       "0"},
      {"0", 
       RowBox[{
        RowBox[{"ConstInt", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"gfun", "[", 
            RowBox[{"1", ",", "0.", ",", "x", ",", "t"}], "]"}], "-", 
           RowBox[{"gfun", "[", 
            RowBox[{
             RowBox[{"-", "1"}], ",", "0.", ",", "x", ",", "t"}], "]"}]}], 
          ",", 
          RowBox[{"a1", "[", 
           RowBox[{"x", ",", "t"}], "]"}], ",", 
          RowBox[{"k", "-", 
           RowBox[{"I", "*", 
            RowBox[{"10", "^", 
             RowBox[{"(", 
              RowBox[{"-", "16"}], ")"}]}]}]}]}], "]"}], "//", "Exp"}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ConstInt", "[", 
    RowBox[{"cc_", ",", "aa_", ",", "z_"}], "]"}], ":=", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", "cc", " ", 
     RowBox[{"ArcTanh", "[", 
      FractionBox["aa", "z"], "]"}]}], "\[Pi]"]}], 
  ";"}], "\[IndentingNewLine]"}], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.568553397555667*^9, {3.568553449403825*^9, 3.56855345203559*^9}, {
   3.568553772837674*^9, 3.568553818750169*^9}, {3.568554052643714*^9, 
   3.568554063818709*^9}, {3.568554160134904*^9, 3.568554161391015*^9}, {
   3.568554205088578*^9, 3.568554211839402*^9}, {3.568554324036275*^9, 
   3.5685544847216873`*^9}, {3.568554601326907*^9, 3.568554752249818*^9}, {
   3.5685547965331593`*^9, 3.568554799270925*^9}, {3.568554846408194*^9, 
   3.5685548784821177`*^9}, 3.5685553326636887`*^9, {3.568555426372445*^9, 
   3.5685554997613087`*^9}, {3.568556654363599*^9, 3.568556699083126*^9}, {
   3.568556765603969*^9, 3.5685567777584047`*^9}, {3.568556811337154*^9, 
   3.568556864286745*^9}, {3.568556978774241*^9, 3.568557019646803*^9}, 
   3.568557128104793*^9, {3.5685604088845882`*^9, 3.568560423468215*^9}, {
   3.568570447942103*^9, 3.568570448377982*^9}, {3.568571181392728*^9, 
   3.568571181747373*^9}, {3.568571293253327*^9, 3.568571303543618*^9}, {
   3.568648680469138*^9, 3.568648716457201*^9}, {3.568648814137488*^9, 
   3.568648870367041*^9}, 3.5686489407346277`*^9, {3.568648970847946*^9, 
   3.5686489989966087`*^9}, {3.568649066560121*^9, 3.568649072709372*^9}, 
   3.568651562312509*^9, {3.568651801063848*^9, 3.568651869189157*^9}, {
   3.568652112175404*^9, 3.568652163038739*^9}, {3.5686524909628153`*^9, 
   3.568652491164792*^9}, {3.568653058627043*^9, 3.568653070540358*^9}, {
   3.5686583320482807`*^9, 3.568658339454233*^9}, {3.5686587468688507`*^9, 
   3.568658747144731*^9}, {3.568659576283032*^9, 3.568659583489526*^9}, {
   3.568663890632555*^9, 3.56866392529419*^9}, {3.568665469948277*^9, 
   3.5686654758827457`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Multiplied jumps", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.568573155255061*^9, 3.568573158829356*^9}}],

Cell[CellGroupData[{

Cell["Normal jumps", "Subsubsection",
 CellChangeTimes->{{3.568643674322945*^9, 3.5686436761966553`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"\[Phi]QinsPsQs\[Phi]", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Ps", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]QinsPinsQs\[Phi]", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"MultFun", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"InvFun", "[", 
        RowBox[{
         RowBox[{"\[Phi]", "[", 
          RowBox[{"+", "1"}], "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"InvFun", "[", 
        RowBox[{"Qs", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"InvFun", "[", 
        RowBox[{"Ps", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]QinsUinsQs\[Phi]", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"MultFun", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"InvFun", "[", 
         RowBox[{
          RowBox[{"\[Phi]", "[", 
           RowBox[{"+", "1"}], "]"}], "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
        RowBox[{"InvFun", "[", 
         RowBox[{"Qs", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
        RowBox[{"InvFun", "[", 
         RowBox[{"Us", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
        RowBox[{"Qs", "[", 
         RowBox[{"x", ",", "t"}], "]"}], ",", 
        RowBox[{
         RowBox[{"\[Phi]", "[", 
          RowBox[{"+", "1"}], "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], ";"}], 
   "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[Phi]QinsMsQs\[Phi]", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Ms", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]QinsGsQs\[Phi]", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"MultFun", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"InvFun", "[", 
        RowBox[{
         RowBox[{"\[Phi]", "[", 
          RowBox[{"-", "1"}], "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"InvFun", "[", 
        RowBox[{"Qs", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"Gs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]QinsLsQs\[Phi]", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"MultFun", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"InvFun", "[", 
         RowBox[{
          RowBox[{"\[Phi]", "[", 
           RowBox[{"-", "1"}], "]"}], "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
        RowBox[{"InvFun", "[", 
         RowBox[{"Qs", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
        RowBox[{"Ls", "[", 
         RowBox[{"x", ",", "t"}], "]"}], ",", 
        RowBox[{"Qs", "[", 
         RowBox[{"x", ",", "t"}], "]"}], ",", 
        RowBox[{
         RowBox[{"\[Phi]", "[", 
          RowBox[{"-", "1"}], "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], ";", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsUsQs\[Phi]", "[", 
       RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
     RowBox[{
      RowBox[{"MultFun", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"InvFun", "[", 
          RowBox[{
           RowBox[{"\[Phi]", "[", 
            RowBox[{"+", "1"}], "]"}], "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
         RowBox[{"InvFun", "[", 
          RowBox[{"Qs", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
         RowBox[{"Us", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ",", 
         RowBox[{"Qs", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ",", 
         RowBox[{
          RowBox[{"\[Phi]", "[", 
           RowBox[{"+", "1"}], "]"}], "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}], "[", "k", "]"}]}], 
    ";"}], "*)"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]QinsPinsQs\[Phi]a", "[", 
     RowBox[{"+", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Pins", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"Z", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"+", "1"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Psi]1", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]QinsMsQs\[Phi]a", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"MultFun", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"InvFun", "[", 
        RowBox[{
         RowBox[{"\[Phi]", "[", 
          RowBox[{"-", "1"}], "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"InvFun", "[", 
        RowBox[{"Qs", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"Ms", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"Z", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"+", "1"}], "]"}], ",", 
       RowBox[{
        RowBox[{"\[Psi]1", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]QinsPinsQs\[Phi]a", "[", 
     RowBox[{"-", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Pins", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"Z", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"-", "1"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Psi]1", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]QinsMsQs\[Phi]a", "[", 
      RowBox[{"-", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"MultFun", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"InvFun", "[", 
        RowBox[{
         RowBox[{"\[Phi]", "[", 
          RowBox[{"-", "1"}], "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"InvFun", "[", 
        RowBox[{"Qs", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"Ms", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"Z", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"-", "1"}], "]"}], ",", 
       RowBox[{
        RowBox[{"\[Psi]1", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]QinsPinsQs\[Phi]b", "[", 
     RowBox[{"+", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Pins", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"Z", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"+", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]QinsMsQs\[Phi]b", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"MultFun", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"InvFun", "[", 
        RowBox[{
         RowBox[{"\[Phi]", "[", 
          RowBox[{"-", "1"}], "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"InvFun", "[", 
        RowBox[{"Qs", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"Ms", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"Z", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"+", "1"}], "]"}]}], "}"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]QinsPinsQs\[Phi]b", "[", 
     RowBox[{"-", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Pins", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"Z", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"-", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]QinsMsQs\[Phi]b", "[", 
     RowBox[{"-", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Ms", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"Z", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"-", "1"}], "]"}]}], "}"}], "]"}]}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.513711472307995*^9, 3.51371150688424*^9}, {
   3.513711559410595*^9, 3.513711606964553*^9}, {3.513711648397517*^9, 
   3.513711936409362*^9}, {3.5139442600508213`*^9, 3.513944356354691*^9}, {
   3.513944486092828*^9, 3.5139446411607943`*^9}, {3.513951549753932*^9, 
   3.513951699728924*^9}, {3.5139517397524*^9, 3.5139518129172707`*^9}, {
   3.513952232831436*^9, 3.513952249103897*^9}, {3.513953371131137*^9, 
   3.513953378053*^9}, {3.513953475209831*^9, 3.5139534934360037`*^9}, {
   3.513953588050322*^9, 3.513953588555484*^9}, {3.513953862581882*^9, 
   3.513953869531641*^9}, {3.513953913176754*^9, 3.51395391839988*^9}, {
   3.513954124364752*^9, 3.513954124771585*^9}, {3.513963109152478*^9, 
   3.513963188824473*^9}, {3.513963351166205*^9, 3.513963487001848*^9}, {
   3.513963527828042*^9, 3.51396353762601*^9}, {3.5139635929170723`*^9, 
   3.5139636917580643`*^9}, {3.5139637974682493`*^9, 3.513963802783566*^9}, {
   3.513963881140173*^9, 3.513963913320346*^9}, {3.513964002763595*^9, 
   3.5139640804451647`*^9}, {3.5139641459974127`*^9, 
   3.5139642689184113`*^9}, {3.513964342352106*^9, 3.513964382632668*^9}, {
   3.5139644334878397`*^9, 3.513964445439907*^9}, {3.513965136438246*^9, 
   3.513965142087782*^9}, {3.513968925722913*^9, 3.51396897056528*^9}, {
   3.51482264314016*^9, 3.514822674472425*^9}, {3.514823299014048*^9, 
   3.514823433520165*^9}, {3.5148259836929407`*^9, 3.514826021246005*^9}, {
   3.5148260568909473`*^9, 3.514826093709059*^9}, {3.514829266130872*^9, 
   3.514829267269014*^9}, 3.514907738691799*^9, 3.514909645865958*^9, {
   3.514909725826643*^9, 3.514909749867633*^9}, {3.514909835600062*^9, 
   3.514909849756949*^9}, {3.514909927551237*^9, 3.514909958608143*^9}, {
   3.51490999142306*^9, 3.514910014969368*^9}, {3.514910105977624*^9, 
   3.514910107150621*^9}, {3.514910138545768*^9, 3.514910187921351*^9}, {
   3.514910242162261*^9, 3.514910545155949*^9}, {3.514910580487647*^9, 
   3.514910592845335*^9}, {3.514910650014436*^9, 3.514910654005959*^9}, 
   3.51491115722192*^9, {3.51491234211771*^9, 3.514912347366127*^9}, {
   3.514913668250718*^9, 3.514913684476367*^9}, {3.51491537864764*^9, 
   3.5149154086481857`*^9}, {3.514916004664308*^9, 3.514916111631596*^9}, {
   3.514916343684166*^9, 3.514916350939144*^9}, {3.514919628933803*^9, 
   3.514919629374105*^9}, {3.514919814051456*^9, 3.51491981431933*^9}, {
   3.5149210615292788`*^9, 3.5149210618093157`*^9}, {3.514921376696*^9, 
   3.514921409459235*^9}, {3.514921550975163*^9, 3.514921553118926*^9}, {
   3.5149216002292137`*^9, 3.514921601651573*^9}, {3.514921714355473*^9, 
   3.5149217145617943`*^9}, {3.514921849323975*^9, 3.51492184966441*^9}, {
   3.515014018484*^9, 3.5150140807939997`*^9}, {3.515014414274*^9, 
   3.515014419104*^9}, {3.515014499744*^9, 3.515014556984*^9}, {
   3.516031623738676*^9, 3.51603164993892*^9}, {3.5163613960938*^9, 
   3.5163614987928*^9}, {3.5163621515938*^9, 3.5163622450538*^9}, {
   3.5163623032008*^9, 3.5163623226138*^9}, {3.5163624113227997`*^9, 
   3.5163624414667997`*^9}, {3.5163625423188*^9, 3.5163625712748003`*^9}, {
   3.5163626344598*^9, 3.5163626494598*^9}, {3.5163626897988*^9, 
   3.5163627142928*^9}, {3.5163629553128*^9, 3.5163629783718*^9}, {
   3.5163630349268*^9, 3.5163630547258*^9}, {3.5163630874538*^9, 
   3.5163631165657997`*^9}, {3.5163631505308*^9, 3.5163631901438*^9}, {
   3.5163633288018*^9, 3.5163633291317997`*^9}, {3.5163633932418003`*^9, 
   3.5163634088087997`*^9}, {3.516369647737257*^9, 3.516369666853775*^9}, {
   3.516371046201543*^9, 3.516371081347692*^9}, {3.516371202900103*^9, 
   3.516371207766354*^9}, {3.51637143189148*^9, 3.516371468582984*^9}, {
   3.516371641584278*^9, 3.5163716420861*^9}, 3.516371673320703*^9, {
   3.516371728651638*^9, 3.516371729847763*^9}, {3.516382638594038*^9, 
   3.51638286034973*^9}, {3.516383039819622*^9, 3.516383111119877*^9}, {
   3.516383169368149*^9, 3.516383194658242*^9}, {3.5163838852752867`*^9, 
   3.516383951945406*^9}, {3.5163850545636597`*^9, 3.516385067352995*^9}, {
   3.51638526865587*^9, 3.51638530022193*^9}, {3.5163853567862177`*^9, 
   3.51638542827841*^9}, {3.5163859079277563`*^9, 3.516385912132997*^9}, {
   3.516386199386607*^9, 3.516386213286777*^9}, {3.516386388653233*^9, 
   3.5163863996118517`*^9}, {3.5163867067406063`*^9, 3.516386715759712*^9}, 
   3.516396963584313*^9, {3.516397541951666*^9, 3.5163975535653048`*^9}, 
   3.516457120472746*^9, {3.516457151655912*^9, 3.516457235975576*^9}, {
   3.516457284491362*^9, 3.5164573214514093`*^9}, {3.516457387922867*^9, 
   3.516457454287363*^9}, {3.516457625199349*^9, 3.516457672991251*^9}, {
   3.51645831600738*^9, 3.5164583295493793`*^9}, 3.516458360081263*^9, {
   3.516458779434746*^9, 3.516458802316041*^9}, {3.516459394550309*^9, 
   3.5164595175171347`*^9}, {3.516459604960169*^9, 3.516459625340931*^9}, {
   3.5164596662234*^9, 3.516459666637341*^9}, {3.516459998653577*^9, 
   3.516460006124365*^9}, {3.516460354942031*^9, 3.516460396324935*^9}, {
   3.51646053069876*^9, 3.516460614685628*^9}, {3.5164607527937717`*^9, 
   3.516460897803651*^9}, {3.516462501534315*^9, 3.516462526604961*^9}, {
   3.516462611112651*^9, 3.516462611383815*^9}, {3.516463021154703*^9, 
   3.516463061529895*^9}, {3.5164633717215548`*^9, 3.5164633976508627`*^9}, {
   3.516463429116269*^9, 3.516463481201078*^9}, {3.5164638398603697`*^9, 
   3.5164638593406963`*^9}, {3.516464312638464*^9, 3.516464312912916*^9}, {
   3.516464914674979*^9, 3.516465085200768*^9}, {3.516465337676579*^9, 
   3.516465372496331*^9}, {3.516465405274722*^9, 3.516465406270146*^9}, {
   3.516465552779982*^9, 3.516465588691825*^9}, {3.5164672510061007`*^9, 
   3.516467293942857*^9}, {3.516467474603541*^9, 3.5164675231703978`*^9}, {
   3.51646760024851*^9, 3.516467739960421*^9}, {3.5164677717917137`*^9, 
   3.516467988712447*^9}, {3.516469159296556*^9, 3.516469161955163*^9}, {
   3.5164698074427032`*^9, 3.516469904905406*^9}, {3.5164707554820213`*^9, 
   3.516470756292307*^9}, {3.516471149654396*^9, 3.516471249685629*^9}, {
   3.516471405122237*^9, 3.516471412333644*^9}, {3.516472156919051*^9, 
   3.51647219331638*^9}, {3.516472854954361*^9, 3.5164729609522753`*^9}, {
   3.568572773889769*^9, 3.5685728183678083`*^9}, {3.568573181918784*^9, 
   3.5685732296762877`*^9}, {3.568573342255867*^9, 3.568573749688448*^9}, {
   3.568573787742181*^9, 3.568573813907329*^9}, {3.568573850661203*^9, 
   3.568573919559573*^9}, {3.5685739707975807`*^9, 3.5685739999250727`*^9}, {
   3.568574594174342*^9, 3.568574676348609*^9}, {3.568574730907544*^9, 
   3.568574813674015*^9}, {3.568575547486279*^9, 3.568575589757934*^9}, {
   3.5685834303597527`*^9, 3.568583436207259*^9}, {3.568584066859976*^9, 
   3.5685841512933702`*^9}, {3.568584762036027*^9, 3.568584762654573*^9}, {
   3.5685848204813633`*^9, 3.568584839282544*^9}, 3.568643683007687*^9, {
   3.568649147868464*^9, 3.568649364926387*^9}, {3.568652918503582*^9, 
   3.56865293091017*^9}, {3.568653015556768*^9, 3.568653024430928*^9}, {
   3.568659656154059*^9, 3.568659744463448*^9}, {3.568659778894001*^9, 
   3.568659779745195*^9}, {3.5686637404458857`*^9, 3.5686637450530252`*^9}, {
   3.568663780948853*^9, 3.568663841357082*^9}, {3.5686639458052673`*^9, 
   3.568664068187275*^9}, {3.56866410906429*^9, 3.568664162978807*^9}, {
   3.568664210269896*^9, 3.5686642125527067`*^9}, {3.568664340654105*^9, 
   3.5686644446758537`*^9}, {3.5686645203752193`*^9, 3.56866453379126*^9}, {
   3.568665014329318*^9, 3.568665016680511*^9}, {3.568665804195015*^9, 
   3.568665805533679*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Sample jumps", "Subsubsection",
 CellChangeTimes->{{3.568643674322945*^9, 3.5686436761966553`*^9}, {
  3.5686437676644373`*^9, 3.568643768960115*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"\[Phi]QinsPsQs\[Phi]samp", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Psamps", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]samp", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[Phi]QinsPinsQs\[Phi]samp", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"InvFun", "[", 
        RowBox[{
         RowBox[{"\[Phi]samp", "[", 
          RowBox[{"+", "1"}], "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], 
       RowBox[{"InvFun", "[", 
        RowBox[{"Psamps", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ",", 
      RowBox[{
       RowBox[{"\[Phi]samp", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[Phi]QinsMsQs\[Phi]samp", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Msamps", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]samp", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]QinsGsQs\[Phi]samp", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"MultFun", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"InvFun", "[", 
        RowBox[{
         RowBox[{"\[Phi]samp", "[", 
          RowBox[{"-", "1"}], "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"Gsamps", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]QinsPinsQs\[Phi]asamp", "[", 
     RowBox[{"+", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Pinsamps", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]samp", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]QinsMsQs\[Phi]asamp", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"MultFun", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"InvFun", "[", 
        RowBox[{
         RowBox[{"\[Phi]samp", "[", 
          RowBox[{"-", "1"}], "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"Msamps", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]QinsPinsQs\[Phi]asamp", "[", 
     RowBox[{"-", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Pinsamps", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]samp", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]QinsMsQs\[Phi]asamp", "[", 
      RowBox[{"-", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"MultFun", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"InvFun", "[", 
        RowBox[{
         RowBox[{"\[Phi]samp", "[", 
          RowBox[{"-", "1"}], "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"Msamps", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]QinsPinsQs\[Phi]bsamp", "[", 
     RowBox[{"+", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Pinsamps", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]samp", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]QinsMsQs\[Phi]bsamp", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"MultFun", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"InvFun", "[", 
        RowBox[{
         RowBox[{"\[Phi]samp", "[", 
          RowBox[{"-", "1"}], "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"Msamps", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]QinsPinsQs\[Phi]bsamp", "[", 
     RowBox[{"-", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Pinsamps", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]samp", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]QinsMsQs\[Phi]bsamp", "[", 
     RowBox[{"-", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]samp", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Msamps", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]samp", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.513711472307995*^9, 3.51371150688424*^9}, {
   3.513711559410595*^9, 3.513711606964553*^9}, {3.513711648397517*^9, 
   3.513711936409362*^9}, {3.5139442600508213`*^9, 3.513944356354691*^9}, {
   3.513944486092828*^9, 3.5139446411607943`*^9}, {3.513951549753932*^9, 
   3.513951699728924*^9}, {3.5139517397524*^9, 3.5139518129172707`*^9}, {
   3.513952232831436*^9, 3.513952249103897*^9}, {3.513953371131137*^9, 
   3.513953378053*^9}, {3.513953475209831*^9, 3.5139534934360037`*^9}, {
   3.513953588050322*^9, 3.513953588555484*^9}, {3.513953862581882*^9, 
   3.513953869531641*^9}, {3.513953913176754*^9, 3.51395391839988*^9}, {
   3.513954124364752*^9, 3.513954124771585*^9}, {3.513963109152478*^9, 
   3.513963188824473*^9}, {3.513963351166205*^9, 3.513963487001848*^9}, {
   3.513963527828042*^9, 3.51396353762601*^9}, {3.5139635929170723`*^9, 
   3.5139636917580643`*^9}, {3.5139637974682493`*^9, 3.513963802783566*^9}, {
   3.513963881140173*^9, 3.513963913320346*^9}, {3.513964002763595*^9, 
   3.5139640804451647`*^9}, {3.5139641459974127`*^9, 
   3.5139642689184113`*^9}, {3.513964342352106*^9, 3.513964382632668*^9}, {
   3.5139644334878397`*^9, 3.513964445439907*^9}, {3.513965136438246*^9, 
   3.513965142087782*^9}, {3.513968925722913*^9, 3.51396897056528*^9}, {
   3.51482264314016*^9, 3.514822674472425*^9}, {3.514823299014048*^9, 
   3.514823433520165*^9}, {3.5148259836929407`*^9, 3.514826021246005*^9}, {
   3.5148260568909473`*^9, 3.514826093709059*^9}, {3.514829266130872*^9, 
   3.514829267269014*^9}, 3.514907738691799*^9, 3.514909645865958*^9, {
   3.514909725826643*^9, 3.514909749867633*^9}, {3.514909835600062*^9, 
   3.514909849756949*^9}, {3.514909927551237*^9, 3.514909958608143*^9}, {
   3.51490999142306*^9, 3.514910014969368*^9}, {3.514910105977624*^9, 
   3.514910107150621*^9}, {3.514910138545768*^9, 3.514910187921351*^9}, {
   3.514910242162261*^9, 3.514910545155949*^9}, {3.514910580487647*^9, 
   3.514910592845335*^9}, {3.514910650014436*^9, 3.514910654005959*^9}, 
   3.51491115722192*^9, {3.51491234211771*^9, 3.514912347366127*^9}, {
   3.514913668250718*^9, 3.514913684476367*^9}, {3.51491537864764*^9, 
   3.5149154086481857`*^9}, {3.514916004664308*^9, 3.514916111631596*^9}, {
   3.514916343684166*^9, 3.514916350939144*^9}, {3.514919628933803*^9, 
   3.514919629374105*^9}, {3.514919814051456*^9, 3.51491981431933*^9}, {
   3.5149210615292788`*^9, 3.5149210618093157`*^9}, {3.514921376696*^9, 
   3.514921409459235*^9}, {3.514921550975163*^9, 3.514921553118926*^9}, {
   3.5149216002292137`*^9, 3.514921601651573*^9}, {3.514921714355473*^9, 
   3.5149217145617943`*^9}, {3.514921849323975*^9, 3.51492184966441*^9}, {
   3.515014018484*^9, 3.5150140807939997`*^9}, {3.515014414274*^9, 
   3.515014419104*^9}, {3.515014499744*^9, 3.515014556984*^9}, {
   3.516031623738676*^9, 3.51603164993892*^9}, {3.5163613960938*^9, 
   3.5163614987928*^9}, {3.5163621515938*^9, 3.5163622450538*^9}, {
   3.5163623032008*^9, 3.5163623226138*^9}, {3.5163624113227997`*^9, 
   3.5163624414667997`*^9}, {3.5163625423188*^9, 3.5163625712748003`*^9}, {
   3.5163626344598*^9, 3.5163626494598*^9}, {3.5163626897988*^9, 
   3.5163627142928*^9}, {3.5163629553128*^9, 3.5163629783718*^9}, {
   3.5163630349268*^9, 3.5163630547258*^9}, {3.5163630874538*^9, 
   3.5163631165657997`*^9}, {3.5163631505308*^9, 3.5163631901438*^9}, {
   3.5163633288018*^9, 3.5163633291317997`*^9}, {3.5163633932418003`*^9, 
   3.5163634088087997`*^9}, {3.516369647737257*^9, 3.516369666853775*^9}, {
   3.516371046201543*^9, 3.516371081347692*^9}, {3.516371202900103*^9, 
   3.516371207766354*^9}, {3.51637143189148*^9, 3.516371468582984*^9}, {
   3.516371641584278*^9, 3.5163716420861*^9}, 3.516371673320703*^9, {
   3.516371728651638*^9, 3.516371729847763*^9}, {3.516382638594038*^9, 
   3.51638286034973*^9}, {3.516383039819622*^9, 3.516383111119877*^9}, {
   3.516383169368149*^9, 3.516383194658242*^9}, {3.5163838852752867`*^9, 
   3.516383951945406*^9}, {3.5163850545636597`*^9, 3.516385067352995*^9}, {
   3.51638526865587*^9, 3.51638530022193*^9}, {3.5163853567862177`*^9, 
   3.51638542827841*^9}, {3.5163859079277563`*^9, 3.516385912132997*^9}, {
   3.516386199386607*^9, 3.516386213286777*^9}, {3.516386388653233*^9, 
   3.5163863996118517`*^9}, {3.5163867067406063`*^9, 3.516386715759712*^9}, 
   3.516396963584313*^9, {3.516397541951666*^9, 3.5163975535653048`*^9}, 
   3.516457120472746*^9, {3.516457151655912*^9, 3.516457235975576*^9}, {
   3.516457284491362*^9, 3.5164573214514093`*^9}, {3.516457387922867*^9, 
   3.516457454287363*^9}, {3.516457625199349*^9, 3.516457672991251*^9}, {
   3.51645831600738*^9, 3.5164583295493793`*^9}, 3.516458360081263*^9, {
   3.516458779434746*^9, 3.516458802316041*^9}, {3.516459394550309*^9, 
   3.5164595175171347`*^9}, {3.516459604960169*^9, 3.516459625340931*^9}, {
   3.5164596662234*^9, 3.516459666637341*^9}, {3.516459998653577*^9, 
   3.516460006124365*^9}, {3.516460354942031*^9, 3.516460396324935*^9}, {
   3.51646053069876*^9, 3.516460614685628*^9}, {3.5164607527937717`*^9, 
   3.516460897803651*^9}, {3.516462501534315*^9, 3.516462526604961*^9}, {
   3.516462611112651*^9, 3.516462611383815*^9}, {3.516463021154703*^9, 
   3.516463061529895*^9}, {3.5164633717215548`*^9, 3.5164633976508627`*^9}, {
   3.516463429116269*^9, 3.516463481201078*^9}, {3.5164638398603697`*^9, 
   3.5164638593406963`*^9}, {3.516464312638464*^9, 3.516464312912916*^9}, {
   3.516464914674979*^9, 3.516465085200768*^9}, {3.516465337676579*^9, 
   3.516465372496331*^9}, {3.516465405274722*^9, 3.516465406270146*^9}, {
   3.516465552779982*^9, 3.516465588691825*^9}, {3.5164672510061007`*^9, 
   3.516467293942857*^9}, {3.516467474603541*^9, 3.5164675231703978`*^9}, {
   3.51646760024851*^9, 3.516467739960421*^9}, {3.5164677717917137`*^9, 
   3.516467988712447*^9}, {3.516469159296556*^9, 3.516469161955163*^9}, {
   3.5164698074427032`*^9, 3.516469904905406*^9}, {3.5164707554820213`*^9, 
   3.516470756292307*^9}, {3.516471149654396*^9, 3.516471249685629*^9}, {
   3.516471405122237*^9, 3.516471412333644*^9}, {3.516472156919051*^9, 
   3.51647219331638*^9}, {3.516472854954361*^9, 3.5164729609522753`*^9}, {
   3.568572773889769*^9, 3.5685728183678083`*^9}, {3.568573181918784*^9, 
   3.5685732296762877`*^9}, {3.568573342255867*^9, 3.568573749688448*^9}, {
   3.568573787742181*^9, 3.568573813907329*^9}, {3.568573850661203*^9, 
   3.568573919559573*^9}, {3.5685739707975807`*^9, 3.5685739999250727`*^9}, {
   3.568574594174342*^9, 3.568574676348609*^9}, {3.568574730907544*^9, 
   3.568574813674015*^9}, {3.568575547486279*^9, 3.568575589757934*^9}, {
   3.5685834303597527`*^9, 3.568583436207259*^9}, {3.568584066859976*^9, 
   3.5685841512933702`*^9}, {3.568584762036027*^9, 3.568584762654573*^9}, {
   3.5685848204813633`*^9, 3.568584839282544*^9}, 3.568643683007687*^9, {
   3.568649147868464*^9, 3.568649364926387*^9}, {3.568652918503582*^9, 
   3.56865293091017*^9}, {3.568653015556768*^9, 3.568653024430928*^9}, {
   3.568659656154059*^9, 3.568659744463448*^9}, {3.568659778894001*^9, 
   3.568659779745195*^9}, {3.5686637404458857`*^9, 3.5686637450530252`*^9}, {
   3.568663780948853*^9, 3.568663841357082*^9}, {3.5686639458052673`*^9, 
   3.568664068187275*^9}, {3.56866410906429*^9, 3.568664162978807*^9}, {
   3.568664210269896*^9, 3.5686642125527067`*^9}, {3.568664340654105*^9, 
   3.5686644446758537`*^9}, {3.5686645203752193`*^9, 3.56866453379126*^9}, {
   3.568665014329318*^9, 3.568665016680511*^9}, {3.568665804195015*^9, 
   3.568665805533679*^9}, {3.568665965952354*^9, 3.568665987838311*^9}, {
   3.568666110391972*^9, 3.56866623324951*^9}, {3.568734948189703*^9, 
   3.5687350483893137`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Derivatives of jumps", "Subsubsection",
 CellChangeTimes->{{3.568643687267159*^9, 3.568643692073745*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"\[Phi]QinsPsQs\[Phi]x", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Psx", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[Phi]QinsPinsQs\[Phi]x", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Pinsx", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[Phi]QinsMsQs\[Phi]x", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Msx", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]QinsGsQs\[Phi]x", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"MultFun", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"InvFun", "[", 
        RowBox[{
         RowBox[{"\[Phi]", "[", 
          RowBox[{"-", "1"}], "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"InvFun", "[", 
        RowBox[{"Qs", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"Gsx", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]QinsPinsQs\[Phi]ax", "[", 
     RowBox[{"+", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Pinsx", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"Z", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"+", "1"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Psi]1", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]QinsMsQs\[Phi]ax", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"MultFun", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"InvFun", "[", 
        RowBox[{
         RowBox[{"\[Phi]", "[", 
          RowBox[{"-", "1"}], "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"InvFun", "[", 
        RowBox[{"Qs", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"Msx", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"Z", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"+", "1"}], "]"}], ",", 
       RowBox[{
        RowBox[{"\[Psi]1", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]QinsPinsQs\[Phi]ax", "[", 
     RowBox[{"-", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Pinsx", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"Z", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"-", "1"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Psi]1", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]QinsMsQs\[Phi]ax", "[", 
      RowBox[{"-", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"MultFun", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"InvFun", "[", 
        RowBox[{
         RowBox[{"\[Phi]", "[", 
          RowBox[{"-", "1"}], "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"InvFun", "[", 
        RowBox[{"Qs", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"Msx", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"Z", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"-", "1"}], "]"}], ",", 
       RowBox[{
        RowBox[{"\[Psi]1", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "}"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]QinsPinsQs\[Phi]bx", "[", 
     RowBox[{"+", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Pinsx", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"Z", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"+", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"\[Phi]QinsMsQs\[Phi]bx", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"MultFun", "[", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"InvFun", "[", 
        RowBox[{
         RowBox[{"\[Phi]", "[", 
          RowBox[{"-", "1"}], "]"}], "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"InvFun", "[", 
        RowBox[{"Qs", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
       RowBox[{"Msx", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", 
       RowBox[{
        RowBox[{"Z", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"+", "1"}], "]"}]}], "}"}], "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]QinsPinsQs\[Phi]bx", "[", 
     RowBox[{"-", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"+", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Pinsx", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"Z", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"-", "1"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Phi]QinsMsQs\[Phi]bx", "[", 
     RowBox[{"-", "1"}], "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"MultFun", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"InvFun", "[", 
       RowBox[{
        RowBox[{"\[Phi]", "[", 
         RowBox[{"-", "1"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"InvFun", "[", 
       RowBox[{"Qs", "[", 
        RowBox[{"x", ",", "t"}], "]"}], "]"}], ",", 
      RowBox[{"Msx", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Qs", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"\[Phi]", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{
       RowBox[{"Z", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"-", "1"}], "]"}]}], "}"}], "]"}]}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.513711472307995*^9, 3.51371150688424*^9}, {
   3.513711559410595*^9, 3.513711606964553*^9}, {3.513711648397517*^9, 
   3.513711936409362*^9}, {3.5139442600508213`*^9, 3.513944356354691*^9}, {
   3.513944486092828*^9, 3.5139446411607943`*^9}, {3.513951549753932*^9, 
   3.513951699728924*^9}, {3.5139517397524*^9, 3.5139518129172707`*^9}, {
   3.513952232831436*^9, 3.513952249103897*^9}, {3.513953371131137*^9, 
   3.513953378053*^9}, {3.513953475209831*^9, 3.5139534934360037`*^9}, {
   3.513953588050322*^9, 3.513953588555484*^9}, {3.513953862581882*^9, 
   3.513953869531641*^9}, {3.513953913176754*^9, 3.51395391839988*^9}, {
   3.513954124364752*^9, 3.513954124771585*^9}, {3.513963109152478*^9, 
   3.513963188824473*^9}, {3.513963351166205*^9, 3.513963487001848*^9}, {
   3.513963527828042*^9, 3.51396353762601*^9}, {3.5139635929170723`*^9, 
   3.5139636917580643`*^9}, {3.5139637974682493`*^9, 3.513963802783566*^9}, {
   3.513963881140173*^9, 3.513963913320346*^9}, {3.513964002763595*^9, 
   3.5139640804451647`*^9}, {3.5139641459974127`*^9, 
   3.5139642689184113`*^9}, {3.513964342352106*^9, 3.513964382632668*^9}, {
   3.5139644334878397`*^9, 3.513964445439907*^9}, {3.513965136438246*^9, 
   3.513965142087782*^9}, {3.513968925722913*^9, 3.51396897056528*^9}, {
   3.51482264314016*^9, 3.514822674472425*^9}, {3.514823299014048*^9, 
   3.514823433520165*^9}, {3.5148259836929407`*^9, 3.514826021246005*^9}, {
   3.5148260568909473`*^9, 3.514826093709059*^9}, {3.514829266130872*^9, 
   3.514829267269014*^9}, 3.514907738691799*^9, 3.514909645865958*^9, {
   3.514909725826643*^9, 3.514909749867633*^9}, {3.514909835600062*^9, 
   3.514909849756949*^9}, {3.514909927551237*^9, 3.514909958608143*^9}, {
   3.51490999142306*^9, 3.514910014969368*^9}, {3.514910105977624*^9, 
   3.514910107150621*^9}, {3.514910138545768*^9, 3.514910187921351*^9}, {
   3.514910242162261*^9, 3.514910545155949*^9}, {3.514910580487647*^9, 
   3.514910592845335*^9}, {3.514910650014436*^9, 3.514910654005959*^9}, 
   3.51491115722192*^9, {3.51491234211771*^9, 3.514912347366127*^9}, {
   3.514913668250718*^9, 3.514913684476367*^9}, {3.51491537864764*^9, 
   3.5149154086481857`*^9}, {3.514916004664308*^9, 3.514916111631596*^9}, {
   3.514916343684166*^9, 3.514916350939144*^9}, {3.514919628933803*^9, 
   3.514919629374105*^9}, {3.514919814051456*^9, 3.51491981431933*^9}, {
   3.5149210615292788`*^9, 3.5149210618093157`*^9}, {3.514921376696*^9, 
   3.514921409459235*^9}, {3.514921550975163*^9, 3.514921553118926*^9}, {
   3.5149216002292137`*^9, 3.514921601651573*^9}, {3.514921714355473*^9, 
   3.5149217145617943`*^9}, {3.514921849323975*^9, 3.51492184966441*^9}, {
   3.515014018484*^9, 3.5150140807939997`*^9}, {3.515014414274*^9, 
   3.515014419104*^9}, {3.515014499744*^9, 3.515014556984*^9}, {
   3.516031623738676*^9, 3.51603164993892*^9}, {3.5163613960938*^9, 
   3.5163614987928*^9}, {3.5163621515938*^9, 3.5163622450538*^9}, {
   3.5163623032008*^9, 3.5163623226138*^9}, {3.5163624113227997`*^9, 
   3.5163624414667997`*^9}, {3.5163625423188*^9, 3.5163625712748003`*^9}, {
   3.5163626344598*^9, 3.5163626494598*^9}, {3.5163626897988*^9, 
   3.5163627142928*^9}, {3.5163629553128*^9, 3.5163629783718*^9}, {
   3.5163630349268*^9, 3.5163630547258*^9}, {3.5163630874538*^9, 
   3.5163631165657997`*^9}, {3.5163631505308*^9, 3.5163631901438*^9}, {
   3.5163633288018*^9, 3.5163633291317997`*^9}, {3.5163633932418003`*^9, 
   3.5163634088087997`*^9}, {3.516369647737257*^9, 3.516369666853775*^9}, {
   3.516371046201543*^9, 3.516371081347692*^9}, {3.516371202900103*^9, 
   3.516371207766354*^9}, {3.51637143189148*^9, 3.516371468582984*^9}, {
   3.516371641584278*^9, 3.5163716420861*^9}, 3.516371673320703*^9, {
   3.516371728651638*^9, 3.516371729847763*^9}, {3.516382638594038*^9, 
   3.51638286034973*^9}, {3.516383039819622*^9, 3.516383111119877*^9}, {
   3.516383169368149*^9, 3.516383194658242*^9}, {3.5163838852752867`*^9, 
   3.516383951945406*^9}, {3.5163850545636597`*^9, 3.516385067352995*^9}, {
   3.51638526865587*^9, 3.51638530022193*^9}, {3.5163853567862177`*^9, 
   3.51638542827841*^9}, {3.5163859079277563`*^9, 3.516385912132997*^9}, {
   3.516386199386607*^9, 3.516386213286777*^9}, {3.516386388653233*^9, 
   3.5163863996118517`*^9}, {3.5163867067406063`*^9, 3.516386715759712*^9}, 
   3.516396963584313*^9, {3.516397541951666*^9, 3.5163975535653048`*^9}, 
   3.516457120472746*^9, {3.516457151655912*^9, 3.516457235975576*^9}, {
   3.516457284491362*^9, 3.5164573214514093`*^9}, {3.516457387922867*^9, 
   3.516457454287363*^9}, {3.516457625199349*^9, 3.516457672991251*^9}, {
   3.51645831600738*^9, 3.5164583295493793`*^9}, 3.516458360081263*^9, {
   3.516458779434746*^9, 3.516458802316041*^9}, {3.516459394550309*^9, 
   3.5164595175171347`*^9}, {3.516459604960169*^9, 3.516459625340931*^9}, {
   3.5164596662234*^9, 3.516459666637341*^9}, {3.516459998653577*^9, 
   3.516460006124365*^9}, {3.516460354942031*^9, 3.516460396324935*^9}, {
   3.51646053069876*^9, 3.516460614685628*^9}, {3.5164607527937717`*^9, 
   3.516460897803651*^9}, {3.516462501534315*^9, 3.516462526604961*^9}, {
   3.516462611112651*^9, 3.516462611383815*^9}, {3.516463021154703*^9, 
   3.516463061529895*^9}, {3.5164633717215548`*^9, 3.5164633976508627`*^9}, {
   3.516463429116269*^9, 3.516463481201078*^9}, {3.5164638398603697`*^9, 
   3.5164638593406963`*^9}, {3.516464312638464*^9, 3.516464312912916*^9}, {
   3.516464914674979*^9, 3.516465085200768*^9}, {3.516465337676579*^9, 
   3.516465372496331*^9}, {3.516465405274722*^9, 3.516465406270146*^9}, {
   3.516465552779982*^9, 3.516465588691825*^9}, {3.5164672510061007`*^9, 
   3.516467293942857*^9}, {3.516467474603541*^9, 3.5164675231703978`*^9}, {
   3.51646760024851*^9, 3.516467739960421*^9}, {3.5164677717917137`*^9, 
   3.516467988712447*^9}, {3.516469159296556*^9, 3.516469161955163*^9}, {
   3.5164698074427032`*^9, 3.516469904905406*^9}, {3.5164707554820213`*^9, 
   3.516470756292307*^9}, {3.516471149654396*^9, 3.516471249685629*^9}, {
   3.516471405122237*^9, 3.516471412333644*^9}, {3.516472156919051*^9, 
   3.51647219331638*^9}, {3.516472854954361*^9, 3.5164729609522753`*^9}, {
   3.568572773889769*^9, 3.5685728183678083`*^9}, {3.568573181918784*^9, 
   3.5685732296762877`*^9}, {3.568573342255867*^9, 3.568573749688448*^9}, {
   3.568573787742181*^9, 3.568573813907329*^9}, {3.568573850661203*^9, 
   3.568573919559573*^9}, {3.5685739707975807`*^9, 3.5685739999250727`*^9}, {
   3.568574594174342*^9, 3.568574676348609*^9}, {3.568574730907544*^9, 
   3.568574813674015*^9}, {3.568575547486279*^9, 3.568575589757934*^9}, {
   3.5685834303597527`*^9, 3.568583436207259*^9}, {3.568584066859976*^9, 
   3.5685841512933702`*^9}, {3.568584762036027*^9, 3.568584762654573*^9}, {
   3.5685848204813633`*^9, 3.568584839282544*^9}, 3.568643683007687*^9, {
   3.568649147868464*^9, 3.568649364926387*^9}, {3.568652918503582*^9, 
   3.56865293091017*^9}, {3.568653015556768*^9, 3.568653024430928*^9}, {
   3.568659656154059*^9, 3.568659744463448*^9}, {3.568659778894001*^9, 
   3.568659779745195*^9}, {3.5686637404458857`*^9, 3.5686637450530252`*^9}, {
   3.568663780948853*^9, 3.568663841357082*^9}, {3.5686639458052673`*^9, 
   3.568664068187275*^9}, {3.56866410906429*^9, 3.568664162978807*^9}, {
   3.568664210269896*^9, 3.5686642125527067`*^9}, {3.568664340654105*^9, 
   3.5686644446758537`*^9}, {3.5686645203752193`*^9, 3.56866453379126*^9}, {
   3.568665014329318*^9, 3.568665016680511*^9}, {3.568665804195015*^9, 
   3.568665805533679*^9}, {3.5686662584783697`*^9, 3.5686662622930527`*^9}, {
   3.568666309800907*^9, 3.568666358857277*^9}, 3.56866672731892*^9}]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["RHP construction", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.56864670932092*^9, 3.568646712689498*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"shrinkt", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{
     RowBox[{"Abs", "[", 
      RowBox[{
       RowBox[{"x", "/", "6"}], "*", 
       RowBox[{"K0", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], "^", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", "2"}], "/", "3"}], ")"}]}], "/", "2."}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"radatran", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Min", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"a1", "[", 
       RowBox[{"x", ",", "t"}], "]"}], "/", "1.2"}], ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"b1", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "-", 
        RowBox[{"a1", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], ")"}], "/", "2.2"}], ",", 
     RowBox[{
      RowBox[{"shrinkt", "[", 
       RowBox[{"x", ",", "t"}], "]"}], "/", "2"}], ",", "\[Nu]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"radbtran", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Min", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"a1", "[", 
       RowBox[{"x", ",", "t"}], "]"}], "/", "1.2"}], ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"b1", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "-", 
        RowBox[{"a1", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], ")"}], "/", "2.2"}], ",", 
     RowBox[{
      RowBox[{"shrinkt", "[", 
       RowBox[{"x", ",", "t"}], "]"}], "/", "2"}], ",", "\[Nu]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"modulusa", "[", 
   RowBox[{"x_", ",", "t_"}], "]"}], ":=", " ", 
  RowBox[{
   RowBox[{"Sqrt", "[", "2", "]"}], "\[Nu]"}]}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.568658371347681*^9, 3.5686584562154417`*^9}}],

Cell[CellGroupData[{

Cell["Full jumps", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.568646722205043*^9, 3.5686467240656652`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "5", "]"}], "[", "1", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsGsQs\[Phi]", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a1", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}], "+", 
         RowBox[{"radatran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"a1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "-", 
         RowBox[{"radatran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}]}], "}"}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "5", "]"}], "[", "2", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsGsQs\[Phi]", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b1", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}], "+", 
         RowBox[{"radbtran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a1", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}], "-", 
         RowBox[{"radatran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}]}], "}"}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "5", "]"}], "[", "3", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsGsQs\[Phi]", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"a1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "+", 
         RowBox[{"radatran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"b1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "-", 
         RowBox[{"radbtran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}]}], "}"}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "5", "]"}], "[", "4", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsPsQs\[Phi]", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"b1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "+", 
         RowBox[{
          RowBox[{"radbtran", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"b1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "+", 
         RowBox[{
          RowBox[{"modulusa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "5", "]"}], "[", "5", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsPsQs\[Phi]", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b1", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"modulusa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b1", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"radbtran", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "5", "]"}], "[", "6", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsMsQs\[Phi]", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"b1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "+", 
         RowBox[{
          RowBox[{"radbtran", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"b1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "+", 
         RowBox[{
          RowBox[{"modulusa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJ", "[", "5", "]"}], "[", "7", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Phi]QinsMsQs\[Phi]", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{"b1", "[", 
            RowBox[{"x", ",", "t"}], "]"}]}], "+", 
          RowBox[{
           RowBox[{"modulusa", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "*", 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "3"}], "I", " ", 
             RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"b1", "[", 
            RowBox[{"x", ",", "t"}], "]"}]}], "+", 
          RowBox[{
           RowBox[{"radbtran", "[", 
            RowBox[{"x", ",", "t"}], "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "3"}], "I", " ", 
             RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
      "smallN"}], "}"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "5", "]"}], "[", "8", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsPinsQs\[Phi]b", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{"0", ",", 
            RowBox[{"3", 
             RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}], 
        RowBox[{"radbtran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"b1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "5", "]"}], "[", "9", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Z", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"-", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"3", 
             RowBox[{"Pi", "/", "4"}]}], ",", "Pi"}], "}"}]}], "]"}], 
        RowBox[{"radbtran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"b1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "5", "]"}], "[", "10", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Z", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"-", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{"Pi", ",", 
            RowBox[{"5", 
             RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}], 
        RowBox[{"radbtran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"b1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJ", "[", "5", "]"}], "[", "11", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Phi]QinsMsQs\[Phi]b", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Exp", "[", 
          RowBox[{"I", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"5", 
              RowBox[{"Pi", "/", "4"}]}], ",", 
             RowBox[{"2", "Pi"}]}], "}"}]}], "]"}], 
         RowBox[{"radbtran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "-", 
        RowBox[{"b1", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
   ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "5", "]"}], "[", "12", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsPinsQs\[Phi]a", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "2"}]}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "Pi"}], "]"}]}], "}"}], 
        RowBox[{"radatran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"a1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "5", "]"}], "[", "13", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsPinsQs\[Phi]a", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "0"}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "2"}]}], "]"}]}], "}"}], 
        RowBox[{"radatran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"a1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "5", "]"}], "[", "14", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsMsQs\[Phi]a", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", "Pi"}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"Pi", "/", "2"}]}], "]"}]}], "}"}], 
        RowBox[{"radatran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"a1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJ", "[", "5", "]"}], "[", "15", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Phi]QinsMsQs\[Phi]a", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", 
             RowBox[{"Pi", "/", "2"}]}], "]"}], ",", 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", "0"}], "]"}]}], "}"}], 
         RowBox[{"radatran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "-", 
        RowBox[{"a1", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
   ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "5", "]"}], "[", "16", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsPinsQs\[Phi]b", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Pi", "/", "4"}], ",", "Pi"}], "}"}]}], "]"}], 
        RowBox[{"radbtran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"b1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "5", "]"}], "[", "17", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsMsQs\[Phi]b", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{"Pi", ",", 
            RowBox[{"7", 
             RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}], 
        RowBox[{"radbtran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"b1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "5", "]"}], "[", "18", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Z", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"+", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "Pi"}], "/", "4"}], ",", "0"}], "}"}]}], "]"}], 
        RowBox[{"radbtran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"b1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJ", "[", "5", "]"}], "[", "19", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"Z", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"+", "1"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Exp", "[", 
          RowBox[{"I", 
           RowBox[{"{", 
            RowBox[{"0", ",", 
             RowBox[{"Pi", "/", "4"}]}], "}"}]}], "]"}], 
         RowBox[{"radbtran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "+", 
        RowBox[{"b1", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
   ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "5", "]"}], "[", "20", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsPinsQs\[Phi]a", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "2"}]}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "Pi"}], "]"}]}], "}"}], 
        RowBox[{"radatran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"a1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "5", "]"}], "[", "21", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsPinsQs\[Phi]a", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "0"}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "2"}]}], "]"}]}], "}"}], 
        RowBox[{"radatran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"a1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "5", "]"}], "[", "22", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsMsQs\[Phi]a", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", "Pi"}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"Pi", "/", "2"}]}], "]"}]}], "}"}], 
        RowBox[{"radatran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"a1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJ", "[", "5", "]"}], "[", "23", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Phi]QinsMsQs\[Phi]a", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", 
             RowBox[{"Pi", "/", "2"}]}], "]"}], ",", 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", "0"}], "]"}]}], "}"}], 
         RowBox[{"radatran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "+", 
        RowBox[{"a1", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
   ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "5", "]"}], "[", "24", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsPsQs\[Phi]", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "el"}], " ", "+", " ", 
         RowBox[{"I", " ", "\[Nu]"}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b1", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}], "+", 
         RowBox[{"\[Nu]", " ", 
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJ", "[", "5", "]"}], "[", "25", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Phi]QinsMsQs\[Phi]", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "el"}], " ", "-", 
          RowBox[{"I", " ", "\[Nu]"}]}], ",", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"b1", "[", 
            RowBox[{"x", ",", "t"}], "]"}]}], "+", 
          RowBox[{"\[Nu]", " ", 
           RowBox[{"Sqrt", "[", "2", "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", "3", " ", 
             RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
      "smallN"}], "}"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "5", "]"}], "[", "26", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsPsQs\[Phi]", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"b1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "+", 
         RowBox[{"\[Nu]", " ", 
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"I", " ", "\[Nu]"}], "+", " ", "el"}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJ", "[", "5", "]"}], "[", "27", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsMsQs\[Phi]", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"b1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "+", 
         RowBox[{"\[Nu]", " ", 
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "I"}], " ", "\[Nu]"}], "+", " ", "el"}]}], "}"}], 
      "]"}], ",", "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]"}], "Input",\

 InitializationCell->True,
 CellChangeTimes->{{3.514825267948341*^9, 3.514825292676878*^9}, {
   3.5148253366078253`*^9, 3.514825427904633*^9}, {3.5148255311926517`*^9, 
   3.5148255943071117`*^9}, {3.514825669646573*^9, 3.514825745629958*^9}, {
   3.5148258988524733`*^9, 3.514825926785774*^9}, {3.5148261039825363`*^9, 
   3.514826258039207*^9}, {3.514826288823626*^9, 3.514826543012664*^9}, {
   3.514826573528026*^9, 3.514826574581785*^9}, {3.514826728774248*^9, 
   3.514826758756071*^9}, {3.514828935462126*^9, 3.5148290533272142`*^9}, {
   3.5149096829953327`*^9, 3.514909723142671*^9}, {3.5149098576453648`*^9, 
   3.514909889448564*^9}, {3.514912127190716*^9, 3.51491213623287*^9}, {
   3.514912195772973*^9, 3.514912204226186*^9}, {3.514912303442165*^9, 
   3.5149123042055283`*^9}, 3.514921423035899*^9, {3.5150131270220003`*^9, 
   3.515013127442*^9}, {3.515013592144*^9, 3.515013629584*^9}, {
   3.5150136607939997`*^9, 3.515013665744*^9}, {3.515013883704*^9, 
   3.515013948974*^9}, {3.515014575494*^9, 3.515014585174*^9}, {
   3.515510466019574*^9, 3.5155104702861137`*^9}, {3.516032532081387*^9, 
   3.516032632290653*^9}, 3.516032718421742*^9, {3.516116258863737*^9, 
   3.516116280731205*^9}, {3.5163598914918003`*^9, 3.5163599052258*^9}, {
   3.5163605163677998`*^9, 3.5163605337567997`*^9}, {3.5163605964997997`*^9, 
   3.5163606026288*^9}, {3.5163608507637997`*^9, 3.5163608508698*^9}, {
   3.516382888805026*^9, 3.5163830068406553`*^9}, {3.516383277038055*^9, 
   3.5163833555156527`*^9}, {3.5163833998893337`*^9, 
   3.5163834453055067`*^9}, {3.516383476857959*^9, 3.51638351945615*^9}, {
   3.5163854577936897`*^9, 3.5163856049352503`*^9}, {3.516385952783477*^9, 
   3.516385986208107*^9}, {3.5163867387806797`*^9, 3.516386778242505*^9}, {
   3.516388278925934*^9, 3.516388314455687*^9}, {3.51638840971177*^9, 
   3.51638841247635*^9}, {3.516397565665082*^9, 3.5163976921604767`*^9}, {
   3.516397899028942*^9, 3.516397899322949*^9}, {3.5164076030158243`*^9, 
   3.516407638210791*^9}, {3.516408006778857*^9, 3.516408016021781*^9}, {
   3.51640975918292*^9, 3.516409767769142*^9}, {3.516457465894291*^9, 
   3.516457481056218*^9}, {3.516457559479982*^9, 3.516457610709281*^9}, {
   3.516457646901187*^9, 3.51645764800912*^9}, {3.516457700012754*^9, 
   3.516457786011052*^9}, {3.516457820881053*^9, 3.516458122097102*^9}, 
   3.51645820755171*^9, {3.516458351867426*^9, 3.5164584977762203`*^9}, {
   3.516458531496792*^9, 3.516458550968389*^9}, {3.516458649058075*^9, 
   3.516458748494729*^9}, {3.516458824332374*^9, 3.516458838505898*^9}, {
   3.5164588709359703`*^9, 3.516459125857703*^9}, {3.516459181357025*^9, 
   3.516459184898267*^9}, {3.516459232991765*^9, 3.516459242845731*^9}, {
   3.516459533041588*^9, 3.516459555124057*^9}, {3.516459631924409*^9, 
   3.516459679085615*^9}, {3.516460197000605*^9, 3.516460206808563*^9}, {
   3.5164602992598553`*^9, 3.516460311067243*^9}, {3.5164609129823503`*^9, 
   3.516460968519927*^9}, {3.516462676865677*^9, 3.516462684467883*^9}, {
   3.5164631772156067`*^9, 3.516463276695409*^9}, {3.516463488358333*^9, 
   3.516463493706519*^9}, {3.516463910894813*^9, 3.5164639193960238`*^9}, {
   3.516464097606217*^9, 3.51646419164781*^9}, 3.5164643626396723`*^9, {
   3.516464411015307*^9, 3.516464431083765*^9}, {3.5164646185531282`*^9, 
   3.516464634286481*^9}, {3.516465101313439*^9, 3.5164651163310337`*^9}, {
   3.516465390538862*^9, 3.516465401256546*^9}, {3.516465809366246*^9, 
   3.5164658223352957`*^9}, {3.516468032786314*^9, 3.51646805284263*^9}, {
   3.516468116424253*^9, 3.516468136815833*^9}, {3.516468247182317*^9, 
   3.516468349452812*^9}, {3.5164683848684*^9, 3.516468387922887*^9}, {
   3.5164685154818697`*^9, 3.5164685841468763`*^9}, {3.516469921949301*^9, 
   3.5164699711692543`*^9}, {3.5164700483876743`*^9, 
   3.5164701636501837`*^9}, {3.516471263308015*^9, 3.5164712746994658`*^9}, {
   3.516471352359535*^9, 3.5164713546883163`*^9}, {3.5164714262637653`*^9, 
   3.516471432301978*^9}, {3.516473517538863*^9, 3.516473523403447*^9}, {
   3.516474982691085*^9, 3.516474983982852*^9}, {3.516475459381082*^9, 
   3.5164755286888514`*^9}, {3.5178368227708817`*^9, 3.517836867940639*^9}, 
   3.517836922921733*^9, {3.5685748879394712`*^9, 3.568574942003587*^9}, {
   3.568574975568761*^9, 3.568575073669878*^9}, {3.568575199214121*^9, 
   3.5685753730555277`*^9}, {3.568575430728529*^9, 3.56857544416363*^9}, {
   3.568575639798094*^9, 3.5685757167565727`*^9}, {3.568575765502276*^9, 
   3.568575875843132*^9}, {3.568575964817031*^9, 3.568575996598225*^9}, {
   3.5685815788707113`*^9, 3.5685816231848097`*^9}, {3.568581802957261*^9, 
   3.5685818768577337`*^9}, {3.568583185807536*^9, 3.568583193174665*^9}, {
   3.568583564587915*^9, 3.568583650082548*^9}, {3.568585532768119*^9, 
   3.56858556451052*^9}, {3.56858638617171*^9, 3.568586411272897*^9}, {
   3.568586562870647*^9, 3.5685865750782146`*^9}, 3.568586700783231*^9, {
   3.5685869892028027`*^9, 3.568587050334168*^9}, {3.568587113451311*^9, 
   3.568587177428643*^9}, {3.568587208818534*^9, 3.568587278029192*^9}, {
   3.568587315618803*^9, 3.5685874272201843`*^9}, {3.5685875764556723`*^9, 
   3.568587688191224*^9}, {3.5685877325224733`*^9, 3.568588053803132*^9}, {
   3.568588118269617*^9, 3.5685881777593327`*^9}, {3.568645003388166*^9, 
   3.5686450240213947`*^9}, {3.568646726459577*^9, 3.568646726775237*^9}, {
   3.568649788006895*^9, 3.5686498989509974`*^9}, {3.568650033415969*^9, 
   3.5686500761406317`*^9}, {3.568650167098319*^9, 3.5686502354571133`*^9}, {
   3.568650291820149*^9, 3.568650327769712*^9}, {3.568650393073289*^9, 
   3.568650407501232*^9}, {3.5686504659936028`*^9, 3.5686504802890778`*^9}, {
   3.5686505146719007`*^9, 3.568650843951952*^9}, {3.568650942316934*^9, 
   3.568650957751766*^9}, {3.56865104279845*^9, 3.568651113100993*^9}, {
   3.568651166033194*^9, 3.568651170050005*^9}, {3.56865241412368*^9, 
   3.568652450736114*^9}, {3.568652720434972*^9, 3.568652733011682*^9}, {
   3.568658479051959*^9, 3.5686585053433867`*^9}, {3.568658599308311*^9, 
   3.568658612467614*^9}, {3.5686591241274633`*^9, 3.568659360378166*^9}, {
   3.568659434524927*^9, 3.568659435897852*^9}, {3.5686644799014597`*^9, 
   3.568664507678816*^9}, {3.568664551609149*^9, 3.568664657887267*^9}, {
   3.568664720996027*^9, 3.568664721530136*^9}, {3.568664752934039*^9, 
   3.5686647909283037`*^9}, {3.568665560981145*^9, 3.5686655612924147`*^9}, {
   3.568665714616829*^9, 3.56866572107321*^9}, {3.568665782068348*^9, 
   3.5686657826620283`*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Sample jumps", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.56864674561305*^9, 3.5686467471847677`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "5", "]"}], "[", "1", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsGsQs\[Phi]samp", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a1", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}], "+", 
         RowBox[{"radatran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"a1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "-", 
         RowBox[{"radatran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}]}], "}"}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "5", "]"}], "[", "2", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsGsQs\[Phi]samp", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b1", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}], "+", 
         RowBox[{"radbtran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a1", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}], "-", 
         RowBox[{"radatran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}]}], "}"}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "5", "]"}], "[", "3", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsGsQs\[Phi]samp", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"a1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "+", 
         RowBox[{"radatran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"b1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "-", 
         RowBox[{"radbtran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}]}], "}"}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "5", "]"}], "[", "4", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsPsQs\[Phi]samp", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"b1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "+", 
         RowBox[{
          RowBox[{"radbtran", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"b1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "+", 
         RowBox[{
          RowBox[{"modulusa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "5", "]"}], "[", "5", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsPsQs\[Phi]samp", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b1", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"modulusa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b1", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"radbtran", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "5", "]"}], "[", "6", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsMsQs\[Phi]samp", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"b1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "+", 
         RowBox[{
          RowBox[{"radbtran", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"b1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "+", 
         RowBox[{
          RowBox[{"modulusa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJsamp", "[", "5", "]"}], "[", "7", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Phi]QinsMsQs\[Phi]samp", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{"b1", "[", 
            RowBox[{"x", ",", "t"}], "]"}]}], "+", 
          RowBox[{
           RowBox[{"modulusa", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "*", 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "3"}], "I", " ", 
             RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"b1", "[", 
            RowBox[{"x", ",", "t"}], "]"}]}], "+", 
          RowBox[{
           RowBox[{"radbtran", "[", 
            RowBox[{"x", ",", "t"}], "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "3"}], "I", " ", 
             RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
      "smallN"}], "}"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "5", "]"}], "[", "8", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsPinsQs\[Phi]bsamp", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{"0", ",", 
            RowBox[{"3", 
             RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}], 
        RowBox[{"radbtran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"b1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "5", "]"}], "[", "9", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Zsamp", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"-", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"3", 
             RowBox[{"Pi", "/", "4"}]}], ",", "Pi"}], "}"}]}], "]"}], 
        RowBox[{"radbtran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"b1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "5", "]"}], "[", "10", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Zsamp", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"-", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{"Pi", ",", 
            RowBox[{"5", 
             RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}], 
        RowBox[{"radbtran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"b1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJsamp", "[", "5", "]"}], "[", "11", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Phi]QinsMsQs\[Phi]bsamp", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Exp", "[", 
          RowBox[{"I", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"5", 
              RowBox[{"Pi", "/", "4"}]}], ",", 
             RowBox[{"2", "Pi"}]}], "}"}]}], "]"}], 
         RowBox[{"radbtran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "-", 
        RowBox[{"b1", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
   ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "5", "]"}], "[", "12", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsPinsQs\[Phi]asamp", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "2"}]}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "Pi"}], "]"}]}], "}"}], 
        RowBox[{"radatran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"a1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "5", "]"}], "[", "13", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsPinsQs\[Phi]asamp", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "0"}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "2"}]}], "]"}]}], "}"}], 
        RowBox[{"radatran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"a1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "5", "]"}], "[", "14", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsMsQs\[Phi]asamp", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", "Pi"}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"Pi", "/", "2"}]}], "]"}]}], "}"}], 
        RowBox[{"radatran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"a1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJsamp", "[", "5", "]"}], "[", "15", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Phi]QinsMsQs\[Phi]asamp", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", 
             RowBox[{"Pi", "/", "2"}]}], "]"}], ",", 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", "0"}], "]"}]}], "}"}], 
         RowBox[{"radatran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "-", 
        RowBox[{"a1", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
   ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "5", "]"}], "[", "16", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsPinsQs\[Phi]bsamp", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Pi", "/", "4"}], ",", "Pi"}], "}"}]}], "]"}], 
        RowBox[{"radbtran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"b1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "5", "]"}], "[", "17", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsMsQs\[Phi]bsamp", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{"Pi", ",", 
            RowBox[{"7", 
             RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}], 
        RowBox[{"radbtran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"b1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "5", "]"}], "[", "18", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Zsamp", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"+", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "Pi"}], "/", "4"}], ",", "0"}], "}"}]}], "]"}], 
        RowBox[{"radbtran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"b1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJsamp", "[", "5", "]"}], "[", "19", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"Zsamp", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"+", "1"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Exp", "[", 
          RowBox[{"I", 
           RowBox[{"{", 
            RowBox[{"0", ",", 
             RowBox[{"Pi", "/", "4"}]}], "}"}]}], "]"}], 
         RowBox[{"radbtran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "+", 
        RowBox[{"b1", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
   ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "5", "]"}], "[", "20", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsPinsQs\[Phi]asamp", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "2"}]}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "Pi"}], "]"}]}], "}"}], 
        RowBox[{"radatran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"a1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "5", "]"}], "[", "21", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsPinsQs\[Phi]asamp", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "0"}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "2"}]}], "]"}]}], "}"}], 
        RowBox[{"radatran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"a1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "5", "]"}], "[", "22", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsMsQs\[Phi]asamp", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", "Pi"}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"Pi", "/", "2"}]}], "]"}]}], "}"}], 
        RowBox[{"radatran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"a1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJsamp", "[", "5", "]"}], "[", "23", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Phi]QinsMsQs\[Phi]asamp", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", 
             RowBox[{"Pi", "/", "2"}]}], "]"}], ",", 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", "0"}], "]"}]}], "}"}], 
         RowBox[{"radatran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "+", 
        RowBox[{"a1", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
   ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "5", "]"}], "[", "24", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsPsQs\[Phi]samp", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "el"}], " ", "+", " ", 
         RowBox[{"I", " ", "\[Nu]"}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b1", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}], "+", 
         RowBox[{"\[Nu]", " ", 
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJsamp", "[", "5", "]"}], "[", "25", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Phi]QinsMsQs\[Phi]samp", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "el"}], " ", "-", 
          RowBox[{"I", " ", "\[Nu]"}]}], ",", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"b1", "[", 
            RowBox[{"x", ",", "t"}], "]"}]}], "+", 
          RowBox[{"\[Nu]", " ", 
           RowBox[{"Sqrt", "[", "2", "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", "3", " ", 
             RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
      "smallN"}], "}"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "5", "]"}], "[", "26", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsPsQs\[Phi]samp", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"b1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "+", 
         RowBox[{"\[Nu]", " ", 
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"I", " ", "\[Nu]"}], "+", " ", "el"}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJsamp", "[", "5", "]"}], "[", "27", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsMsQs\[Phi]samp", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"b1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "+", 
         RowBox[{"\[Nu]", " ", 
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "I"}], " ", "\[Nu]"}], "+", " ", "el"}]}], "}"}], 
      "]"}], ",", "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]"}], "Input",\

 InitializationCell->True,
 CellChangeTimes->{{3.514825267948341*^9, 3.514825292676878*^9}, {
   3.5148253366078253`*^9, 3.514825427904633*^9}, {3.5148255311926517`*^9, 
   3.5148255943071117`*^9}, {3.514825669646573*^9, 3.514825745629958*^9}, {
   3.5148258988524733`*^9, 3.514825926785774*^9}, {3.5148261039825363`*^9, 
   3.514826258039207*^9}, {3.514826288823626*^9, 3.514826543012664*^9}, {
   3.514826573528026*^9, 3.514826574581785*^9}, {3.514826728774248*^9, 
   3.514826758756071*^9}, {3.514828935462126*^9, 3.5148290533272142`*^9}, {
   3.5149096829953327`*^9, 3.514909723142671*^9}, {3.5149098576453648`*^9, 
   3.514909889448564*^9}, {3.514912127190716*^9, 3.51491213623287*^9}, {
   3.514912195772973*^9, 3.514912204226186*^9}, {3.514912303442165*^9, 
   3.5149123042055283`*^9}, 3.514921423035899*^9, {3.5150131270220003`*^9, 
   3.515013127442*^9}, {3.515013592144*^9, 3.515013629584*^9}, {
   3.5150136607939997`*^9, 3.515013665744*^9}, {3.515013883704*^9, 
   3.515013948974*^9}, {3.515014575494*^9, 3.515014585174*^9}, {
   3.515510466019574*^9, 3.5155104702861137`*^9}, {3.516032532081387*^9, 
   3.516032632290653*^9}, 3.516032718421742*^9, {3.516116258863737*^9, 
   3.516116280731205*^9}, {3.5163598914918003`*^9, 3.5163599052258*^9}, {
   3.5163605163677998`*^9, 3.5163605337567997`*^9}, {3.5163605964997997`*^9, 
   3.5163606026288*^9}, {3.5163608507637997`*^9, 3.5163608508698*^9}, {
   3.516382888805026*^9, 3.5163830068406553`*^9}, {3.516383277038055*^9, 
   3.5163833555156527`*^9}, {3.5163833998893337`*^9, 
   3.5163834453055067`*^9}, {3.516383476857959*^9, 3.51638351945615*^9}, {
   3.5163854577936897`*^9, 3.5163856049352503`*^9}, {3.516385952783477*^9, 
   3.516385986208107*^9}, {3.5163867387806797`*^9, 3.516386778242505*^9}, {
   3.516388278925934*^9, 3.516388314455687*^9}, {3.51638840971177*^9, 
   3.51638841247635*^9}, {3.516397565665082*^9, 3.5163976921604767`*^9}, {
   3.516397899028942*^9, 3.516397899322949*^9}, {3.5164076030158243`*^9, 
   3.516407638210791*^9}, {3.516408006778857*^9, 3.516408016021781*^9}, {
   3.51640975918292*^9, 3.516409767769142*^9}, {3.516457465894291*^9, 
   3.516457481056218*^9}, {3.516457559479982*^9, 3.516457610709281*^9}, {
   3.516457646901187*^9, 3.51645764800912*^9}, {3.516457700012754*^9, 
   3.516457786011052*^9}, {3.516457820881053*^9, 3.516458122097102*^9}, 
   3.51645820755171*^9, {3.516458351867426*^9, 3.5164584977762203`*^9}, {
   3.516458531496792*^9, 3.516458550968389*^9}, {3.516458649058075*^9, 
   3.516458748494729*^9}, {3.516458824332374*^9, 3.516458838505898*^9}, {
   3.5164588709359703`*^9, 3.516459125857703*^9}, {3.516459181357025*^9, 
   3.516459184898267*^9}, {3.516459232991765*^9, 3.516459242845731*^9}, {
   3.516459533041588*^9, 3.516459555124057*^9}, {3.516459631924409*^9, 
   3.516459679085615*^9}, {3.516460197000605*^9, 3.516460206808563*^9}, {
   3.5164602992598553`*^9, 3.516460311067243*^9}, {3.5164609129823503`*^9, 
   3.516460968519927*^9}, {3.516462676865677*^9, 3.516462684467883*^9}, {
   3.5164631772156067`*^9, 3.516463276695409*^9}, {3.516463488358333*^9, 
   3.516463493706519*^9}, {3.516463910894813*^9, 3.5164639193960238`*^9}, {
   3.516464097606217*^9, 3.51646419164781*^9}, 3.5164643626396723`*^9, {
   3.516464411015307*^9, 3.516464431083765*^9}, {3.5164646185531282`*^9, 
   3.516464634286481*^9}, {3.516465101313439*^9, 3.5164651163310337`*^9}, {
   3.516465390538862*^9, 3.516465401256546*^9}, {3.516465809366246*^9, 
   3.5164658223352957`*^9}, {3.516468032786314*^9, 3.51646805284263*^9}, {
   3.516468116424253*^9, 3.516468136815833*^9}, {3.516468247182317*^9, 
   3.516468349452812*^9}, {3.5164683848684*^9, 3.516468387922887*^9}, {
   3.5164685154818697`*^9, 3.5164685841468763`*^9}, {3.516469921949301*^9, 
   3.5164699711692543`*^9}, {3.5164700483876743`*^9, 
   3.5164701636501837`*^9}, {3.516471263308015*^9, 3.5164712746994658`*^9}, {
   3.516471352359535*^9, 3.5164713546883163`*^9}, {3.5164714262637653`*^9, 
   3.516471432301978*^9}, {3.516473517538863*^9, 3.516473523403447*^9}, {
   3.516474982691085*^9, 3.516474983982852*^9}, {3.516475459381082*^9, 
   3.5164755286888514`*^9}, {3.5178368227708817`*^9, 3.517836867940639*^9}, 
   3.517836922921733*^9, {3.5685748879394712`*^9, 3.568574942003587*^9}, {
   3.568574975568761*^9, 3.568575073669878*^9}, {3.568575199214121*^9, 
   3.5685753730555277`*^9}, {3.568575430728529*^9, 3.56857544416363*^9}, {
   3.568575639798094*^9, 3.5685757167565727`*^9}, {3.568575765502276*^9, 
   3.568575875843132*^9}, {3.568575964817031*^9, 3.568575996598225*^9}, {
   3.5685815788707113`*^9, 3.5685816231848097`*^9}, {3.568581802957261*^9, 
   3.5685818768577337`*^9}, {3.568583185807536*^9, 3.568583193174665*^9}, {
   3.568583564587915*^9, 3.568583650082548*^9}, {3.568585532768119*^9, 
   3.56858556451052*^9}, {3.56858638617171*^9, 3.568586411272897*^9}, {
   3.568586562870647*^9, 3.5685865750782146`*^9}, 3.568586700783231*^9, {
   3.5685869892028027`*^9, 3.568587050334168*^9}, {3.568587113451311*^9, 
   3.568587177428643*^9}, {3.568587208818534*^9, 3.568587278029192*^9}, {
   3.568587315618803*^9, 3.5685874272201843`*^9}, {3.5685875764556723`*^9, 
   3.568587688191224*^9}, {3.5685877325224733`*^9, 3.568588053803132*^9}, {
   3.568588118269617*^9, 3.5685881777593327`*^9}, {3.568645003388166*^9, 
   3.5686450240213947`*^9}, {3.568646726459577*^9, 3.568646726775237*^9}, {
   3.568649788006895*^9, 3.5686498989509974`*^9}, {3.568650033415969*^9, 
   3.5686500761406317`*^9}, {3.568650167098319*^9, 3.5686502354571133`*^9}, {
   3.568650291820149*^9, 3.568650327769712*^9}, {3.568650393073289*^9, 
   3.568650407501232*^9}, {3.5686504659936028`*^9, 3.5686504802890778`*^9}, {
   3.5686505146719007`*^9, 3.568650843951952*^9}, {3.568650942316934*^9, 
   3.568650957751766*^9}, {3.56865104279845*^9, 3.568651113100993*^9}, {
   3.568651166033194*^9, 3.568651170050005*^9}, {3.56865241412368*^9, 
   3.568652450736114*^9}, {3.568652720434972*^9, 3.568652733011682*^9}, {
   3.568658479051959*^9, 3.5686585053433867`*^9}, {3.568658599308311*^9, 
   3.568658612467614*^9}, {3.5686591241274633`*^9, 3.568659360378166*^9}, {
   3.568659434524927*^9, 3.568659435897852*^9}, {3.5686644799014597`*^9, 
   3.568664507678816*^9}, {3.568664551609149*^9, 3.568664657887267*^9}, {
   3.568664720996027*^9, 3.568664721530136*^9}, {3.568664752934039*^9, 
   3.5686647909283037`*^9}, {3.568665560981145*^9, 3.5686655612924147`*^9}, {
   3.568665714616829*^9, 3.56866572107321*^9}, {3.568665782068348*^9, 
   3.5686657826620283`*^9}, {3.568666373853428*^9, 3.568666470066934*^9}, {
   3.5686665113027563`*^9, 3.568666519389391*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Differentiated Jumps", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.568646777515478*^9, 3.568646782201989*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "5", "]"}], "[", "1", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsGsQs\[Phi]x", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a1", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}], "+", 
         RowBox[{"radatran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"a1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "-", 
         RowBox[{"radatran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}]}], "}"}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "5", "]"}], "[", "2", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsGsQs\[Phi]x", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b1", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}], "+", 
         RowBox[{"radbtran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"a1", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}], "-", 
         RowBox[{"radatran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}]}], "}"}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "5", "]"}], "[", "3", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsGsQs\[Phi]x", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"a1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "+", 
         RowBox[{"radatran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"b1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "-", 
         RowBox[{"radbtran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}]}], "}"}], "]"}], ",", "smallN"}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "5", "]"}], "[", "4", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsPsQs\[Phi]x", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"b1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "+", 
         RowBox[{
          RowBox[{"radbtran", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"b1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "+", 
         RowBox[{
          RowBox[{"modulusa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "5", "]"}], "[", "5", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsPsQs\[Phi]x", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b1", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"modulusa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b1", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}], "+", 
         RowBox[{
          RowBox[{"radbtran", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "5", "]"}], "[", "6", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsMsQs\[Phi]x", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"b1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "+", 
         RowBox[{
          RowBox[{"radbtran", "[", 
           RowBox[{"x", ",", "t"}], "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"b1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "+", 
         RowBox[{
          RowBox[{"modulusa", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJx", "[", "5", "]"}], "[", "7", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Phi]QinsMsQs\[Phi]x", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"-", 
           RowBox[{"b1", "[", 
            RowBox[{"x", ",", "t"}], "]"}]}], "+", 
          RowBox[{
           RowBox[{"modulusa", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "*", 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "3"}], "I", " ", 
             RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"b1", "[", 
            RowBox[{"x", ",", "t"}], "]"}]}], "+", 
          RowBox[{
           RowBox[{"radbtran", "[", 
            RowBox[{"x", ",", "t"}], "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "3"}], "I", " ", 
             RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
      "smallN"}], "}"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "5", "]"}], "[", "8", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsPinsQs\[Phi]bx", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{"0", ",", 
            RowBox[{"3", 
             RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}], 
        RowBox[{"radbtran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"b1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "5", "]"}], "[", "9", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Zx", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"-", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"3", 
             RowBox[{"Pi", "/", "4"}]}], ",", "Pi"}], "}"}]}], "]"}], 
        RowBox[{"radbtran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"b1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "5", "]"}], "[", "10", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Zx", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"-", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{"Pi", ",", 
            RowBox[{"5", 
             RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}], 
        RowBox[{"radbtran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"b1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJx", "[", "5", "]"}], "[", "11", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Phi]QinsMsQs\[Phi]bx", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Exp", "[", 
          RowBox[{"I", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"5", 
              RowBox[{"Pi", "/", "4"}]}], ",", 
             RowBox[{"2", "Pi"}]}], "}"}]}], "]"}], 
         RowBox[{"radbtran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "-", 
        RowBox[{"b1", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
   ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "5", "]"}], "[", "12", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsPinsQs\[Phi]ax", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "2"}]}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "Pi"}], "]"}]}], "}"}], 
        RowBox[{"radatran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"a1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "5", "]"}], "[", "13", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsPinsQs\[Phi]ax", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "0"}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "2"}]}], "]"}]}], "}"}], 
        RowBox[{"radatran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"a1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "5", "]"}], "[", "14", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsMsQs\[Phi]ax", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", "Pi"}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"Pi", "/", "2"}]}], "]"}]}], "}"}], 
        RowBox[{"radatran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "-", 
       RowBox[{"a1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJx", "[", "5", "]"}], "[", "15", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Phi]QinsMsQs\[Phi]ax", "[", 
        RowBox[{"-", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", 
             RowBox[{"Pi", "/", "2"}]}], "]"}], ",", 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", "0"}], "]"}]}], "}"}], 
         RowBox[{"radatran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "-", 
        RowBox[{"a1", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
   ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "5", "]"}], "[", "16", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsPinsQs\[Phi]bx", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"Pi", "/", "4"}], ",", "Pi"}], "}"}]}], "]"}], 
        RowBox[{"radbtran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"b1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "5", "]"}], "[", "17", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsMsQs\[Phi]bx", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{"Pi", ",", 
            RowBox[{"7", 
             RowBox[{"Pi", "/", "4"}]}]}], "}"}]}], "]"}], 
        RowBox[{"radbtran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"b1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "5", "]"}], "[", "18", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Zx", "[", 
       RowBox[{"-", "1"}], "]"}], "[", 
      RowBox[{"+", "1"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Exp", "[", 
         RowBox[{"I", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "Pi"}], "/", "4"}], ",", "0"}], "}"}]}], "]"}], 
        RowBox[{"radbtran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"b1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJx", "[", "5", "]"}], "[", "19", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"Zx", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"+", "1"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Exp", "[", 
          RowBox[{"I", 
           RowBox[{"{", 
            RowBox[{"0", ",", 
             RowBox[{"Pi", "/", "4"}]}], "}"}]}], "]"}], 
         RowBox[{"radbtran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "+", 
        RowBox[{"b1", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
   ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "5", "]"}], "[", "20", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsPinsQs\[Phi]ax", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "2"}]}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "Pi"}], "]"}]}], "}"}], 
        RowBox[{"radatran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"a1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "5", "]"}], "[", "21", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsPinsQs\[Phi]ax", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "0"}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "2"}]}], "]"}]}], "}"}], 
        RowBox[{"radatran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"a1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "5", "]"}], "[", "22", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"\[Phi]QinsMsQs\[Phi]ax", "[", 
       RowBox[{"+", "1"}], "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"{", 
         RowBox[{
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", "Pi"}], "]"}], ",", 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"Pi", "/", "2"}]}], "]"}]}], "}"}], 
        RowBox[{"radatran", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "+", 
       RowBox[{"a1", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJx", "[", "5", "]"}], "[", "23", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Phi]QinsMsQs\[Phi]ax", "[", 
        RowBox[{"+", "1"}], "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", 
             RowBox[{"Pi", "/", "2"}]}], "]"}], ",", 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", "0"}], "]"}]}], "}"}], 
         RowBox[{"radatran", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "+", 
        RowBox[{"a1", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], "]"}], ",", "smallN"}], "}"}]}], 
   ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "5", "]"}], "[", "24", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsPsQs\[Phi]x", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "el"}], " ", "+", " ", 
         RowBox[{"I", " ", "\[Nu]"}]}], ",", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"b1", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}], "+", 
         RowBox[{"\[Nu]", " ", 
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", "3", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"JJx", "[", "5", "]"}], "[", "25", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"\[Phi]QinsMsQs\[Phi]x", "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", 
      RowBox[{"Line", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "el"}], " ", "-", 
          RowBox[{"I", " ", "\[Nu]"}]}], ",", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"b1", "[", 
            RowBox[{"x", ",", "t"}], "]"}]}], "+", 
          RowBox[{"\[Nu]", " ", 
           RowBox[{"Sqrt", "[", "2", "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", "3", " ", 
             RowBox[{"Pi", "/", "4"}]}], "]"}]}]}]}], "}"}], "]"}], ",", 
      "smallN"}], "}"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "5", "]"}], "[", "26", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsPsQs\[Phi]x", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"b1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "+", 
         RowBox[{"\[Nu]", " ", 
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{"I", " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{"I", " ", "\[Nu]"}], "+", " ", "el"}]}], "}"}], "]"}], ",", 
     "smallN"}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"JJx", "[", "5", "]"}], "[", "27", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"\[Phi]QinsMsQs\[Phi]x", "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"b1", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "+", 
         RowBox[{"\[Nu]", " ", 
          RowBox[{"Sqrt", "[", "2", "]"}], 
          RowBox[{"Exp", "[", 
           RowBox[{
            RowBox[{"-", "I"}], " ", 
            RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "I"}], " ", "\[Nu]"}], "+", " ", "el"}]}], "}"}], 
      "]"}], ",", "smallN"}], "}"}]}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.514825267948341*^9, 3.514825292676878*^9}, {
   3.5148253366078253`*^9, 3.514825427904633*^9}, {3.5148255311926517`*^9, 
   3.5148255943071117`*^9}, {3.514825669646573*^9, 3.514825745629958*^9}, {
   3.5148258988524733`*^9, 3.514825926785774*^9}, {3.5148261039825363`*^9, 
   3.514826258039207*^9}, {3.514826288823626*^9, 3.514826543012664*^9}, {
   3.514826573528026*^9, 3.514826574581785*^9}, {3.514826728774248*^9, 
   3.514826758756071*^9}, {3.514828935462126*^9, 3.5148290533272142`*^9}, {
   3.5149096829953327`*^9, 3.514909723142671*^9}, {3.5149098576453648`*^9, 
   3.514909889448564*^9}, {3.514912127190716*^9, 3.51491213623287*^9}, {
   3.514912195772973*^9, 3.514912204226186*^9}, {3.514912303442165*^9, 
   3.5149123042055283`*^9}, 3.514921423035899*^9, {3.5150131270220003`*^9, 
   3.515013127442*^9}, {3.515013592144*^9, 3.515013629584*^9}, {
   3.5150136607939997`*^9, 3.515013665744*^9}, {3.515013883704*^9, 
   3.515013948974*^9}, {3.515014575494*^9, 3.515014585174*^9}, {
   3.515510466019574*^9, 3.5155104702861137`*^9}, {3.516032532081387*^9, 
   3.516032632290653*^9}, 3.516032718421742*^9, {3.516116258863737*^9, 
   3.516116280731205*^9}, {3.5163598914918003`*^9, 3.5163599052258*^9}, {
   3.5163605163677998`*^9, 3.5163605337567997`*^9}, {3.5163605964997997`*^9, 
   3.5163606026288*^9}, {3.5163608507637997`*^9, 3.5163608508698*^9}, {
   3.516382888805026*^9, 3.5163830068406553`*^9}, {3.516383277038055*^9, 
   3.5163833555156527`*^9}, {3.5163833998893337`*^9, 
   3.5163834453055067`*^9}, {3.516383476857959*^9, 3.51638351945615*^9}, {
   3.5163854577936897`*^9, 3.5163856049352503`*^9}, {3.516385952783477*^9, 
   3.516385986208107*^9}, {3.5163867387806797`*^9, 3.516386778242505*^9}, {
   3.516388278925934*^9, 3.516388314455687*^9}, {3.51638840971177*^9, 
   3.51638841247635*^9}, {3.516397565665082*^9, 3.5163976921604767`*^9}, {
   3.516397899028942*^9, 3.516397899322949*^9}, {3.5164076030158243`*^9, 
   3.516407638210791*^9}, {3.516408006778857*^9, 3.516408016021781*^9}, {
   3.51640975918292*^9, 3.516409767769142*^9}, {3.516457465894291*^9, 
   3.516457481056218*^9}, {3.516457559479982*^9, 3.516457610709281*^9}, {
   3.516457646901187*^9, 3.51645764800912*^9}, {3.516457700012754*^9, 
   3.516457786011052*^9}, {3.516457820881053*^9, 3.516458122097102*^9}, 
   3.51645820755171*^9, {3.516458351867426*^9, 3.5164584977762203`*^9}, {
   3.516458531496792*^9, 3.516458550968389*^9}, {3.516458649058075*^9, 
   3.516458748494729*^9}, {3.516458824332374*^9, 3.516458838505898*^9}, {
   3.5164588709359703`*^9, 3.516459125857703*^9}, {3.516459181357025*^9, 
   3.516459184898267*^9}, {3.516459232991765*^9, 3.516459242845731*^9}, {
   3.516459533041588*^9, 3.516459555124057*^9}, {3.516459631924409*^9, 
   3.516459679085615*^9}, {3.516460197000605*^9, 3.516460206808563*^9}, {
   3.5164602992598553`*^9, 3.516460311067243*^9}, {3.5164609129823503`*^9, 
   3.516460968519927*^9}, {3.516462676865677*^9, 3.516462684467883*^9}, {
   3.5164631772156067`*^9, 3.516463276695409*^9}, {3.516463488358333*^9, 
   3.516463493706519*^9}, {3.516463910894813*^9, 3.5164639193960238`*^9}, {
   3.516464097606217*^9, 3.51646419164781*^9}, 3.5164643626396723`*^9, {
   3.516464411015307*^9, 3.516464431083765*^9}, {3.5164646185531282`*^9, 
   3.516464634286481*^9}, {3.516465101313439*^9, 3.5164651163310337`*^9}, {
   3.516465390538862*^9, 3.516465401256546*^9}, {3.516465809366246*^9, 
   3.5164658223352957`*^9}, {3.516468032786314*^9, 3.51646805284263*^9}, {
   3.516468116424253*^9, 3.516468136815833*^9}, {3.516468247182317*^9, 
   3.516468349452812*^9}, {3.5164683848684*^9, 3.516468387922887*^9}, {
   3.5164685154818697`*^9, 3.5164685841468763`*^9}, {3.516469921949301*^9, 
   3.5164699711692543`*^9}, {3.5164700483876743`*^9, 
   3.5164701636501837`*^9}, {3.516471263308015*^9, 3.5164712746994658`*^9}, {
   3.516471352359535*^9, 3.5164713546883163`*^9}, {3.5164714262637653`*^9, 
   3.516471432301978*^9}, {3.516473517538863*^9, 3.516473523403447*^9}, {
   3.516474982691085*^9, 3.516474983982852*^9}, {3.516475459381082*^9, 
   3.5164755286888514`*^9}, {3.5178368227708817`*^9, 3.517836867940639*^9}, 
   3.517836922921733*^9, {3.5685748879394712`*^9, 3.568574942003587*^9}, {
   3.568574975568761*^9, 3.568575073669878*^9}, {3.568575199214121*^9, 
   3.5685753730555277`*^9}, {3.568575430728529*^9, 3.56857544416363*^9}, {
   3.568575639798094*^9, 3.5685757167565727`*^9}, {3.568575765502276*^9, 
   3.568575875843132*^9}, {3.568575964817031*^9, 3.568575996598225*^9}, {
   3.5685815788707113`*^9, 3.5685816231848097`*^9}, {3.568581802957261*^9, 
   3.5685818768577337`*^9}, {3.568583185807536*^9, 3.568583193174665*^9}, {
   3.568583564587915*^9, 3.568583650082548*^9}, {3.568585532768119*^9, 
   3.56858556451052*^9}, {3.56858638617171*^9, 3.568586411272897*^9}, {
   3.568586562870647*^9, 3.5685865750782146`*^9}, 3.568586700783231*^9, {
   3.5685869892028027`*^9, 3.568587050334168*^9}, {3.568587113451311*^9, 
   3.568587177428643*^9}, {3.568587208818534*^9, 3.568587278029192*^9}, {
   3.568587315618803*^9, 3.5685874272201843`*^9}, {3.5685875764556723`*^9, 
   3.568587688191224*^9}, {3.5685877325224733`*^9, 3.568588053803132*^9}, {
   3.568588118269617*^9, 3.5685881777593327`*^9}, {3.568645003388166*^9, 
   3.5686450240213947`*^9}, {3.568646726459577*^9, 3.568646726775237*^9}, {
   3.568649788006895*^9, 3.5686498989509974`*^9}, {3.568650033415969*^9, 
   3.5686500761406317`*^9}, {3.568650167098319*^9, 3.5686502354571133`*^9}, {
   3.568650291820149*^9, 3.568650327769712*^9}, {3.568650393073289*^9, 
   3.568650407501232*^9}, {3.5686504659936028`*^9, 3.5686504802890778`*^9}, {
   3.5686505146719007`*^9, 3.568650843951952*^9}, {3.568650942316934*^9, 
   3.568650957751766*^9}, {3.56865104279845*^9, 3.568651113100993*^9}, {
   3.568651166033194*^9, 3.568651170050005*^9}, {3.56865241412368*^9, 
   3.568652450736114*^9}, {3.568652720434972*^9, 3.568652733011682*^9}, {
   3.568658479051959*^9, 3.5686585053433867`*^9}, {3.568658599308311*^9, 
   3.568658612467614*^9}, {3.5686591241274633`*^9, 3.568659360378166*^9}, {
   3.568659434524927*^9, 3.568659435897852*^9}, {3.5686644799014597`*^9, 
   3.568664507678816*^9}, {3.568664551609149*^9, 3.568664657887267*^9}, {
   3.568664720996027*^9, 3.568664721530136*^9}, {3.568664752934039*^9, 
   3.5686647909283037`*^9}, {3.568665560981145*^9, 3.5686655612924147`*^9}, {
   3.568665714616829*^9, 3.56866572107321*^9}, {3.568665782068348*^9, 
   3.5686657826620283`*^9}, {3.568666373853428*^9, 3.568666597918047*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Assembly", "Subsubsection",
 InitializationCell->True,
 CellChangeTimes->{{3.56865121529117*^9, 3.568651216471094*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"J", "[", "5", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"r", ",", "end"}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"end", "=", "27"}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"JJ", "[", "5", "]"}], "[", "i", "]"}], "[", 
           RowBox[{"x", ",", "t"}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "1", ",", "end"}], "}"}]}], "]"}], "//", 
        "Transpose"}]}]}], "\[IndentingNewLine]", "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Jsamp", "[", "5", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"r", ",", "end"}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"end", "=", "27"}], ";", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"JJsamp", "[", "5", "]"}], "[", "i", "]"}], "[", 
           RowBox[{"x", ",", "t"}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "1", ",", "end"}], "}"}]}], "]"}], "//", 
        "Transpose"}]}]}], "\[IndentingNewLine]", "]"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Jx", "[", "5", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"r", ",", "end"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"end", "=", "27"}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Table", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"JJx", "[", "5", "]"}], "[", "i", "]"}], "[", 
          RowBox[{"x", ",", "t"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "1", ",", "end"}], "}"}]}], "]"}], "//", 
       "Transpose"}]}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Jadapt", "[", "5", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"Adapt", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Jsamp", "[", "5", "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", "globalTol"}], "]"}], "[", 
    RowBox[{
     RowBox[{"J", "[", "5", "]"}], "[", 
     RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Jadaptx", "[", "5", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{
    RowBox[{"Adapt", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Jsamp", "[", "5", "]"}], "[", 
       RowBox[{"x", ",", "t"}], "]"}], ",", "globalTol"}], "]"}], "[", 
    RowBox[{
     RowBox[{"Jx", "[", "5", "]"}], "[", 
     RowBox[{"x", ",", "t"}], "]"}], "]"}]}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.514825267948341*^9, 3.514825292676878*^9}, {
   3.5148253366078253`*^9, 3.514825427904633*^9}, {3.5148255311926517`*^9, 
   3.5148255943071117`*^9}, {3.514825669646573*^9, 3.514825745629958*^9}, {
   3.5148258988524733`*^9, 3.514825926785774*^9}, {3.5148261039825363`*^9, 
   3.514826258039207*^9}, {3.514826288823626*^9, 3.514826543012664*^9}, {
   3.514826573528026*^9, 3.514826574581785*^9}, {3.514826728774248*^9, 
   3.514826758756071*^9}, {3.514828935462126*^9, 3.5148290533272142`*^9}, {
   3.5149096829953327`*^9, 3.514909723142671*^9}, {3.5149098576453648`*^9, 
   3.514909889448564*^9}, {3.514912127190716*^9, 3.51491213623287*^9}, {
   3.514912195772973*^9, 3.514912204226186*^9}, {3.514912303442165*^9, 
   3.5149123042055283`*^9}, 3.514921423035899*^9, {3.5150131270220003`*^9, 
   3.515013127442*^9}, {3.515013592144*^9, 3.515013629584*^9}, {
   3.5150136607939997`*^9, 3.515013665744*^9}, {3.515013883704*^9, 
   3.515013948974*^9}, {3.515014575494*^9, 3.515014585174*^9}, {
   3.515510466019574*^9, 3.5155104702861137`*^9}, {3.516032532081387*^9, 
   3.516032632290653*^9}, 3.516032718421742*^9, {3.516116258863737*^9, 
   3.516116280731205*^9}, {3.5163598914918003`*^9, 3.5163599052258*^9}, {
   3.5163605163677998`*^9, 3.5163605337567997`*^9}, {3.5163605964997997`*^9, 
   3.5163606026288*^9}, {3.5163608507637997`*^9, 3.5163608508698*^9}, {
   3.516382888805026*^9, 3.5163830068406553`*^9}, {3.516383277038055*^9, 
   3.5163833555156527`*^9}, {3.5163833998893337`*^9, 
   3.5163834453055067`*^9}, {3.516383476857959*^9, 3.51638351945615*^9}, {
   3.5163854577936897`*^9, 3.5163856049352503`*^9}, {3.516385952783477*^9, 
   3.516385986208107*^9}, {3.5163867387806797`*^9, 3.516386778242505*^9}, {
   3.516388278925934*^9, 3.516388314455687*^9}, {3.51638840971177*^9, 
   3.51638841247635*^9}, {3.516397565665082*^9, 3.5163976921604767`*^9}, {
   3.516397899028942*^9, 3.516397899322949*^9}, {3.5164076030158243`*^9, 
   3.516407638210791*^9}, {3.516408006778857*^9, 3.516408016021781*^9}, {
   3.51640975918292*^9, 3.516409767769142*^9}, {3.516457465894291*^9, 
   3.516457481056218*^9}, {3.516457559479982*^9, 3.516457610709281*^9}, {
   3.516457646901187*^9, 3.51645764800912*^9}, {3.516457700012754*^9, 
   3.516457786011052*^9}, {3.516457820881053*^9, 3.516458122097102*^9}, 
   3.51645820755171*^9, {3.516458351867426*^9, 3.5164584977762203`*^9}, {
   3.516458531496792*^9, 3.516458550968389*^9}, {3.516458649058075*^9, 
   3.516458748494729*^9}, {3.516458824332374*^9, 3.516458838505898*^9}, {
   3.5164588709359703`*^9, 3.516459125857703*^9}, {3.516459181357025*^9, 
   3.516459184898267*^9}, {3.516459232991765*^9, 3.516459242845731*^9}, {
   3.516459533041588*^9, 3.516459555124057*^9}, {3.516459631924409*^9, 
   3.516459679085615*^9}, {3.516460197000605*^9, 3.516460206808563*^9}, {
   3.5164602992598553`*^9, 3.516460311067243*^9}, {3.5164609129823503`*^9, 
   3.516460968519927*^9}, {3.516462676865677*^9, 3.516462684467883*^9}, {
   3.5164631772156067`*^9, 3.516463276695409*^9}, {3.516463488358333*^9, 
   3.516463493706519*^9}, {3.516463910894813*^9, 3.5164639193960238`*^9}, {
   3.516464097606217*^9, 3.51646419164781*^9}, 3.5164643626396723`*^9, {
   3.516464411015307*^9, 3.516464431083765*^9}, {3.5164646185531282`*^9, 
   3.516464634286481*^9}, {3.516465101313439*^9, 3.5164651163310337`*^9}, {
   3.516465390538862*^9, 3.516465401256546*^9}, {3.516465809366246*^9, 
   3.5164658223352957`*^9}, {3.516468032786314*^9, 3.51646805284263*^9}, {
   3.516468116424253*^9, 3.516468136815833*^9}, {3.516468247182317*^9, 
   3.516468349452812*^9}, {3.5164683848684*^9, 3.516468387922887*^9}, {
   3.5164685154818697`*^9, 3.5164685841468763`*^9}, {3.516469921949301*^9, 
   3.5164699711692543`*^9}, {3.5164700483876743`*^9, 
   3.5164701636501837`*^9}, {3.516471263308015*^9, 3.5164712746994658`*^9}, {
   3.516471352359535*^9, 3.5164713546883163`*^9}, {3.5164714262637653`*^9, 
   3.516471432301978*^9}, {3.516473517538863*^9, 3.516473523403447*^9}, {
   3.516474982691085*^9, 3.516474983982852*^9}, {3.516475459381082*^9, 
   3.5164755286888514`*^9}, {3.5178368227708817`*^9, 3.517836867940639*^9}, 
   3.517836922921733*^9, {3.5685748879394712`*^9, 3.568574942003587*^9}, {
   3.568574975568761*^9, 3.568575073669878*^9}, {3.568575199214121*^9, 
   3.5685753730555277`*^9}, {3.568575430728529*^9, 3.56857544416363*^9}, {
   3.568575639798094*^9, 3.5685757167565727`*^9}, {3.568575765502276*^9, 
   3.568575875843132*^9}, {3.568575964817031*^9, 3.568575996598225*^9}, {
   3.5685815788707113`*^9, 3.5685816231848097`*^9}, {3.568581802957261*^9, 
   3.5685818768577337`*^9}, {3.568583185807536*^9, 3.568583193174665*^9}, {
   3.568583564587915*^9, 3.568583650082548*^9}, {3.568585532768119*^9, 
   3.56858556451052*^9}, {3.56858638617171*^9, 3.568586411272897*^9}, {
   3.568586562870647*^9, 3.5685865750782146`*^9}, 3.568586700783231*^9, {
   3.5685869892028027`*^9, 3.568587050334168*^9}, {3.568587113451311*^9, 
   3.568587177428643*^9}, {3.568587208818534*^9, 3.568587278029192*^9}, {
   3.568587315618803*^9, 3.5685874272201843`*^9}, {3.5685875764556723`*^9, 
   3.568587688191224*^9}, {3.5685877325224733`*^9, 3.568588053803132*^9}, {
   3.568588118269617*^9, 3.5685881777593327`*^9}, 3.568645003388166*^9, {
   3.56864533834613*^9, 3.568645346114463*^9}, {3.5686454301385927`*^9, 
   3.568645448411878*^9}, {3.56864686230995*^9, 3.5686468698342247`*^9}, {
   3.568651210061797*^9, 3.568651358931919*^9}, {3.5686593795667458`*^9, 
   3.5686593799571466`*^9}, {3.568732648299584*^9, 3.568732650992077*^9}}]
}, Closed]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Soliton Contours (Use PFuns later)", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.540663327743*^9, 3.5406633359230003`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"T", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_", ",", "a_", ",", "c_"}], "]"}], "[", "z_", "]"}],
    ":=", 
   RowBox[{"(", GridBox[{
      {"1", "0"},
      {
       RowBox[{
        RowBox[{"-", "c"}], " ", 
        RowBox[{
         RowBox[{"Exp", "[", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}], "]"}], "/", 
         RowBox[{"(", 
          RowBox[{"z", "-", "a"}], ")"}]}]}], "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"T", "[", 
      RowBox[{"-", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_", ",", "a_", ",", "c_"}], "]"}], "[", "z_", "]"}],
    ":=", 
   RowBox[{"(", GridBox[{
      {"1", 
       RowBox[{"\[Lambda]", " ", 
        RowBox[{"cc", "[", 
         RowBox[{
          RowBox[{"-", "c"}], " ", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"\[Theta]", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}], "]"}], "/", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"cc", "[", "z", "]"}], "-", "a"}], ")"}]}]}], "]"}]}]},
      {
       RowBox[{" ", "0"}], "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"S", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_", ",", "a_", ",", "c_"}], "]"}], "[", "z_", "]"}],
    ":=", 
   RowBox[{"(", GridBox[{
      {"1", 
       RowBox[{
        FractionBox[
         RowBox[{"-", 
          RowBox[{"(", 
           RowBox[{"z", "-", "a"}], ")"}]}], "c"], "*", 
        RowBox[{"Exp", "[", 
         RowBox[{"-", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}]}], "]"}]}]},
      {"0", "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"S", "[", 
      RowBox[{"-", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_", ",", "a_", ",", "c_"}], "]"}], "[", "z_", "]"}],
    ":=", 
   RowBox[{"(", GridBox[{
      {"1", "0"},
      {
       RowBox[{"\[Lambda]", " ", 
        RowBox[{"cc", "[", 
         RowBox[{
          FractionBox[
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"cc", "[", "z", "]"}], "-", "a"}], ")"}]}], "c"], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"-", 
            RowBox[{
             RowBox[{"\[Theta]", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}]}], "]"}]}], 
         "]"}]}], "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Tx", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_", ",", "a_", ",", "c_"}], "]"}], "[", "z_", "]"}],
    ":=", 
   RowBox[{"(", GridBox[{
      {"0", "0"},
      {
       RowBox[{
        RowBox[{"-", "c"}], " ", 
        RowBox[{
         RowBox[{"\[Theta]x", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}], 
        RowBox[{
         RowBox[{"Exp", "[", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}], "]"}], "/", 
         RowBox[{"(", 
          RowBox[{"z", "-", "a"}], ")"}]}]}], "0"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Tx", "[", 
      RowBox[{"-", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_", ",", "a_", ",", "c_"}], "]"}], "[", "z_", "]"}],
    ":=", 
   RowBox[{"(", GridBox[{
      {"0", 
       RowBox[{"\[Lambda]", " ", 
        RowBox[{"cc", "[", 
         RowBox[{
          RowBox[{"\[Theta]x", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}], "]"}], 
        RowBox[{"cc", "[", 
         RowBox[{
          RowBox[{"-", "c"}], " ", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"\[Theta]", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}], "]"}], "/", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"cc", "[", "z", "]"}], "-", "a"}], ")"}]}]}], "]"}]}]},
      {
       RowBox[{" ", "0"}], "0"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Sx", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_", ",", "a_", ",", "c_"}], "]"}], "[", "z_", "]"}],
    ":=", 
   RowBox[{"(", GridBox[{
      {"0", 
       RowBox[{
        FractionBox[
         RowBox[{"(", 
          RowBox[{"z", "-", "a"}], ")"}], "c"], "*", 
        RowBox[{
         RowBox[{"\[Theta]x", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}], 
        RowBox[{"Exp", "[", 
         RowBox[{"-", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}]}], "]"}]}]},
      {"0", "0"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Sx", "[", 
      RowBox[{"-", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_", ",", "a_", ",", "c_"}], "]"}], "[", "z_", "]"}],
    ":=", 
   RowBox[{"(", GridBox[{
      {"0", "0"},
      {
       RowBox[{"\[Lambda]", " ", 
        RowBox[{"cc", "[", 
         RowBox[{"-", 
          RowBox[{
           RowBox[{"\[Theta]x", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}]}], "]"}], 
        RowBox[{"cc", "[", 
         RowBox[{
          FractionBox[
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"cc", "[", "z", "]"}], "-", "a"}], ")"}]}], "c"], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"-", 
            RowBox[{
             RowBox[{"\[Theta]", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}]}], "]"}]}], 
         "]"}]}], "0"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Tx", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_", ",", "a_", ",", "c_"}], "]"}], "[", "k_", "]"}],
    ":=", 
   RowBox[{"(", GridBox[{
      {"0", "0"},
      {
       RowBox[{
        RowBox[{"-", 
         RowBox[{
          RowBox[{"\[Theta]x", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}]}], "*", "c", " ", 
        RowBox[{
         RowBox[{"Exp", "[", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}], "]"}], "/", 
         RowBox[{"(", 
          RowBox[{"k", "-", "a"}], ")"}]}]}], "0"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Tx", "[", 
      RowBox[{"-", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_", ",", "a_", ",", "c_"}], "]"}], "[", "k_", "]"}],
    ":=", 
   RowBox[{"(", GridBox[{
      {"0", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{
          RowBox[{"\[Theta]x", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}]}], "*", "c", " ", 
        RowBox[{
         RowBox[{"Exp", "[", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}], "]"}], "/", 
         RowBox[{"(", 
          RowBox[{"k", "+", "a"}], ")"}]}]}]},
      {"0", "0"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Sx", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_", ",", "a_", ",", "c_"}], "]"}], "[", "k_", "]"}],
    ":=", 
   RowBox[{"(", GridBox[{
      {"0", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Theta]x", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}], "*", 
        FractionBox[
         RowBox[{"(", 
          RowBox[{"k", "-", "a"}], ")"}], "c"], "*", 
        RowBox[{"Exp", "[", 
         RowBox[{"-", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}]}], "]"}]}]},
      {"0", "0"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Sx", "[", 
      RowBox[{"-", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_", ",", "a_", ",", "c_"}], "]"}], "[", "k_", "]"}],
    ":=", 
   RowBox[{"(", GridBox[{
      {"0", "0"},
      {
       RowBox[{
        RowBox[{
         RowBox[{"\[Theta]x", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}], "*", 
        FractionBox[
         RowBox[{"(", 
          RowBox[{"k", "+", "a"}], ")"}], "c"], "*", 
        RowBox[{"Exp", "[", 
         RowBox[{"-", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}]}], "]"}]}], "0"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ContourArray", "[", 
    RowBox[{
     RowBox[{"+", "1"}], ",", "r_", ",", "a_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Arc", "[", 
      RowBox[{"a", ",", "r", ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "Pi"}], "}"}]}], "]"}], ",", 
     RowBox[{"Arc", "[", 
      RowBox[{"a", ",", "r", ",", 
       RowBox[{"{", 
        RowBox[{"Pi", ",", 
         RowBox[{"2", "Pi"}]}], "}"}]}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ContourArray", "[", 
    RowBox[{
     RowBox[{"-", "1"}], ",", "r_", ",", "a_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Arc", "[", 
      RowBox[{
       RowBox[{"a", "//", "cc"}], ",", "r", ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", 
         RowBox[{"-", "Pi"}]}], "}"}]}], "]"}], ",", 
     RowBox[{"Arc", "[", 
      RowBox[{
       RowBox[{"a", "//", "cc"}], ",", "r", ",", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"-", "Pi"}], ",", 
         RowBox[{
          RowBox[{"-", "2"}], "Pi"}]}], "}"}]}], "]"}]}], "}"}]}], 
  ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.540663356863*^9, 3.5406634779230003`*^9}, {
   3.540663514553*^9, 3.540663559523*^9}, {3.5406636903546*^9, 
   3.5406638634946003`*^9}, {3.5406639640346003`*^9, 3.5406640204646*^9}, {
   3.5406641670945997`*^9, 3.5406641983646*^9}, {3.5406642496246*^9, 
   3.5406643193246*^9}, {3.5406643716646*^9, 3.5406643812946*^9}, {
   3.5406644820945997`*^9, 3.5406645177646*^9}, 3.5406645531018*^9, {
   3.5406646915018*^9, 3.5406648196538*^9}, {3.540668185589525*^9, 
   3.540668263873556*^9}, {3.54066831648492*^9, 3.5406684168554087`*^9}, {
   3.54066844904218*^9, 3.540668517764138*^9}, {3.540668618759264*^9, 
   3.540668717261815*^9}, {3.540669354468211*^9, 3.540669359280778*^9}, {
   3.540669502128993*^9, 3.5406695823656063`*^9}, {3.54066961440208*^9, 
   3.540669662983015*^9}, {3.54067189344142*^9, 3.540671925802581*^9}, {
   3.5406731652249823`*^9, 3.5406732638866568`*^9}, {3.5406741647813997`*^9, 
   3.5406741678613997`*^9}, {3.5406741991954*^9, 3.5406742011074*^9}, {
   3.5406826199754*^9, 3.5406826249944*^9}, {3.5406834491594*^9, 
   3.5406834534044*^9}, {3.5406835138184*^9, 3.5406835198314*^9}, {
   3.5406839573473997`*^9, 3.5406839578014*^9}, {3.5406875816144*^9, 
   3.5406875942444*^9}, {3.5406893661443996`*^9, 3.5406893835144*^9}, {
   3.5406894255744*^9, 3.5406894318104*^9}, {3.5407264300074*^9, 
   3.5407264358624*^9}, {3.540735266297641*^9, 3.54073530536005*^9}, {
   3.540735803202181*^9, 3.540735809259849*^9}, {3.540736099083493*^9, 
   3.540736100324586*^9}, {3.540737377234128*^9, 3.540737392702014*^9}, {
   3.540738683107176*^9, 3.540738743177516*^9}, {3.540738779460793*^9, 
   3.540738786570944*^9}, {3.540740941006283*^9, 3.540740955773658*^9}, {
   3.5407412010663157`*^9, 3.540741317799933*^9}, {3.5407610939393997`*^9, 
   3.5407612605094*^9}, {3.5407620108694*^9, 3.5407620122693996`*^9}, {
   3.5407621572894*^9, 3.5407621698494*^9}, {3.5407622649393997`*^9, 
   3.5407622724894*^9}, 3.5407625509994*^9, {3.5407640913234*^9, 
   3.5407640999234*^9}, {3.5407690321454*^9, 3.5407690325254*^9}, {
   3.5409028140464*^9, 3.5409028143864*^9}, {3.5656323663862*^9, 
   3.5656324283422003`*^9}, {3.5656324906851997`*^9, 
   3.5656325725522003`*^9}, {3.5656573568952*^9, 3.5656574370852003`*^9}, {
   3.5657050454421997`*^9, 3.5657051195582*^9}, {3.5657052135692*^9, 
   3.5657052213722*^9}, {3.5657053117812*^9, 3.5657053210292*^9}, {
   3.568131470694765*^9, 3.5681315057104*^9}, {3.568387207215556*^9, 
   3.568387265090479*^9}, {3.568387306381442*^9, 3.568387400253421*^9}, {
   3.568401487663148*^9, 3.5684016046455173`*^9}, {3.5684016631059103`*^9, 
   3.5684016871999893`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"PoleList", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"out", ",", "i", ",", "Domain", ",", "Jumps", ",", "Points"}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Domain", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Jumps", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Points", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"i", "=", "1"}], ",", 
        RowBox[{"i", "\[LessEqual]", 
         RowBox[{"Length", "[", "A", "]"}]}], ",", 
        RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"1", "/", "globalTol"}], ">", 
             RowBox[{
              RowBox[{"Abs", "[", 
               RowBox[{"c", "[", 
                RowBox[{"[", "i", "]"}], "]"}], "]"}], 
              RowBox[{"Exp", "[", 
               RowBox[{"Re", "[", 
                RowBox[{
                 RowBox[{"\[Theta]", "[", 
                  RowBox[{"x", ",", "t"}], "]"}], "[", 
                 RowBox[{"A", "[", 
                  RowBox[{"[", "i", "]"}], "]"}], "]"}], "]"}], "]"}]}]}], "&&", 
            RowBox[{
             RowBox[{
              RowBox[{"Abs", "[", 
               RowBox[{"c", "[", 
                RowBox[{"[", "i", "]"}], "]"}], "]"}], 
              RowBox[{"Exp", "[", 
               RowBox[{"Re", "[", 
                RowBox[{
                 RowBox[{"\[Theta]", "[", 
                  RowBox[{"x", ",", "t"}], "]"}], "[", 
                 RowBox[{"A", "[", 
                  RowBox[{"[", "i", "]"}], "]"}], "]"}], "]"}], "]"}]}], ">", 
             " ", "globalTol"}]}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{"Abs", "[", 
                 RowBox[{"c", "[", 
                  RowBox[{"[", "i", "]"}], "]"}], "]"}], 
                RowBox[{"Exp", "[", 
                 RowBox[{"Re", "[", 
                  RowBox[{
                   RowBox[{"\[Theta]", "[", 
                    RowBox[{"x", ",", "t"}], "]"}], "[", 
                   RowBox[{"A", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], "]"}], "]"}], "]"}]}], 
               ">", "expconst"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"Jumps", "=", 
                RowBox[{"Join", "[", 
                 RowBox[{"Jumps", ",", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"S", "[", 
                    RowBox[{"+", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"A", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
                    RowBox[{
                    RowBox[{"S", "[", 
                    RowBox[{"+", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"A", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], "}"}]}], 
                 "]"}]}], ";", "\[IndentingNewLine]", 
               RowBox[{"Jumps", "=", 
                RowBox[{"Join", "[", 
                 RowBox[{"Jumps", ",", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"S", "[", 
                    RowBox[{"-", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"A", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
                    RowBox[{
                    RowBox[{"S", "[", 
                    RowBox[{"-", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"A", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], "}"}]}], 
                 "]"}]}], ";", "\[IndentingNewLine]", 
               RowBox[{"flag", "=", "\"\<pos\>\""}], ";"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"Jumps", " ", "=", " ", 
                RowBox[{"Join", "[", 
                 RowBox[{"Jumps", ",", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"T", "[", 
                    RowBox[{"+", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"A", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
                    RowBox[{
                    RowBox[{"T", "[", 
                    RowBox[{"+", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"A", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], "}"}]}], 
                 "]"}]}], ";", "\[IndentingNewLine]", 
               RowBox[{"Jumps", " ", "=", " ", 
                RowBox[{"Join", "[", 
                 RowBox[{"Jumps", ",", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"T", "[", 
                    RowBox[{"-", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"A", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
                    RowBox[{
                    RowBox[{"T", "[", 
                    RowBox[{"-", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"A", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], "}"}]}], 
                 "]"}]}], ";", "\[IndentingNewLine]", 
               RowBox[{"flag", "=", "\"\<neg\>\""}], ";"}]}], 
             "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
            RowBox[{"Domain", "=", " ", 
             RowBox[{"Join", "[", 
              RowBox[{"Domain", ",", 
               RowBox[{"Join", "[", 
                RowBox[{
                 RowBox[{"ContourArray", "[", 
                  RowBox[{
                   RowBox[{"+", "1"}], ",", "rad", ",", 
                   RowBox[{"A", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
                 RowBox[{"ContourArray", "[", 
                  RowBox[{
                   RowBox[{"-", "1"}], ",", "rad", ",", 
                   RowBox[{"A", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], "]"}]}], 
              "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"Points", "=", 
             RowBox[{"Join", "[", 
              RowBox[{"Points", ",", 
               RowBox[{"{", 
                RowBox[{
                "smallN", ",", "smallN", ",", "smallN", ",", "smallN"}], 
                "}"}]}], "]"}]}], ";"}]}], "\[IndentingNewLine]", "]"}], 
         ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"Jumps", ",", "Domain", ",", "Points"}], "}"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"PoleListx", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"out", ",", "i", ",", "Domain", ",", "Jumps", ",", "Points"}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Jumps", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"i", "=", "1"}], ",", 
        RowBox[{"i", "\[LessEqual]", 
         RowBox[{"Length", "[", "A", "]"}]}], ",", 
        RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"1", "/", "globalTol"}], ">", 
             RowBox[{
              RowBox[{"Abs", "[", 
               RowBox[{"c", "[", 
                RowBox[{"[", "i", "]"}], "]"}], "]"}], 
              RowBox[{"Exp", "[", 
               RowBox[{"Re", "[", 
                RowBox[{
                 RowBox[{"\[Theta]", "[", 
                  RowBox[{"x", ",", "t"}], "]"}], "[", 
                 RowBox[{"A", "[", 
                  RowBox[{"[", "i", "]"}], "]"}], "]"}], "]"}], "]"}]}]}], "&&", 
            RowBox[{
             RowBox[{
              RowBox[{"Abs", "[", 
               RowBox[{"c", "[", 
                RowBox[{"[", "i", "]"}], "]"}], "]"}], 
              RowBox[{"Exp", "[", 
               RowBox[{"Re", "[", 
                RowBox[{
                 RowBox[{"\[Theta]", "[", 
                  RowBox[{"x", ",", "t"}], "]"}], "[", 
                 RowBox[{"A", "[", 
                  RowBox[{"[", "i", "]"}], "]"}], "]"}], "]"}], "]"}]}], ">", 
             " ", "globalTol"}]}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{"Abs", "[", 
                 RowBox[{"c", "[", 
                  RowBox[{"[", "i", "]"}], "]"}], "]"}], 
                RowBox[{"Exp", "[", 
                 RowBox[{"Re", "[", 
                  RowBox[{
                   RowBox[{"\[Theta]", "[", 
                    RowBox[{"x", ",", "t"}], "]"}], "[", 
                   RowBox[{"A", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], "]"}], "]"}], "]"}]}], 
               ">", "expconst"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"Jumps", "=", 
                RowBox[{"Join", "[", 
                 RowBox[{"Jumps", ",", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"Sx", "[", 
                    RowBox[{"+", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"A", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
                    RowBox[{
                    RowBox[{"Sx", "[", 
                    RowBox[{"+", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"A", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], "}"}]}], 
                 "]"}]}], ";", "\[IndentingNewLine]", 
               RowBox[{"Jumps", "=", 
                RowBox[{"Join", "[", 
                 RowBox[{"Jumps", ",", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"Sx", "[", 
                    RowBox[{"-", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"A", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
                    RowBox[{
                    RowBox[{"Sx", "[", 
                    RowBox[{"-", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"A", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], "}"}]}], 
                 "]"}]}], ";", "\[IndentingNewLine]", 
               RowBox[{"flag", "=", "\"\<pos\>\""}], ";"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"Jumps", " ", "=", " ", 
                RowBox[{"Join", "[", 
                 RowBox[{"Jumps", ",", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"Tx", "[", 
                    RowBox[{"+", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"A", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
                    RowBox[{
                    RowBox[{"Tx", "[", 
                    RowBox[{"+", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"A", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], "}"}]}], 
                 "]"}]}], ";", "\[IndentingNewLine]", 
               RowBox[{"Jumps", " ", "=", " ", 
                RowBox[{"Join", "[", 
                 RowBox[{"Jumps", ",", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"Tx", "[", 
                    RowBox[{"-", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"A", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
                    RowBox[{
                    RowBox[{"Tx", "[", 
                    RowBox[{"-", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"A", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], "}"}]}], 
                 "]"}]}], ";", "\[IndentingNewLine]", 
               RowBox[{"flag", "=", "\"\<neg\>\""}], ";"}]}], 
             "\[IndentingNewLine]", "]"}], ";"}]}], "\[IndentingNewLine]", 
          "]"}], ";"}]}], "\[IndentingNewLine]", "]"}], ";", 
      "\[IndentingNewLine]", "Jumps"}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.540663356863*^9, 3.5406634779230003`*^9}, {
   3.540663514553*^9, 3.540663559523*^9}, {3.5406636903546*^9, 
   3.5406638634946003`*^9}, {3.5406639640346003`*^9, 3.5406640204646*^9}, {
   3.5406641670945997`*^9, 3.5406641983646*^9}, {3.5406642496246*^9, 
   3.5406643193246*^9}, {3.5406643716646*^9, 3.5406643812946*^9}, {
   3.5406644820945997`*^9, 3.5406645177646*^9}, 3.5406645531018*^9, {
   3.5406646915018*^9, 3.5406648196538*^9}, {3.540668185589525*^9, 
   3.540668263873556*^9}, {3.54066831648492*^9, 3.5406684168554087`*^9}, {
   3.54066844904218*^9, 3.540668517764138*^9}, {3.540668618759264*^9, 
   3.540668717261815*^9}, {3.540669354468211*^9, 3.540669359280778*^9}, {
   3.540669502128993*^9, 3.5406695823656063`*^9}, {3.54066961440208*^9, 
   3.540669662983015*^9}, {3.54067189344142*^9, 3.540671925802581*^9}, {
   3.5406731652249823`*^9, 3.5406732638866568`*^9}, {3.5406741647813997`*^9, 
   3.5406741678613997`*^9}, {3.5406741991954*^9, 3.5406742011074*^9}, {
   3.5406826199754*^9, 3.5406826249944*^9}, {3.5406834491594*^9, 
   3.5406834534044*^9}, {3.5406835138184*^9, 3.5406835198314*^9}, {
   3.5406839573473997`*^9, 3.5406839578014*^9}, {3.5406875816144*^9, 
   3.5406875942444*^9}, {3.5406893661443996`*^9, 3.5406893835144*^9}, {
   3.5406894255744*^9, 3.5406894318104*^9}, {3.5407264300074*^9, 
   3.5407264358624*^9}, {3.540735266297641*^9, 3.54073530536005*^9}, {
   3.540735803202181*^9, 3.540735809259849*^9}, {3.540736099083493*^9, 
   3.540736100324586*^9}, {3.540737377234128*^9, 3.540737392702014*^9}, {
   3.540738683107176*^9, 3.540738743177516*^9}, {3.540738779460793*^9, 
   3.540738786570944*^9}, {3.540740941006283*^9, 3.540740955773658*^9}, {
   3.5407412010663157`*^9, 3.540741317799933*^9}, {3.5407610939393997`*^9, 
   3.5407612605094*^9}, {3.5407620108694*^9, 3.5407620122693996`*^9}, {
   3.5407621572894*^9, 3.5407621698494*^9}, {3.5407622649393997`*^9, 
   3.5407622724894*^9}, {3.5407625509994*^9, 3.5407625539294*^9}, 
   3.5407628404994*^9, {3.5407631811373997`*^9, 3.5407631847774*^9}, {
   3.5407638581274*^9, 3.5407638642874002`*^9}, {3.5407639867474003`*^9, 
   3.5407639944874*^9}, {3.5409021567874002`*^9, 3.5409021699004*^9}, {
   3.5409024156774*^9, 3.5409024266174*^9}, {3.5418962414278*^9, 
   3.5418962696868*^9}, {3.5656325796591997`*^9, 3.5656326739922*^9}, {
   3.5656408345022*^9, 3.5656408553482*^9}, {3.5656409296572*^9, 
   3.5656409311712*^9}, {3.5656410927402*^9, 3.5656410944161997`*^9}, {
   3.5656569424672003`*^9, 3.5656569628112*^9}, {3.5656570735601997`*^9, 
   3.5656570739122*^9}, {3.5656574485282*^9, 3.5656574568632*^9}, {
   3.5656575081242*^9, 3.5656575201902*^9}, 3.5657051296362*^9, {
   3.5657051636382*^9, 3.5657051928532*^9}, {3.5657052255741997`*^9, 
   3.5657052641122*^9}, {3.5657053266972*^9, 3.5657053442622004`*^9}, {
   3.5657055039852*^9, 3.5657056261212*^9}, {3.5657056569862003`*^9, 
   3.5657058077992*^9}, 3.5657058821342*^9, {3.5657059386932*^9, 
   3.5657060435632*^9}, {3.5657299619792*^9, 3.5657300165132*^9}, {
   3.5657300566001997`*^9, 3.5657300661042*^9}, {3.5657301133512*^9, 
   3.5657301772592*^9}, {3.5657304984102*^9, 3.5657305008901997`*^9}, {
   3.5657309373652*^9, 3.5657309378262*^9}, {3.5657310008402*^9, 
   3.5657310015192003`*^9}, {3.5657311318742*^9, 3.5657311407702*^9}, {
   3.5657311733612003`*^9, 3.5657311765502*^9}, {3.5657312242911997`*^9, 
   3.5657312298772*^9}, {3.5657312702562*^9, 3.5657312727232*^9}, {
   3.5657319452152*^9, 3.5657319568372*^9}, 3.5657319998802*^9, {
   3.5657320713552*^9, 3.5657321651472*^9}, 3.5657321965702*^9, {
   3.5657322322392*^9, 3.5657322623932*^9}, {3.5657325516442003`*^9, 
   3.5657325518982*^9}, {3.568131486325077*^9, 3.568131530817397*^9}, {
   3.568387418587628*^9, 3.568387476289146*^9}, {3.568397033062167*^9, 
   3.568397035241928*^9}, {3.568737524407069*^9, 3.5687375871719637`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Add Poles and Choose", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.5656330942762003`*^9, 3.5656330964582*^9}, {
  3.565800226807107*^9, 3.565800228918049*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"DomainIntegrate", "[", "0.", "]"}], ":=", "0"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"DomainIntegrate", "[", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"0.", ",", "0."}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0.", ",", "0."}], "}"}]}], "}"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0.", ",", "0."}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"0.", ",", "0."}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"tcrit", "=", "8"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"RMM1", "[", "M_", "]"}], ":=", 
   RowBox[{"-", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "M"}], "//", "AddIdentityMatrix"}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"KdV", "[", "i_", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "rhp1", ",", "rhp2", ",", "rhp1x", ",", "rhp2x", ",", "scale", ",", 
       "\[CapitalPhi]1", ",", "\[CapitalPhi]2", ",", "\[CapitalPhi]", ",", 
       "out", ",", "t0", ",", "t1", ",", "t2", ",", "t3", ",", "t4", ",", 
       "poles", ",", "pl", ",", "ja", ",", "sol", ",", "Cm1", ",", "solx", 
       ",", "t1x", ",", "rh1", ",", "rh2", ",", "rh3", ",", 
       "\[CapitalPhi]1in", ",", "\[CapitalPhi]1x", ",", "\[CapitalPhi]1inx", 
       ",", "plx"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"mRHSolver", "//", "Clear"}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"mRHSolver", "[", "X_", "]"}], ":=", 
       RowBox[{
        RowBox[{"mRHSolver", "[", "X", "]"}], "=", 
        RowBox[{"RHSolver", "[", "X", "]"}]}]}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"RHSolvedx", "[", 
        RowBox[{"X_", ",", "XX_"}], "]"}], ":=", 
       RowBox[{"Module", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"Cm", ",", "rhs"}], "}"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"If", "[", 
           RowBox[{
            RowBox[{
             RowBox[{"X", "\[Equal]", 
              RowBox[{"{", "}"}]}], "&&", 
             RowBox[{"XX", "\[Equal]", 
              RowBox[{"{", "}"}]}]}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{
              RowBox[{"sol", "[", 
               RowBox[{"{", "}"}], "]"}], "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0.", ",", "0."}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.", ",", "0."}], "}"}]}], "}"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"solx", "[", 
               RowBox[{
                RowBox[{"{", "}"}], ",", 
                RowBox[{"{", "}"}]}], "]"}], "=", 
              RowBox[{"{", 
               RowBox[{
                RowBox[{"{", 
                 RowBox[{"0.", ",", "0."}], "}"}], ",", 
                RowBox[{"{", 
                 RowBox[{"0.", ",", "0."}], "}"}]}], "}"}]}], ";"}], ",", 
            "\[IndentingNewLine]", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{
              RowBox[{"sol", "[", "X", "]"}], "=", 
              RowBox[{
               RowBox[{"mRHSolver", "[", "X", "]"}], "[", "X", "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"Cm", "=", 
              RowBox[{"CauchyOperator", "[", 
               RowBox[{
                RowBox[{"-", "1"}], ",", 
                RowBox[{"mRHSolver", "[", "X", "]"}]}], "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"rhs", "=", 
              RowBox[{
               RowBox[{"Cm", "[", 
                RowBox[{"sol", "[", "X", "]"}], "]"}], "//", 
               "AddIdentityMatrix"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"solx", "[", 
               RowBox[{"X", ",", "XX"}], "]"}], "=", 
              RowBox[{
               RowBox[{"mRHSolver", "[", "X", "]"}], "[", 
               RowBox[{"X", ",", 
                RowBox[{"rhs", "~", "FunListDot", "~", "XX"}]}], "]"}]}], 
             ";"}]}], "\[IndentingNewLine]", "]"}], ";"}]}], 
        "\[IndentingNewLine]", "]"}]}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"RHSolved", "//", "Clear"}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"RHSolved", "[", "X_", "]"}], ":=", 
       RowBox[{
        RowBox[{"RHSolved", "[", "X", "]"}], "=", 
        RowBox[{"mRHSolver", "[", "X", "]"}]}]}], ";", "\[IndentingNewLine]", 
      
      RowBox[{
       RowBox[{"RHSolved", "[", 
        RowBox[{"{", "}"}], "]"}], ":=", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"0.", ",", "0."}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0.", ",", "0"}], "}"}]}], "}"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Cauchy", "[", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"0.", ",", "0."}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"0.", ",", "0."}], "}"}]}], "}"}], ",", "k_"}], "]"}], ":=", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"0.", ",", "0."}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{"0.", ",", "0."}], "}"}]}], "}"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"i", "\[Equal]", "4"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"CSGFUN", "[", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"Generate\[Delta]cs", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"Generate\[Zeta]", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ";"}]}], "\[IndentingNewLine]", 
       "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"i", "\[Equal]", "5"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"TransitionGFUN", "[", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"Generate\[Zeta]t", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ";"}]}], "\[IndentingNewLine]", 
       "]"}], ";", "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
       "Construct", " ", "first", " ", "RHP", " ", "and", " ", "RHPx"}], " ", 
       "*)"}], "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
        RowBox[{"Return", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Jadapt", "[", "i", "]"}], "[", 
             RowBox[{"x", ",", "t"}], "]"}], "//", "MakeListFun"}], ",", 
           RowBox[{
            RowBox[{
             RowBox[{"Jadaptx", "[", "i", "]"}], "[", 
             RowBox[{"x", ",", "t"}], "]"}], "//", "MakeListFun"}]}], "}"}], 
         "]"}], ";"}], "*)"}], "\[IndentingNewLine]", 
      RowBox[{"t0", "=", 
       RowBox[{"AbsoluteTiming", "[", 
        RowBox[{"ja", "=", 
         RowBox[{
          RowBox[{"Jadapt", "[", "i", "]"}], "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"ja", "\[NotEqual]", 
         RowBox[{"{", 
          RowBox[{"{", "}"}], "}"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"i", "<", "4"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"t1", "=", 
             RowBox[{"AbsoluteTiming", "[", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"rhp1", "=", 
                RowBox[{
                 RowBox[{"ConjugateList", "[", 
                  RowBox[{"ja", ",", 
                   RowBox[{"Q", "[", 
                    RowBox[{"x", ",", "t"}], "]"}]}], "]"}], "//", 
                 "MakeListFun"}]}], ";", "\[IndentingNewLine]", 
               RowBox[{"rhp1x", "=", 
                RowBox[{
                 RowBox[{"ConjugateList", "[", 
                  RowBox[{
                   RowBox[{
                    RowBox[{"Jadaptx", "[", "i", "]"}], "[", 
                    RowBox[{"x", ",", "t"}], "]"}], ",", 
                   RowBox[{"Q", "[", 
                    RowBox[{"x", ",", "t"}], "]"}]}], "]"}], "//", 
                 "MakeListFun"}]}], ";"}], "\[IndentingNewLine]", "]"}]}], 
            ";"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"t1", "=", 
             RowBox[{"AbsoluteTiming", "[", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"rhp1", "=", 
                RowBox[{"ja", "//", "MakeListFun"}]}], ";", 
               "\[IndentingNewLine]", 
               RowBox[{"rhp1x", "=", 
                RowBox[{
                 RowBox[{
                  RowBox[{"Jadaptx", "[", "i", "]"}], "[", 
                  RowBox[{"x", ",", "t"}], "]"}], "//", "MakeListFun"}]}], 
               ";"}], "\[IndentingNewLine]", "]"}]}], ";"}]}], 
          "\[IndentingNewLine]", "]"}], ";"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"t1", "=", 
          RowBox[{"{", "0", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"rhp1", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"rhp1x", "=", 
          RowBox[{"{", "}"}]}], ";"}]}], "\[IndentingNewLine]", 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"t2", " ", "=", " ", 
       RowBox[{"AbsoluteTiming", "[", 
        RowBox[{
         RowBox[{"RHSolvedx", "[", 
          RowBox[{"rhp1", ",", "rhp1x"}], "]"}], ";"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
        RowBox[{"Return", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"sol", "[", "rhp1", "]"}], ",", 
           RowBox[{"solx", "[", 
            RowBox[{"rhp1", ",", "rhp1x"}], "]"}]}], "}"}], "]"}], ";"}], 
       "*)"}], "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"(*", " ", 
       RowBox[{
       "Solution", " ", "of", " ", "first", " ", "RHP", " ", "and", " ", 
        "derivative", " ", "with", " ", "correct", " ", "inverse", " ", 
        "scaling"}], " ", "*)"}], "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"InvScale", "[", "xx_", "]"}], ":=", 
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"i", " ", "<", "4"}], ",", " ", "xx", ",", 
         RowBox[{"xx", "/", 
          RowBox[{"K0", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"\[CapitalPhi]1", "//", "Clear"}], ";", 
      RowBox[{"\[CapitalPhi]1x", "//", "Clear"}], ";", "\[IndentingNewLine]", 
      
      RowBox[{
       RowBox[{"\[CapitalPhi]1", "[", "z_", "]"}], ":=", 
       RowBox[{
        RowBox[{"\[CapitalPhi]1", "[", "z", "]"}], "=", 
        RowBox[{
         RowBox[{"Cauchy", "[", 
          RowBox[{
           RowBox[{"sol", "[", "rhp1", "]"}], ",", 
           RowBox[{"z", "//", "InvScale"}]}], "]"}], "+", 
         RowBox[{"IdentityMatrix", "[", "2", "]"}]}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"\[CapitalPhi]1x", "[", "z_", "]"}], ":=", 
       RowBox[{
        RowBox[{"\[CapitalPhi]1x", "[", "z", "]"}], "=", 
        RowBox[{"Cauchy", "[", 
         RowBox[{
          RowBox[{"solx", "[", 
           RowBox[{"rhp1", ",", "rhp1x"}], "]"}], ",", 
          RowBox[{"z", "//", "InvScale"}]}], "]"}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"\[CapitalPhi]1inx", "[", "z_", "]"}], ":=", 
       RowBox[{"Module", "[", 
        RowBox[{
         RowBox[{"{", "temp", "}"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"temp", " ", "=", " ", 
           RowBox[{"\[CapitalPhi]1x", "[", "z", "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
              RowBox[{"temp", "[", 
               RowBox[{"[", 
                RowBox[{"2", ",", "2"}], "]"}], "]"}], ",", 
              RowBox[{"-", 
               RowBox[{"temp", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ",", "2"}], "]"}], "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"-", 
               RowBox[{"temp", "[", 
                RowBox[{"[", 
                 RowBox[{"2", ",", "1"}], "]"}], "]"}]}], ",", 
              RowBox[{"temp", "[", 
               RowBox[{"[", 
                RowBox[{"1", ",", "1"}], "]"}], "]"}]}], "}"}]}], "}"}]}]}], 
        "\[IndentingNewLine]", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"\[CapitalPhi]1in", "[", "z_", "]"}], ":=", 
       RowBox[{"Module", "[", 
        RowBox[{
         RowBox[{"{", "temp", "}"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"temp", " ", "=", " ", 
           RowBox[{"\[CapitalPhi]1", "[", "z", "]"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{
              RowBox[{"temp", "[", 
               RowBox[{"[", 
                RowBox[{"2", ",", "2"}], "]"}], "]"}], ",", 
              RowBox[{"-", 
               RowBox[{"temp", "[", 
                RowBox[{"[", 
                 RowBox[{"1", ",", "2"}], "]"}], "]"}]}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"-", 
               RowBox[{"temp", "[", 
                RowBox[{"[", 
                 RowBox[{"2", ",", "1"}], "]"}], "]"}]}], ",", 
              RowBox[{"temp", "[", 
               RowBox[{"[", 
                RowBox[{"1", ",", "1"}], "]"}], "]"}]}], "}"}]}], "}"}]}]}], 
        "\[IndentingNewLine]", "]"}]}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"pl", "=", 
       RowBox[{"PoleList", "[", 
        RowBox[{"x", ",", "t"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"pl", "\[Equal]", 
         RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"rhp2", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"t3", "=", 
          RowBox[{"{", "0", "}"}]}], ";"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"rhp2", "=", " ", 
          RowBox[{"ConjugateList", "[", 
           RowBox[{"pl", ",", 
            RowBox[{"Q", "[", 
             RowBox[{"x", ",", "t"}], "]"}]}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"i", "\[Equal]", "3"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"rhp2", "=", " ", 
             RowBox[{"ConjugateList", "[", 
              RowBox[{"rhp2", ",", 
               RowBox[{"\[CapitalPhi]tin", "[", 
                RowBox[{"x", ",", "t"}], "]"}]}], "]"}]}], ";"}]}], 
          "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"i", "==", "4"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"rhp2", "=", " ", 
             RowBox[{"ConjugateList", "[", 
              RowBox[{"rhp2", ",", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[CapitalDelta]k", "[", "1", "]"}], "[", 
                 RowBox[{"#", "//", "InvScale"}], "]"}], "&"}]}], "]"}]}], 
            ";", "\[IndentingNewLine]", 
            RowBox[{"rhp2", "=", " ", 
             RowBox[{"ConjugateList", "[", 
              RowBox[{"rhp2", ",", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"\[Phi]", "[", "1", "]"}], "[", 
                  RowBox[{"x", ",", "t"}], "]"}], "[", 
                 RowBox[{"#", "//", "InvScale"}], "]"}], "&"}]}], "]"}]}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"i", "\[Equal]", "5"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"rhp2", "=", " ", 
             RowBox[{"ConjugateList", "[", 
              RowBox[{"rhp2", ",", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"\[Phi]", "[", "1", "]"}], "[", 
                  RowBox[{"x", ",", "t"}], "]"}], "[", 
                 RowBox[{"#", "//", "InvScale"}], "]"}], "&"}]}], "]"}]}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"pl", " ", "=", " ", "rhp2"}], ";", "\[IndentingNewLine]", 
         RowBox[{"t3", "=", 
          RowBox[{"AbsoluteTiming", "[", 
           RowBox[{
            RowBox[{"rhp2", "=", " ", 
             RowBox[{
              RowBox[{"ConjugateList", "[", 
               RowBox[{"rhp2", ",", "\[CapitalPhi]1in"}], "]"}], "//", 
              "MakeListFun"}]}], ";"}], "]"}]}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      "\[IndentingNewLine]", 
      RowBox[{"plx", "=", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"PoleListx", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ",", 
         RowBox[{"pl", "[", 
          RowBox[{"[", "2", "]"}], "]"}], ",", 
         RowBox[{"pl", "[", 
          RowBox[{"[", "3", "]"}], "]"}]}], "}"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"plx", "[", 
          RowBox[{"[", "1", "]"}], "]"}], "\[Equal]", 
         RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"rhp2x", "=", 
          RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"solx", "[", 
           RowBox[{"rhp2", ",", "rhp2x"}], "]"}], "=", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"{", 
             RowBox[{"0.", ",", "0."}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"0.", ",", "0."}], "}"}]}], "}"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"t3", "=", 
          RowBox[{"{", "0", "}"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"t4", " ", "=", 
          RowBox[{"{", "0", "}"}]}], ";"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"rhp2x", "=", " ", 
          RowBox[{"ConjugateList", "[", 
           RowBox[{"plx", ",", 
            RowBox[{"Q", "[", 
             RowBox[{"x", ",", "t"}], "]"}]}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"i", "\[Equal]", "3"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"rhp2x", "=", " ", 
             RowBox[{"ConjugateList", "[", 
              RowBox[{"rhp2x", ",", 
               RowBox[{"\[CapitalPhi]tin", "[", 
                RowBox[{"x", ",", "t"}], "]"}]}], "]"}]}], ";"}]}], 
          "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"i", "==", "4"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"rhp2x", "=", " ", 
             RowBox[{"ConjugateList", "[", 
              RowBox[{"rhp2x", ",", 
               RowBox[{
                RowBox[{
                 RowBox[{"\[CapitalDelta]k", "[", "1", "]"}], "[", 
                 RowBox[{"#", "//", "InvScale"}], "]"}], "&"}]}], "]"}]}], 
            ";", "\[IndentingNewLine]", 
            RowBox[{"rhp2x", "=", " ", 
             RowBox[{"ConjugateList", "[", 
              RowBox[{"rhp2x", ",", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"\[Phi]", "[", "1", "]"}], "[", 
                  RowBox[{"x", ",", "t"}], "]"}], "[", 
                 RowBox[{"#", "//", "InvScale"}], "]"}], "&"}]}], "]"}]}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{"i", "\[Equal]", "5"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"rhp2x", "=", " ", 
             RowBox[{"ConjugateList", "[", 
              RowBox[{"rhp2x", ",", 
               RowBox[{
                RowBox[{
                 RowBox[{
                  RowBox[{"\[Phi]", "[", "1", "]"}], "[", 
                  RowBox[{"x", ",", "t"}], "]"}], "[", 
                 RowBox[{"#", "//", "InvScale"}], "]"}], "&"}]}], "]"}]}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
         RowBox[{"t3", "=", 
          RowBox[{"t3", "+", 
           RowBox[{"AbsoluteTiming", "[", 
            RowBox[{
             RowBox[{"rh1", "=", " ", 
              RowBox[{"MakeListFun", "[", 
               RowBox[{"ConjugateList", "[", 
                RowBox[{"rhp2x", ",", "\[CapitalPhi]1in"}], "]"}], "]"}]}], 
             ";", "\[IndentingNewLine]", 
             RowBox[{"rh2", "=", 
              RowBox[{"MakeListFun", "[", 
               RowBox[{"RightMultiplyList", "[", 
                RowBox[{
                 RowBox[{"LeftMultiplyList", "[", 
                  RowBox[{"pl", ",", "\[CapitalPhi]1x"}], "]"}], ",", 
                 "\[CapitalPhi]1in"}], "]"}], "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"rh3", "=", 
              RowBox[{"MakeListFun", "[", 
               RowBox[{"RightMultiplyList", "[", 
                RowBox[{
                 RowBox[{"LeftMultiplyList", "[", 
                  RowBox[{"pl", ",", "\[CapitalPhi]1"}], "]"}], ",", 
                 "\[CapitalPhi]1inx"}], "]"}], "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"rhp2x", "=", 
              RowBox[{"rh1", "+", "rh2", " ", "+", "rh3"}]}], ";"}], 
            "\[IndentingNewLine]", "]"}]}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"t4", "=", 
          RowBox[{"AbsoluteTiming", "[", 
           RowBox[{"RHSolvedx", "[", 
            RowBox[{"rhp2", ",", "rhp2x"}], "]"}], "]"}]}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"out", "=", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            RowBox[{"DomainIntegrate", "[", 
             RowBox[{"solx", "[", 
              RowBox[{"rhp1", ",", "rhp1x"}], "]"}], "]"}], 
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{"i", "\[GreaterEqual]", "4"}], ",", 
              RowBox[{"K0", "[", 
               RowBox[{"x", ",", "t"}], "]"}], ",", "1"}], "]"}]}], "+", 
           RowBox[{"DomainIntegrate", "[", 
            RowBox[{"solx", "[", 
             RowBox[{"rhp2", ",", "rhp2x"}], "]"}], "]"}]}], ")"}]}], "/", 
        "Pi"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"(*", 
       RowBox[{
        RowBox[{"out", "=", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{"DomainIntegrate", "[", 
             RowBox[{"solx", "[", 
              RowBox[{"rhp1", ",", "rhp1x"}], "]"}], "]"}], ")"}]}], "/", 
          "Pi"}]}], ";"}], "*)"}], "\[IndentingNewLine]", 
      RowBox[{"out", " ", "=", " ", 
       RowBox[{
        RowBox[{"out", "[", 
         RowBox[{"[", 
          RowBox[{"1", ",", "1"}], "]"}], "]"}], "+", 
        RowBox[{"out", "[", 
         RowBox[{"[", 
          RowBox[{"2", ",", "1"}], "]"}], "]"}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"domainOutput", " ", "=", " ", 
       RowBox[{"DomainPlot", "[", 
        RowBox[{"Join", "[", 
         RowBox[{"rhp1", ",", "rhp2"}], "]"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"timestring", "=", " ", 
       RowBox[{"\"\<Region: \>\"", " ", "<>", 
        RowBox[{"ToString", "[", "i", "]"}], "<>", "\"\< (\>\"", "<>", 
        RowBox[{"ToString", "[", "x", "]"}], "<>", "\"\<,\>\"", "<>", 
        RowBox[{"ToString", "[", "t", "]"}], "<>", "\"\<) \>\"", "<>", 
        "\"\<0) Adapt: \>\"", " ", "<>", " ", 
        RowBox[{"ToString", "[", 
         RowBox[{"t0", "//", "First"}], "]"}], "<>", 
        "\"\< 1) Construct: \>\"", " ", "<>", " ", 
        RowBox[{"ToString", "[", 
         RowBox[{"t1", "//", "First"}], "]"}], "<>", "\"\<  \>\"", "<>", 
        "\"\<1) Solve: \>\"", " ", "<>", " ", 
        RowBox[{"ToString", "[", 
         RowBox[{"t2", "//", "First"}], "]"}], "<>", "\"\<  \>\"", "<>", 
        "\"\<2) Construct: \>\"", " ", "<>", " ", 
        RowBox[{"ToString", "[", 
         RowBox[{"t3", "//", "First"}], "]"}], "<>", "\"\<  \>\"", "<>", 
        "\"\<2) Solve: \>\"", " ", "<>", " ", 
        RowBox[{"ToString", "[", 
         RowBox[{"t4", "//", "First"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{"timeflag", ",", 
        RowBox[{
         RowBox[{"Print", "[", "timestring", "]"}], ";"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"out", ",", 
        RowBox[{"{", 
         RowBox[{"rhp1", ",", "rhp1x"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"rhp2", ",", "rhp2x"}], "}"}], ",", "timestring", ",", 
        RowBox[{"solx", "[", 
         RowBox[{"rhp1", ",", "rhp1x"}], "]"}], ",", 
        RowBox[{"solx", "[", 
         RowBox[{"rhp2", ",", "rhp2x"}], "]"}]}], "}"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"CSTSwitch", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"midT", ",", "midCS", ",", "Tflag", ",", "CSflag"}], "}"}], ",",
      "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{".1", "<", 
         RowBox[{".1", "/", 
          RowBox[{"tau", "[", 
           RowBox[{"x", ",", "t"}], "]"}]}], " ", "<", " ", "15.9"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"midT", "=", "8"}], ";", "\[IndentingNewLine]", 
         RowBox[{"midT", " ", "=", " ", 
          RowBox[{"Abs", "[", 
           RowBox[{
            RowBox[{".1", "/", 
             RowBox[{"tau", "[", 
              RowBox[{"x", ",", "t"}], "]"}]}], "-", "midT"}], "]"}]}], ";"}],
         ",", "\[IndentingNewLine]", " ", 
        RowBox[{
         RowBox[{"midT", "=", "\[Infinity]"}], ";"}]}], "\[IndentingNewLine]",
        "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{".1", "<", 
         RowBox[{"s", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "<", " ", "22.6"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"midCS", "=", "11"}], ";", "\[IndentingNewLine]", 
         RowBox[{"midCS", " ", "=", " ", 
          RowBox[{"Abs", "[", 
           RowBox[{
            RowBox[{"s", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "-", "midCS"}], "]"}]}], ";"}], 
        ",", "\[IndentingNewLine]", " ", 
        RowBox[{
         RowBox[{"midCS", "=", "\[Infinity]"}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"midCS", "\[Equal]", "\[Infinity]"}], " ", "&&", " ", 
         RowBox[{"midT", "\[Equal]", "\[Infinity]"}]}], ",", " ", 
        RowBox[{"Return", "[", "3", "]"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"midCS", " ", "<", " ", "midT"}], ",", " ", 
        RowBox[{"Return", "[", "4", "]"}], ",", 
        RowBox[{"Return", "[", "5", "]"}]}], "]"}], ";"}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"KdVAuto", "[", 
   RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
  RowBox[{"\[Piecewise]", GridBox[{
     {
      RowBox[{
       RowBox[{
        RowBox[{"KdV", "[", "0", "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "[", 
       RowBox[{"[", "1", "]"}], "]"}], 
      RowBox[{
       RowBox[{
        RowBox[{"Abs", "[", "x", "]"}], "\[LessEqual]", "3"}], "&&", 
       RowBox[{"t", "<", ".1"}]}]},
     {
      RowBox[{
       RowBox[{
        RowBox[{"KdV", "[", "1", "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "[", 
       RowBox[{"[", "1", "]"}], "]"}], 
      RowBox[{"x", " ", "\[GreaterEqual]", "0"}]},
     {
      RowBox[{
       RowBox[{
        RowBox[{"KdV", "[", "2.5", "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "[", 
       RowBox[{"[", "1", "]"}], "]"}], 
      RowBox[{
       RowBox[{"x", " ", "<", "0"}], " ", "&&", " ", 
       RowBox[{
        RowBox[{"Abs", "[", "x", "]"}], "<", 
        RowBox[{"3", 
         RowBox[{"t", "^", 
          RowBox[{"(", 
           RowBox[{"1", "/", "3"}], ")"}]}]}]}]}]},
     {
      RowBox[{
       RowBox[{
        RowBox[{"KdV", "[", "3", "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "[", 
       RowBox[{"[", "1", "]"}], "]"}], 
      RowBox[{
       RowBox[{"x", "<", "0"}], " ", "&&", " ", 
       RowBox[{
        RowBox[{"K0", "[", 
         RowBox[{"x", ",", "t"}], "]"}], ">", 
        RowBox[{"1", "/", "10"}]}]}]},
     {
      RowBox[{
       RowBox[{
        RowBox[{"KdV", "[", 
         RowBox[{"CSTSwitch", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "[", 
       RowBox[{"[", "1", "]"}], "]"}], "True"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxItemSize->{
     "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.84]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, 
      "RowsIndexed" -> {}}]}]}], "\[IndentingNewLine]"}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5656331021261997`*^9, 3.5656331507042*^9}, {
   3.5656332214231997`*^9, 3.5656332560642*^9}, {3.5656333681882*^9, 
   3.5656335261731997`*^9}, {3.5656335846042*^9, 3.5656335872632*^9}, {
   3.5656336435452003`*^9, 3.5656336860112*^9}, {3.5656338535052*^9, 
   3.5656338537632*^9}, {3.5656339276542*^9, 3.5656339286672*^9}, {
   3.5656592020192003`*^9, 3.5656592088892*^9}, {3.5656598958532*^9, 
   3.5656598966331997`*^9}, {3.5656619660292*^9, 3.5656620028492002`*^9}, 
   3.5656620931552*^9, {3.5656622786682*^9, 3.5656622915782003`*^9}, {
   3.5656634907882*^9, 3.5656635025982*^9}, {3.5656638034382*^9, 
   3.5656638245382*^9}, {3.5656644434452*^9, 3.5656644476521997`*^9}, {
   3.5656646513442*^9, 3.5656646589042*^9}, {3.5657047691472*^9, 
   3.5657047774302*^9}, {3.5657088696521997`*^9, 3.5657088700362*^9}, {
   3.5657089777462*^9, 3.5657090848772*^9}, {3.5657091151042*^9, 
   3.5657094998262*^9}, {3.5657095724021997`*^9, 3.5657095743862*^9}, {
   3.5657097344792*^9, 3.5657099280262003`*^9}, {3.5657109429972*^9, 
   3.5657110409142*^9}, {3.5657116229172*^9, 3.5657116266252003`*^9}, {
   3.5657271296342*^9, 3.5657272376172*^9}, {3.5657272685972*^9, 
   3.5657274588002*^9}, {3.5657275441552*^9, 3.5657275569462*^9}, {
   3.5657276169692*^9, 3.5657277268622*^9}, {3.5657277573022003`*^9, 
   3.5657277899292*^9}, {3.5657278648361998`*^9, 3.5657278652111998`*^9}, {
   3.5657279281822*^9, 3.5657280276302*^9}, 3.5657280680662003`*^9, {
   3.5657288935922003`*^9, 3.5657289195042*^9}, {3.5657303695222*^9, 
   3.5657304360642*^9}, {3.5657305933142*^9, 3.5657306444212*^9}, {
   3.565737475873736*^9, 3.5657374771709967`*^9}, {3.5657470873494787`*^9, 
   3.565747115534953*^9}, {3.5657549388660297`*^9, 3.565754942524865*^9}, {
   3.565754991522559*^9, 3.565754996829166*^9}, {3.5657555443336363`*^9, 
   3.5657555447033873`*^9}, {3.565797002050454*^9, 3.565797002546124*^9}, {
   3.5658025961582413`*^9, 3.565802605219771*^9}, {3.568127580519804*^9, 
   3.5681276101348543`*^9}, {3.568131542263668*^9, 3.5681315712070007`*^9}, {
   3.568131605785861*^9, 3.5681316949229507`*^9}, {3.568131732696384*^9, 
   3.568131740509767*^9}, {3.568322969266074*^9, 3.568323077591319*^9}, {
   3.568323294278069*^9, 3.5683233042625923`*^9}, {3.568324033184499*^9, 
   3.5683240550258636`*^9}, {3.56832940766568*^9, 3.5683294105702553`*^9}, {
   3.56833056230602*^9, 3.568330575438127*^9}, {3.568330607219866*^9, 
   3.568330632792687*^9}, {3.568330776498214*^9, 3.568330780459249*^9}, {
   3.568331372233078*^9, 3.568331390111599*^9}, {3.5683314338443937`*^9, 
   3.568331457094502*^9}, {3.568331527233041*^9, 3.568331543343507*^9}, {
   3.56833163003648*^9, 3.568331653829433*^9}, {3.56833310834986*^9, 
   3.5683331527307243`*^9}, {3.568333189641114*^9, 3.56833319117452*^9}, {
   3.56833482023983*^9, 3.568334820479849*^9}, {3.568336214323735*^9, 
   3.568336327812791*^9}, {3.568336368339611*^9, 3.568336389775852*^9}, {
   3.568336427292983*^9, 3.568336436168269*^9}, {3.5683795065557327`*^9, 
   3.56837950976192*^9}, {3.568387982174502*^9, 3.568387986618891*^9}, {
   3.568388024651102*^9, 3.5683880595168247`*^9}, {3.568388098883113*^9, 
   3.5683882589339123`*^9}, {3.568388296362535*^9, 3.568388299762309*^9}, {
   3.5683883923596783`*^9, 3.568388426990519*^9}, {3.568388461452135*^9, 
   3.568388538871482*^9}, {3.5683886556979027`*^9, 3.5683887993445377`*^9}, {
   3.568388838116487*^9, 3.568388893482264*^9}, {3.568389563133095*^9, 
   3.568389951047016*^9}, {3.568389994034773*^9, 3.568390030922117*^9}, {
   3.568390102044835*^9, 3.568390105050002*^9}, {3.568390148798381*^9, 
   3.5683901795655193`*^9}, {3.568390216094982*^9, 3.568390221100861*^9}, {
   3.5683902823058567`*^9, 3.568390309000457*^9}, {3.5683903716628933`*^9, 
   3.5683903981499243`*^9}, {3.568390442146215*^9, 3.568390521151321*^9}, {
   3.568390551917953*^9, 3.568390562104188*^9}, 3.568390673067088*^9, {
   3.5683908860866823`*^9, 3.568391111767688*^9}, {3.568391173647171*^9, 
   3.5683911857806396`*^9}, {3.568391230094933*^9, 3.568391321459133*^9}, {
   3.5683913536757307`*^9, 3.5683914116403723`*^9}, {3.568391473242125*^9, 
   3.568391644613901*^9}, {3.568391720019744*^9, 3.56839175916954*^9}, {
   3.568391837604789*^9, 3.568391867330896*^9}, 3.568392365484915*^9, {
   3.568392407277775*^9, 3.568392443239044*^9}, {3.568392570544106*^9, 
   3.568392588259688*^9}, {3.568392618489296*^9, 3.568392621612311*^9}, {
   3.56839270259394*^9, 3.568392706466072*^9}, {3.5683927402945623`*^9, 
   3.56839274556012*^9}, {3.5683928935143213`*^9, 3.5683928956768312`*^9}, {
   3.568392954408515*^9, 3.568392966974045*^9}, {3.568393018786117*^9, 
   3.5683930259061127`*^9}, {3.5683930624178963`*^9, 3.568393067481061*^9}, 
   3.568393103145616*^9, {3.568394073257638*^9, 3.56839407794663*^9}, {
   3.568396251771574*^9, 3.5683962624854927`*^9}, {3.56839630814607*^9, 
   3.568396311072048*^9}, {3.5683963601989393`*^9, 3.5683963703397408`*^9}, 
   3.568396401204441*^9, {3.568396434265355*^9, 3.568396498327415*^9}, {
   3.568396528771764*^9, 3.568396571449573*^9}, {3.5683966087616053`*^9, 
   3.5683966138137283`*^9}, {3.568396657026207*^9, 3.5683966660880404`*^9}, {
   3.5683967144823847`*^9, 3.5683967398058357`*^9}, {3.56839687141849*^9, 
   3.5683968915146646`*^9}, {3.568396945335568*^9, 3.5683969567241993`*^9}, {
   3.568397003235179*^9, 3.568397012952146*^9}, {3.5683971008523197`*^9, 
   3.5683971010973682`*^9}, {3.568397285214775*^9, 3.568397289330278*^9}, {
   3.568397325113253*^9, 3.56839741496196*^9}, {3.568397456295931*^9, 
   3.568397464191244*^9}, {3.5683974971906347`*^9, 3.5683975002195053`*^9}, 
   3.568397541534284*^9, {3.568397597798299*^9, 3.568397631789215*^9}, {
   3.568397755636788*^9, 3.568397759155779*^9}, {3.568397802646673*^9, 
   3.568397818410656*^9}, {3.568397850841462*^9, 3.568397852966152*^9}, {
   3.56839823476803*^9, 3.5683982365101013`*^9}, {3.568398444893667*^9, 
   3.568398526365587*^9}, {3.568398565474111*^9, 3.568398566556937*^9}, {
   3.568398763231718*^9, 3.568398766842605*^9}, {3.568398797510002*^9, 
   3.568398802148403*^9}, {3.568398872981185*^9, 3.568398880581168*^9}, {
   3.568398930792556*^9, 3.568398946671358*^9}, {3.5683989884520817`*^9, 
   3.568398988797374*^9}, 3.568399053868208*^9, {3.568399087470334*^9, 
   3.568399097216434*^9}, {3.568399148729767*^9, 3.56839916028084*^9}, {
   3.5683992158748302`*^9, 3.568399218289949*^9}, {3.568399553293961*^9, 
   3.568399596208366*^9}, {3.568399877779551*^9, 3.568399878402133*^9}, 
   3.5684001678682117`*^9, {3.568400437894327*^9, 3.568400439431645*^9}, {
   3.568400622830792*^9, 3.568400631802414*^9}, 3.568400993603409*^9, {
   3.5684020720699787`*^9, 3.568402091491928*^9}, {3.568402126564946*^9, 
   3.568402212284721*^9}, {3.568402245511846*^9, 3.568402246894141*^9}, {
   3.568402332776824*^9, 3.56840233807232*^9}, {3.568402440863908*^9, 
   3.568402444722144*^9}, {3.568402533956018*^9, 3.568402552918429*^9}, {
   3.568407754692683*^9, 3.568407779799582*^9}, 3.5684078274257517`*^9, {
   3.56840803472453*^9, 3.5684080382890797`*^9}, {3.568408078888413*^9, 
   3.568408082576736*^9}, {3.568408121793377*^9, 3.5684082140304317`*^9}, {
   3.5684082555203457`*^9, 3.568408257986475*^9}, {3.56840831070268*^9, 
   3.5684083109513493`*^9}, {3.568408756464539*^9, 3.568408762083352*^9}, {
   3.5684100036414127`*^9, 3.568410012015223*^9}, {3.568410078621039*^9, 
   3.5684100787802067`*^9}, {3.5684716651261253`*^9, 3.568471666879898*^9}, {
   3.56847172964369*^9, 3.56847173122777*^9}, {3.568471922653213*^9, 
   3.568471923192724*^9}, {3.568471960463667*^9, 3.568472025999669*^9}, {
   3.568472085314908*^9, 3.5684721367265778`*^9}, {3.568472185106748*^9, 
   3.5684722023423653`*^9}, {3.568472774271091*^9, 3.568472783763074*^9}, {
   3.568472840714575*^9, 3.568472886248034*^9}, {3.568473560517717*^9, 
   3.568473560886444*^9}, {3.568473778329823*^9, 3.568473783395595*^9}, {
   3.5684739301773643`*^9, 3.568473940180463*^9}, {3.56847406079665*^9, 
   3.568474085118724*^9}, {3.568474577879418*^9, 3.5684745901304502`*^9}, {
   3.5684795426379557`*^9, 3.568479575164258*^9}, {3.56848014070854*^9, 
   3.568480147923603*^9}, {3.568480348576867*^9, 3.568480348933045*^9}, {
   3.568581919661132*^9, 3.568581967953225*^9}, {3.568582028602015*^9, 
   3.5685820533103733`*^9}, {3.568582497655973*^9, 3.568582523614801*^9}, {
   3.568582778116692*^9, 3.568582779276524*^9}, {3.5685828653010683`*^9, 
   3.5685829011055813`*^9}, {3.5685843788395567`*^9, 3.568584383778249*^9}, {
   3.568584415953185*^9, 3.568584444598184*^9}, {3.568584485191106*^9, 
   3.568584488837747*^9}, {3.568585333970175*^9, 3.56858533423798*^9}, {
   3.5685853675369577`*^9, 3.5685853713510437`*^9}, {3.568585447477066*^9, 
   3.568585468552409*^9}, {3.56858559347997*^9, 3.5685856524703903`*^9}, {
   3.5685857304038467`*^9, 3.5685857308058147`*^9}, {3.5685858837587748`*^9, 
   3.5685858855762978`*^9}, {3.568585983791616*^9, 3.568586014999186*^9}, {
   3.5685862116535597`*^9, 3.5685862191546583`*^9}, {3.5685865812616453`*^9, 
   3.568586581441317*^9}, {3.568645469665984*^9, 3.5686454702061357`*^9}, {
   3.568645571127146*^9, 3.568645628115777*^9}, {3.568646451069393*^9, 
   3.568646454004428*^9}, {3.568646944607242*^9, 3.5686470838108683`*^9}, {
   3.568647141921887*^9, 3.568647215523678*^9}, {3.568647282845615*^9, 
   3.5686473056551657`*^9}, {3.568647423989724*^9, 3.568647436453032*^9}, {
   3.568647508134008*^9, 3.5686475619563*^9}, {3.568647594959404*^9, 
   3.56864762741774*^9}, {3.568647661026108*^9, 3.568647681059523*^9}, {
   3.56864771193158*^9, 3.56864773262218*^9}, {3.568647804457287*^9, 
   3.568647806555976*^9}, {3.568648201447899*^9, 3.568648217711245*^9}, 
   3.568648282196034*^9, {3.56865151726235*^9, 3.568651596918689*^9}, {
   3.568651637674362*^9, 3.568651687669922*^9}, {3.568651749253045*^9, 
   3.5686517781462097`*^9}, {3.568652576321905*^9, 3.568652578653221*^9}, 
   3.5686589525041103`*^9, {3.56865901484396*^9, 3.5686590170502462`*^9}, {
   3.5686666151859703`*^9, 3.5686666314285803`*^9}, {3.568666798717568*^9, 
   3.5686668030656633`*^9}, 3.568666841005889*^9, {3.568666878566256*^9, 
   3.568666917073008*^9}, {3.568666956167425*^9, 3.568666959878756*^9}, {
   3.56873418493346*^9, 3.568734578656803*^9}, {3.568734650716769*^9, 
   3.568734832766488*^9}, {3.5687348822689734`*^9, 3.568734904359148*^9}, {
   3.5687373671003447`*^9, 3.56873737002731*^9}, {3.5687376003236723`*^9, 
   3.568737600556678*^9}, {3.568737779128393*^9, 3.5687377846392736`*^9}, {
   3.5687378157693043`*^9, 3.568737818656995*^9}, {3.568737885332623*^9, 
   3.568737919077574*^9}, {3.5687380177024384`*^9, 3.568738048612997*^9}, {
   3.5687381890028353`*^9, 3.568738281584836*^9}, {3.5687383926228456`*^9, 
   3.5687383929131603`*^9}, {3.5687392184685507`*^9, 3.5687392222848387`*^9}, 
   3.5687456734488897`*^9, {3.568745994426301*^9, 3.568745994615178*^9}, {
   3.6063379293654213`*^9, 3.6063379364173098`*^9}, {3.606338386654126*^9, 
   3.606338386933234*^9}, {3.6063407074777*^9, 3.606340725610653*^9}, {
   3.6063410668918133`*^9, 3.6063410961309547`*^9}, {3.606341349999525*^9, 
   3.6063413531344213`*^9}, {3.6063414216652184`*^9, 3.606341426701655*^9}, {
   3.6063414622087708`*^9, 3.606341484839341*^9}, {3.606341624893113*^9, 
   3.60634166170229*^9}, {3.6063417071942472`*^9, 3.6063417735293293`*^9}, {
   3.606341836293603*^9, 3.606341908208742*^9}, {3.6063419731467133`*^9, 
   3.606342072434659*^9}, {3.606342109304785*^9, 3.606342139000354*^9}, {
   3.606342443622944*^9, 3.606342449701515*^9}, 3.60634251100965*^9, {
   3.6063426860410957`*^9, 3.606342770317103*^9}, {3.606342811503175*^9, 
   3.606342811783106*^9}, {3.606342875420898*^9, 3.606342879450938*^9}, {
   3.6063429317285357`*^9, 3.6063429795436153`*^9}, {3.606343035622061*^9, 
   3.6063430358192673`*^9}, {3.6063431353591423`*^9, 
   3.6063431501686163`*^9}, {3.606343232371183*^9, 3.606343246902844*^9}, {
   3.606343326224182*^9, 3.606343349604835*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Finish", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.5463644907164*^9, 3.5463644913004*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"End", "[", "]"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5463644931024*^9, 3.5463644945224*^9}}]
}, Closed]]
},
AutoGeneratedPackage->Automatic,
WindowSize->{720, 852},
WindowMargins->{{Automatic, 0}, {Automatic, 0}},
ShowSelection->True,
FrontEndVersion->"10.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (December 4, \
2014)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[557, 20, 408, 9, 63, "Input"],
Cell[CellGroupData[{
Cell[990, 33, 125, 2, 64, "Section",
 InitializationGroup->True],
Cell[1118, 37, 11388, 197, 1457, "Input",
 InitializationGroup->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[12543, 239, 162, 3, 64, "Section",
 InitializationCell->True],
Cell[12708, 244, 9996, 246, 794, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[22741, 495, 179, 3, 80, "Section",
 InitializationCell->True],
Cell[22923, 500, 17122, 436, 1219, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[40082, 941, 224, 4, 80, "Section",
 InitializationGroup->True],
Cell[40309, 947, 11170, 282, 896, "Input"]
}, Open  ]],
Cell[CellGroupData[{
Cell[51516, 1234, 98, 1, 80, "Section"],
Cell[CellGroupData[{
Cell[51639, 1239, 186, 3, 37, "Subsection",
 InitializationCell->True],
Cell[51828, 1244, 37027, 1002, 2619, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[88892, 2251, 252, 4, 37, "Subsection",
 InitializationCell->True],
Cell[89147, 2257, 10640, 266, 565, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[99824, 2528, 233, 4, 37, "Subsection",
 InitializationCell->True],
Cell[100060, 2534, 14474, 383, 487, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[114571, 2922, 298, 4, 37, "Subsection",
 InitializationCell->True],
Cell[114872, 2928, 9102, 193, 183, "Input",
 InitializationCell->True]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[124023, 3127, 198, 3, 50, "Section",
 InitializationCell->True],
Cell[124224, 3132, 4426, 132, 164, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[128687, 3269, 133, 2, 50, "Section",
 InitializationCell->True],
Cell[128823, 3273, 15509, 444, 772, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[144369, 3722, 228, 3, 50, "Section"],
Cell[144600, 3727, 8901, 270, 449, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[153538, 4002, 196, 3, 50, "Section",
 InitializationCell->True],
Cell[153737, 4007, 11569, 350, 544, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[165343, 4362, 180, 3, 50, "Section",
 InitializationCell->True],
Cell[165526, 4367, 53317, 1385, 2902, "Input",
 InitializationCell->True],
Cell[218846, 5754, 38597, 1006, 2069, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[257480, 6765, 179, 2, 50, "Section"],
Cell[257662, 6769, 36904, 964, 2123, "Input"],
Cell[294569, 7735, 24736, 650, 1441, "Input"]
}, Closed]],
Cell[CellGroupData[{
Cell[319342, 8390, 143, 2, 50, "Section",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[319510, 8396, 176, 3, 37, "Subsection",
 InitializationCell->True],
Cell[319689, 8401, 14270, 430, 1034, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[333996, 8836, 208, 3, 37, "Subsection",
 InitializationCell->True],
Cell[334207, 8841, 33711, 993, 2049, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[367955, 9839, 131, 2, 29, "Subsection",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[368111, 9845, 105, 1, 28, "Subsubsection"],
Cell[368219, 9848, 25915, 650, 1038, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[394171, 10503, 156, 2, 22, "Subsubsection"],
Cell[394330, 10507, 17517, 415, 620, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[411884, 10927, 111, 1, 22, "Subsubsection"],
Cell[411998, 10930, 23601, 582, 924, "Input",
 InitializationCell->True]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[435648, 11518, 103, 1, 37, "Subsection"],
Cell[435754, 11521, 5493, 125, 145, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[441272, 11650, 103, 1, 28, "Subsubsection"],
Cell[441378, 11653, 49883, 1445, 2292, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[491298, 13103, 104, 1, 22, "Subsubsection"],
Cell[491405, 13106, 37359, 1061, 1760, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[528801, 14172, 111, 1, 22, "Subsubsection"],
Cell[528915, 14175, 37156, 1053, 1722, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[566108, 15233, 98, 1, 28, "Subsubsection"],
Cell[566209, 15236, 14210, 318, 677, "Input",
 InitializationCell->True]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[580480, 15561, 183, 3, 50, "Section",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[580688, 15568, 229, 4, 37, "Subsection",
 InitializationCell->True],
Cell[580920, 15574, 4538, 127, 259, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[585495, 15706, 110, 1, 37, "Subsection"],
Cell[585608, 15709, 11560, 316, 685, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[597205, 16030, 131, 2, 37, "Subsection",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[597361, 16036, 105, 1, 28, "Subsubsection"],
Cell[597469, 16039, 21695, 538, 848, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[619201, 16582, 156, 2, 22, "Subsubsection"],
Cell[619360, 16586, 15682, 356, 544, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[635079, 16947, 111, 1, 28, "Subsubsection"],
Cell[635193, 16950, 19151, 461, 772, "Input",
 InitializationCell->True]
}, Closed]]
}, Open  ]],
Cell[CellGroupData[{
Cell[654393, 17417, 130, 2, 37, "Subsection",
 InitializationCell->True],
Cell[654526, 17421, 1896, 62, 107, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[656447, 17487, 130, 2, 28, "Subsubsection",
 InitializationCell->True],
Cell[656580, 17491, 30107, 844, 1494, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[686724, 18340, 131, 2, 22, "Subsubsection",
 InitializationCell->True],
Cell[686858, 18344, 30421, 845, 1551, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[717316, 19194, 138, 2, 28, "Subsubsection",
 InitializationCell->True],
Cell[717457, 19198, 30182, 843, 1475, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[747676, 20046, 125, 2, 22, "Subsubsection",
 InitializationCell->True],
Cell[747804, 20050, 8760, 173, 316, "Input",
 InitializationCell->True]
}, Closed]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[756625, 20230, 145, 2, 50, "Section",
 InitializationCell->True],
Cell[756773, 20234, 12715, 376, 603, "Input",
 InitializationCell->True],
Cell[769491, 20612, 18780, 406, 811, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[788308, 21023, 181, 3, 80, "Section",
 InitializationCell->True],
Cell[788492, 21028, 42087, 929, 2628, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[830616, 21962, 114, 2, 80, "Section",
 InitializationCell->True],
Cell[830733, 21966, 158, 4, 30, "Input",
 InitializationCell->True]
}, Closed]]
}
]
*)

(* End of internal cache information *)
