(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 7.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[    194785,       5149]
NotebookOptionsPosition[    191941,       5047]
NotebookOutlinePosition[    192329,       5064]
CellTagsIndexPosition[    192286,       5061]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"Copyright", " ", "2013", " ", "Thomas", " ", "Trogdon"}], " ", 
   "*)"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
   "This", " ", "software", " ", "is", " ", "distributed", " ", "under", " ", 
    "the", " ", "terms", " ", "of", " ", "the", " ", "GNU", " ", "General", 
    " ", "Public", " ", "License"}], " ", "*)"}]}]], "Input"],

Cell[CellGroupData[{

Cell["Documentation", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.5463622250573997`*^9, 3.5463622273694*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Needs", "[", "\"\<RiemannHilbert`\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"HillNLS", "::", "usage"}], "=", 
   "\"\<HillNLS[q,n] gives the eigenvalues of the modified Zakharov-Shabat \
scattering (including continuous spectrum) problem with n Fourier modes using \
Hill's method.\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"LocatePolesNLS", "::", "usage"}], "=", 
   "\"\<LocatePoles[q,n] gives the true eigenvalues of the modified \
Zakharov-Shabat scattering problem.\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"NLS", "::", "usage"}], "=", 
   "\"\<NLS[i][x,t] = {out,\[CapitalPhi],rhp1,rhp2,timestring} where out is \
the solution of NLS, \[CapitalPhi] is the solution of the Riemann-Hilbert \
problem, rhp1 and rhp2 are the Riemann-Hilbert problems and timestring is the \
string of computation times.  This is done using the deformation in Region i.\
\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"NLSAuto", "::", "usage"}], "=", 
  "\"\<NLSAuto[x,t] returns the solution of the NLS equation with deformation \
selected automagically.  Many things must be initialized.  Sample code:\n\n\t\
q[x_]:=.1 Exp[-x^2+I x];\[IndentingNewLine]Defocusing[];\n\t\
H=ScatteringMatrixFiniteNLS[q,50,6];\[IndentingNewLine]aa//Clear;bb//Clear;\
\[IndentingNewLine]aa[k_]:=aa[k]=H[k][[1,1]];\[IndentingNewLine]bb[k_]:=bb[k]=\
H[k][[2,1]];\n\t\
SetParams[.6,.1,10.^(-9),15,30];\[IndentingNewLine]h[k_]:=3/(1+Abs[k/2+1/3.2]^\
8);\[IndentingNewLine]Setrsamp[h];\[IndentingNewLine]Settimeflag[False];\
\[IndentingNewLine]\[Nu]=Getnu[];\[IndentingNewLine]\[Rho][k_]:=bb[k]/aa[k];\n\
\tup=I(\[Nu]+.0001);m=40;el=8;\[IndentingNewLine]f={Fun[\[Rho],Line[{-el,0}-\
up],m],Fun[\[Rho],Line[{0,el}-up],m],Fun[\[Rho],Line[{el,0}+up],m],Fun[\[Rho],\
Line[{0,-el}+up],m]};\n\n\t\
up=I(\[Nu]+.0001);m=40;el=8;\[IndentingNewLine]f={Fun[\[Rho],Line[{-el,0}-up],\
m],Fun[\[Rho],Line[{0,el}-up],m],Fun[\[Rho],Line[{el,0}+up],m],Fun[\[Rho],\
Line[{0,-el}+up],m]};\n\
SetScatteringData[aa,bb,m\[Rho],LocatePoles[q,40]]\>\""}], "\
\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Focusing", "::", "usage"}], "=", 
   "\"\<Focusing[], switch to focusing NLS\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Defocusing", "::", "usage"}], "=", 
   "\"\<Defocusing[], switch to focusing NLS\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"SetScatteringData", "::", "usage"}], "=", 
   "\"\<SetScatteringData[a,b,\[Rho],poles] or \
SetScatteringData[a,b,\[Rho],poles,normingConstants] sets the scattering data \
for the NLS equation\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"SetParams", "::", "usage"}], "=", 
   "\"\<SetParams[\[Nu],rad,globalTol,smallN,bigN] sets the parameters for \
the rest of the code:\n\t\[Nu]: half of width of strip of analyticity\n\trad: \
radius of soliton contours\n\tglobalTol: contour truncation tolerance\n\t\
smallN: small number of collocation points\n\tbigN: big number of collocation \
points\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"GetScatteringData", "::", "usage"}], "=", 
   "\"\<GetScatteringData[] = {\[Rho],poles,normingConstants}\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ScatteringMatrixFiniteNLS", "::", "usage"}], "=", 
   "\"\<ScatteringMatrixFiniteNLS[q,n,L] returns a function f[w] that \
produces an approximation of the scattering matrix at w. The parameter n \
indicates the number of collocation points and L sets the problem on [-L,L]\n\
Sample code:\n\tq[x_]:=.1 Exp[-x^2+I x];\[IndentingNewLine]Defocusing[];\n\t\
H=ScatteringMatrixFiniteNLS[q,50,6];\[IndentingNewLine]aa//Clear;bb//Clear;\
\[IndentingNewLine]aa[k_]:=aa[k]=H[k][[1,1]];\[IndentingNewLine]bb[k_]:=bb[k]=\
H[k][[2,1]];\n\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Getnu", "::", "usage"}], "=", "\"\<Getnu[] returns \[Nu]\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Setrsamp", "::", "usage"}], "=", 
   "\"\<Setrsamp[h] sets a function that should be chosen so that that \
|\[Rho]| < h\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"timestring", "::", "usage"}], "=", 
   "\"\<A string updated by NLS[i][x,t]\n(x,t) 1) Construct: --time to \
construct rhp1-- 1) Solve: --time to solve rhp1-- 2) Construct: --time to \
construct rhp2-- 2) Solve: --time to solve rhp2--\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Settimeflag", "::", "usage"}], "=", 
   "\"\<Settimeflag[bool] bool = True forces timestring to be printed each \
time NLS[i][x,t] is called\>\""}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"startift", ";"}], "\[IndentingNewLine]", 
 RowBox[{"domainOutput", ";"}], "\[IndentingNewLine]", 
 RowBox[{"ift", ";"}], "\[IndentingNewLine]", 
 RowBox[{"\[CapitalPhi]t", ";"}], "\[IndentingNewLine]", 
 RowBox[{"dom", ";"}], "\[IndentingNewLine]", 
 RowBox[{"Jadapt", ";", "J", ";", "Jsamp", ";"}], "\[IndentingNewLine]", 
 RowBox[{"PoleListn", ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"f1", ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Begin", "[", "\"\<Private`\>\"", "]"}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.503865094224187*^9, 3.5038651020296*^9}, {
   3.503927907403412*^9, 3.503927909713172*^9}, {3.5463622424474*^9, 
   3.5463622642414*^9}, {3.5463626582194*^9, 3.5463626735304003`*^9}, {
   3.5463627171584*^9, 3.5463627549664*^9}, {3.5463630470550003`*^9, 
   3.546363053007*^9}, {3.5463636741546*^9, 3.5463636807546*^9}, {
   3.5463641965566*^9, 3.5463641987186003`*^9}, {3.5463649081344*^9, 
   3.5463649091744003`*^9}, {3.5463651229344*^9, 3.5463651320044003`*^9}, {
   3.5463652879944*^9, 3.5463652884544*^9}, {3.5463658662204*^9, 
   3.5463658844244003`*^9}, {3.5463661544224*^9, 3.5463661639164*^9}, {
   3.5463662109983997`*^9, 3.5463662153784*^9}, {3.5463675756984*^9, 
   3.5463675807684*^9}, {3.5463676634723997`*^9, 3.5463676657024*^9}, {
   3.546389444761112*^9, 3.546389456597519*^9}, {3.546391697049466*^9, 
   3.546391698452414*^9}, {3.5464015603071537`*^9, 3.546401590871516*^9}, {
   3.546434403958077*^9, 3.5464344147152987`*^9}, {3.563910225793398*^9, 
   3.563910227918174*^9}, {3.5656137687682*^9, 3.5656137697542*^9}, {
   3.5656139003392*^9, 3.5656139288342*^9}, {3.5656151920592003`*^9, 
   3.5656151970192003`*^9}, {3.5656156395151997`*^9, 3.5656156445502*^9}, {
   3.5656157463582*^9, 3.5656157484662*^9}, {3.5656159228382*^9, 
   3.5656159263012*^9}, {3.5656174040372*^9, 3.5656174070202*^9}, {
   3.5656177920601997`*^9, 3.5656177959472*^9}, {3.5656190027962*^9, 
   3.5656190070972*^9}, {3.5656291442992*^9, 3.5656291485092*^9}, {
   3.5656305555502*^9, 3.5656306099792*^9}, {3.5656333098972*^9, 
   3.5656333107482*^9}, {3.5656339377812*^9, 3.5656339379602003`*^9}, {
   3.5656345203662*^9, 3.5656345212532*^9}, {3.5656346861972*^9, 
   3.5656346887962*^9}, {3.5656352908831997`*^9, 3.5656352937382*^9}, {
   3.5656380208282003`*^9, 3.5656380229882*^9}, {3.5656626102882*^9, 
   3.5656626208182*^9}, {3.5656646656742*^9, 3.5656646676742*^9}, 
   3.5656647127002*^9, {3.5657063129751997`*^9, 3.5657063150542*^9}, {
   3.5657085673072*^9, 3.5657085693552*^9}, {3.5657106763982*^9, 
   3.5657106766582003`*^9}, {3.5657108974332*^9, 3.5657109289282*^9}, {
   3.5657112059252*^9, 3.5657112089561996`*^9}, {3.5657119385382*^9, 
   3.5657119412162*^9}, {3.5657191417841997`*^9, 3.5657191465492*^9}, {
   3.5657193088412*^9, 3.5657193090162*^9}, {3.5657213945302*^9, 
   3.5657213956292*^9}, {3.5657214903452*^9, 3.5657215001092*^9}, {
   3.5657276459312*^9, 3.5657276473922*^9}, {3.5657280965452003`*^9, 
   3.5657280993452*^9}, {3.5657304429572*^9, 3.5657304456022*^9}, {
   3.5657305464231997`*^9, 3.5657305507342*^9}, {3.565742611453149*^9, 
   3.5657427171054707`*^9}, {3.56574277865187*^9, 3.565742869146963*^9}, {
   3.565742980436193*^9, 3.5657431898856077`*^9}, {3.5657432503045673`*^9, 
   3.565743264006617*^9}, {3.56574333818435*^9, 3.565743342266161*^9}, {
   3.565743416325479*^9, 3.5657434710920467`*^9}, {3.5657435014216003`*^9, 
   3.5657435144636393`*^9}, {3.565743583840477*^9, 3.5657437287947283`*^9}, {
   3.5657437682505836`*^9, 3.565743808785096*^9}, {3.565743854229642*^9, 
   3.565743981397606*^9}, {3.5657440541607656`*^9, 3.5657441106114883`*^9}, {
   3.565744147396241*^9, 3.565744148127984*^9}, {3.56574418639472*^9, 
   3.565744267538278*^9}, {3.565744472266567*^9, 3.565744539531228*^9}, {
   3.565744640232654*^9, 3.5657447146254187`*^9}, {3.5657447532043657`*^9, 
   3.5657447701282787`*^9}, {3.565744803503544*^9, 3.5657448370120697`*^9}, {
   3.565744873278673*^9, 3.5657449429227867`*^9}, {3.565744976404983*^9, 
   3.5657449773873863`*^9}, {3.565745127535184*^9, 3.565745150433978*^9}, {
   3.565745801755686*^9, 3.565745839129191*^9}, {3.5657458760558653`*^9, 
   3.565745905863186*^9}, {3.565745966784422*^9, 3.5657460022073174`*^9}, {
   3.565746048799855*^9, 3.5657460682812347`*^9}, {3.5657461057046137`*^9, 
   3.5657461388764*^9}, {3.565746178691865*^9, 3.565746183667274*^9}, {
   3.5657535451222*^9, 3.5657535462162*^9}, {3.565753855140954*^9, 
   3.565753855840127*^9}, {3.5657539853194313`*^9, 3.5657539870126247`*^9}, {
   3.565754408445376*^9, 3.565754408749222*^9}, {3.5657550866242857`*^9, 
   3.5657550884680643`*^9}, {3.565755149738063*^9, 3.565755152755093*^9}, {
   3.5657552771517467`*^9, 3.565755278189406*^9}, {3.5657607684613123`*^9, 
   3.565760770041016*^9}, {3.565802574600819*^9, 3.565802579155362*^9}, {
   3.568128176969741*^9, 3.568128177188424*^9}, {3.568764568236958*^9, 
   3.568764568499837*^9}, {3.5687789570461273`*^9, 3.568778959651313*^9}, {
   3.568778997645636*^9, 3.5687789978647327`*^9}, {3.5687791469567947`*^9, 
   3.568779148611024*^9}, {3.6712175881054363`*^9, 3.67121759198326*^9}, {
   3.671219068253974*^9, 3.671219069446898*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Setup", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.546363064814*^9, 3.546363067493*^9}, {
  3.565745327189906*^9, 3.5657453277187843`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"Focusing", "[", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"\[Lambda]", "=", "1"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Defocusing", "[", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"\[Lambda]", "=", 
      RowBox[{"-", "1"}]}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"SetScatteringData", "[", 
    RowBox[{"aa_", ",", "bb_", ",", "rr_", ",", "ma_", ",", "mc_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"r", "[", "k_", "]"}], ":=", 
       RowBox[{"rr", "[", "k", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"a", "=", "ma"}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"\[Tau]", "[", "k_", "]"}], ":=", 
       RowBox[{"1", "+", 
        RowBox[{"\[Lambda]", " ", 
         RowBox[{"r", "[", "k", "]"}], 
         RowBox[{"cc", "[", 
          RowBox[{"r", "[", 
           RowBox[{"cc", "[", "k", "]"}], "]"}], "]"}]}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"\[Tau]", "[", 
        RowBox[{"_", "?", "InfinityQ"}], "]"}], ":=", "1"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"\[Tau]", "[", "k_List", "]"}], ":=", 
       RowBox[{"\[Tau]", "/@", "k"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"dom", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"Truncate", "[", "globalTol", "]"}], "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{"Log", "[", 
             RowBox[{"1", "+", 
              RowBox[{
               RowBox[{"rsamp", "[", "#", "]"}], "^", "2"}]}], "]"}], "&"}], 
           ",", 
           RowBox[{"Line", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"-", "el"}], ",", "el"}], "}"}], "]"}], ",", "100"}], 
          "}"}], "]"}], "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"c", "=", "mc"}], ";"}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"SetScatteringData", "[", 
    RowBox[{"aa_", ",", "bb_", ",", "rr_", ",", "ma_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{
       RowBox[{"r", "[", "k_", "]"}], ":=", 
       RowBox[{"rr", "[", "k", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"a", "=", "ma"}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"\[Tau]", "[", "k_", "]"}], ":=", 
       RowBox[{"1", "+", 
        RowBox[{"\[Lambda]", " ", 
         RowBox[{"r", "[", "k", "]"}], 
         RowBox[{"cc", "[", 
          RowBox[{"r", "[", 
           RowBox[{"cc", "[", "k", "]"}], "]"}], "]"}]}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"\[Tau]", "[", 
        RowBox[{"_", "?", "InfinityQ"}], "]"}], ":=", "1"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"\[Tau]", "[", "k_List", "]"}], ":=", 
       RowBox[{"\[Tau]", "/@", "k"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"dom", "=", 
       RowBox[{
        RowBox[{
         RowBox[{"Truncate", "[", "globalTol", "]"}], "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{"Log", "[", 
             RowBox[{"1", "+", 
              RowBox[{
               RowBox[{"rsamp", "[", "#", "]"}], "^", "2"}]}], "]"}], "&"}], 
           ",", 
           RowBox[{"Line", "[", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"-", "el"}], ",", "el"}], "}"}], "]"}], ",", "100"}], 
          "}"}], "]"}], "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"f1", "=", 
       RowBox[{"Fun", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"aa", "[", "#", "]"}], "-", "1"}], "&"}], ",", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "\[Infinity]"}], "}"}], "]"}], ",", 
         RowBox[{"4", "bigN"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"f2", "=", 
       RowBox[{"Fun", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"aa", "[", "#", "]"}], "-", "1"}], "&"}], ",", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "\[Infinity]"}], ",", "0"}], "}"}], "]"}], ",", 
         RowBox[{"4", " ", "bigN"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"df1", "=", 
       RowBox[{"f1", "'"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"df2", "=", 
       RowBox[{"f2", "'"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"da", "[", "k_", "]"}], ":=", 
       RowBox[{
        RowBox[{"Cauchy", "[", 
         RowBox[{"df1", ",", "k"}], "]"}], "+", 
        RowBox[{"Cauchy", "[", 
         RowBox[{"df2", ",", "k"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"c", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"i", "=", "1"}], ",", 
        RowBox[{"i", "\[LessEqual]", 
         RowBox[{"Length", "[", "ma", "]"}]}], ",", 
        RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"c", "=", 
          RowBox[{"Join", "[", 
           RowBox[{"c", ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"Quiet", "[", 
               RowBox[{"bb", "[", 
                RowBox[{"ma", "[", 
                 RowBox[{"[", "i", "]"}], "]"}], "]"}], "]"}], "/", 
              RowBox[{"da", "[", 
               RowBox[{"ma", "[", 
                RowBox[{"[", "i", "]"}], "]"}], "]"}]}], "}"}]}], "]"}]}], 
         ";"}]}], 
       RowBox[{"(*", 
        RowBox[{"compute", " ", "norming", " ", "constants"}], "*)"}], 
       "\[IndentingNewLine]", "]"}], ";"}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"SetParams", "[", 
    RowBox[{"nu_", ",", "rad1_", ",", "tol_", ",", "sN_", ",", "bN_"}], "]"}],
    ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"\[Nu]", "=", "nu"}], ";", " ", 
      RowBox[{"(*", 
       RowBox[{
       "parameter", " ", "for", " ", "width", " ", "of", " ", "strip", " ", 
        "of", " ", "analyticity"}], "*)"}], "\[IndentingNewLine]", 
      RowBox[{"rad", "=", "rad1"}], ";", "\[IndentingNewLine]", 
      RowBox[{"globalTol", "=", "tol"}], ";", "\[IndentingNewLine]", 
      RowBox[{"smallN", "=", "sN"}], ";", "\[IndentingNewLine]", 
      RowBox[{"bigN", "=", "bN"}], ";"}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"GetScatteringData", "[", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"r", ",", "a", ",", "c"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Getnu", "[", "]"}], ":=", "\[Nu]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Setrsamp", "[", "h_", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"rsamp", "=", "h"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Settimeflag", "[", "h_", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{"timeflag", "=", "h"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"SetN", "[", "x_", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", 
     RowBox[{
      RowBox[{"bigN", "=", 
       RowBox[{"x", "[", 
        RowBox[{"[", "1", "]"}], "]"}]}], ";", 
      RowBox[{"smallN", "=", 
       RowBox[{"x", "[", 
        RowBox[{"[", "2", "]"}], "]"}]}], ";"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"expconst", "=", "1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"el", "=", "100"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"bigN", "=", "20"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"smallN", "=", "10"}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5463630695*^9, 3.546363080922*^9}, {3.546363115111*^9, 
   3.546363141222*^9}, {3.5463634376070004`*^9, 3.546363492512*^9}, {
   3.5463635258719997`*^9, 3.546363635209*^9}, {3.5463659693424*^9, 
   3.5463659740664*^9}, {3.5463660119104*^9, 3.5463661504323997`*^9}, {
   3.5463664883764*^9, 3.5463664944384003`*^9}, {3.5463666192464*^9, 
   3.5463666459524*^9}, {3.5463667113484*^9, 3.5463667373324003`*^9}, {
   3.5463676399364*^9, 3.5463676562564*^9}, {3.546390936499014*^9, 
   3.546390947510488*^9}, {3.546401541105667*^9, 3.5464015544196997`*^9}, {
   3.546434422083152*^9, 3.546434437782632*^9}, {3.563910175270574*^9, 
   3.563910220797307*^9}, {3.564799718336*^9, 3.564799720823*^9}, {
   3.5649679651156*^9, 3.5649679782296*^9}, {3.5655714809302*^9, 
   3.5655714994402*^9}, {3.5655715372522*^9, 3.5655715645132*^9}, {
   3.5656148581191998`*^9, 3.5656148586572*^9}, {3.5656173795022*^9, 
   3.5656173983492002`*^9}, {3.5656200626132*^9, 3.5656200714432*^9}, {
   3.5656203941162*^9, 3.5656204067612*^9}, {3.5656338952332*^9, 
   3.5656339009142*^9}, {3.5656343322771997`*^9, 3.5656343385042*^9}, {
   3.5657105818142*^9, 3.5657105820962*^9}, {3.5657106368512*^9, 
   3.5657106371591997`*^9}, {3.5657106923602*^9, 3.5657107507892*^9}, 
   3.5657108510002003`*^9, {3.5657130676252003`*^9, 3.5657130795922003`*^9}, {
   3.5657280484491997`*^9, 3.5657280561782*^9}, {3.5657313869382*^9, 
   3.5657314205002003`*^9}, {3.565744033573463*^9, 3.565744042518571*^9}, {
   3.565745333305562*^9, 3.565745362905344*^9}, {3.5657454278631887`*^9, 
   3.565745444786531*^9}, {3.5657455096709547`*^9, 3.5657455425153027`*^9}, {
   3.565746618412477*^9, 3.565746650258099*^9}, {3.565753896592372*^9, 
   3.56575390058217*^9}, {3.565753960980502*^9, 3.5657539672129097`*^9}, {
   3.565754087442314*^9, 3.565754191303976*^9}, {3.565754253265995*^9, 
   3.565754273708042*^9}, {3.565754484213047*^9, 3.5657544921511374`*^9}, {
   3.565755271816646*^9, 3.565755314332326*^9}, {3.568778838468502*^9, 
   3.568778844504279*^9}, {3.6369966981721087`*^9, 3.636996717030632*^9}, {
   3.636996851466588*^9, 3.636996865624813*^9}, {3.6660136237157383`*^9, 
   3.666013670821316*^9}, {3.666013893248465*^9, 3.6660139364020987`*^9}, {
   3.6660139687341423`*^9, 3.66601397001274*^9}, {3.6660141238590117`*^9, 
   3.6660141307290163`*^9}, {3.668348624015904*^9, 3.6683486288118877`*^9}, {
   3.671217871017108*^9, 3.671217914017391*^9}, {3.6712180373012943`*^9, 
   3.6712180413874903`*^9}, {3.671219062993865*^9, 3.6712190635847397`*^9}, {
   3.671219493756735*^9, 3.6712195384749804`*^9}, {3.671219609254385*^9, 
   3.671219614748374*^9}, {3.671219713949239*^9, 3.6712197162506523`*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Other Functions", "Section",
 CellChangeTimes->{{3.5463623180694*^9, 3.5463623214114*^9}, {
   3.5657108715811996`*^9, 3.5657108824351997`*^9}, 3.568127509238703*^9}],

Cell[CellGroupData[{

Cell["Some Functions", "Subsection",
 InitializationCell->True,
 CellChangeTimes->{{3.5408983275643997`*^9, 3.5408983375383997`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Theta]", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "z_", "]"}], ":=", 
   RowBox[{"2", " ", "I", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"2", " ", "t", " ", 
       SuperscriptBox["z", "2"]}], "+", 
      RowBox[{"x", " ", "z"}]}], ")"}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"rb", "[", "k_", "]"}], ":=", 
   RowBox[{"\[Lambda]", " ", 
    RowBox[{"cc", "[", 
     RowBox[{"r", "[", 
      RowBox[{"cc", "[", "k", "]"}], "]"}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"rbsamp", "[", "k_", "]"}], ":=", 
   RowBox[{"\[Lambda]", " ", 
    RowBox[{"cc", "[", 
     RowBox[{"rsamp", "[", 
      RowBox[{"cc", "[", "k", "]"}], "]"}], "]"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Tau]samp", "[", "k_", "]"}], ":=", 
    RowBox[{"1", "+", 
     RowBox[{"\[Lambda]", " ", 
      RowBox[{"rsamp", "[", "k", "]"}], 
      RowBox[{"cc", "[", 
       RowBox[{"rsamp", "[", 
        RowBox[{"cc", "[", "k", "]"}], "]"}], "]"}]}]}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Ln", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", "0"},
      {
       RowBox[{
        FractionBox[
         RowBox[{"r", "[", "k", "]"}], 
         RowBox[{"\[Tau]", "[", "k", "]"}]], 
        RowBox[{"Exp", "[", 
         RowBox[{
          RowBox[{"\[Theta]", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Lnsamp", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {"1", "0"},
       {
        RowBox[{
         FractionBox[
          RowBox[{"rsamp", "[", "k", "]"}], 
          RowBox[{"\[Tau]samp", "[", "k", "]"}]], 
         RowBox[{"Exp", "[", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "1"}
      }], ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Linn", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", "0"},
      {
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{"r", "[", "k", "]"}], 
          RowBox[{"\[Tau]", "[", "k", "]"}]]}], 
        RowBox[{"Exp", "[", 
         RowBox[{
          RowBox[{"\[Theta]", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Linnsamp", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {"1", "0"},
       {
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{"rsamp", "[", "k", "]"}], 
           RowBox[{"\[Tau]samp", "[", "k", "]"}]]}], 
         RowBox[{"Exp", "[", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "1"}
      }], ")"}]}], ";"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Un", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", 
       RowBox[{
        FractionBox[
         RowBox[{"rb", "[", "k", "]"}], 
         RowBox[{"\[Tau]", "[", "k", "]"}]], 
        RowBox[{"Exp", "[", 
         RowBox[{"-", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "]"}]}]},
      {"0", "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Unsamp", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {"1", 
        RowBox[{
         FractionBox[
          RowBox[{"rbsamp", "[", "k", "]"}], 
          RowBox[{"\[Tau]samp", "[", "k", "]"}]], 
         RowBox[{"Exp", "[", 
          RowBox[{"-", 
           RowBox[{
            RowBox[{"\[Theta]", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "]"}]}]},
       {"0", "1"}
      }], ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DDn", "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {
        RowBox[{"\[Tau]", "[", "k", "]"}], "0"},
       {"0", 
        RowBox[{"1", "/", 
         RowBox[{"\[Tau]", "[", "k", "]"}]}]}
      }], ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DDnsamp", "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {
        RowBox[{"\[Tau]samp", "[", "k", "]"}], "0"},
       {"0", 
        RowBox[{"1", "/", 
         RowBox[{"\[Tau]samp", "[", "k", "]"}]}]}
      }], ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"G", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "z_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"1", "+", 
        RowBox[{
         RowBox[{"r", "[", "z", "]"}], 
         RowBox[{"rb", "[", "z", "]"}]}]}], 
       RowBox[{
        RowBox[{"rb", "[", "z", "]"}], 
        RowBox[{"Exp", "[", 
         RowBox[{
          RowBox[{"-", "2"}], " ", "I", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "t", " ", 
             SuperscriptBox["z", "2"]}], "+", 
            RowBox[{"x", " ", "z"}]}], ")"}]}], "]"}]}]},
      {
       RowBox[{
        RowBox[{"r", "[", "z", "]"}], " ", 
        RowBox[{"Exp", "[", 
         RowBox[{"2", " ", "I", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "t", " ", 
             SuperscriptBox["z", "2"]}], "+", " ", 
            RowBox[{"x", " ", "z"}]}], ")"}]}], "]"}]}], "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"G", "[", 
     RowBox[{"_", ",", "_"}], "]"}], "[", 
    RowBox[{"_", "?", "InfinityQ"}], "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1.", "0"},
      {"0", "1."}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Gsamp", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "z_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"1", "+", 
        RowBox[{
         RowBox[{"rsamp", "[", "z", "]"}], 
         RowBox[{"rbsamp", "[", "z", "]"}]}]}], 
       RowBox[{
        RowBox[{"rbsamp", "[", "z", "]"}], 
        RowBox[{"Exp", "[", 
         RowBox[{
          RowBox[{"-", "2"}], " ", "I", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "t", " ", 
             SuperscriptBox["z", "2"]}], "+", 
            RowBox[{"x", " ", "z"}]}], ")"}]}], "]"}]}]},
      {
       RowBox[{
        RowBox[{"rsamp", "[", "z", "]"}], " ", 
        RowBox[{"Exp", "[", 
         RowBox[{"2", " ", "I", " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", "t", " ", 
             SuperscriptBox["z", "2"]}], "+", " ", 
            RowBox[{"x", " ", "z"}]}], ")"}]}], "]"}]}], "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Gsamp", "[", 
     RowBox[{"_", ",", "_"}], "]"}], "[", 
    RowBox[{"_", "?", "InfinityQ"}], "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1.", "0"},
      {"0", "1."}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Mn", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", 
       RowBox[{
        RowBox[{"rb", "[", "k", "]"}], " ", 
        RowBox[{"Exp", "[", 
         RowBox[{"-", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "]"}]}]},
      {"0", "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Mnsamp", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", 
       RowBox[{
        RowBox[{"rbsamp", "[", "k", "]"}], " ", 
        RowBox[{"Exp", "[", 
         RowBox[{"-", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "]"}]}]},
      {"0", "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Pn", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", "0"},
      {
       RowBox[{
        RowBox[{"r", "[", "k", "]"}], " ", 
        RowBox[{"Exp", "[", 
         RowBox[{
          RowBox[{"\[Theta]", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Pnsamp", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {"1", "0"},
       {
        RowBox[{
         RowBox[{"rsamp", "[", "k", "]"}], " ", 
         RowBox[{"Exp", "[", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "1"}
      }], ")"}]}], ";"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"qn", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"out", ",", "i"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"out", " ", "=", " ", "1"}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"i", "=", "1"}], ",", 
        RowBox[{"i", "\[LessEqual]", 
         RowBox[{"Length", "[", "a", "]"}]}], ",", 
        RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Abs", "[", 
              RowBox[{"c", "[", 
               RowBox[{"[", "i", "]"}], "]"}], "]"}], 
             RowBox[{"Exp", "[", 
              RowBox[{"Re", "[", 
               RowBox[{
                RowBox[{"\[Theta]", "[", 
                 RowBox[{"x", ",", "t"}], "]"}], "[", 
                RowBox[{"a", "[", 
                 RowBox[{"[", "i", "]"}], "]"}], "]"}], "]"}], "]"}]}], ">", 
            "expconst"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"out", " ", "=", " ", 
             RowBox[{"out", "*", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"k", "-", 
                 RowBox[{"a", "[", 
                  RowBox[{"[", "i", "]"}], "]"}]}], ")"}], "/", 
               RowBox[{"(", 
                RowBox[{"k", "-", 
                 RowBox[{"Conjugate", "[", 
                  RowBox[{"a", "[", 
                   RowBox[{"[", "i", "]"}], "]"}], "]"}]}], ")"}]}]}]}], 
            ";"}]}], "\[IndentingNewLine]", "]"}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", "out"}]}], 
    "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Qn", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{
        RowBox[{"qn", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "0"},
      {"0", 
       RowBox[{"1", "/", 
        RowBox[{
         RowBox[{"qn", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Qn", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", 
    RowBox[{"_", "?", "InfinityQ"}], "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1.", "0"},
      {"0", "1."}
     }], ")"}]}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.538103142394384*^9, 3.538103151534226*^9}, {
   3.5381035800010853`*^9, 3.5381035806070843`*^9}, {3.540653533861*^9, 
   3.540653534101*^9}, {3.5406547841210003`*^9, 3.540654804311*^9}, {
   3.5406548650150003`*^9, 3.540654883401*^9}, {3.5406583132382*^9, 
   3.5406583318182*^9}, {3.5406593163606*^9, 3.5406593424706*^9}, {
   3.5406606785154*^9, 3.5406606829554*^9}, {3.540670035786598*^9, 
   3.540670045469647*^9}, {3.5406701523856287`*^9, 3.540670159109951*^9}, {
   3.540671043884227*^9, 3.5406710445151052`*^9}, {3.54067107622948*^9, 
   3.540671117438383*^9}, 3.540671223981215*^9, 3.540673675932254*^9, {
   3.5406742109124002`*^9, 3.5406742112953997`*^9}, {3.5406777028814*^9, 
   3.5406777049323997`*^9}, {3.5406826023554*^9, 3.5406826068993998`*^9}, {
   3.5406828924653997`*^9, 3.5406828948314*^9}, {3.5406831061644*^9, 
   3.5406831144644003`*^9}, {3.5406844599924*^9, 3.5406844614054003`*^9}, {
   3.5406847746944*^9, 3.5406847750914*^9}, {3.5406849669924*^9, 
   3.5406849737824*^9}, {3.5406851745424*^9, 3.5406852291823997`*^9}, {
   3.5406853996924*^9, 3.5406853998624*^9}, 3.5406876104944*^9, {
   3.5406888923224*^9, 3.5406889113124*^9}, {3.5407250148524*^9, 
   3.5407250177324*^9}, {3.5407255652974*^9, 3.5407255687424*^9}, {
   3.5407264058993998`*^9, 3.5407264153334*^9}, {3.5407270816303997`*^9, 
   3.5407270841644*^9}, {3.5407272503754*^9, 3.5407272625214*^9}, {
   3.5407277902693996`*^9, 3.5407277906574*^9}, {3.5407280626984*^9, 
   3.5407280632244*^9}, {3.540732091156747*^9, 3.540732094732853*^9}, {
   3.5407337077895193`*^9, 3.540733708610265*^9}, {3.540734314953412*^9, 
   3.540734315469853*^9}, {3.540734447769846*^9, 3.540734448600122*^9}, {
   3.5407364063724413`*^9, 3.5407364098191147`*^9}, {3.5407369996439257`*^9, 
   3.54073700378605*^9}, {3.540737269754858*^9, 3.540737270296258*^9}, {
   3.5407374577807903`*^9, 3.540737460562327*^9}, 3.5407388991890717`*^9, {
   3.540739981122607*^9, 3.540739982269462*^9}, {3.540740493804627*^9, 
   3.540740495873988*^9}, {3.540740792568321*^9, 3.540740792845584*^9}, {
   3.540741429871428*^9, 3.540741430139724*^9}, {3.540743857872079*^9, 
   3.5407438622771*^9}, {3.540749237001852*^9, 3.5407492383893623`*^9}, {
   3.5407608990193996`*^9, 3.5407609003594*^9}, {3.5407636848574*^9, 
   3.5407637029354*^9}, {3.5407637497794*^9, 3.5407637906194*^9}, {
   3.5407639569974003`*^9, 3.5407639572774*^9}, 3.5407685188574*^9, {
   3.5407690527953997`*^9, 3.5407690690114*^9}, {3.540839541133526*^9, 
   3.540839547433312*^9}, {3.5409029240854*^9, 3.5409029383283997`*^9}, 
   3.5409035117384*^9, 3.5409416746544*^9, 3.5656171862981997`*^9, {
   3.5656174193571997`*^9, 3.5656174476022*^9}, {3.5656175091342*^9, 
   3.5656175348802*^9}, {3.5656175759962*^9, 3.5656176103392*^9}, {
   3.5656176410592003`*^9, 3.5656176662782*^9}, {3.5656177086092*^9, 
   3.5656177351282*^9}, {3.5656204709592*^9, 3.5656205127052*^9}, {
   3.5656212498292*^9, 3.5656214052762003`*^9}, {3.5656411683152*^9, 
   3.5656411980062*^9}, {3.5656586528082*^9, 3.5656587202482*^9}, {
   3.565754330008767*^9, 3.565754333067934*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Shift", "Subsection",
 InitializationGroup->True,
 CellChangeTimes->{{3.538856529637267*^9, 3.538856530444125*^9}, {
   3.5656143675142*^9, 3.5656143832502003`*^9}, 3.5656214290852003`*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalPhi]s", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "z_", "]"}], ":=", 
   RowBox[{"\[CapitalPhi]", "[", 
    RowBox[{
     RowBox[{"z", "/", 
      RowBox[{"Sqrt", "[", "t", "]"}]}], "-", 
     RowBox[{"x", "/", 
      RowBox[{"(", 
       RowBox[{"4", " ", "t"}], ")"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalPhi]ins", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], "[", "z_", "]"}], ":=", 
  RowBox[{"\[CapitalPhi]in", "[", 
   RowBox[{
    RowBox[{"z", "/", 
     RowBox[{"Sqrt", "[", "t", "]"}]}], "-", 
    RowBox[{"x", "/", 
     RowBox[{"(", 
      RowBox[{"4", " ", "t"}], ")"}]}]}], "]"}]}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"rs", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "z_", "]"}], ":=", 
   RowBox[{"r", "[", 
    RowBox[{
     RowBox[{"z", "/", 
      RowBox[{"Sqrt", "[", "t", "]"}]}], "-", 
     RowBox[{"x", "/", 
      RowBox[{"(", 
       RowBox[{"4", " ", "t"}], ")"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Tau]s", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "z_", "]"}], ":=", 
   RowBox[{"\[Tau]", "[", 
    RowBox[{
     RowBox[{"z", "/", 
      RowBox[{"Sqrt", "[", "t", "]"}]}], "-", 
     RowBox[{"x", "/", 
      RowBox[{"(", 
       RowBox[{"4", " ", "t"}], ")"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"rsb", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "z_", "]"}], ":=", 
    RowBox[{"rb", "[", 
     RowBox[{
      RowBox[{"z", "/", 
       RowBox[{"Sqrt", "[", "t", "]"}]}], "-", 
      RowBox[{"x", "/", 
       RowBox[{"(", 
        RowBox[{"4", " ", "t"}], ")"}]}]}], "]"}]}], ";"}], 
  "\n"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalPhi]ssamp", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "z_", "]"}], ":=", 
   RowBox[{"\[CapitalPhi]samp", "[", 
    RowBox[{
     RowBox[{"z", "/", 
      RowBox[{"Sqrt", "[", "t", "]"}]}], "-", 
     RowBox[{"x", "/", 
      RowBox[{"(", 
       RowBox[{"4", " ", "t"}], ")"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[CapitalPhi]inssamp", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], "[", "z_", "]"}], ":=", 
  RowBox[{"\[CapitalPhi]insamp", "[", 
   RowBox[{
    RowBox[{"z", "/", 
     RowBox[{"Sqrt", "[", "t", "]"}]}], "-", 
    RowBox[{"x", "/", 
     RowBox[{"(", 
      RowBox[{"4", " ", "t"}], ")"}]}]}], "]"}]}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"rssamp", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "z_", "]"}], ":=", 
   RowBox[{"rsamp", "[", 
    RowBox[{
     RowBox[{"z", "/", 
      RowBox[{"Sqrt", "[", "t", "]"}]}], "-", 
     RowBox[{"x", "/", 
      RowBox[{"(", 
       RowBox[{"4", " ", "t"}], ")"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Tau]ssamp", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "z_", "]"}], ":=", 
   RowBox[{"\[Tau]samp", "[", 
    RowBox[{
     RowBox[{"z", "/", 
      RowBox[{"Sqrt", "[", "t", "]"}]}], "-", 
     RowBox[{"x", "/", 
      RowBox[{"(", 
       RowBox[{"4", " ", "t"}], ")"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"rsbsamp", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "z_", "]"}], ":=", 
   RowBox[{"rbsamp", "[", 
    RowBox[{
     RowBox[{"z", "/", 
      RowBox[{"Sqrt", "[", "t", "]"}]}], "-", 
     RowBox[{"x", "/", 
      RowBox[{"(", 
       RowBox[{"4", " ", "t"}], ")"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Theta]s", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "z_", "]"}], ":=", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"\[ImaginaryI]", " ", 
        SuperscriptBox["x", "2"]}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{"4", " ", "t"}], ")"}]}]}], "+", 
    RowBox[{"4", " ", "\[ImaginaryI]", " ", 
     SuperscriptBox["z", "2"]}]}]}], "\n"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"L", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", "0"},
      {
       RowBox[{
        FractionBox[
         RowBox[{
          RowBox[{"rs", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], 
         RowBox[{
          RowBox[{"\[Tau]s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]], 
        RowBox[{"Exp", "[", 
         RowBox[{
          RowBox[{"\[Theta]s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Lsamp", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {"1", "0"},
       {
        RowBox[{
         FractionBox[
          RowBox[{
           RowBox[{"rssamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], 
          RowBox[{
           RowBox[{"\[Tau]ssamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]], 
         RowBox[{"Exp", "[", 
          RowBox[{
           RowBox[{"\[Theta]s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "1"}
      }], ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Lin", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", "0"},
      {
       RowBox[{
        FractionBox[
         RowBox[{"-", 
          RowBox[{
           RowBox[{"rs", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], 
         RowBox[{
          RowBox[{"\[Tau]s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]], 
        RowBox[{"Exp", "[", 
         RowBox[{
          RowBox[{"\[Theta]s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Linsamp", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {"1", "0"},
       {
        RowBox[{
         FractionBox[
          RowBox[{"-", 
           RowBox[{
            RowBox[{"rssamp", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], 
          RowBox[{
           RowBox[{"\[Tau]ssamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]], 
         RowBox[{"Exp", "[", 
          RowBox[{
           RowBox[{"\[Theta]s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "1"}
      }], ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"U", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", 
       RowBox[{
        FractionBox[
         RowBox[{
          RowBox[{"rsb", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], 
         RowBox[{
          RowBox[{"\[Tau]s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]], 
        RowBox[{"Exp", "[", 
         RowBox[{"-", 
          RowBox[{
           RowBox[{"\[Theta]s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "]"}]}]},
      {"0", "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Usamp", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {"1", 
        RowBox[{
         FractionBox[
          RowBox[{
           RowBox[{"rsbsamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], 
          RowBox[{
           RowBox[{"\[Tau]ssamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]], 
         RowBox[{"Exp", "[", 
          RowBox[{"-", 
           RowBox[{
            RowBox[{"\[Theta]s", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "]"}]}]},
       {"0", "1"}
      }], ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Uin", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", 
       RowBox[{
        FractionBox[
         RowBox[{"-", 
          RowBox[{
           RowBox[{"rsb", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], 
         RowBox[{
          RowBox[{"\[Tau]s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]], 
        RowBox[{"Exp", "[", 
         RowBox[{"-", 
          RowBox[{
           RowBox[{"\[Theta]s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "]"}]}]},
      {"0", "1"}
     }], ")"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Uinsamp", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {"1", 
        RowBox[{
         FractionBox[
          RowBox[{"-", 
           RowBox[{
            RowBox[{"rsbsamp", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], 
          RowBox[{
           RowBox[{"\[Tau]ssamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]], 
         RowBox[{"Exp", "[", 
          RowBox[{"-", 
           RowBox[{
            RowBox[{"\[Theta]s", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "]"}]}]},
       {"0", "1"}
      }], ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"M", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", 
       RowBox[{
        RowBox[{
         RowBox[{"rsb", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], " ", 
        RowBox[{"Exp", "[", 
         RowBox[{"-", 
          RowBox[{
           RowBox[{"\[Theta]s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "]"}]}]},
      {"0", "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Msamp", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {"1", 
        RowBox[{
         RowBox[{
          RowBox[{"rsbsamp", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], " ", 
         RowBox[{"Exp", "[", 
          RowBox[{"-", 
           RowBox[{
            RowBox[{"\[Theta]s", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "]"}]}]},
       {"0", "1"}
      }], ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Mi", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{
          RowBox[{"rsb", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], " ", 
        RowBox[{"Exp", "[", 
         RowBox[{"-", 
          RowBox[{
           RowBox[{"\[Theta]s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "]"}]}]},
      {"0", "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Misamp", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {"1", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{
           RowBox[{"rsbsamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], " ", 
         RowBox[{"Exp", "[", 
          RowBox[{"-", 
           RowBox[{
            RowBox[{"\[Theta]s", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], "]"}]}]},
       {"0", "1"}
      }], ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"P", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", "0"},
      {
       RowBox[{
        RowBox[{
         RowBox[{"rs", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], " ", 
        RowBox[{"Exp", "[", 
         RowBox[{
          RowBox[{"\[Theta]s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Psamp", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {"1", "0"},
       {
        RowBox[{
         RowBox[{
          RowBox[{"rssamp", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], " ", 
         RowBox[{"Exp", "[", 
          RowBox[{
           RowBox[{"\[Theta]s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "1"}
      }], ")"}]}], ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Pin", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {"1", "0"},
      {
       RowBox[{
        RowBox[{"-", 
         RowBox[{
          RowBox[{"rs", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], " ", 
        RowBox[{"Exp", "[", 
         RowBox[{
          RowBox[{"\[Theta]s", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Pinsamp", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {"1", "0"},
       {
        RowBox[{
         RowBox[{"-", 
          RowBox[{
           RowBox[{"rssamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}], " ", 
         RowBox[{"Exp", "[", 
          RowBox[{
           RowBox[{"\[Theta]s", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}]}], "1"}
      }], ")"}]}], ";"}], "\[IndentingNewLine]", 
  "\n"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DD", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{
        RowBox[{"\[Tau]s", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "0"},
      {"0", 
       RowBox[{"1", "/", 
        RowBox[{
         RowBox[{"\[Tau]s", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"DDsamp", "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
    RowBox[{"(", GridBox[{
       {
        RowBox[{
         RowBox[{"\[Tau]ssamp", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "0"},
       {"0", 
        RowBox[{"1", "/", 
         RowBox[{
          RowBox[{"\[Tau]ssamp", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}]}]}
      }], ")"}]}], ";"}], "\[IndentingNewLine]"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Q", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "z_", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{
        RowBox[{"qn", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "[", 
        RowBox[{
         RowBox[{"z", "/", 
          RowBox[{"Sqrt", "[", "t", "]"}]}], "-", 
         RowBox[{"x", "/", 
          RowBox[{"(", 
           RowBox[{"4", " ", "t"}], ")"}]}]}], "]"}], "0"},
      {"0", 
       RowBox[{"1", "/", 
        RowBox[{
         RowBox[{"qn", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", 
         RowBox[{
          RowBox[{"z", "/", 
           RowBox[{"Sqrt", "[", "t", "]"}]}], "-", 
          RowBox[{"x", "/", 
           RowBox[{"(", 
            RowBox[{"4", " ", "t"}], ")"}]}]}], "]"}]}]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]"}], "Input",
 InitializationGroup->True,
 CellChangeTimes->{{3.5656143858612003`*^9, 3.5656143858622*^9}, {
  3.5656183566622*^9, 3.5656184064021997`*^9}, {3.5656204620852003`*^9, 
  3.5656204884452*^9}, {3.5656214407462*^9, 3.5656214603812*^9}, {
  3.5656214921212*^9, 3.5656214964632*^9}, {3.5656215725942*^9, 
  3.5656218388712*^9}}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Reflection Coefficient", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.5407685463534*^9, 3.5407685478054*^9}, {
  3.5657108306542*^9, 3.5657108348501997`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"CME", "[", "x_", "]"}], ":=", 
   RowBox[{"Chop", "[", 
    RowBox[{"x", ",", "$MachineEpsilon"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ScatteringMatrixFiniteNLS", "[", 
    RowBox[{"q_", ",", "n_", ",", "el_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "qf", ",", "qfb", ",", "Dm", ",", "Dmb", ",", "IIm", ",", "IImb", ",", 
       "P", ",", "Pb", ",", "id", ",", "Q", ",", "R", ",", "Qb", ",", "Rb", 
       ",", "DM", ",", "DMb", ",", "q\[Sigma]1", ",", "q\[Sigma]1b", ",", 
       "J\[Sigma]3", ",", "J\[Sigma]31", ",", "J\[Sigma]3b", ",", 
       "J\[Sigma]31b", ",", "rhs", ",", "A", ",", "Ab"}], "}"}], ",", "\n", 
     " ", 
     RowBox[{
      RowBox[{"qf", "=", 
       RowBox[{"Fun", "[", 
        RowBox[{"q", ",", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", "el"}], ",", "0"}], "}"}], "]"}], ",", "n"}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"qfb", "=", 
       RowBox[{"Fun", "[", 
        RowBox[{"q", ",", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{"0", ",", "el"}], "}"}], "]"}], ",", "n"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"qf", "//", "Values"}], "//", "First"}], "//", "Abs"}], 
         ",", 
         RowBox[{"qf", "//", "DCTPlot"}]}], "}"}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Dm", "=", 
       RowBox[{"DerivativeMatrix", "[", "qf", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Dmb", "=", 
       RowBox[{"DerivativeMatrix", "[", "qfb", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"IIm", "=", 
       RowBox[{"ReduceDimensionIntegrateMatrix", "[", "qf", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"IImb", "=", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"ReduceDimensionIntegrateMatrix", "[", 
             RowBox[{"(", 
              RowBox[{"qfb", "//", "ReverseOrientation"}], ")"}], "]"}], "//",
             "Transpose"}], "//", "Reverse"}], "//", "Transpose"}], "//", 
         "Reverse"}], ")"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"id", "=", 
       RowBox[{"IdentityMatrix", "[", "n", "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"Q", " ", "=", " ", 
       RowBox[{"DiagonalMatrix", "[", 
        RowBox[{"qf", "//", "Values"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"R", "=", " ", 
       RowBox[{
        RowBox[{"-", "\[Lambda]"}], " ", 
        RowBox[{"DiagonalMatrix", "[", 
         RowBox[{
          RowBox[{"qf", "//", "Conjugate"}], "//", "Values"}], "]"}]}]}], ";",
       "\[IndentingNewLine]", 
      RowBox[{"Qb", " ", "=", " ", 
       RowBox[{"DiagonalMatrix", "[", 
        RowBox[{"qfb", "//", "Values"}], "]"}]}], ";", "\[IndentingNewLine]", 
      
      RowBox[{"Rb", " ", "=", " ", 
       RowBox[{
        RowBox[{"-", "\[Lambda]"}], " ", 
        RowBox[{"DiagonalMatrix", "[", 
         RowBox[{
          RowBox[{"qfb", "//", "Conjugate"}], "//", "Values"}], "]"}]}]}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"A", "=", 
       RowBox[{"BlockMatrix", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"id", ",", 
            RowBox[{"-", 
             RowBox[{"IIm", ".", "Q"}]}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"IIm", ".", "R"}]}], ",", "id"}], "}"}]}], "}"}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Ab", "=", 
       RowBox[{"BlockMatrix", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"id", ",", 
            RowBox[{"-", 
             RowBox[{"IImb", ".", "Qb"}]}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"IImb", ".", "Rb"}]}], ",", "id"}], "}"}]}], "}"}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"J\[Sigma]3", "=", 
       RowBox[{"BlockMatrix", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"0", "*", "id"}], ",", "0"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", 
            RowBox[{
             RowBox[{"-", "2"}], "I", " ", "IIm"}]}], "}"}]}], "}"}], "]"}]}],
       ";", "\[IndentingNewLine]", 
      RowBox[{"J\[Sigma]31", "=", 
       RowBox[{"BlockMatrix", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"2", "I", " ", "IIm"}], ",", "0"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", 
            RowBox[{"0", "*", "id"}]}], "}"}]}], "}"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"J\[Sigma]3b", "=", 
       RowBox[{"BlockMatrix", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"0", "*", "id"}], ",", "0"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", 
            RowBox[{
             RowBox[{"-", "2"}], "I", " ", "IImb"}]}], "}"}]}], "}"}], 
        "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"J\[Sigma]31b", "=", 
       RowBox[{"BlockMatrix", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"2", "I", " ", "IImb"}], ",", "0"}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", 
            RowBox[{"0", "*", "id"}]}], "}"}]}], "}"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"ScatteringMatrixFiniteNLS", "[", 
       RowBox[{
       "qf", ",", "qfb", ",", "A", ",", "Ab", ",", "J\[Sigma]3", ",", 
        "J\[Sigma]31", ",", "J\[Sigma]3b", ",", "J\[Sigma]31b", ",", "IIm", 
        ",", "IImb"}], "]"}]}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ScatteringMatrixFiniteNLS", "[", 
     RowBox[{
     "qf_", ",", "qfb_", ",", "A_", ",", "Ab_", ",", "J\[Sigma]3_", ",", 
      "J\[Sigma]31_", ",", "J\[Sigma]3b_", ",", "J\[Sigma]31b_", ",", "IIm_", 
      ",", "IImb_"}], "]"}], "[", "w_", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "ans0", ",", "ans1", ",", "rhs", ",", "lhs", ",", "n", ",", "s1", ",", 
       "s2", ",", "qb", ",", "q", ",", "r", ",", "rb"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"q", " ", "=", " ", 
       RowBox[{"qf", "//", "Values"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"qb", " ", "=", " ", 
       RowBox[{"qfb", "//", "Values"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"r", " ", "=", " ", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "\[Lambda]"}], " ", "q"}], "//", "Conjugate"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"rb", "=", " ", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "\[Lambda]"}], " ", "qb"}], "//", "Conjugate"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"n", " ", "=", " ", 
       RowBox[{"q", "//", "Length"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"lhs", "=", 
       RowBox[{"A", "+", " ", 
        RowBox[{"w", "*", "J\[Sigma]3"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"rhs", " ", "=", " ", 
       RowBox[{"Join", "[", 
        RowBox[{
         RowBox[{"ConstantArray", "[", 
          RowBox[{"0.", ",", "n"}], "]"}], ",", 
         RowBox[{"IIm", ".", "r"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"ans0", " ", "=", " ", 
       RowBox[{"LinearSolve", "[", 
        RowBox[{
         RowBox[{"lhs", "//", "CME"}], ",", 
         RowBox[{"rhs", "//", "CME"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
      
      RowBox[{"lhs", "=", 
       RowBox[{"A", "+", 
        RowBox[{"w", "*", "J\[Sigma]31"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"rhs", "=", 
       RowBox[{"Join", "[", 
        RowBox[{
         RowBox[{"IIm", ".", "q"}], ",", 
         RowBox[{"ConstantArray", "[", 
          RowBox[{"0.", ",", "n"}], "]"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"ans1", " ", "=", " ", 
       RowBox[{"LinearSolve", "[", 
        RowBox[{
         RowBox[{"lhs", "//", "CME"}], ",", 
         RowBox[{"rhs", "//", "CME"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
      
      RowBox[{"s1", "=", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{"ans0", "[", 
             RowBox[{"[", "n", "]"}], "]"}], "+", "1"}], ",", 
           RowBox[{"-", 
            RowBox[{"ans1", "[", 
             RowBox[{"[", "n", "]"}], "]"}]}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"ans0", "[", 
            RowBox[{"[", 
             RowBox[{"2", "n"}], "]"}], "]"}], ",", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"ans1", "[", 
              RowBox[{"[", 
               RowBox[{"2", "n"}], "]"}], "]"}]}], "-", "1"}]}], "}"}]}], 
        "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"lhs", "=", 
       RowBox[{"Ab", "+", " ", 
        RowBox[{"w", "*", "J\[Sigma]3b"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Off", "[", 
       RowBox[{"LinearSolve", "::", "luc"}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"rhs", " ", "=", " ", 
       RowBox[{"Join", "[", 
        RowBox[{
         RowBox[{"ConstantArray", "[", 
          RowBox[{"0.", ",", "n"}], "]"}], ",", 
         RowBox[{"IImb", ".", "rb"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"ans0", " ", "=", " ", 
       RowBox[{"LinearSolve", "[", 
        RowBox[{
         RowBox[{"lhs", "//", "CME"}], ",", 
         RowBox[{"rhs", "//", "CME"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
      
      RowBox[{"lhs", "=", 
       RowBox[{"Ab", "+", " ", 
        RowBox[{"w", "*", "J\[Sigma]31b"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"rhs", " ", "=", " ", 
       RowBox[{"Join", "[", 
        RowBox[{
         RowBox[{"IImb", ".", "qb"}], ",", 
         RowBox[{"ConstantArray", "[", 
          RowBox[{"0.", ",", "n"}], "]"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"ans1", " ", "=", " ", 
       RowBox[{"LinearSolve", "[", 
        RowBox[{
         RowBox[{"lhs", "//", "CME"}], ",", 
         RowBox[{"rhs", "//", "CME"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
      
      RowBox[{"On", "[", 
       RowBox[{"LinearSolve", "::", "luc"}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"s2", "=", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{
            RowBox[{"ans0", "[", 
             RowBox[{"[", "1", "]"}], "]"}], "+", "1"}], ",", 
           RowBox[{"ans1", "[", 
            RowBox[{"[", "1", "]"}], "]"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"ans0", "[", 
            RowBox[{"[", 
             RowBox[{"n", "+", "1"}], "]"}], "]"}], ",", 
           RowBox[{
            RowBox[{"ans1", "[", 
             RowBox[{"[", 
              RowBox[{"n", "+", "1"}], "]"}], "]"}], "+", "1"}]}], "}"}]}], 
        "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"s1", "=", 
       RowBox[{
        RowBox[{"Inverse", "[", "s2", "]"}], ".", "s1"}]}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5407710794914*^9, 3.5407710807813997`*^9}, {
   3.5407711951854*^9, 3.5407712059854*^9}, {3.5407712856654*^9, 
   3.5407713737354*^9}, {3.540827282042986*^9, 3.540827333330905*^9}, {
   3.54083242807616*^9, 3.5408324852837*^9}, {3.540832522921386*^9, 
   3.5408325930834007`*^9}, {3.540832938848312*^9, 3.540832960757762*^9}, {
   3.540832994793727*^9, 3.5408330192890873`*^9}, {3.540833097880999*^9, 
   3.540833104398241*^9}, {3.5408331497947607`*^9, 3.540833157137931*^9}, {
   3.540833189894033*^9, 3.540833239916215*^9}, {3.5408333534680634`*^9, 
   3.540833375999275*^9}, {3.540833439903254*^9, 3.540833457391617*^9}, 
   3.5409039382404003`*^9, {3.5409378931194*^9, 3.5409379107794*^9}, {
   3.5409397365383997`*^9, 3.5409397675624*^9}, 3.5409398277864*^9, {
   3.5409399836464*^9, 3.5409399892994003`*^9}, {3.5409422127864*^9, 
   3.5409422401823997`*^9}, {3.541851859895*^9, 3.541851909367*^9}, {
   3.5418522254849997`*^9, 3.541852231517*^9}, {3.5418533264820004`*^9, 
   3.541853326612*^9}, {3.5463628431070004`*^9, 3.5463628486429996`*^9}, {
   3.5463662630164003`*^9, 3.5463662636764*^9}, {3.5656340548582*^9, 
   3.5656340797651997`*^9}, {3.5656341116712*^9, 3.5656341309472*^9}, {
   3.5656341613592*^9, 3.5656341633571997`*^9}, {3.5657105213012*^9, 
   3.5657105408712*^9}, {3.5657557671034803`*^9, 3.5657557769551697`*^9}, {
   3.5657559916336403`*^9, 3.5657559932735453`*^9}, {3.6660139868449707`*^9, 
   3.666014008588172*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Eigenvalues", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.5407715647813997`*^9, 3.5407715708213997`*^9}, {
   3.5407718315214*^9, 3.5407718358213997`*^9}, 3.568127365394457*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"HillNLS", "[", 
    RowBox[{"F_", ",", "n_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "c2", ",", "c1", ",", "c0", ",", "f11", ",", "f12", ",", "zz", ",", 
       "f02", ",", "f01", ",", "t", ",", "A", ",", "mu", ",", "L", ",", "P", 
       ",", "fun"}], "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Off", "[", 
       RowBox[{"General", "::", "unfl"}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Off", "[", 
       RowBox[{"General", "::", "ovfl"}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"mu", " ", "=", " ", ".25"}], ";", "\[IndentingNewLine]", 
      RowBox[{"L", " ", "=", " ", "Pi"}], ";", "\[IndentingNewLine]", 
      RowBox[{"P", " ", "=", "1"}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"f11", "[", "x_", "]"}], ":=", " ", 
       RowBox[{"2", "I", " ", 
        RowBox[{
         RowBox[{"Cos", "[", 
          RowBox[{"x", "/", "2"}], "]"}], "^", "2"}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"f12", "[", "x_", "]"}], ":=", 
       RowBox[{
        RowBox[{"-", "2"}], "I", " ", 
        RowBox[{
         RowBox[{"Cos", "[", 
          RowBox[{"x", "/", "2"}], "]"}], "^", "2"}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"zz", "[", "x_", "]"}], ":=", 
       RowBox[{"ConstantArray", "[", 
        RowBox[{"0.", ",", 
         RowBox[{"Length", "[", "x", "]"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"t", "[", "x_", "]"}], " ", ":=", " ", 
       RowBox[{"Tan", "[", 
        RowBox[{"x", "/", "2"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"f02", "[", "x_", "]"}], ":=", " ", 
       RowBox[{
        RowBox[{"-", "I"}], " ", 
        RowBox[{"F", "[", 
         RowBox[{"t", "[", "x", "]"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
      
      RowBox[{
       RowBox[{"f01", "[", "x_", "]"}], ":=", 
       RowBox[{
        RowBox[{"-", "\[Lambda]"}], " ", "I", " ", 
        RowBox[{"Conjugate", "[", 
         RowBox[{"F", "[", 
          RowBox[{"t", "[", "x", "]"}], "]"}], "]"}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"fun", "=", " ", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"zz", ",", "f11"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"f02", ",", "zz"}], "}"}]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{"f01", ",", "zz"}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"zz", ",", "f12"}], "}"}]}], "}"}]}], "}"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"A", "=", " ", 
       RowBox[{"Hill", "[", 
        RowBox[{"mu", ",", "L", ",", "n", ",", "P", ",", "fun"}], "]"}]}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"On", "[", 
       RowBox[{"General", "::", "unfl"}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"On", "[", 
       RowBox[{"General", "::", "ovfl"}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Eigenvalues", "[", 
       RowBox[{"Chop", "[", 
        RowBox[{"A", ",", "$MachineEpsilon"}], "]"}], "]"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"LocatePolesNLS", "[", 
    RowBox[{"F_", ",", "n_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"eval", ",", "s", ",", "i", ",", "sys", ",", "vecs"}], "}"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Off", "[", 
       RowBox[{"General", "::", "unfl"}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"eval", "=", 
       RowBox[{"HillNLS", "[", 
        RowBox[{"F", ",", "n"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"s", " ", "=", " ", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"i", "=", "1"}], ",", 
        RowBox[{"i", "\[LessEqual]", 
         RowBox[{"Length", "[", "eval", "]"}]}], ",", 
        RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"Im", "[", 
              RowBox[{"eval", "[", 
               RowBox[{"[", "i", "]"}], "]"}], "]"}], ">", 
             RowBox[{"10", "^", 
              RowBox[{"(", 
               RowBox[{"-", "4"}], ")"}]}]}], "&&", " ", 
            RowBox[{
             RowBox[{"Abs", "[", 
              RowBox[{"eval", "[", 
               RowBox[{"[", "i", "]"}], "]"}], "]"}], "<", "30"}]}], ",", 
           "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"s", "=", 
             RowBox[{"Join", "[", 
              RowBox[{"s", ",", 
               RowBox[{"{", 
                RowBox[{"eval", "[", 
                 RowBox[{"[", "i", "]"}], "]"}], "}"}]}], "]"}]}], ";"}]}], 
          "\[IndentingNewLine]", "]"}], ";"}]}], "\[IndentingNewLine]", "]"}],
       ";", "\[IndentingNewLine]", 
      RowBox[{"On", "[", 
       RowBox[{"General", "::", "unfl"}], "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"Chop", "[", 
       RowBox[{"s", ",", "$MachineEpsilon"}], "]"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.503864765345394*^9, 3.503864805342846*^9}, {
   3.50386734288988*^9, 3.503867354117487*^9}, {3.503927760898873*^9, 
   3.503927788744951*^9}, {3.503927896065843*^9, 3.503927898017088*^9}, {
   3.503927946305841*^9, 3.503927987465645*^9}, {3.503928052874394*^9, 
   3.503928083824047*^9}, {3.503928147881987*^9, 3.503928148234082*^9}, {
   3.503928256325563*^9, 3.503928300122334*^9}, {3.503928344716576*^9, 
   3.503928374380585*^9}, {3.50392844123773*^9, 3.503928460781448*^9}, 
   3.503928811980692*^9, {3.503929274135363*^9, 3.503929285314468*^9}, {
   3.503929315402057*^9, 3.503929319956047*^9}, {3.503929410566998*^9, 
   3.503929421191896*^9}, {3.503929577459712*^9, 3.503929582154504*^9}, {
   3.503929719644224*^9, 3.503929721700133*^9}, {3.503929866254088*^9, 
   3.503929911131696*^9}, {3.503930134680891*^9, 3.503930136866754*^9}, {
   3.503930189879069*^9, 3.50393023334754*^9}, {3.503930273900549*^9, 
   3.503930293646057*^9}, {3.503930944205041*^9, 3.503931066589187*^9}, {
   3.503931179513111*^9, 3.503931190473869*^9}, {3.503931261887096*^9, 
   3.503931274581136*^9}, {3.503931328583484*^9, 3.503931370004759*^9}, {
   3.503931407748326*^9, 3.503931407914923*^9}, {3.503933307782749*^9, 
   3.503933338694513*^9}, {3.503933407486908*^9, 3.503933415562467*^9}, {
   3.503937036128636*^9, 3.503937097560886*^9}, {3.503937144941339*^9, 
   3.5039372344975*^9}, {3.503942000708616*^9, 3.503942006005069*^9}, {
   3.504021886951573*^9, 3.504021908326655*^9}, 3.504022004000711*^9, {
   3.504026086567231*^9, 3.5040260878016148`*^9}, {3.50403068418861*^9, 
   3.504030757608275*^9}, {3.504030876667343*^9, 3.5040308791785603`*^9}, {
   3.504030997464014*^9, 3.504031027125536*^9}, 3.504038920402371*^9, 
   3.504209954034*^9, {3.504288326676*^9, 3.5042883269779997`*^9}, {
   3.504288441027*^9, 3.504288477814*^9}, {3.504289857642*^9, 
   3.504289858402*^9}, {3.5407716014814*^9, 3.5407716027114*^9}, 
   3.5407716386614*^9, {3.5407719168714*^9, 3.5407719180313997`*^9}, {
   3.540819210863092*^9, 3.540819242095166*^9}, {3.5408193660846148`*^9, 
   3.540819366450569*^9}, {3.5408193964676313`*^9, 3.5408194758801203`*^9}, {
   3.540819555536408*^9, 3.540819556390916*^9}, {3.5408196042185926`*^9, 
   3.540819612305895*^9}, {3.540819671349736*^9, 3.540819675616293*^9}, {
   3.540819729135867*^9, 3.540819757771803*^9}, {3.540819789000448*^9, 
   3.5408198461157084`*^9}, {3.540819888599984*^9, 3.540819890027162*^9}, {
   3.540820020626029*^9, 3.540820021597898*^9}, {3.540820098695648*^9, 
   3.5408201005555162`*^9}, {3.5408202408286858`*^9, 
   3.5408202687408543`*^9}, {3.540820321710685*^9, 3.540820328094551*^9}, {
   3.540820553869779*^9, 3.540820554671226*^9}, {3.540822490776183*^9, 
   3.540822493533906*^9}, 3.540822752068866*^9, 3.540824355901135*^9, {
   3.5408246853957*^9, 3.540824692701099*^9}, {3.540824833322504*^9, 
   3.540824834876959*^9}, 3.5408259865319223`*^9, 3.540826038947308*^9, {
   3.540826808489683*^9, 3.5408268708637342`*^9}, {3.540826915162272*^9, 
   3.5408269581927977`*^9}, {3.5408271078611097`*^9, 
   3.5408271324618607`*^9}, {3.54082718124622*^9, 3.540827188794232*^9}, {
   3.540827337957547*^9, 3.540827349109482*^9}, {3.540827412064855*^9, 
   3.5408274331891947`*^9}, {3.540827549372154*^9, 3.540827550711824*^9}, {
   3.540827628516138*^9, 3.5408276295311623`*^9}, {3.540827877893024*^9, 
   3.5408279490002537`*^9}, {3.5408279927693577`*^9, 
   3.5408280131059647`*^9}, {3.540828059326911*^9, 3.540828061706418*^9}, {
   3.540829392842729*^9, 3.540829401968701*^9}, {3.5408295466109047`*^9, 
   3.540829622330606*^9}, {3.540829953239996*^9, 3.540830067552009*^9}, {
   3.5408301538417063`*^9, 3.540830244152267*^9}, {3.540830321272368*^9, 
   3.5408303830032377`*^9}, {3.5408304162939*^9, 3.540830421493327*^9}, {
   3.540830456136663*^9, 3.5408304918362827`*^9}, {3.540830587455862*^9, 
   3.540830617324831*^9}, {3.540830691921343*^9, 3.540831056314954*^9}, {
   3.540831608134359*^9, 3.5408316306763678`*^9}, {3.5408316674284143`*^9, 
   3.540831669679337*^9}, {3.5408317019517603`*^9, 3.5408318074256*^9}, {
   3.540832026525626*^9, 3.540832035502232*^9}, {3.540832158612021*^9, 
   3.540832243751718*^9}, {3.540832501772859*^9, 3.540832506780499*^9}, {
   3.540833501624042*^9, 3.540833513093803*^9}, {3.5408335771994057`*^9, 
   3.5408336144965353`*^9}, {3.540833648919447*^9, 3.5408337244834538`*^9}, {
   3.540833784451839*^9, 3.54083379823851*^9}, {3.5409036826043997`*^9, 
   3.5409036950864*^9}, {3.5409394960243998`*^9, 3.5409394988104*^9}, {
   3.5409396370864*^9, 3.5409396403943996`*^9}, {3.5409401144063997`*^9, 
   3.5409401188244*^9}, {3.5409401593434*^9, 3.5409402168314*^9}, {
   3.5409412717883997`*^9, 3.5409412754384003`*^9}, {3.5409415184994*^9, 
   3.5409415298993998`*^9}, {3.5409415622944*^9, 3.5409415657074003`*^9}, {
   3.5409416069434*^9, 3.5409416072334003`*^9}, {3.5657105550022*^9, 
   3.5657105555122004`*^9}, {3.568126182652104*^9, 3.568126182853517*^9}, {
   3.666014040757195*^9, 3.666014090145177*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Region 0: Undeformed", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.538443201592167*^9, 3.538443212095222*^9}, {
   3.5656156095022*^9, 3.5656156113522*^9}, 3.5656208942292*^9}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"J", "[", "0", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"G", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"G", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}]}], "}"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "el"}], ",", "0"}], "}"}], "]"}], ",", 
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{"0", ",", "el"}], "}"}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"2", "*", "bigN"}], ",", 
       RowBox[{"2", "*", "bigN"}]}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Jsamp", "[", "0", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"Gsamp", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
       RowBox[{
        RowBox[{
         RowBox[{"Gsamp", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}]}], "}"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "el"}], ",", "0"}], "}"}], "]"}], ",", 
       RowBox[{"Line", "[", 
        RowBox[{"{", 
         RowBox[{"0", ",", "el"}], "}"}], "]"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"2", "*", "bigN"}], ",", 
       RowBox[{"2", "*", "bigN"}]}], "}"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Jadapt", "[", "0", "]"}], "[", 
   RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"Adapt", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Jsamp", "[", "0", "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", "globalTol"}], "]"}], "[", 
   RowBox[{
    RowBox[{"J", "[", "0", "]"}], "[", 
    RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5406576785462*^9, 3.5406577161661997`*^9}, {
   3.5406582201982*^9, 3.5406582273082*^9}, 3.5406586749382*^9, 
   3.5407609243294*^9, {3.5408422052614202`*^9, 3.540842205632593*^9}, {
   3.540842429716239*^9, 3.540842429846467*^9}, {3.5409332732904*^9, 
   3.5409333148454*^9}, {3.541861619785755*^9, 3.541861623452838*^9}, {
   3.541861656327001*^9, 3.541861658902041*^9}, {3.5423735388424*^9, 
   3.5423735413424*^9}, {3.5424025499314003`*^9, 3.5424025521113997`*^9}, {
   3.5656156244642*^9, 3.5656156262852*^9}, {3.5656183001152*^9, 
   3.5656183043241997`*^9}, {3.5656205355512*^9, 3.5656205799202003`*^9}, {
   3.5656206584592*^9, 3.5656206776342*^9}, {3.5656208891191998`*^9, 
   3.5656208914672003`*^9}, {3.5656305467581997`*^9, 3.5656305490722*^9}, {
   3.5657106489052*^9, 3.5657106655822*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Region 1: Small t", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.5656277702742*^9, 3.5656277744342003`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{"SmallTimeContour", "=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "el"}], " ", "+", " ", 
         RowBox[{"I", " ", "\[Nu]"}]}], ",", 
        RowBox[{
         RowBox[{"Sqrt", "[", "2", "]"}], "\[Nu]", " ", 
         RowBox[{"Exp", "[", 
          RowBox[{"I", " ", "3", " ", 
           RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], "}"}], "]"}], ",", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"Sqrt", "[", "2", "]"}], "\[Nu]", " ", 
         RowBox[{"Exp", "[", 
          RowBox[{"I", " ", "3", " ", 
           RowBox[{"Pi", "/", "4"}]}], "]"}]}], ",", "0"}], "}"}], "]"}], ",",
      "\[IndentingNewLine]", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"-", "el"}], ",", "0"}], "}"}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "el"}], " ", "-", " ", 
         RowBox[{"I", " ", "\[Nu]"}]}], ",", 
        RowBox[{
         RowBox[{"Sqrt", "[", "2", "]"}], "\[Nu]", " ", 
         RowBox[{"Exp", "[", 
          RowBox[{"I", " ", "5", " ", 
           RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], "}"}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"Sqrt", "[", "2", "]"}], "\[Nu]", " ", 
         RowBox[{"Exp", "[", 
          RowBox[{"I", " ", "5", " ", 
           RowBox[{"Pi", "/", "4"}]}], "]"}]}], ",", "0"}], "}"}], "]"}], ",",
      "\[IndentingNewLine]", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{"0", ",", 
        RowBox[{
         RowBox[{"Sqrt", "[", "2", "]"}], "\[Nu]", " ", 
         RowBox[{"Exp", "[", 
          RowBox[{"I", " ", 
           RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], "}"}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"Sqrt", "[", "2", "]"}], "\[Nu]", " ", 
         RowBox[{"Exp", "[", 
          RowBox[{"I", " ", 
           RowBox[{"Pi", "/", "4"}]}], "]"}]}], ",", 
        RowBox[{
         RowBox[{"I", " ", "\[Nu]"}], " ", "+", " ", "el"}]}], "}"}], "]"}], 
     ",", "\[IndentingNewLine]", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{"0", ",", 
        RowBox[{
         RowBox[{"Sqrt", "[", "2", "]"}], "\[Nu]", " ", 
         RowBox[{"Exp", "[", 
          RowBox[{
           RowBox[{"-", "I"}], " ", 
           RowBox[{"Pi", "/", "4"}]}], "]"}]}]}], "}"}], "]"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{"Line", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{
         RowBox[{"Sqrt", "[", "2", "]"}], "\[Nu]", " ", 
         RowBox[{"Exp", "[", 
          RowBox[{
           RowBox[{"-", "I"}], " ", 
           RowBox[{"Pi", "/", "4"}]}], "]"}]}], ",", 
        RowBox[{
         RowBox[{
          RowBox[{"-", "I"}], " ", "\[Nu]"}], " ", "+", " ", "el"}]}], "}"}], 
      "]"}]}], "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Modk0", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"t", " ", "<", " ", 
      RowBox[{"10", "*", "$MachineEpsilon"}]}], ",", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"Sign", "[", "x", "]"}]}], "*", "100"}], ",", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"Sign", "[", "x", "]"}]}], "*", 
      RowBox[{"Min", "[", 
       RowBox[{
        RowBox[{"Abs", "[", 
         RowBox[{"x", "/", 
          RowBox[{"(", 
           RowBox[{"4", "t"}], ")"}]}], "]"}], ",", "100"}], "]"}]}]}], 
    "]"}]}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5656280583422003`*^9, 3.5656280652251997`*^9}, {
   3.5656285659852*^9, 3.5656287457122*^9}, {3.5656292517061996`*^9, 
   3.5656292519751997`*^9}, {3.5656349460212*^9, 3.5656349839392*^9}, {
   3.5656350144561996`*^9, 3.5656350657841997`*^9}, {3.5656351399231997`*^9, 
   3.5656352555312*^9}, {3.5656353864282*^9, 3.5656353951102*^9}, 
   3.5656354339372*^9, {3.5657106309172*^9, 3.5657106321812*^9}}],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"J", "[", "1", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"Un", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"Un", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"DDn", "[", "#", "]"}], "&"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"Ln", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"Ln", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"Pn", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"Pn", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"Mn", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"Mn", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}]}], "}"}], 
       ",", " ", 
       RowBox[{"SmallTimeContour", " ", "+", " ", 
        RowBox[{"Modk0", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], ",", " ", 
       RowBox[{"{", 
        RowBox[{
        "bigN", ",", "smallN", ",", "bigN", ",", "bigN", ",", "smallN", ",", 
         "smallN", ",", "bigN", ",", "smallN", ",", "bigN"}], "}"}]}], 
      "}"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"Jsamp", "[", "1", "]"}], "[", 
      RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"Unsamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"Unsamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"DDnsamp", "[", "#", "]"}], "&"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"Lnsamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"Lnsamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"Pnsamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"Pnsamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"Mnsamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         RowBox[{
          RowBox[{
           RowBox[{"Mnsamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}]}], "}"}], 
       ",", " ", 
       RowBox[{"SmallTimeContour", " ", "+", " ", 
        RowBox[{"Modk0", "[", 
         RowBox[{"x", ",", "t"}], "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{
        "bigN", ",", "smallN", ",", "bigN", ",", "bigN", ",", "smallN", ",", 
         "smallN", ",", "bigN", ",", "smallN", ",", "bigN"}], "}"}]}], 
      "}"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"Jadapt", "[", "1", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{
     RowBox[{"Adapt", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Jsamp", "[", "1", "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], ",", "globalTol"}], "]"}], "[", 
     RowBox[{
      RowBox[{"J", "[", "1", "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}]}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5656277803342*^9, 3.5656277942142*^9}, {
   3.5656278543152*^9, 3.5656280347212*^9}, 3.5656285803831997`*^9, {
   3.5656287904302*^9, 3.5656289367002*^9}, {3.5656354431261997`*^9, 
   3.5656354619342003`*^9}, {3.5656355936562*^9, 3.5656356693892*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Region 2: Large t", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.5378236069191504`*^9, 3.537823619029093*^9}, {
  3.5656307009522*^9, 3.5656307035861998`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"DDI", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Im", "[", "k", "]"}], " ", "\[GreaterEqual]", "0"}], ",", 
     RowBox[{
      RowBox[{"DD", "[", 
       RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], ",", 
     RowBox[{"IdentityMatrix", "[", "2", "]"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"startift", "[", "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Clear", "[", "ift", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"ift", "[", "k0_", "]"}], ":=", 
       RowBox[{
        RowBox[{"ift", "[", "k0", "]"}], "=", 
        RowBox[{"Fun", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"Log", "[", 
            RowBox[{"\[Tau]", "[", "#", "]"}], "]"}], "&"}], ",", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"LeftEndpoint", "[", "dom", "]"}], ",", "k0"}], "}"}], "//",
            "Line"}], ",", "100"}], "]"}]}]}], ";"}]}], "\[IndentingNewLine]",
     "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"startift", "[", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Delta]t", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", 
    RowBox[{"s_", ",", "k_"}], "]"}], ":=", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Cauchy", "[", 
      RowBox[{"s", ",", 
       RowBox[{"ift", "[", 
        RowBox[{
         RowBox[{"-", "x"}], "/", 
         RowBox[{"(", 
          RowBox[{"4", " ", "t"}], ")"}]}], "]"}], ",", "k"}], "]"}], "//", 
     "Exp"}], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[Delta]t", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"Cauchy", "[", 
      RowBox[{
       RowBox[{"ift", "[", 
        RowBox[{
         RowBox[{"-", "x"}], "/", 
         RowBox[{"(", 
          RowBox[{"4", " ", "t"}], ")"}]}], "]"}], ",", "k"}], "]"}], "//", 
     "Exp"}], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalPhi]t", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "z__", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{
        RowBox[{"\[Delta]t", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "[", "z", "]"}], "0"},
      {"0", 
       FractionBox["1", 
        RowBox[{
         RowBox[{"\[Delta]t", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", "z", "]"}]]}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalPhi]tin", "[", "t__", "]"}], "[", "z__", "]"}], ":=", 
   RowBox[{"(", GridBox[{
      {
       RowBox[{"1", "/", 
        RowBox[{
         RowBox[{"\[Delta]t", "[", "t", "]"}], "[", "z", "]"}]}], "0"},
      {"0", 
       RowBox[{
        RowBox[{"\[Delta]t", "[", "t", "]"}], "[", "z", "]"}]}
     }], ")"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalPhi]ts", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "z_", "]"}], ":=", 
   RowBox[{
    RowBox[{"\[CapitalPhi]t", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"z", "/", 
      RowBox[{"Sqrt", "[", "t", "]"}]}], "-", 
     RowBox[{"x", "/", 
      RowBox[{"(", 
       RowBox[{"4", " ", "t"}], ")"}]}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"\[CapitalPhi]tins", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "z_", "]"}], ":=", 
   RowBox[{
    RowBox[{"\[CapitalPhi]tin", "[", 
     RowBox[{"x", ",", "t"}], "]"}], "[", 
    RowBox[{
     RowBox[{"z", "/", 
      RowBox[{"Sqrt", "[", "t", "]"}]}], "-", 
     RowBox[{"x", "/", 
      RowBox[{"(", 
       RowBox[{"4", " ", "t"}], ")"}]}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"MaxRads", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Min", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Sqrt", "[", "2", "]"}], "\[Nu]", " ", "*", 
        RowBox[{"Sqrt", "[", "t", "]"}]}], ",", "100"}], "]"}], ",", 
     RowBox[{"Min", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"Sqrt", "[", "2", "]"}], "\[Nu]", " ", "*", 
        RowBox[{
         RowBox[{"Sqrt", "[", "t", "]"}], "/", "2"}]}], ",", "1"}], "]"}]}], 
    "}"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"J", "[", "2", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Jumps", "=", 
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"\[CapitalPhi]ts", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"L", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"\[CapitalPhi]tins", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"\[CapitalPhi]ts", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"L", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"\[CapitalPhi]tins", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"\[CapitalPhi]ts", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"U", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"\[CapitalPhi]tins", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"\[CapitalPhi]ts", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"U", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"\[CapitalPhi]tins", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"\[CapitalPhi]ts", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"M", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"\[CapitalPhi]tins", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"\[CapitalPhi]ts", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"M", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"\[CapitalPhi]tins", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"\[CapitalPhi]ts", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"P", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"\[CapitalPhi]tins", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"\[CapitalPhi]ts", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"P", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"\[CapitalPhi]tins", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"DDI", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"\[CapitalPhi]tins", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"DD", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"U", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"\[CapitalPhi]tins", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Lin", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"M", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"\[CapitalPhi]tins", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Lin", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"\[CapitalPhi]tins", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}]}], 
        "\[IndentingNewLine]", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Domains", "=", 
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"-", "el"}], " ", 
              RowBox[{"Sqrt", "[", "t", "]"}]}], " ", "-", " ", 
             RowBox[{"I", " ", "\[Nu]", " ", 
              RowBox[{"Sqrt", "[", "t", "]"}]}]}], ",", 
            RowBox[{
             RowBox[{
              RowBox[{"MaxRads", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], 
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", "3"}], "I", " ", 
               RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}]}], "}"}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"MaxRads", "[", 
            RowBox[{"x", ",", "t"}], "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "3"}], "I", " ", 
             RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"-", "el"}], " ", 
              RowBox[{"Sqrt", "[", "t", "]"}]}], " ", "+", " ", 
             RowBox[{"I", " ", "\[Nu]", " ", 
              RowBox[{"Sqrt", "[", "t", "]"}]}]}], ",", 
            RowBox[{
             RowBox[{
              RowBox[{"MaxRads", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], 
             RowBox[{"Exp", "[", 
              RowBox[{"3", "I", " ", 
               RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}]}], "}"}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"MaxRads", "[", 
            RowBox[{"x", ",", "t"}], "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{"3", "I", " ", 
             RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"Reverse", "[", 
            RowBox[{"MaxRads", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", 
             RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"MaxRads", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], 
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", "I"}], " ", 
               RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}], ",", 
            RowBox[{
             RowBox[{"el", " ", 
              RowBox[{"Sqrt", "[", "t", "]"}]}], " ", "-", " ", 
             RowBox[{"I", " ", "\[Nu]", " ", 
              RowBox[{"Sqrt", "[", "t", "]"}]}]}]}], "}"}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"Reverse", "[", 
            RowBox[{"MaxRads", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", 
             RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"MaxRads", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], 
             RowBox[{"Exp", "[", 
              RowBox[{"I", " ", 
               RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}], ",", 
            RowBox[{
             RowBox[{"el", " ", 
              RowBox[{"Sqrt", "[", "t", "]"}]}], " ", "+", " ", 
             RowBox[{"I", " ", "\[Nu]", " ", 
              RowBox[{"Sqrt", "[", "t", "]"}]}]}]}], "}"}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", "3"}], "I", " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}], ",", 
             RowBox[{"Exp", "[", 
              RowBox[{"3", "I", " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}]}], "}"}], "*", 
           RowBox[{
            RowBox[{"MaxRads", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Exp", "[", 
              RowBox[{"3", "I", " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}], ",", 
             RowBox[{"Exp", "[", 
              RowBox[{"I", " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}]}], "}"}], "*", 
           RowBox[{
            RowBox[{"MaxRads", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Exp", "[", 
              RowBox[{"I", " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}], ",", 
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", " ", "I"}], " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}]}], "}"}], "*", 
           RowBox[{
            RowBox[{"MaxRads", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", "I"}], " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}], ",", 
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", "3"}], " ", "I", " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}]}], "}"}], "*", 
           RowBox[{
            RowBox[{"MaxRads", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
        "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"NumPts", "=", 
       RowBox[{"Table", "[", 
        RowBox[{"smallN", ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "1", ",", "12"}], "}"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"NumPts", "[", 
        RowBox[{"[", "1", "]"}], "]"}], "=", "bigN"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"NumPts", "[", 
        RowBox[{"[", "3", "]"}], "]"}], "=", "bigN"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"NumPts", "[", 
        RowBox[{"[", "6", "]"}], "]"}], "=", "bigN"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"NumPts", "[", 
        RowBox[{"[", "8", "]"}], "]"}], "=", "bigN"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Jsamp", "[", "2", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Jumps", "=", 
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"Lsamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"Lsamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"Usamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"Usamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"Msamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"Msamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"Psamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"Psamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"DDsamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"DDsamp", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"Usamp", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Linsamp", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"Msamp", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"Linsamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}]}], "}"}]}],
       ";", "\[IndentingNewLine]", 
      RowBox[{"Domains", "=", 
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"-", "el"}], " ", 
              RowBox[{"Sqrt", "[", "t", "]"}]}], " ", "-", " ", 
             RowBox[{"I", " ", "\[Nu]", " ", 
              RowBox[{"Sqrt", "[", "t", "]"}]}]}], ",", 
            RowBox[{
             RowBox[{
              RowBox[{"MaxRads", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], 
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", "3"}], "I", " ", 
               RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}]}], "}"}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"MaxRads", "[", 
            RowBox[{"x", ",", "t"}], "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "3"}], "I", " ", 
             RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"-", "el"}], " ", 
              RowBox[{"Sqrt", "[", "t", "]"}]}], " ", "+", " ", 
             RowBox[{"I", " ", "\[Nu]", " ", 
              RowBox[{"Sqrt", "[", "t", "]"}]}]}], ",", 
            RowBox[{
             RowBox[{
              RowBox[{"MaxRads", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], 
             RowBox[{"Exp", "[", 
              RowBox[{"3", "I", " ", 
               RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}]}], "}"}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"MaxRads", "[", 
            RowBox[{"x", ",", "t"}], "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{"3", "I", " ", 
             RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"Reverse", "[", 
            RowBox[{"MaxRads", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", 
             RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"MaxRads", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], 
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", "I"}], " ", 
               RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}], ",", 
            RowBox[{
             RowBox[{"el", " ", 
              RowBox[{"Sqrt", "[", "t", "]"}]}], " ", "-", " ", 
             RowBox[{"I", " ", "\[Nu]", " ", 
              RowBox[{"Sqrt", "[", "t", "]"}]}]}]}], "}"}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"Reverse", "[", 
            RowBox[{"MaxRads", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", 
             RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"MaxRads", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], 
             RowBox[{"Exp", "[", 
              RowBox[{"I", " ", 
               RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}], ",", 
            RowBox[{
             RowBox[{"el", " ", 
              RowBox[{"Sqrt", "[", "t", "]"}]}], " ", "+", " ", 
             RowBox[{"I", " ", "\[Nu]", " ", 
              RowBox[{"Sqrt", "[", "t", "]"}]}]}]}], "}"}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", "3"}], "I", " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}], ",", 
             RowBox[{"Exp", "[", 
              RowBox[{"3", "I", " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}]}], "}"}], "*", 
           RowBox[{
            RowBox[{"MaxRads", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Exp", "[", 
              RowBox[{"3", "I", " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}], ",", 
             RowBox[{"Exp", "[", 
              RowBox[{"I", " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}]}], "}"}], "*", 
           RowBox[{
            RowBox[{"MaxRads", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Exp", "[", 
              RowBox[{"I", " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}], ",", 
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", " ", "I"}], " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}]}], "}"}], "*", 
           RowBox[{
            RowBox[{"MaxRads", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", "I"}], " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}], ",", 
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", "3"}], " ", "I", " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}]}], "}"}], "*", 
           RowBox[{
            RowBox[{"MaxRads", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
        "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"NumPts", "=", 
       RowBox[{"Table", "[", 
        RowBox[{"smallN", ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "1", ",", "12"}], "}"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"NumPts", "[", 
        RowBox[{"[", "1", "]"}], "]"}], "=", "bigN"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"NumPts", "[", 
        RowBox[{"[", "3", "]"}], "]"}], "=", "bigN"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"NumPts", "[", 
        RowBox[{"[", "6", "]"}], "]"}], "=", "bigN"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"NumPts", "[", 
        RowBox[{"[", "8", "]"}], "]"}], "=", "bigN"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Jadapt", "[", "2", "]"}], "[", 
   RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"Adapt", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Jsamp", "[", "2", "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", "globalTol"}], "]"}], "[", 
   RowBox[{
    RowBox[{"J", "[", "2", "]"}], "[", 
    RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.537823686437846*^9, 3.537823759286895*^9}, 
   3.5407610818094*^9, {3.540842388424041*^9, 3.540842390117072*^9}, {
   3.541861872987379*^9, 3.5418618934527607`*^9}, {3.5656296622852*^9, 
   3.5656296673232*^9}, {3.5656299847592*^9, 3.5656302347792*^9}, {
   3.5656303393562*^9, 3.5656303396532*^9}, {3.5656303719302*^9, 
   3.5656304390572*^9}, {3.5656307288932*^9, 3.5656307325512*^9}, {
   3.5656308164342003`*^9, 3.5656308820222*^9}, {3.5656309179982*^9, 
   3.5656309243332*^9}, {3.5656310882972*^9, 3.5656312752002*^9}, {
   3.5656313309462*^9, 3.5656314091772003`*^9}, {3.5656316170811996`*^9, 
   3.5656316182111998`*^9}, {3.5656317798342*^9, 3.5656318154072*^9}, 
   3.5656318858171997`*^9, {3.5656372521222*^9, 3.5656372585842*^9}, {
   3.5656372950052*^9, 3.5656372959932003`*^9}, {3.5656398476602*^9, 
   3.5656398616612*^9}, {3.5656411402262*^9, 3.5656411477642*^9}, 
   3.5656412028792*^9, {3.5656415538431997`*^9, 3.5656416110102*^9}, {
   3.5656542369542*^9, 3.5656542447452*^9}, {3.5656584703272*^9, 
   3.5656584795872*^9}, {3.5656588940092*^9, 3.5656589494091997`*^9}, {
   3.5657100848472*^9, 3.5657101201442003`*^9}, {3.5657101507042*^9, 
   3.5657101995982*^9}, {3.5657528650822*^9, 3.5657528721812*^9}, {
   3.5657543897637043`*^9, 3.565754390366497*^9}, {3.5657607080606127`*^9, 
   3.565760756787821*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Region 3: Large t -- no scale", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.5378236069191504`*^9, 3.537823619029093*^9}, {
  3.5656307009522*^9, 3.5656307035861998`*^9}, {3.5656582268232*^9, 
  3.5656582317152*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"DDIn", "[", "k_", "]"}], ":=", 
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Im", "[", "k", "]"}], " ", "\[GreaterEqual]", "0"}], ",", 
     RowBox[{"DDn", "[", "k", "]"}], ",", 
     RowBox[{"IdentityMatrix", "[", "2", "]"}]}], "]"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{
   RowBox[{"MaxRadsn", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"Sqrt", "[", "2", "]"}], "\[Nu]"}], " ", ",", 
     RowBox[{
      RowBox[{"Sqrt", "[", "2", "]"}], 
      RowBox[{"\[Nu]", " ", "/", 
       RowBox[{"(", 
        RowBox[{"2", 
         RowBox[{"(", 
          RowBox[{"1", "+", 
           RowBox[{"Sqrt", "[", "t", "]"}]}], ")"}]}], ")"}]}]}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"J", "[", "3", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Jumps", "=", 
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"\[CapitalPhi]t", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"Ln", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"\[CapitalPhi]tin", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"\[CapitalPhi]t", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"Ln", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"\[CapitalPhi]tin", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"\[CapitalPhi]t", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"Un", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"\[CapitalPhi]tin", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"\[CapitalPhi]t", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"Un", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"\[CapitalPhi]tin", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"\[CapitalPhi]t", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"Mn", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"\[CapitalPhi]tin", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"\[CapitalPhi]t", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"Mn", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"\[CapitalPhi]tin", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"\[CapitalPhi]t", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"Pn", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"\[CapitalPhi]tin", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"\[CapitalPhi]t", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"Pn", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"\[CapitalPhi]tin", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"DDIn", "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"\[CapitalPhi]tin", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"DDn", "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"Un", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"\[CapitalPhi]tin", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Linn", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"Mn", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"\[CapitalPhi]tin", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Linn", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"\[CapitalPhi]tin", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}]}], 
        "\[IndentingNewLine]", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Domains", "=", 
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "el"}], "  ", "-", " ", 
             RowBox[{"I", " ", "\[Nu]"}]}], " ", ",", 
            RowBox[{
             RowBox[{
              RowBox[{"MaxRadsn", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], 
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", "3"}], "I", " ", 
               RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}]}], "}"}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"MaxRadsn", "[", 
            RowBox[{"x", ",", "t"}], "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "3"}], "I", " ", 
             RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "el"}], " ", "+", " ", 
             RowBox[{"I", " ", "\[Nu]"}]}], " ", ",", 
            RowBox[{
             RowBox[{
              RowBox[{"MaxRadsn", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], 
             RowBox[{"Exp", "[", 
              RowBox[{"3", "I", " ", 
               RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}]}], "}"}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"MaxRadsn", "[", 
            RowBox[{"x", ",", "t"}], "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{"3", "I", " ", 
             RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"Reverse", "[", 
            RowBox[{"MaxRadsn", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", 
             RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"MaxRadsn", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], 
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", "I"}], " ", 
               RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}], ",", 
            RowBox[{"el", " ", "-", " ", 
             RowBox[{"I", " ", "\[Nu]"}]}]}], " ", "}"}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"Reverse", "[", 
            RowBox[{"MaxRadsn", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", 
             RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"MaxRadsn", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], 
             RowBox[{"Exp", "[", 
              RowBox[{"I", " ", 
               RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}], ",", 
            RowBox[{"el", "  ", "+", " ", 
             RowBox[{"I", " ", "\[Nu]"}]}]}], " ", "}"}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", "3"}], "I", " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}], ",", 
             RowBox[{"Exp", "[", 
              RowBox[{"3", "I", " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}]}], "}"}], "*", 
           RowBox[{
            RowBox[{"MaxRadsn", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Exp", "[", 
              RowBox[{"3", "I", " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}], ",", 
             RowBox[{"Exp", "[", 
              RowBox[{"I", " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}]}], "}"}], "*", 
           RowBox[{
            RowBox[{"MaxRadsn", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Exp", "[", 
              RowBox[{"I", " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}], ",", 
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", " ", "I"}], " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}]}], "}"}], "*", 
           RowBox[{
            RowBox[{"MaxRadsn", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", "I"}], " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}], ",", 
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", "3"}], " ", "I", " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}]}], "}"}], "*", 
           RowBox[{
            RowBox[{"MaxRadsn", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
        "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Domains", "=", 
       RowBox[{"Domains", "-", 
        RowBox[{"x", "/", 
         RowBox[{"(", 
          RowBox[{"4", "t"}], ")"}]}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"NumPts", "=", 
       RowBox[{"Table", "[", 
        RowBox[{"smallN", ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "1", ",", "12"}], "}"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"NumPts", "[", 
        RowBox[{"[", "1", "]"}], "]"}], "=", "bigN"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"NumPts", "[", 
        RowBox[{"[", "3", "]"}], "]"}], "=", "bigN"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"NumPts", "[", 
        RowBox[{"[", "6", "]"}], "]"}], "=", "bigN"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"NumPts", "[", 
        RowBox[{"[", "8", "]"}], "]"}], "=", "bigN"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Jsamp", "[", "3", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Jumps", "=", 
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"Lnsamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"Lnsamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"Unsamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"Unsamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"Mnsamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"Mnsamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"Pnsamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"Pnsamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"Lnsamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Lnsamp", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"Unsamp", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{
            RowBox[{"Lnsamp", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], ".", 
           RowBox[{
            RowBox[{"Mnsamp", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}]}], "&"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{
           RowBox[{"Lnsamp", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "#", "]"}], "&"}]}], 
        "\[IndentingNewLine]", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Domains", "=", 
       RowBox[{"{", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "el"}], "  ", "-", " ", 
             RowBox[{"I", " ", "\[Nu]"}]}], " ", ",", 
            RowBox[{
             RowBox[{
              RowBox[{"MaxRadsn", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], 
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", "3"}], "I", " ", 
               RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}]}], "}"}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"MaxRadsn", "[", 
            RowBox[{"x", ",", "t"}], "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "3"}], "I", " ", 
             RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{"-", "el"}], " ", "+", " ", 
             RowBox[{"I", " ", "\[Nu]"}]}], " ", ",", 
            RowBox[{
             RowBox[{
              RowBox[{"MaxRadsn", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], 
             RowBox[{"Exp", "[", 
              RowBox[{"3", "I", " ", 
               RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}]}], "}"}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"MaxRadsn", "[", 
            RowBox[{"x", ",", "t"}], "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{"3", "I", " ", 
             RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"Reverse", "[", 
            RowBox[{"MaxRadsn", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "I"}], " ", 
             RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"MaxRadsn", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], 
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", "I"}], " ", 
               RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}], ",", 
            RowBox[{"el", " ", "-", " ", 
             RowBox[{"I", " ", "\[Nu]"}]}]}], " ", "}"}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"Reverse", "[", 
            RowBox[{"MaxRadsn", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "]"}], 
           RowBox[{"Exp", "[", 
            RowBox[{"I", " ", 
             RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"MaxRadsn", "[", 
               RowBox[{"x", ",", "t"}], "]"}], "[", 
              RowBox[{"[", "1", "]"}], "]"}], 
             RowBox[{"Exp", "[", 
              RowBox[{"I", " ", 
               RowBox[{"\[Pi]", "/", "4."}]}], "]"}]}], ",", 
            RowBox[{"el", "  ", "+", " ", 
             RowBox[{"I", " ", "\[Nu]"}]}]}], " ", "}"}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", "3"}], "I", " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}], ",", 
             RowBox[{"Exp", "[", 
              RowBox[{"3", "I", " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}]}], "}"}], "*", 
           RowBox[{
            RowBox[{"MaxRadsn", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Exp", "[", 
              RowBox[{"3", "I", " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}], ",", 
             RowBox[{"Exp", "[", 
              RowBox[{"I", " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}]}], "}"}], "*", 
           RowBox[{
            RowBox[{"MaxRadsn", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Exp", "[", 
              RowBox[{"I", " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}], ",", 
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", " ", "I"}], " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}]}], "}"}], "*", 
           RowBox[{
            RowBox[{"MaxRadsn", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], "]"}], ",", 
         "\[IndentingNewLine]", 
         RowBox[{"Line", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", "I"}], " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}], ",", 
             RowBox[{"Exp", "[", 
              RowBox[{
               RowBox[{"-", "3"}], " ", "I", " ", 
               RowBox[{"\[Pi]", "/", "4"}]}], "]"}]}], "}"}], "*", 
           RowBox[{
            RowBox[{"MaxRadsn", "[", 
             RowBox[{"x", ",", "t"}], "]"}], "[", 
            RowBox[{"[", "2", "]"}], "]"}]}], "]"}]}], "\[IndentingNewLine]", 
        "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Domains", "=", 
       RowBox[{"Domains", "-", 
        RowBox[{"x", "/", 
         RowBox[{"(", 
          RowBox[{"4", "t"}], ")"}]}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"NumPts", "=", 
       RowBox[{"Table", "[", 
        RowBox[{"smallN", ",", 
         RowBox[{"{", 
          RowBox[{"i", ",", "1", ",", "12"}], "}"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"NumPts", "[", 
        RowBox[{"[", "1", "]"}], "]"}], "=", "bigN"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"NumPts", "[", 
        RowBox[{"[", "3", "]"}], "]"}], "=", "bigN"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"NumPts", "[", 
        RowBox[{"[", "6", "]"}], "]"}], "=", "bigN"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"NumPts", "[", 
        RowBox[{"[", "8", "]"}], "]"}], "=", "bigN"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"Jumps", ",", "Domains", ",", "NumPts"}], "}"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"Jadapt", "[", "3", "]"}], "[", 
   RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{"Adapt", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Jsamp", "[", "3", "]"}], "[", 
      RowBox[{"x", ",", "t"}], "]"}], ",", "globalTol"}], "]"}], "[", 
   RowBox[{
    RowBox[{"J", "[", "3", "]"}], "[", 
    RowBox[{"x", ",", "t"}], "]"}], "]"}]}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.537823686437846*^9, 3.537823759286895*^9}, 
   3.5407610818094*^9, {3.540842388424041*^9, 3.540842390117072*^9}, {
   3.541861872987379*^9, 3.5418618934527607`*^9}, {3.5656296622852*^9, 
   3.5656296673232*^9}, {3.5656299847592*^9, 3.5656302347792*^9}, {
   3.5656303393562*^9, 3.5656303396532*^9}, {3.5656303719302*^9, 
   3.5656304390572*^9}, {3.5656307288932*^9, 3.5656307325512*^9}, {
   3.5656308164342003`*^9, 3.5656308820222*^9}, {3.5656309179982*^9, 
   3.5656309243332*^9}, {3.5656310882972*^9, 3.5656312752002*^9}, {
   3.5656313309462*^9, 3.5656314091772003`*^9}, {3.5656316170811996`*^9, 
   3.5656316182111998`*^9}, {3.5656317798342*^9, 3.5656318154072*^9}, 
   3.5656318858171997`*^9, {3.5656372521222*^9, 3.5656372585842*^9}, {
   3.5656372950052*^9, 3.5656372959932003`*^9}, {3.5656398476602*^9, 
   3.5656398616612*^9}, {3.5656411402262*^9, 3.5656411477642*^9}, 
   3.5656412028792*^9, {3.5656415538431997`*^9, 3.5656416110102*^9}, {
   3.5656542369542*^9, 3.5656542447452*^9}, {3.5656584154161997`*^9, 
   3.5656584404812*^9}, {3.5656585558582*^9, 3.5656585897382*^9}, {
   3.5656587338582*^9, 3.5656587691882*^9}, {3.5656589649392*^9, 
   3.5656590056191998`*^9}, {3.5656590380192003`*^9, 3.5656591716692*^9}, {
   3.5656593925052*^9, 3.5656594057651997`*^9}, {3.5656594795271997`*^9, 
   3.5656594797872*^9}, {3.5656604244112*^9, 3.5656604637712*^9}, {
   3.5656606509692*^9, 3.5656606939012003`*^9}, {3.5656609436672*^9, 
   3.5656609557372*^9}, {3.5657102291372004`*^9, 3.5657102434161997`*^9}, {
   3.5657117928272*^9, 3.5657117994252*^9}, {3.5657520745312*^9, 
   3.5657520807662*^9}, {3.5657527798691998`*^9, 3.5657527829602003`*^9}, {
   3.5657528163462*^9, 3.5657528176132*^9}, {3.5657531574052*^9, 
   3.5657531787082*^9}}]
}, Closed]],

Cell[CellGroupData[{

Cell["Soliton Contours (Use PFuns later)", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.540663327743*^9, 3.5406633359230003`*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"InvScale", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"Sqrt", "[", "t", "]"}], 
    RowBox[{"(", 
     RowBox[{"k", "+", 
      RowBox[{"x", "/", 
       RowBox[{"(", 
        RowBox[{"4", "t"}], ")"}]}]}], ")"}]}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Scaling", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], "[", "k_", "]"}], ":=", 
   RowBox[{
    RowBox[{"k", "/", 
     RowBox[{"Sqrt", "[", "t", "]"}]}], " ", "-", 
    RowBox[{"x", "/", 
     RowBox[{"(", 
      RowBox[{"4", "t"}], ")"}]}]}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Tn", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_", ",", "a_", ",", "c_"}], "]"}], "[", "z_", "]"}],
    ":=", 
   RowBox[{"(", GridBox[{
      {"1", "0"},
      {
       RowBox[{
        RowBox[{"-", "c"}], " ", 
        RowBox[{
         RowBox[{"Exp", "[", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}], "]"}], "/", 
         RowBox[{"(", 
          RowBox[{"z", "-", "a"}], ")"}]}]}], "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Tn", "[", 
      RowBox[{"-", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_", ",", "a_", ",", "c_"}], "]"}], "[", "z_", "]"}],
    ":=", 
   RowBox[{"(", GridBox[{
      {"1", 
       RowBox[{"\[Lambda]", " ", 
        RowBox[{"cc", "[", 
         RowBox[{
          RowBox[{"-", "c"}], " ", 
          RowBox[{
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"\[Theta]", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}], "]"}], "/", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"cc", "[", "z", "]"}], "-", "a"}], ")"}]}]}], "]"}]}]},
      {
       RowBox[{" ", "0"}], "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Sn", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_", ",", "a_", ",", "c_"}], "]"}], "[", "z_", "]"}],
    ":=", 
   RowBox[{"(", GridBox[{
      {"1", 
       RowBox[{
        FractionBox[
         RowBox[{"-", 
          RowBox[{"(", 
           RowBox[{"z", "-", "a"}], ")"}]}], "c"], "*", 
        RowBox[{"Exp", "[", 
         RowBox[{"-", 
          RowBox[{
           RowBox[{"\[Theta]", "[", 
            RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}]}], "]"}]}]},
      {"0", "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Sn", "[", 
      RowBox[{"-", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_", ",", "a_", ",", "c_"}], "]"}], "[", "z_", "]"}],
    ":=", 
   RowBox[{"(", GridBox[{
      {"1", "0"},
      {
       RowBox[{"\[Lambda]", " ", 
        RowBox[{"cc", "[", 
         RowBox[{
          FractionBox[
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"cc", "[", "z", "]"}], "-", "a"}], ")"}]}], "c"], "*", 
          RowBox[{"Exp", "[", 
           RowBox[{"-", 
            RowBox[{
             RowBox[{"\[Theta]", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}]}], "]"}]}], 
         "]"}]}], "1"}
     }], ")"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"ContourArrayn", "[", 
    RowBox[{
     RowBox[{"+", "1"}], ",", "r_", ",", "a_"}], "]"}], ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Arc", "[", 
      RowBox[{"a", ",", "r", ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "Pi"}], "}"}]}], "]"}], ",", 
     RowBox[{"Arc", "[", 
      RowBox[{"a", ",", "r", ",", 
       RowBox[{"{", 
        RowBox[{"Pi", ",", 
         RowBox[{"2", "Pi"}]}], "}"}]}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ContourArrayn", "[", 
     RowBox[{
      RowBox[{"-", "1"}], ",", "r_", ",", "a_"}], "]"}], ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Arc", "[", 
       RowBox[{
        RowBox[{"a", "//", "cc"}], ",", "r", ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", 
          RowBox[{"-", "Pi"}]}], "}"}]}], "]"}], ",", 
      RowBox[{"Arc", "[", 
       RowBox[{
        RowBox[{"a", "//", "cc"}], ",", "r", ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "Pi"}], ",", 
          RowBox[{
           RowBox[{"-", "2"}], "Pi"}]}], "}"}]}], "]"}]}], "}"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"T", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_", ",", "a_", ",", "c_"}], "]"}], "[", "z_", "]"}],
    ":=", 
   RowBox[{
    RowBox[{
     RowBox[{"Tn", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"x", ",", "t", ",", "a", ",", "c"}], "]"}], "[", 
    RowBox[{
     RowBox[{"Scaling", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "[", "z", "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"T", "[", 
      RowBox[{"-", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_", ",", "a_", ",", "c_"}], "]"}], "[", "z_", "]"}],
    ":=", 
   RowBox[{
    RowBox[{
     RowBox[{"Tn", "[", 
      RowBox[{"-", "1"}], "]"}], "[", 
     RowBox[{"x", ",", "t", ",", "a", ",", "c"}], "]"}], "[", 
    RowBox[{
     RowBox[{"Scaling", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "[", "z", "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"S", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_", ",", "a_", ",", "c_"}], "]"}], "[", "z_", "]"}],
    ":=", 
   RowBox[{
    RowBox[{
     RowBox[{"Sn", "[", 
      RowBox[{"+", "1"}], "]"}], "[", 
     RowBox[{"x", ",", "t", ",", "a", ",", "c"}], "]"}], "[", 
    RowBox[{
     RowBox[{"Scaling", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "[", "z", "]"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"S", "[", 
      RowBox[{"-", "1"}], "]"}], "[", 
     RowBox[{"x_", ",", "t_", ",", "a_", ",", "c_"}], "]"}], "[", "z_", "]"}],
    ":=", 
   RowBox[{
    RowBox[{
     RowBox[{"Sn", "[", 
      RowBox[{"-", "1"}], "]"}], "[", 
     RowBox[{"x", ",", "t", ",", "a", ",", "c"}], "]"}], "[", 
    RowBox[{
     RowBox[{"Scaling", "[", 
      RowBox[{"x", ",", "t"}], "]"}], "[", "z", "]"}], "]"}]}], 
  ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.540663356863*^9, 3.5406634779230003`*^9}, {
   3.540663514553*^9, 3.540663559523*^9}, {3.5406636903546*^9, 
   3.5406638634946003`*^9}, {3.5406639640346003`*^9, 3.5406640204646*^9}, {
   3.5406641670945997`*^9, 3.5406641983646*^9}, {3.5406642496246*^9, 
   3.5406643193246*^9}, {3.5406643716646*^9, 3.5406643812946*^9}, {
   3.5406644820945997`*^9, 3.5406645177646*^9}, 3.5406645531018*^9, {
   3.5406646915018*^9, 3.5406648196538*^9}, {3.540668185589525*^9, 
   3.540668263873556*^9}, {3.54066831648492*^9, 3.5406684168554087`*^9}, {
   3.54066844904218*^9, 3.540668517764138*^9}, {3.540668618759264*^9, 
   3.540668717261815*^9}, {3.540669354468211*^9, 3.540669359280778*^9}, {
   3.540669502128993*^9, 3.5406695823656063`*^9}, {3.54066961440208*^9, 
   3.540669662983015*^9}, {3.54067189344142*^9, 3.540671925802581*^9}, {
   3.5406731652249823`*^9, 3.5406732638866568`*^9}, {3.5406741647813997`*^9, 
   3.5406741678613997`*^9}, {3.5406741991954*^9, 3.5406742011074*^9}, {
   3.5406826199754*^9, 3.5406826249944*^9}, {3.5406834491594*^9, 
   3.5406834534044*^9}, {3.5406835138184*^9, 3.5406835198314*^9}, {
   3.5406839573473997`*^9, 3.5406839578014*^9}, {3.5406875816144*^9, 
   3.5406875942444*^9}, {3.5406893661443996`*^9, 3.5406893835144*^9}, {
   3.5406894255744*^9, 3.5406894318104*^9}, {3.5407264300074*^9, 
   3.5407264358624*^9}, {3.540735266297641*^9, 3.54073530536005*^9}, {
   3.540735803202181*^9, 3.540735809259849*^9}, {3.540736099083493*^9, 
   3.540736100324586*^9}, {3.540737377234128*^9, 3.540737392702014*^9}, {
   3.540738683107176*^9, 3.540738743177516*^9}, {3.540738779460793*^9, 
   3.540738786570944*^9}, {3.540740941006283*^9, 3.540740955773658*^9}, {
   3.5407412010663157`*^9, 3.540741317799933*^9}, {3.5407610939393997`*^9, 
   3.5407612605094*^9}, {3.5407620108694*^9, 3.5407620122693996`*^9}, {
   3.5407621572894*^9, 3.5407621698494*^9}, {3.5407622649393997`*^9, 
   3.5407622724894*^9}, 3.5407625509994*^9, {3.5407640913234*^9, 
   3.5407640999234*^9}, {3.5407690321454*^9, 3.5407690325254*^9}, {
   3.5409028140464*^9, 3.5409028143864*^9}, {3.5656323663862*^9, 
   3.5656324283422003`*^9}, {3.5656324906851997`*^9, 
   3.5656325725522003`*^9}, {3.5656573568952*^9, 3.5656574370852003`*^9}, {
   3.5657050454421997`*^9, 3.5657051195582*^9}, {3.5657052135692*^9, 
   3.5657052213722*^9}, {3.5657053117812*^9, 3.5657053210292*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"ContourArray", "[", 
    RowBox[{
     RowBox[{"+", "1"}], ",", "r_", ",", "a_", ",", "x_", ",", "t_"}], "]"}], 
   ":=", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"Arc", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"InvScale", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}], ",", 
       RowBox[{"r", "*", 
        RowBox[{"Sqrt", "[", "t", "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"0", ",", "Pi"}], "}"}]}], "]"}], ",", 
     RowBox[{"Arc", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"InvScale", "[", 
         RowBox[{"x", ",", "t"}], "]"}], "[", "a", "]"}], ",", 
       RowBox[{"r", "*", 
        RowBox[{"Sqrt", "[", "t", "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"Pi", ",", 
         RowBox[{"2", "Pi"}]}], "}"}]}], "]"}]}], "}"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ContourArray", "[", 
     RowBox[{
      RowBox[{"-", "1"}], ",", "r_", ",", "a_", ",", "x_", ",", "t_"}], "]"}],
     ":=", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"Arc", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"InvScale", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", 
         RowBox[{"a", "//", "cc"}], "]"}], ",", 
        RowBox[{"r", "*", 
         RowBox[{"Sqrt", "[", "t", "]"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", 
          RowBox[{"-", "Pi"}]}], "}"}]}], "]"}], ",", 
      RowBox[{"Arc", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"InvScale", "[", 
          RowBox[{"x", ",", "t"}], "]"}], "[", 
         RowBox[{"a", "//", "cc"}], "]"}], ",", 
        RowBox[{"r", "*", 
         RowBox[{"Sqrt", "[", "t", "]"}]}], ",", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"-", "Pi"}], ",", 
          RowBox[{
           RowBox[{"-", "2"}], "Pi"}]}], "}"}]}], "]"}]}], "}"}]}], ";"}], 
  "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"PoleListn", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"out", ",", "i", ",", "Domain", ",", "Jumps", ",", "Points"}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Domain", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Jumps", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Points", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"i", "=", "1"}], ",", 
        RowBox[{"i", "\[LessEqual]", 
         RowBox[{"Length", "[", "a", "]"}]}], ",", 
        RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"1", "/", "globalTol"}], ">", 
             RowBox[{
              RowBox[{"Abs", "[", 
               RowBox[{"c", "[", 
                RowBox[{"[", "i", "]"}], "]"}], "]"}], 
              RowBox[{"Exp", "[", 
               RowBox[{"Re", "[", 
                RowBox[{
                 RowBox[{"\[Theta]", "[", 
                  RowBox[{"x", ",", "t"}], "]"}], "[", 
                 RowBox[{"a", "[", 
                  RowBox[{"[", "i", "]"}], "]"}], "]"}], "]"}], "]"}]}]}], "&&", 
            RowBox[{
             RowBox[{
              RowBox[{"Abs", "[", 
               RowBox[{"c", "[", 
                RowBox[{"[", "i", "]"}], "]"}], "]"}], 
              RowBox[{"Exp", "[", 
               RowBox[{"Re", "[", 
                RowBox[{
                 RowBox[{"\[Theta]", "[", 
                  RowBox[{"x", ",", "t"}], "]"}], "[", 
                 RowBox[{"a", "[", 
                  RowBox[{"[", "i", "]"}], "]"}], "]"}], "]"}], "]"}]}], ">", 
             " ", "globalTol"}]}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{"Abs", "[", 
                 RowBox[{"c", "[", 
                  RowBox[{"[", "i", "]"}], "]"}], "]"}], 
                RowBox[{"Exp", "[", 
                 RowBox[{"Re", "[", 
                  RowBox[{
                   RowBox[{"\[Theta]", "[", 
                    RowBox[{"x", ",", "t"}], "]"}], "[", 
                   RowBox[{"a", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], "]"}], "]"}], "]"}]}], 
               ">", "expconst"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"Jumps", "=", 
                RowBox[{"Join", "[", 
                 RowBox[{"Jumps", ",", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"Sn", "[", 
                    RowBox[{"+", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
                    RowBox[{
                    RowBox[{"Sn", "[", 
                    RowBox[{"+", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], "}"}]}], 
                 "]"}]}], ";", "\[IndentingNewLine]", 
               RowBox[{"Jumps", "=", 
                RowBox[{"Join", "[", 
                 RowBox[{"Jumps", ",", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"Sn", "[", 
                    RowBox[{"-", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
                    RowBox[{
                    RowBox[{"Sn", "[", 
                    RowBox[{"-", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], "}"}]}], 
                 "]"}]}], ";", "\[IndentingNewLine]", 
               RowBox[{"flag", "=", "\"\<pos\>\""}], ";"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"Jumps", " ", "=", " ", 
                RowBox[{"Join", "[", 
                 RowBox[{"Jumps", ",", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"Tn", "[", 
                    RowBox[{"+", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
                    RowBox[{
                    RowBox[{"Tn", "[", 
                    RowBox[{"+", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], "}"}]}], 
                 "]"}]}], ";", "\[IndentingNewLine]", 
               RowBox[{"Jumps", " ", "=", " ", 
                RowBox[{"Join", "[", 
                 RowBox[{"Jumps", ",", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"Tn", "[", 
                    RowBox[{"-", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
                    RowBox[{
                    RowBox[{"Tn", "[", 
                    RowBox[{"-", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], "}"}]}], 
                 "]"}]}], ";", "\[IndentingNewLine]", 
               RowBox[{"flag", "=", "\"\<neg\>\""}], ";"}]}], 
             "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
            RowBox[{"Domain", "=", " ", 
             RowBox[{"Join", "[", 
              RowBox[{"Domain", ",", 
               RowBox[{"Join", "[", 
                RowBox[{
                 RowBox[{"ContourArrayn", "[", 
                  RowBox[{
                   RowBox[{"+", "1"}], ",", "rad", ",", 
                   RowBox[{"a", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
                 RowBox[{"ContourArrayn", "[", 
                  RowBox[{
                   RowBox[{"-", "1"}], ",", "rad", ",", 
                   RowBox[{"a", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], "]"}]}], 
              "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"Points", "=", 
             RowBox[{"Join", "[", 
              RowBox[{"Points", ",", 
               RowBox[{"{", 
                RowBox[{
                "smallN", ",", "smallN", ",", "smallN", ",", "smallN"}], 
                "}"}]}], "]"}]}], ";"}]}], "\[IndentingNewLine]", "]"}], 
         ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"Jumps", ",", "Domain", ",", "Points"}], "}"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"PoleList", "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"out", ",", "i", ",", "Domain", ",", "Jumps", ",", "Points"}], 
      "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Domain", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Jumps", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Points", "=", 
       RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"i", "=", "1"}], ",", 
        RowBox[{"i", "\[LessEqual]", 
         RowBox[{"Length", "[", "a", "]"}]}], ",", 
        RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{"1", "/", "tol"}], " ", ">", 
            RowBox[{
             RowBox[{"Abs", "[", 
              RowBox[{"c", "[", 
               RowBox[{"[", "i", "]"}], "]"}], "]"}], 
             RowBox[{"Exp", "[", 
              RowBox[{"Re", "[", 
               RowBox[{
                RowBox[{"\[Theta]", "[", 
                 RowBox[{"x", ",", "t"}], "]"}], "[", 
                RowBox[{"a", "[", 
                 RowBox[{"[", "i", "]"}], "]"}], "]"}], "]"}], "]"}]}], ">", 
            " ", "tol"}], ",", "\[IndentingNewLine]", 
           RowBox[{
            RowBox[{"If", "[", 
             RowBox[{
              RowBox[{
               RowBox[{
                RowBox[{"Abs", "[", 
                 RowBox[{"c", "[", 
                  RowBox[{"[", "i", "]"}], "]"}], "]"}], 
                RowBox[{"Exp", "[", 
                 RowBox[{"Re", "[", 
                  RowBox[{
                   RowBox[{"\[Theta]", "[", 
                    RowBox[{"x", ",", "t"}], "]"}], "[", 
                   RowBox[{"a", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], "]"}], "]"}], "]"}]}], 
               ">", "expconst"}], ",", "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"Jumps", "=", 
                RowBox[{"Join", "[", 
                 RowBox[{"Jumps", ",", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"S", "[", 
                    RowBox[{"+", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
                    RowBox[{
                    RowBox[{"Sn", "[", 
                    RowBox[{"+", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], "}"}]}], 
                 "]"}]}], ";", "\[IndentingNewLine]", 
               RowBox[{"Jumps", "=", 
                RowBox[{"Join", "[", 
                 RowBox[{"Jumps", ",", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"S", "[", 
                    RowBox[{"-", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
                    RowBox[{
                    RowBox[{"Sn", "[", 
                    RowBox[{"-", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], "}"}]}], 
                 "]"}]}], ";", "\[IndentingNewLine]", 
               RowBox[{"flag", "=", "\"\<pos\>\""}], ";"}], ",", 
              "\[IndentingNewLine]", 
              RowBox[{
               RowBox[{"Jumps", " ", "=", " ", 
                RowBox[{"Join", "[", 
                 RowBox[{"Jumps", ",", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"T", "[", 
                    RowBox[{"+", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
                    RowBox[{
                    RowBox[{"Tn", "[", 
                    RowBox[{"+", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], "}"}]}], 
                 "]"}]}], ";", "\[IndentingNewLine]", 
               RowBox[{"Jumps", " ", "=", " ", 
                RowBox[{"Join", "[", 
                 RowBox[{"Jumps", ",", 
                  RowBox[{"{", 
                   RowBox[{
                    RowBox[{
                    RowBox[{"T", "[", 
                    RowBox[{"-", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
                    RowBox[{
                    RowBox[{"Tn", "[", 
                    RowBox[{"-", "1"}], "]"}], "[", 
                    RowBox[{"x", ",", "t", ",", 
                    RowBox[{"a", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", 
                    RowBox[{"c", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], "}"}]}], 
                 "]"}]}], ";", "\[IndentingNewLine]", 
               RowBox[{"flag", "=", "\"\<neg\>\""}], ";"}]}], 
             "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
            RowBox[{"Domain", "=", " ", 
             RowBox[{"Join", "[", 
              RowBox[{"Domain", ",", 
               RowBox[{"Join", "[", 
                RowBox[{
                 RowBox[{"ContourArray", "[", 
                  RowBox[{
                   RowBox[{"+", "1"}], ",", "rad", ",", 
                   RowBox[{"a", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
                 RowBox[{"ContourArray", "[", 
                  RowBox[{
                   RowBox[{"-", "1"}], ",", "rad", ",", 
                   RowBox[{"a", "[", 
                    RowBox[{"[", "i", "]"}], "]"}]}], "]"}]}], "]"}]}], 
              "]"}]}], ";", "\[IndentingNewLine]", 
            RowBox[{"Points", "=", 
             RowBox[{"Join", "[", 
              RowBox[{"Points", ",", 
               RowBox[{"{", 
                RowBox[{
                "smallN", ",", "smallN", ",", "smallN", ",", "smallN"}], 
                "}"}]}], "]"}]}], ";"}]}], "\[IndentingNewLine]", "]"}], 
         ";"}]}], "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{"Jumps", ",", "Domain", ",", "Points"}], "}"}]}]}], 
    "\[IndentingNewLine]", "]"}]}], ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.540663356863*^9, 3.5406634779230003`*^9}, {
   3.540663514553*^9, 3.540663559523*^9}, {3.5406636903546*^9, 
   3.5406638634946003`*^9}, {3.5406639640346003`*^9, 3.5406640204646*^9}, {
   3.5406641670945997`*^9, 3.5406641983646*^9}, {3.5406642496246*^9, 
   3.5406643193246*^9}, {3.5406643716646*^9, 3.5406643812946*^9}, {
   3.5406644820945997`*^9, 3.5406645177646*^9}, 3.5406645531018*^9, {
   3.5406646915018*^9, 3.5406648196538*^9}, {3.540668185589525*^9, 
   3.540668263873556*^9}, {3.54066831648492*^9, 3.5406684168554087`*^9}, {
   3.54066844904218*^9, 3.540668517764138*^9}, {3.540668618759264*^9, 
   3.540668717261815*^9}, {3.540669354468211*^9, 3.540669359280778*^9}, {
   3.540669502128993*^9, 3.5406695823656063`*^9}, {3.54066961440208*^9, 
   3.540669662983015*^9}, {3.54067189344142*^9, 3.540671925802581*^9}, {
   3.5406731652249823`*^9, 3.5406732638866568`*^9}, {3.5406741647813997`*^9, 
   3.5406741678613997`*^9}, {3.5406741991954*^9, 3.5406742011074*^9}, {
   3.5406826199754*^9, 3.5406826249944*^9}, {3.5406834491594*^9, 
   3.5406834534044*^9}, {3.5406835138184*^9, 3.5406835198314*^9}, {
   3.5406839573473997`*^9, 3.5406839578014*^9}, {3.5406875816144*^9, 
   3.5406875942444*^9}, {3.5406893661443996`*^9, 3.5406893835144*^9}, {
   3.5406894255744*^9, 3.5406894318104*^9}, {3.5407264300074*^9, 
   3.5407264358624*^9}, {3.540735266297641*^9, 3.54073530536005*^9}, {
   3.540735803202181*^9, 3.540735809259849*^9}, {3.540736099083493*^9, 
   3.540736100324586*^9}, {3.540737377234128*^9, 3.540737392702014*^9}, {
   3.540738683107176*^9, 3.540738743177516*^9}, {3.540738779460793*^9, 
   3.540738786570944*^9}, {3.540740941006283*^9, 3.540740955773658*^9}, {
   3.5407412010663157`*^9, 3.540741317799933*^9}, {3.5407610939393997`*^9, 
   3.5407612605094*^9}, {3.5407620108694*^9, 3.5407620122693996`*^9}, {
   3.5407621572894*^9, 3.5407621698494*^9}, {3.5407622649393997`*^9, 
   3.5407622724894*^9}, {3.5407625509994*^9, 3.5407625539294*^9}, 
   3.5407628404994*^9, {3.5407631811373997`*^9, 3.5407631847774*^9}, {
   3.5407638581274*^9, 3.5407638642874002`*^9}, {3.5407639867474003`*^9, 
   3.5407639944874*^9}, {3.5409021567874002`*^9, 3.5409021699004*^9}, {
   3.5409024156774*^9, 3.5409024266174*^9}, {3.5418962414278*^9, 
   3.5418962696868*^9}, {3.5656325796591997`*^9, 3.5656326739922*^9}, {
   3.5656408345022*^9, 3.5656408553482*^9}, {3.5656409296572*^9, 
   3.5656409311712*^9}, {3.5656410927402*^9, 3.5656410944161997`*^9}, {
   3.5656569424672003`*^9, 3.5656569628112*^9}, {3.5656570735601997`*^9, 
   3.5656570739122*^9}, {3.5656574485282*^9, 3.5656574568632*^9}, {
   3.5656575081242*^9, 3.5656575201902*^9}, 3.5657051296362*^9, {
   3.5657051636382*^9, 3.5657051928532*^9}, {3.5657052255741997`*^9, 
   3.5657052641122*^9}, {3.5657053266972*^9, 3.5657053442622004`*^9}, {
   3.5657055039852*^9, 3.5657056261212*^9}, {3.5657056569862003`*^9, 
   3.5657058077992*^9}, 3.5657058821342*^9, {3.5657059386932*^9, 
   3.5657060435632*^9}, {3.5657299619792*^9, 3.5657300165132*^9}, {
   3.5657300566001997`*^9, 3.5657300661042*^9}, {3.5657301133512*^9, 
   3.5657301772592*^9}, {3.5657304984102*^9, 3.5657305008901997`*^9}, {
   3.5657309373652*^9, 3.5657309378262*^9}, {3.5657310008402*^9, 
   3.5657310015192003`*^9}, {3.5657311318742*^9, 3.5657311407702*^9}, {
   3.5657311733612003`*^9, 3.5657311765502*^9}, {3.5657312242911997`*^9, 
   3.5657312298772*^9}, {3.5657312702562*^9, 3.5657312727232*^9}, {
   3.5657319452152*^9, 3.5657319568372*^9}, 3.5657319998802*^9, {
   3.5657320713552*^9, 3.5657321651472*^9}, 3.5657321965702*^9, {
   3.5657322322392*^9, 3.5657322623932*^9}, {3.5657325516442003`*^9, 
   3.5657325518982*^9}, {3.6712176938363323`*^9, 3.671217696299267*^9}, {
   3.671218175845212*^9, 3.671218176155036*^9}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Add Poles and Choose", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.5656330942762003`*^9, 3.5656330964582*^9}, {
  3.565800226807107*^9, 3.565800228918049*^9}}],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"DomainIntegrate", "[", "0.", "]"}], ":=", "0"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"tcrit", "=", "8"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"NLS", "[", "i_", "]"}], "[", 
    RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "rhp1", ",", "rhp2", ",", "scale", ",", "\[CapitalPhi]1", ",", 
       "\[CapitalPhi]2", ",", "\[CapitalPhi]", ",", "out", ",", "t1", ",", 
       "t2", ",", "t3", ",", "t4", ",", "poles"}], "}"}], ",", 
     "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"i", "\[Equal]", "2"}], ",", 
        RowBox[{"scale", "=", "True"}], ",", 
        RowBox[{"scale", "=", "False"}]}], "]"}], ";", "\[IndentingNewLine]", 
      
      RowBox[{"RHSolved", "//", "Clear"}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"RHSolved", "[", "X_", "]"}], ":=", 
       RowBox[{
        RowBox[{"RHSolved", "[", "X", "]"}], "=", 
        RowBox[{"RHSolve", "[", "X", "]"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"RHSolved", "[", 
        RowBox[{"{", "}"}], "]"}], ":=", "0."}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{
        RowBox[{"RHSolution", "[", "X_", "]"}], "[", "z_", "]"}], ":=", 
       RowBox[{
        RowBox[{"Cauchy", "[", 
         RowBox[{
          RowBox[{"X", "//", "RHSolved"}], ",", "z"}], "]"}], "+", 
        RowBox[{"IdentityMatrix", "[", "2", "]"}]}]}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{"scale", ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"t1", "=", 
          RowBox[{"Timing", "[", 
           RowBox[{
            RowBox[{"rhp1", "=", 
             RowBox[{
              RowBox[{"ConjugateList", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"Jadapt", "[", "i", "]"}], "[", 
                 RowBox[{"x", ",", "t"}], "]"}], ",", 
                RowBox[{"Q", "[", 
                 RowBox[{"x", ",", "t"}], "]"}]}], "]"}], "//", 
              "MakeListFun"}]}], ";"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"t2", "=", 
          RowBox[{"Timing", "[", 
           RowBox[{
            RowBox[{"RHSolved", "[", "rhp1", "]"}], ";"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"\[CapitalPhi]1", "[", "k_", "]"}], ":=", 
          RowBox[{
           RowBox[{
            RowBox[{"RHSolution", "[", "rhp1", "]"}], "[", 
            RowBox[{
             RowBox[{"InvScale", "[", 
              RowBox[{"x", ",", "t"}], "]"}], "[", "k", "]"}], "]"}], "//", 
           "Inverse"}]}], ";"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"t1", "=", 
          RowBox[{"Timing", "[", 
           RowBox[{
            RowBox[{"rhp1", "=", 
             RowBox[{
              RowBox[{"ConjugateList", "[", 
               RowBox[{
                RowBox[{
                 RowBox[{"Jadapt", "[", "i", "]"}], "[", 
                 RowBox[{"x", ",", "t"}], "]"}], ",", 
                RowBox[{"Qn", "[", 
                 RowBox[{"x", ",", "t"}], "]"}]}], "]"}], "//", 
              "MakeListFun"}]}], ";"}], "]"}]}], ";", "\[IndentingNewLine]", 
         RowBox[{"t2", "=", 
          RowBox[{"Timing", "[", 
           RowBox[{
            RowBox[{"RHSolved", "[", "rhp1", "]"}], ";"}], "]"}]}], ";", 
         "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"\[CapitalPhi]1", "[", "k_", "]"}], ":=", 
          RowBox[{
           RowBox[{
            RowBox[{"RHSolution", "[", "rhp1", "]"}], "[", "k", "]"}], "//", 
           "Inverse"}]}], ";"}]}], "\[IndentingNewLine]", "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"domainOutput", " ", "=", " ", 
       RowBox[{"DomainPlot", "[", "rhp1", "]"}]}], ";", "\[IndentingNewLine]",
       "\[IndentingNewLine]", 
      RowBox[{"rhp2", "=", " ", 
       RowBox[{"ConjugateList", "[", 
        RowBox[{
         RowBox[{"PoleListn", "[", 
          RowBox[{"x", ",", "t"}], "]"}], ",", 
         RowBox[{"Qn", "[", 
          RowBox[{"x", ",", "t"}], "]"}]}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"i", ">", "1"}], ",", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"rhp2", "=", " ", 
          RowBox[{"ConjugateList", "[", 
           RowBox[{"rhp2", ",", 
            RowBox[{"\[CapitalPhi]tin", "[", 
             RowBox[{"x", ",", "t"}], "]"}]}], "]"}]}], ";"}]}], 
       "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
      RowBox[{"t3", "=", 
       RowBox[{"Timing", "[", 
        RowBox[{
         RowBox[{"rhp2", "=", " ", 
          RowBox[{
           RowBox[{"ConjugateList", "[", 
            RowBox[{"rhp2", ",", "\[CapitalPhi]1"}], "]"}], "//", 
           "MakeListFun"}]}], ";"}], "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"t4", "=", 
       RowBox[{"Timing", "[", 
        RowBox[{
         RowBox[{"RHSolved", "[", "rhp2", "]"}], ";"}], "]"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"\[CapitalPhi]2", " ", "=", " ", 
       RowBox[{"RHSolution", "[", "rhp2", "]"}]}], ";", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"\[CapitalPhi]", "[", "k_", "]"}], ":=", 
       RowBox[{
        RowBox[{
         RowBox[{"RHSolution", "[", "rhp2", "]"}], "[", "k", "]"}], ".", 
        RowBox[{
         RowBox[{"RHSolution", "[", "rhp1", "]"}], "[", "k", "]"}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{
        RowBox[{"rhp2", "\[NotEqual]", 
         RowBox[{"{", "}"}]}], ",", 
        RowBox[{"domainOutput", "=", " ", 
         RowBox[{"Show", "[", 
          RowBox[{"domainOutput", ",", 
           RowBox[{"DomainPlot", "[", "rhp2", "]"}]}], "]"}]}]}], "]"}], ";", 
      "\[IndentingNewLine]", "\[IndentingNewLine]", 
      RowBox[{"out", "=", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             RowBox[{"DomainIntegrate", "[", 
              RowBox[{"RHSolved", "[", "rhp1", "]"}], "]"}], " ", 
             RowBox[{"If", "[", 
              RowBox[{"scale", ",", 
               RowBox[{"1", "/", 
                RowBox[{"Sqrt", "[", "t", "]"}]}], ",", "1"}], "]"}]}], "+", 
            RowBox[{"DomainIntegrate", "[", 
             RowBox[{"RHSolved", "[", "rhp2", "]"}], "]"}]}], ")"}], "[", 
          RowBox[{"[", 
           RowBox[{"1", ",", "2"}], "]"}], "]"}]}], "/", "Pi"}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"timestring", "=", " ", 
       RowBox[{"\"\<Region: \>\"", " ", "<>", 
        RowBox[{"ToString", "[", "i", "]"}], "<>", "\"\< (\>\"", "<>", 
        RowBox[{"ToString", "[", "x", "]"}], "<>", "\"\<,\>\"", "<>", 
        RowBox[{"ToString", "[", "t", "]"}], "<>", "\"\<) \>\"", "<>", 
        "\"\<1) Construct: \>\"", " ", "<>", " ", 
        RowBox[{"ToString", "[", 
         RowBox[{"t1", "//", "First"}], "]"}], "<>", "\"\<  \>\"", "<>", 
        "\"\<1) Solve: \>\"", " ", "<>", " ", 
        RowBox[{"ToString", "[", 
         RowBox[{"t2", "//", "First"}], "]"}], "<>", "\"\<  \>\"", "<>", 
        "\"\<2) Construct: \>\"", " ", "<>", " ", 
        RowBox[{"ToString", "[", 
         RowBox[{"t3", "//", "First"}], "]"}], "<>", "\"\<  \>\"", "<>", 
        "\"\<2) Solve: \>\"", " ", "<>", " ", 
        RowBox[{"ToString", "[", 
         RowBox[{"t4", "//", "First"}], "]"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"If", "[", 
       RowBox[{"timeflag", ",", 
        RowBox[{
         RowBox[{"Print", "[", "timestring", "]"}], ";"}]}], "]"}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
       "out", ",", "\[CapitalPhi]", ",", "rhp1", ",", "rhp2", ",", 
        "timestring"}], "}"}]}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"NLSAuto", "[", 
   RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
  RowBox[{"\[Piecewise]", GridBox[{
     {
      RowBox[{
       RowBox[{
        RowBox[{"NLS", "[", "0", "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "[", 
       RowBox[{"[", "1", "]"}], "]"}], 
      RowBox[{
       RowBox[{
        RowBox[{"Abs", "[", "x", "]"}], "<=", "5"}], "&&", 
       RowBox[{"t", "<", ".1"}]}]},
     {
      RowBox[{
       RowBox[{
        RowBox[{"NLS", "[", "2", "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "[", 
       RowBox[{"[", "1", "]"}], "]"}], 
      RowBox[{"t", ">", "tcrit"}]},
     {
      RowBox[{
       RowBox[{
        RowBox[{"NLS", "[", "1", "]"}], "[", 
        RowBox[{"x", ",", "t"}], "]"}], "[", 
       RowBox[{"[", "1", "]"}], "]"}], "True"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxItemSize->{
     "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.84]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], 
   "\[IndentingNewLine]"}]}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5656331021261997`*^9, 3.5656331507042*^9}, {
   3.5656332214231997`*^9, 3.5656332560642*^9}, {3.5656333681882*^9, 
   3.5656335261731997`*^9}, {3.5656335846042*^9, 3.5656335872632*^9}, {
   3.5656336435452003`*^9, 3.5656336860112*^9}, {3.5656338535052*^9, 
   3.5656338537632*^9}, {3.5656339276542*^9, 3.5656339286672*^9}, {
   3.5656592020192003`*^9, 3.5656592088892*^9}, {3.5656598958532*^9, 
   3.5656598966331997`*^9}, {3.5656619660292*^9, 3.5656620028492002`*^9}, 
   3.5656620931552*^9, {3.5656622786682*^9, 3.5656622915782003`*^9}, {
   3.5656634907882*^9, 3.5656635025982*^9}, {3.5656638034382*^9, 
   3.5656638245382*^9}, {3.5656644434452*^9, 3.5656644476521997`*^9}, {
   3.5656646513442*^9, 3.5656646589042*^9}, {3.5657047691472*^9, 
   3.5657047774302*^9}, {3.5657088696521997`*^9, 3.5657088700362*^9}, {
   3.5657089777462*^9, 3.5657090848772*^9}, {3.5657091151042*^9, 
   3.5657094998262*^9}, {3.5657095724021997`*^9, 3.5657095743862*^9}, {
   3.5657097344792*^9, 3.5657099280262003`*^9}, {3.5657109429972*^9, 
   3.5657110409142*^9}, {3.5657116229172*^9, 3.5657116266252003`*^9}, {
   3.5657271296342*^9, 3.5657272376172*^9}, {3.5657272685972*^9, 
   3.5657274588002*^9}, {3.5657275441552*^9, 3.5657275569462*^9}, {
   3.5657276169692*^9, 3.5657277268622*^9}, {3.5657277573022003`*^9, 
   3.5657277899292*^9}, {3.5657278648361998`*^9, 3.5657278652111998`*^9}, {
   3.5657279281822*^9, 3.5657280276302*^9}, 3.5657280680662003`*^9, {
   3.5657288935922003`*^9, 3.5657289195042*^9}, {3.5657303695222*^9, 
   3.5657304360642*^9}, {3.5657305933142*^9, 3.5657306444212*^9}, {
   3.565737475873736*^9, 3.5657374771709967`*^9}, {3.5657470873494787`*^9, 
   3.565747115534953*^9}, {3.5657549388660297`*^9, 3.565754942524865*^9}, {
   3.565754991522559*^9, 3.565754996829166*^9}, {3.5657555443336363`*^9, 
   3.5657555447033873`*^9}, {3.565797002050454*^9, 3.565797002546124*^9}, {
   3.5658025961582413`*^9, 3.565802605219771*^9}, {3.568778329962172*^9, 
   3.56877833620391*^9}, {3.568778439786294*^9, 3.568778445013896*^9}, {
   3.5687785062186613`*^9, 3.568778520944726*^9}, {3.568778613432576*^9, 
   3.5687786221517963`*^9}, {3.568779193115464*^9, 3.568779202270108*^9}, {
   3.568779347425478*^9, 3.568779347730536*^9}, {3.67121868426208*^9, 
   3.671218687058913*^9}, {3.671218719103716*^9, 3.6712187274824753`*^9}, {
   3.6712198960221577`*^9, 3.6712199301347723`*^9}, {3.671220361854546*^9, 
   3.6712203880353193`*^9}}],

Cell[BoxData[
 RowBox[{"(*", 
  RowBox[{
   RowBox[{
    RowBox[{"NLS", "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        RowBox[{"DomainIntegrate", "[", 
         RowBox[{"RHSolveTop", "[", 
          RowBox[{"JNLS", "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "]"}], 
        "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], "/", 
      RowBox[{"(", " ", "\[Pi]", "\[NonBreakingSpace]", ")"}]}], "*", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"t", ">", "tcrit"}], ",", 
       RowBox[{"1", "/", 
        RowBox[{"Sqrt", "[", "t", "]"}]}], ",", "1"}], "]"}]}]}], ";", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"NLSSelect", "[", "i_", "]"}], "[", 
     RowBox[{"x_", ",", "t_"}], "]"}], ":=", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        RowBox[{"DomainIntegrate", "[", 
         RowBox[{"RHSolveTop", "[", 
          RowBox[{
           RowBox[{"G", "[", "i", "]"}], "[", 
           RowBox[{"x", ",", "t"}], "]"}], "]"}], "]"}], 
        "\[LeftDoubleBracket]", "2", "\[RightDoubleBracket]"}]}], "/", 
      RowBox[{"(", " ", "\[Pi]", "\[NonBreakingSpace]", ")"}]}], "*", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"i", "\[Equal]", "2"}], ",", 
       RowBox[{"1", "/", 
        RowBox[{"Sqrt", "[", "t", "]"}]}], ",", "1"}], "]"}]}]}], ";"}], 
  "*)"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5656344982122*^9, 3.5656345131792*^9}, {
   3.5656346663412*^9, 3.5656346798782*^9}, 3.5656347371862*^9, {
   3.5656361196472*^9, 3.5656361466612*^9}, {3.5656363575992002`*^9, 
   3.5656363579532003`*^9}, {3.5656363913002*^9, 3.5656363914882*^9}, {
   3.5656364453812*^9, 3.5656364470552*^9}, {3.5656364934222*^9, 
   3.5656364977332*^9}, 3.5657096816722*^9}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Finish", "Section",
 InitializationCell->True,
 CellChangeTimes->{{3.5463644907164*^9, 3.5463644913004*^9}}],

Cell[BoxData[
 RowBox[{
  RowBox[{"End", "[", "]"}], ";"}]], "Input",
 InitializationCell->True,
 CellChangeTimes->{{3.5463644931024*^9, 3.5463644945224*^9}}]
}, Open  ]]
},
AutoGeneratedPackage->Automatic,
WindowSize->{720, 851},
WindowMargins->{{Automatic, 0}, {Automatic, 0}},
FrontEndVersion->"10.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (December 4, \
2014)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[545, 20, 408, 9, 63, "Input"],
Cell[CellGroupData[{
Cell[978, 33, 125, 2, 64, "Section",
 InitializationCell->True],
Cell[1106, 37, 10022, 178, 1508, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[11165, 220, 162, 3, 64, "Section",
 InitializationCell->True],
Cell[11330, 225, 11102, 273, 811, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[22469, 503, 172, 2, 64, "Section"],
Cell[CellGroupData[{
Cell[22666, 509, 133, 2, 44, "Subsection",
 InitializationCell->True],
Cell[22802, 513, 15327, 438, 1225, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[38166, 956, 195, 3, 36, "Subsection",
 InitializationGroup->True],
Cell[38364, 961, 16455, 538, 1248, "Input",
 InitializationGroup->True]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[54868, 1505, 179, 3, 50, "Section",
 InitializationCell->True],
Cell[55050, 1510, 13000, 334, 896, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[68087, 1849, 199, 3, 50, "Section",
 InitializationCell->True],
Cell[68289, 1854, 10403, 211, 539, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[78729, 2070, 198, 3, 50, "Section",
 InitializationCell->True],
Cell[78930, 2075, 3093, 90, 97, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[82060, 2170, 129, 2, 64, "Section",
 InitializationCell->True],
Cell[82192, 2174, 4207, 121, 221, "Input",
 InitializationCell->True],
Cell[86402, 2297, 4488, 126, 221, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[90927, 2428, 180, 3, 50, "Section",
 InitializationCell->True],
Cell[91110, 2433, 29883, 814, 1679, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[121030, 3252, 237, 4, 50, "Section",
 InitializationCell->True],
Cell[121270, 3258, 25873, 682, 1475, "Input",
 InitializationCell->True]
}, Closed]],
Cell[CellGroupData[{
Cell[147180, 3945, 145, 2, 50, "Section",
 InitializationCell->True],
Cell[147328, 3949, 9005, 261, 351, "Input",
 InitializationCell->True],
Cell[156336, 4212, 21353, 482, 998, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[177726, 4699, 181, 3, 64, "Section",
 InitializationCell->True],
Cell[177910, 4704, 11853, 276, 818, "Input",
 InitializationCell->True],
Cell[189766, 4982, 1847, 49, 80, "Input",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[191650, 5036, 114, 2, 64, "Section",
 InitializationCell->True],
Cell[191767, 5040, 158, 4, 28, "Input",
 InitializationCell->True]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
